#!/usr/bin/perl

## agent-stat.pl
## (c) Gnther Alka  2010-2023
## Jan 11, 2023, add fswatch/fswatch2

## debug: start 'perl /var/web-gui/data/napp-it/zfsos/_lib/illumos/agents/agent_stat.pl iostat' manually from console
## print of background tasks -> /tmp/crout...a
## disable all prints after debugging

# info: stat.pl writes unmodified realtime data from stat-tools like iostst to ex. /tmp/iostat.log
# plus average data to ex iostat-lasth.log

#define global var and settings

  use strict;
  use vars qw($par $pid @par $old $date_all $ydat $date_us $date_and_time $isdst $sec $min $hour $mday $mon $year $wday $dow $date $t @t $f %data @datas $debug);
  $old='';
  $pid=$$;
  print "pid=$pid\n";

  $par=$ARGV[0];
  if (!(defined($par))) { $par=""; }
  if ($par eq "") { print "no stat given\n"; exit; }

  $debug=$ARGV[1];
  if (!(defined($debug))) { $debug="0"; }
  if ($debug ne "1")      { $debug="0"; }

  # debug = 1, print to console

  use Fcntl qw(:flock);                                                     # filelocking
  $|=1;                                                                     # keine Pufferung, no buffering


  use strict;
  #use warnings;


$t=`ps axw | grep agent_stat.pl | grep $par | grep -v grep`;              # all stats
#print "agent_stat.pl $par\n$t";

@t=split(/\n/,$t);

if ($par eq "poolstat") {                                                 # only one instance
     if (@t > 1) { print "poolstatservice is already running\n"; exit; }
     &poolstat;
}

if ($par eq "arcstat") {                                                  # only one instance
     if (@t > 1) { print "arcstatservice is already running\n"; exit; }
     &arcstat;
}

if ($par eq "iostat_60s") {                                               # only one instance
     if (@t > 1) { print "iostatservice 60s is already running\n"; exit; }
     &iostat_60s;
}

if ($par eq "iostat_long") {                                              # only one instance
     if (@t > 1) { print "iostat_longservice is already running\n"; exit; }
     &iostat_long;
}

if ($par eq "fsstat") {                                                   # only one instance
     if (@t > 1) { print "fsstatservice is already running\n"; exit; }
     &fsstat;
}

if ($par eq "nicstat") {                                                  # only one instance
     if (@t > 1) { print "nicstatservice is already running\n"; exit; }
     &nicstat;
}

if ($par eq "prstat") {                                                   # only one instance
     if (@t > 1) { print "prstatservice is already running\n"; exit; }
     &prstat;
}

if ($par eq "zilstat") {                                                  # only one instance
     if (@t > 1) { print "zilstatservice is already running\n"; exit; }
     &zilstat;
}

if ($par =~/fswatch/) {                                                  # only one instance
     if (@t > 1) { print "Fswatch is already running, kill in sub\n";  }
     &fswatch;
}

print "no valid stat given: iostat, prstat, netstat, arczilstat, fsstat, fswatch\n"; exit;

######
sub fswatch {
    # create /var/web-gui/_log/monitor/fswatch.day.hour.log (one av value per min)
    my ($t,@p,$p,@w,$w,$tm,@n,$n,$m,$i,$do,$fi,$fi2,$last,$alert,$count);

    # settings /var-web_gui/_log/fswatch.cfg
    # select_mode::select_alert::value_folders1::value_folders2::value_folders3::value_include::value_exclude::value_options::value_events::
    #           0             1               2               3               4              5              6              7             8

    $p=`cat /var/web-gui/_log/fswatch.cfg`;   chomp($p);
    print "config $p\n";

    @p=split(/::/,$p);

    #alert#
    $alert=$p[1];

    $do="/usr/local/bin/fswatch"; #
    # add fswatch options

    #add options
    if ($p[7] ne "") {
      $do.=" $p[7]";
    }

    #include
    if ($p[5] ne "") {
      $do.=" -i \"$p[5]\"";
    }

    # exclude
    if ($p[6] ne "") {
      $do.=" -e \"$p[6]\"";
    }

    # events
    if ($p[8] ne "") {
      $do.=" $p[8]";
    }

    #add folders
    $t="$p[2]";                     # ex "/tank/data 1"
    if ($p[3] ne "") { $t.=" $p[3]"; }
    if ($p[4] ne "") { $t.=" $p[4]"; }

    $do.=" $t";

    #$do="/usr/local/bin/fswatch -t --event 68 --recursive --numeric /daten1/guenther/test";

    # kill all fswatch
    $w=`ps axw | grep fswatch | grep -v agent_stat | grep -v grep`;      # all fswatch beside agents
    @t=split(/\n/,$w);
    foreach my $t (@t) {
      print "kill $t\n";
      chomp $t;
      $t=~s/^ +//;   #leading spaces
      $t=~s/ .*//s;  # all after pid
      `kill -9 $t > /dev/null 2>&1`;
    }

    print "$do\n";
    open (T,"-|","$do");         # request fswatch
         while (my $line=<T>)    {
               &getmydate;
               $tm=time();

               $fi="/var/web-gui/_log/monitor/fswatch.$mday\.log";      # logfile
               $i= -M "$fi";                                            # 1.0 for age=1day

               if ($i>27) {
                  `rm /var/web-gui/_log/monitor/fswatch.$mday\.log`;    # delete if current logfile older 27 days
                   print "delete $fi\n";                                # keep current month
               }

               if ($line ne $last) {

                       open (P, ">>$fi");
                                         flock(P, LOCK_EX);
                                         print P "$line";
                       close (P);
                       print "$line";         # end with nl

                       $count++;
               }
               $last=$line;

               #alert?
               $t=time();
               if (($t-$tm) >60) { $tm=time(); $count="0"; }            # reset error counter

               # generate error
               if ($count > $alert) {
                   print "alert \n";
                  `echo "alert" > /tmp/fswatch.alert`;
               }


         }
    close (T);
    exit;

}


###########  iostat 60s -> last 60min, 24h (last 7 days of week)
sub iostat_long {
###########
 # read /tmp/nappit/iostat60 (60s)
 # create /var/web-gui/_log/monitor/iostat.day.hour.log (one av value per min)

 my (@p,$p,@w,$w,$tm,@n,$n,$m,$i);
 my $current_dow;                                                     # mon,.. sun

 $m="0";
 while (1) {
        $tm=time();
        &getmydate;                                                   # $sec,$min,$hour
        $current_dow=&getdow;

        # once per minute
        # read last 60s every minute
        @datas=();
        if (-f "/tmp/iostat_60s.log") {                               # last 60s
           open (F,"</tmp/iostat_60s.log");
             flock(F, LOCK_EX);
             @datas=<F>;                                              # data sec, 0..59s
           close (F);

           if ($debug) { print "111 iostat long, line 0 =$datas[0]  \nfirst value=$datas[1]\n"; }

           #too old, older 70s, wait for newer data
           if ($tm-$datas[0] > 70) {
             sleep(10);
             next;                                                    # try later
           }
           #no valid data, wait for valid data
           if (!($datas[0] > 0)) {
             sleep(10);
             next;                                                    # try later
           }

           # update valid data for current minute
           # only one update per minute
           if ($min ne $data{'last_min'}) {
                 $data{'last_min'}=$min;
                 &iostat_this_hour();
           }

           # update valid data for current day
           # a value every 24 min
           # on even hours (00,02,..) on min=00, 24 and 48 and
           # on odd hours (01,03,..) on min 12 and 36
           if (("00,24,48"=~/$min/) && ("00,02,04,06,08,10,12,14,16,16,20,22"=~/$hour/)) {
                 &iostat_this_day();
           }
           if (("12,36"=~/$min/)    && ("01,03,05,07,09,11,13,15,17,19,21,23"=~/$hour/)) {
                 &iostat_this_day();
           }

        }  else {
                 if ($debug) { print "139 /tmp/iostat_60s.log not available\n";    }
        }

        $t=61-$sec;
        sleep($t);                             # wait to next minute

 }
}

###################### 00..59 minutes for mday days
sub iostat_this_hour {
######################
           my ($i,$k,$t,@w,$w,@p,$p,@av,%next,@datam,%lastm,@min,@v,@new);

           #@datas => current minute (iostat60.log)                    # data of current hour (00..23
           # create or update hour.log

           # struct of hour.log
           # 03                                # last minute values
           # cpu;idle; 1,2,3,4                 # values of minutes from above
           # rpool;read_60s;0,3,4,7            # ..

           # read parameter from datas
           @par=();
           @av=();
           foreach $p (@datas) {
                if (!($p=~/;/)) { next; }      # no par
                @w=();
                @w=split(/;/,$p);

                $k="$w[0]"."_$w[1]";           # ex rpool_read60s
                push(@par,$k);

                # push average data to @av
                $data{'this_h'}=$w[2];

                # average data current min of par
                $next{$k}=&get_avrg_data('this','h');                 # -> this_h

                # save current next value for day.all.log ex in $data{'current_rpool_data60'}
                $i="current_$k";
                $data{$i}=$next{$k};
           }

           ########
           # read current hour to update once per minute
           @datam=();
           if (-f "/var/web-gui/_log/monitor/iostat.$mday.$hour.log") {
                 open (H,"</var/web-gui/_log/monitor/iostat.$mday.$hour.log");
                   flock(H, LOCK_EX);
                   @datam=<H>;                                        # data min 0 ..59 minutes
                 close (H);
           }
           if ($debug) { print "187 iostat long read hour $hour, line 0 =$datam[0]  \nfirst value=$datam[1], arraysize: @datam\n"; }

           # valid data means $datam[0] ends with min-1   (ex 00,01,02,..59)
           if ($datam[0]=~/^\d+/) {
                $t=$datam[0];                            # date;01:02
                $t=~s/.*://;                             # keep last minute
                if (!($t == ($min-1))) {                 # last value=last minute
                    if ($debug) { print "197 iostat date $datam[0] too old\n"; }
                    @datam=();
                    $datam[0]="$date_us;$hour\:$min";

                }
           }  else {
                if ($debug) { print "201 iostat no date\n"; }

                # set current date; hour:minute
                @datam=();
                $datam[0]="$date_us;$hour\:$min";             # average for this minute (=last 60s)
           }

           # read last min values
           $i="1";
           foreach my $m (@datam) {
                chomp($m);
                if ($i) { $i="0"; next; }                # skip first line (date)
                @w=();
                @w=split(/;/,$m);                        # ex rpool;read;0,1,2

                $k="$w[0]"."_$w[1]";                     # ex rpool_read
                $lastm{$k}=$w[2];                        # data
                if ($debug) { print "213 iostat k=$k -> lastm=$lastm{$k} \n"; }
           }

           @new=();
           push(@new,"$date_us;$hour\:$min\n");

           # process all devices
           foreach my $p (@par) {
                # last minutes + new
                $t=$p;
                $t=~s/_/;/;                              # rpool_write -> rpool;write
#                if ($debug) { print "224 iostat p=$p -> lastm=$lastm{$p} \n"; }

                if ($lastm{$p} ne "") {                  # we have old data

                     push (@new,"$t;$lastm{$p},$next{$p}\n");
                     if ($debug) { print "227 iostat $p\: $lastm{$p},$next{$p} \n"; }

                } else {

                   # set 0 for older hours
                     @v=();
                     $v[$min]=$next{$p};

                     foreach my $v (@v) {
                             if ($v eq "") { $v="0"; }
                     }
                     $next{$p}=join(",",@v);
                     push (@new,"$t;$next{$p}\n");      # rpool;write;1,2,3
#                     if ($debug) { print "240 iostat next only: $next{$p} \n"; }
                }
           }

           # write new hour data
           open (P, ">/var/web-gui/_log/monitor/iostat.$mday.$hour.log");
                flock(P, LOCK_EX);
                print P @new;
           close (P);
}


######################  01..31 last 4
sub iostat_this_day {
######################
     my ($i,$k,$t,@w,$w,@p,$p,@par,%next,@datad,%lastd,@min,@v,@new);

     # this one runs every 24 min and updates the 'day'.all.log
     # as we need 5 values per 2 hour, we runn it
     # on even hours (00,02,..) on min=00, 24 and 48 and
     # on odd hours (01,03,..) on min 12 and 36
     # with 60 values (same graph than hourly), one per 24min  (last average 60s)

     # 1. check if last day log is older 1 day: delete then
     #    line 0 ex: 01.28.2017;06:59
     # 2. read last hour values
     ##########################

     # day data-array:
     # 0=0h:0, 1=0h:24, 2=0h:48, 3=1h:12, 4=1h:36, 5=2h:00, 6=2h;24
     # i=(h*24*2,5) + min/24

      $i=(($hour*24*2.5)+$min)/24;                             # pos of new value


       @datad=();
       if (-f "/var/web-gui/_log/monitor/iostat.$mday.all.log") {
           #
           open (S,"</var/web-gui/_log/monitor/iostat.$mday.all.log");
              flock(S, LOCK_EX);
              @datad=<S>;                                        # data min 0 ..59 minutes
           close (S);

           # not this month.day ex 01.28.2017;06:59
           if (!($datad[0]=~/^$mon\.$mday/)) {
               @datad=();
           }
       }

       # pre-set all devices without values
       if (@datad==0) {
               @datad=@datas;                  # same structure tkan 60s
               foreach my $d (@datad) {
                     $d=~s/\d+,.*//;           # renove all data
               }
       }

       # struct of day.all.log
       # 01.28.2017;06:59                      # last values
       # cpu;idle; 1,2,3,4                     # values of av minutes (max 60 values)
       # rpool;read_60s;0,3,4,7                # a value per 24 min, last value is per 12 min if min = 11, 35, 59

       # # add current next value for day.all.log ex from $data{'current_rpool_data60'}

       $datad[0]="";
       foreach my $d (@datad) {
                chomp($d);                     # no nl

                if (!($d=~/;/)) { next; }      # no par
                @w=();
                @w=split(/;/,$d);

                $k=$w[0]."_$w[1]";             # key

                @new=();                       # new data
                @new=split(/,/,$w[2]);

                $#new=$i;                      # trancate to i; next=newest

                $t="current_$k";
                push(@new,$data{$t});          # add current minute average


                #empty values=0
                foreach my $n (@new) {
                  if (!($n=~/\d/)) { $n="0"; }
                }
                $w[2]=join(",",@new);         # new data
                $d=join(";",@w);
                if ($debug) { print "342, new data: $w[2]<br>"; }
       }

       $datad[0]="$mon.$mday.$year".";$hour\:$min";

       #add nl
       foreach my $d (@datad) {
         chomp($d);
         $d.="\n";
       }

       # write new hour data
       open (P, ">/var/web-gui/_log/monitor/iostat.$mday.all.log");
          flock(P, LOCK_EX);
          print P @datad;
       close (P);
}

###########
 sub zilstat {
###########

#   N-Bytes  N-Bytes/s N-Max-Rate    B-Bytes  B-Bytes/s B-Max-Rate    ops  <=4kB 4-32kB >=32kB
#         0          0          0          0          0          0      0      0      0      0
#         0          0          0          0          0          0      0      0      0      0

#        N-Bytes    - data bytes written to ZIL over the interval
#        N-Bytes/s  - data bytes/s written to ZIL over ther interval
#        N-Max-Rate - maximum data rate during any 1-second sample
#        B-Bytes    - buffer bytes written to ZIL over the interval
#        B-Bytes/s  - buffer bytes/s written to ZIL over ther interval
#        B-Max-Rate - maximum buffer rate during any 1-second sample
#        ops        - number of synchronous iops per interval
#        <=4kB      - number of synchronous iops <= 4kBytes per interval
#        4-32kB     - number of synchronous iops 4-32kBytes per interval
#        >=32kB     - number of synchronous iops >= 32kBytes per interval
#    note: data bytes are actual data, total bytes counts buffer size

   my (@w,$w);
   open (T,"-|","ksh /var/web-gui/data/napp-it/CGI/zilstat.ksh 20");                   # interval 20s
         while (my $line=<T>)
         {

                    chomp($line);
                    if ($line=~/Bytes/) { next; }  # header

                    &getmydate;
                    $line.="   $hour:$min $sec s";

                    push (@w,$line);
                    if (@w > 31) {
                        shift @w;
                    }

                    open (PF, ">/tmp/zilstat.log");
                          flock(PF, LOCK_EX);
                          print PF "   N-Bytes  N-Bytes/s N-Max-Rate    B-Bytes  B-Bytes/s B-Max-Rate    ops  <=4kB 4-32kB >=32kB  Time \n";
                          foreach $w (@w) {
                             print PF "$w\n";
                          }
                    close (PF);
         }
    close (T);
    exit;
 }





########## pool and zfs stat
sub poolstat {
##########
  my ($r,$t);
  while (1) {

       #only if accelerate is not running
       $t=`ps axw | grep agent_acc.pl | grep zfs| grep -v grep`;
       #if (!($t=~/agent_acc.pl zfsxx/)) {                             # opt: run independent from acc

              $r=`/usr/bin/nice /usr/sbin/zpool status`;
              $t=time();
              open (PF, ">/tmp/poolstat.log");
                        flock(PF, LOCK_EX);
                        print PF "$t\n";
                        print PF "$r";
              close (PF);

              #add zfsstat
              #$r=`/usr/sbin/zfs list -o name,used,avail,refer`;
              #$r=`/usr/sbin/zfs list -H -o name,avail,used,readonly,reservation,refreservation,refquota,quota,mountpoint,compression,nbmand,sharesmb,sharenfs -t filesystem`;
              $r=`/usr/bin/nice /usr/sbin/zfs list -o all`;     # NAME                                             TYPE        CREATION                USED  AVAIL  REFER ..
              $r=&parse_mydate($r);

              $t=time();
              open (PF, ">/tmp/zfsstat.log");
                        flock(PF, LOCK_EX);
                        print PF "$t\n";
                        print PF "$r";
              close (PF);
       #}

       #print `cat /tmp/poolstat.log`;
       sleep(60);
  }
}

########## cpu and processes
sub prstat {
##########
   my ($d,$i,@l,$a,@w,$m,$t,$k,$raw,$tm);
   while (1) {
     $i="0";
     $m="";
     $d="";
     $a="";
     open (T,"-|","prstat 1 2");                        # request stat
         while (my $line=<T>)
         {

                chomp($line);
                #print "$line\n";

                if ($line ne "") {
                  $raw.="$line\n";
                }

                $line=~s/^ +//;
                $line=~s/ +/ /g;

                if ($line=~/^\d/) {

                               # pid  user  size  rss  state  pri   nice   time   cpu  name

                               # load average: 1=100%,
                               # 1.5,0.3,0.7 -> values last minute, last 5min, last 15


                               if ($i < 7) {            # max 7 data (%cpu) followed by 1, 5, 15 value

                                        @w=();
                                        @w=split(/ /,$line);
                                        $w[8]=~s/\%//;            # 1.2% - > 1.2

                                        if ($w[9]=~/\w+\.\w+/)    {
                                             $w[9]=~s/.*\.//;     #svc.configd -> configd
                                        }

                                        $w[9]=~s/\/.*//;          # sshs/3 -> sshd

                                        $w[8]=sprintf ("%.1f", $w[8]);  # round 1.55 to 1.6
                                        $d.="$w[8],";             # add data
                                        $a.="$w[9],";             # add label
                                        $i++;
                              }

                } # /\d

                if ($line=~/^Total:/) {
                                        $line=~s/.*averages: *//;
                                        $line=~s/ +//g;
                                        @w=();
                                        @w=split(/,/,$line);                   # 1,5,15 min
                                        $w[0]=sprintf ("%.1f", $w[0]);
                                        $w[1]=sprintf ("%.1f", $w[1]);
                                        $w[2]=sprintf ("%.1f", $w[2]);
                                        $d.="$w[0],$w[1],$w[2]";               # add data
                                        $a.="1min,5min,15min";                 # add label
                                        last;
                 }

        }
        close (T);

        $d=~s/[^0-9\.\,]//g;
        $a=~s///g;
        $m="$a;$d;\n";                 # labels;data

        open (PF, ">/tmp/prstat.log");
                 flock(PF, LOCK_EX);
                 print PF "$m";
        close (PF);

        # raw data
        $tm=time();
        open (PF, ">/tmp/prstat-raw.log");
              flock(PF, LOCK_EX);
              print PF "$tm\n";
              print PF "$raw";
        close (PF);
        $raw="";

        #debug
        # print "$m\n";

        sleep(3);
  } # redo

}

###########
 sub iostat_60s {
###########
#    cpu
# us sy dt id
#  1  3  0 97
#                    extended device statistics
#    r/s    w/s   kr/s   kw/s wait actv wsvc_t asvc_t  %w  %b device
#    0.0    0.0    0.0    0.0  0.0  0.0    0.0    0.0   0   0 rpool
#    0.0    0.0    0.0    0.0  0.0  0.0    0.0    0.0   0   0 c2t0d0
#    0.0    0.0    0.0    0.0  0.0  0.0    0.0    0.0   0   0 c2t1d0
#    ..
#    0.0    0.0    0.0    0.0  0.0  0.0    0.0    0.0   0   0 tank

   my @w;
   my $t="0";
   my $m="";                                         # measurement
   my $dev="cpu";
   my (@a,$k,$a,$s,$tm);


   open (T,"-|","iostat -xcn 1 | grep -v /");         # request stat with CPU, 1s interval? - without mounts
         while (my $line=<T>)    {
              if ($t > 1) {                           # skip first measurement
                    chomp($line);
                    if ($line=~/^ +cpu/) {

                         #$m="    cpu\n$m";

                         $tm=time();
                         open (PF, ">/tmp/iostat.log");
                                flock(PF, LOCK_EX);
                                print PF "$tm\n";
                                print PF "r/s  w/s  kr/s  kw/s  wait  actv  wsvc_t  asvc_t  %w  %b  device  last_60rd  last60wr  last60wait  last60w%  last60b%\n";
                                print PF "$m";

                         close (PF);

                         # debug
                         #  print "stat.pl 164-$t $m\n";


                         # print iostat_60s.log
                         $m="cpu;idle;".$data{'cpu_av60s_idle'}."\n";                                   # last min cpu
                         @a=();
                         @a=split(/,/,$dev);
                         foreach my $d (@a) {
                                 if ($d eq "cpu") { next; }
                                 $a=$d."_av60s_read";  $m.="$d;read60s;" .$data{$a}."\n";          # last min read
                                 $a=$d."_av60s_write"; $m.="$d;write60s;".$data{$a}."\n";          # last min write
                                 $a=$d."_av60s_wait";  $m.="$d;wait60s;" .$data{$a}."\n";          # last min wait

                                 $a=$d."_av60s_wperc";  $m.="$d;av60s_wperc;" .$data{$a}."\n";     # last w%
                                 $a=$d."_av60s_bperc";  $m.="$d;av60s_bperc;" .$data{$a}."\n";     # last b%
                         }

                         #debug
                        #  print "### 184: dev=$dev\n$m\n\n";

                         $tm=time();
                         open (P, ">/tmp/iostat_60s.log");
                                 flock(P, LOCK_EX);
                                 print P "$tm\n";
                                 print P "$m";
                         close (P);

                         $t="2"; $m="";                      # new measurement set

                    } else {                                 # add to last set

                          $s=1;
                          if ($line=~/^ *cpu/)     { $s=0; } # skip cpu
                          if ($line=~/^ *us/)      { $s=0; } # skip cpu header
                          if ($line=~/fd\d+$/)     { $s=0  } # not for fd
                          if ($line=~/^ +extended/){ $s=0  } # comment

                          if ($s) {
                              @w=();
                              $a=$line; $a=~s/^ +//;
                              @w=split(/ +/,$a);

                              if (@w==4) {
                                           $line=~s/.* //;   # CPU idle
                                           $a=100-$line;

                                           #debug
                                           #print "\n##\n185 busy=$a; av=$data{'cpu_av60s_idle'} av10s=".&get_avrg_data('cpu','av60s_idle',5)."\n";

                                           $line="cpu_busy\%(now/av10s) $a";

                                           # add this idle to data{'cpu_av60s_idle'}
                                           &add_to_data('cpu','av60s_idle',$a,60);

                                           # add average last 10s  (this + last 4 = 5) to line;
                                           $line="$line  ".&get_avrg_data('cpu','av60s_idle',5);

                              } else {

                                           # add average 10s to lines
                                           if ($line=~/^ +r\/s/) {  $line.="  av10s_r/ps  av10s_w/s  av10s_wait  av10s_w%  av10s_b%";}

                                           # add this read/write/wait to average ex data{'c3t0d0_av60s_read'};
                                           if ($line=~/ +\d/)   {

                                               # add dev
                                               if (!($dev=~/\b$w[10]\b/)) { $dev.=",$w[10]";}                             # devices

                                               #
                                               $w[0]=sprintf ("%.0f", $w[0]);                                             # integer
                                               $w[1]=sprintf ("%.0f", $w[1]);
                                               $w[4]=sprintf ("%.0f", $w[4]);

                                               &add_to_data($w[10],'av60s_read',$w[0],60);                                # device, key, data, max
                                               &add_to_data($w[10],'av60s_write',$w[1],60);
                                               &add_to_data($w[10],'av60s_wait',$w[4],60);

                                               &add_to_data($w[10],'av60s_wperc',$w[8],60);                               # w%
                                               &add_to_data($w[10],'av60s_bperc',$w[9],60);                               # b%

                                               # add average last 10s  (this + last 4 = 5) to line;
                                               $line="$line  ".&get_avrg_data($w[10],'av60s_read',10);                     # key, n   -> last 10s
                                               $line="$line  ".&get_avrg_data($w[10],'av60s_write',10);
                                               $line="$line  ".&get_avrg_data($w[10],'av60s_wait',10);

                                               $line="$line  ".&get_avrg_data($w[10],'av60s_wperc',10);
                                               $line="$line  ".&get_avrg_data($w[10],'av60s_bperc',10);
                                           }

                                           #debug

                              }
                              $m.="$line\n";    # iostat data
                              #print "## 250 $line\n";
                          }
                    }
              }
              if ($line=~/^ +cpu/) {  $t++; }
         }
    close (T);
    exit;
 }


########### perl /var/web-gui/data/napp-it/zfsos/_lib/illumos/arcstat.pl -f time,read,hits,miss,hit%,l2read,l2hits,l2miss,l2hit%,arcsz,l2size,l2asize 2
sub arcstat {
##########  arc and l2arc
#     time  read  hits  miss  hit%  l2read  l2hits  l2miss  l2hit%  arcsz  l2size  l2asize
# 17:16:16   377   230   147    61       0       0       0       0    43G       0        0

my (@w,$w,$t,$line,@d,@a,@a2,$a,$a2);
@d=();

   open (T,"-|","perl /var/web-gui/data/napp-it/zfsos/_lib/illumos/arcstat.pl -f time,read,hits,miss,hit%,l2read,l2hits,l2miss,l2hit%,arcsz,l2size,l2asize 3");  # last min
         while (my $line=<T>)
         {

                    chomp($line);
                    if ($line=~/time|Changed/) { next; }

                    # add av%
                    if ($line=~/l2asize/) {
                         $line=~s/l2asize/l2asize  last20_hit\%  last20_l2hit\%/;
                    }

                    if ($line=~/^\d/) {                                 # get average
                        @w=();
                        @w=split(/ +/,$line);
                        push (@a,$w[4]);                                # hits in %
                        push (@a2,$w[8]);                               # hits l2 in %
                        while (@a > 20)  { shift(@a);  }                # last 20
                        while (@a2 > 20) { shift(@a2); }

                        #get average
                        $a=0;
                        if (@a >0) {
                          foreach my $d (@a) { $a=$a+$d; }
                          $a=$a/@a;
                          $a=sprintf ("%.0f", $a);
                        }

                        $a2=0;
                        if (@a2 >0) {
                          foreach my $d (@a2) { $a2=$a2+$d; }
                          $a2=$a2/@a2;
                          $a2=sprintf ("%.0f", $a2);
                        }

                        if ($a < 10)  { $a="0$a";   }
                        if ($a2 < 10) { $a2="0$a2"; }

                        $line.="         $a             $a2";
                    }
                    push (@d,$line);
                    while (@d > 20) { shift(@d); }                      # last 20 values

                    $t=join("\n",@d);
                    $t.="\n";

                    open (PF, ">/tmp/arcstat.log");
                               flock(PF, LOCK_EX);
                               print PF"  time  read  hits  miss  hit%  l2read  l2hits  l2miss  l2hit%  arcsz  l2size  l2asize  last20_hit%  last20_l2hit%\n";
                               print PF $t;
                    close (PF);

                    #print "$t";
         }
    close (T);
    exit;


}


###########
 sub nicstat {
###########

#    Time      Int   rMbps   wMbps   rPk/s   wPk/s    rAvs    wAvs %Util    Sat
#15:47:55  e1000g0    0.00    0.00    0.00    0.00   113.8   408.2  0.00   0.00
#15:47:55 vboxnet0    0.00    0.00    0.00    0.00    0.00   93.96  0.00   0.00
#    Time      Int   rMbps   wMbps   rPk/s   wPk/s    rAvs    wAvs %Util    Sat
#15:47:55  e1000g0    0.00    0.00    0.00    0.00   113.8   408.2  0.00   0.00

#or
#15:47:55  e1000g0    0.00    0.00    0.00    0.00   113.8   408.2  0.00   0.00
#15:47:57  e1000g0    0.00    0.00    0.00    0.00   113.8   408.2  0.00   0.00

   my @w;
   my $t="0";
   my $m="";                                         # measurement
   my $n="";
   my (@ar,$k,$d,$l,$w,$tm,$firstnic);
   $d="";
   $l="";
   $w="";
   my $x="100";
   my $raw;

   `chmod 755 /var/web-gui/data/tools/nicstat/nicstat`;
   if (-f "/tmp/nicstat.log")     { `rm /tmp/nicstat.log`; }
   if (-f "/tmp/nicstat-raw.log") { `rm /tmp/nicstat-raw.log`; }

   open (T,"-|","/var/web-gui/data/tools/nicstat/nicstat -Mn 2");                   # request stat with Mb/s
         while (my $line=<T>)
         {

                    chomp($line);
                    $line=~s/^ +//;
                    $line=~s/ +/,/g;
                    if ($line=~/^Time/) { next; }

                    @w=();
                    @w=split(/,/,$line);

                    if ($firstnic eq "") { $firstnic=$w[1]; }        # new measurement start wirh firstnic

                    #print "518 line=$line\nfirstnic=$firstnic\nnic=$w[1]\nm=$m\n\n";

                           ########
                               if (($firstnic eq $w[1]) && ($m ne "")) {                    # next measurement (with one or more nics)

                                     $m=~s/:;/;/gs;
                                     $m=~s/,;/;/gs;

                                     open (PF, ">/tmp/nicstat.log");
                                            flock(PF, LOCK_EX);
                                            print PF "$m";
                                     close (PF);

                                     # raw data
                                     $raw=~s/,/\t/gs;
                                     $tm=time();
                                     open (PF, ">/tmp/nicstat-raw.log");
                                            flock(PF, LOCK_EX);
                                            print PF "$tm\n";
                                            print PF "Time    Int     rMbps   wMbps   rPk/s   wPk/s   rAvs    wAvs    %Util   Sat\n";
                                            print PF "$raw";
                                     close (PF);

                                     # debug, disable on background use!
                                     # print "541 nic=$w[1] first=$firstnic\nm=$m\n\n";

                                     $m=""; $d=""; $l=""; $w=""; $raw="";        # new measurement set
                                }


                    # calculate line

                                      # add to last set
                                      # values in Mbps

                                       $w[2]=sprintf ("%.0f", $w[2]);         # read
                                       $w[3]=sprintf ("%.0f", $w[3]);         # write

                                       while ( $w[2] > $x ) { $x=$x*2  }      # set new x units
                                       while ( $w[3] > $x ) { $x=$x*2  }      # set new x units

                                       $d.=":$w[2],$w[3]";                    # data of link
                                       $d=~s/^://;

                                       $l.=",$w[1]";                          # label e100, vmxnet3
                                       $l=~s/^,//;

                                       $m="$l;$d;$x;Load Mbps/s;\n";
                                      # print "stat 266 m=$m\n";

                                      #raw
                                      if ($line=~/^\d/)  { $raw.="$line\n"; }  # data

         }
    close (T);
    exit;
 }


###########
 sub fsstat {
###########

# new  name   name  attr  attr lookup rddir  read read  write write
# file remov  chng   get   set    ops   ops   ops bytes   ops bytes
#    0     0     0     0     0      0     0     0     0     0     0 ufs
#    0     0     0     0     0      0     0     0     0     0     0 nfs
#    0     0     0   149     0    581     0    45  384K    10 3.33K zfs
#..
#    0     0     0     0     0      0     0     0     0     0     0 autofs

   my @w;
   my $t="0";
   my $m="";                                         # measurement
   my $n="";
   my (@ar,$k,$d,$l,$w,$raw);
   $d="";
   $l="";
   $w="";
   my $x="100";

   open (T,"-|","fsstat -F 3");                   # request stat with CPU, 2s interval
         while (my $line=<T>)
         {

                    chomp($line);
                    $line=~s/^ +//;
                    $line=~s/ +/,/g;

                    if ($m eq "") { $t=0; }
                    #print "231: $line\n";

                    $raw.="$line\n";

                    # only second measurement
                    if ($line=~/ufs$/)   {
                         $t++;
                         $raw="";
                    }

                    if ($line=~/^new/) { $t="0"; }
                    if (($t > 0) && ($line=~/^\d/))           {
                           #
                           if ($w eq "1") {

                                $m=~s/:;/;/gs;
                                $m=~s/,;/;/gs;

                                open (PF, ">/tmp/fsstat.log");
                                       flock(PF, LOCK_EX);
                                       print PF "$m";
                                close (PF);


                                $raw=~s/,/  /gs;
                                open (PF, ">/tmp/fsstat-raw.log");
                                       flock(PF, LOCK_EX);
                                       print PF " new  name   name  attr  attr lookup rddir  read read  write write\n";
                                       print PF " file remov  chng   get   set    ops   ops   ops bytes   ops bytes\n";
                                       print PF "$raw";
                                close (PF);

                               # print "246_$t : $m\n";
                                $m=""; $d=""; $l=""; $w="";  $raw="";        # new measurement set
                                $x="100";

                           } else {
                                                      # add to last set

                                  @w=();
                                  @w=split(/,/,$line);

                                  # values in "" K,M,G
                                  $w[5]=&numeric_data($w[5],'B');
                                  $w[6]=&numeric_data($w[6],'B');
                                  $w[7]=&numeric_data($w[7],'B');
                                  $w[9]=&numeric_data($w[9],'B');

                                  while ( $w[5] > $x ) { $x=$x*2  }      # set new x units
                                  while ( $w[6] > $x ) { $x=$x*2  }      # set new x units
                                  while ( $w[7] > $x ) { $x=$x*2  }      # set new x units
                                  while ( $w[9] > $x ) { $x=$x*2  }      # set new x units

                                  $d.=":$w[5],$w[6],$w[7],$w[9]";        # data of fs
                                  $l.=",$w[11]";                         # label like zfs, nfs
                                  $l=~s/^,//;
                                  $d=~s/^://;

                                  $m="$l;$d;$x;lookup ops,rdir ops, read ops, write ops;\n";
                                  if (($w[11] eq 'nfs4') && ($m=~/zfs/)) {
                                    if ($d=~/:\d/) {                     # not 0,0,0,0: but 0,0,0,0:3,1,7..
                                       $w="1";                           # write
                                       #print "284 stat: $m\n";
                                    }
                                  }



                           }
                     }
         }
    close (T);
    exit;
 }

 ##
sub parse_mydate { # ex  Jan  1:18 -> Jan 01:18 or  Tue  1  -> Tue 01 ex in zfs creation
#############
  my $s=$_[0];
  my @m=('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec');
  for (my $i=0;$i<=9; $i++) { $s=~s/  $i:/ 0$i:/gs; }
  foreach my $m (@m) {  for (my $i=1;$i<=9; $i++) { $s=~s/ $m  $i / $m 0$i /gs; }  }
  return($s);
}


###############  parameter: device, key, data, max      # return none
sub add_to_data  {
###############
  my $key="";
  $key=$_[0]."_$_[1]";                                  # ex c0t0d0_wait
  if ($key eq "")  { return; }

  my $data=$_[2];
  my $max=$_[3];

  $old="";
  $old=$data{$key};
  $old=~s/,+$|^,+//g;

  my @old=split(/,/,$old);

  # add data
  push(@old,$data);
  while ( @old > $max ) { shift @old;   }              # remove first value(s)

  $data{$key}=join(",",@old);
}


###############  parameter: device, key, n  # return average  ex rpool, busy, 60
sub get_avrg_data  {
###############
  my $key="";
  $key=$_[0]."_$_[1]";
  if ($key eq "")  { return; }

  my $n=$_[2];
  $n=~s/\D//g;

  my $data=$data{$key};
  my $i;

  $data=~s/,+$|^,+//g;
  my @data=split(/,/,$data);

  if (@data==0) { return (0); }
  if (!($n>0)) { $n=@data; }                             # default all data

  if ($n==@data) {
       #
       # sum all data
       $data="0";
       foreach my $d (@data) {
            $data=$data+$d;
       }
       $data=$data/@data;                                # average

  } else {    # sum/avarg last n data
       @data=reverse(@data);     # newest top
       $data="0";
       $i=1;
       foreach my $d (@data) {
            if ($i > $n) { last; }
            $data=$data+$d;
            $i++;
       }
       if (@data < $n) { $n=@data; }
       $data=$data/$n;                                   # average
  }

  $data=sprintf ("%.0f", $data);                         # round integer
  return ($data);
}

############  convert 2.9M to 2900 (all values in K)
sub numeric_data {
############
   # asume data is in Bytes ('') or
   # values K,M,G
   my $d=$_[0];

   if ($_[1] eq "B") {                                   # result in B
      if ($d=~/K/) { $d=~s/K//; $d=$d * 1000;  }
      if ($d=~/M/) { $d=~s/M//; $d=$d * 1000000;  }
      if ($d=~/G/) { $d=~s/G//; $d=$d * 1000000000;  }
      $d=sprintf ("%.0f", $d);                           # round integer
      return($d);
   }

   if (!($d=~/\D/)) { $d=$d/1000; }                      # result now in K

   if ($d=~/K/) { $d=~s/K//; $d=$d * 1;  }
   if ($d=~/M/) { $d=~s/M//; $d=$d * 1000;  }
   if ($d=~/G/) { $d=~s/G//; $d=$d * 1000000;  }

   $d=sprintf ("%.0f", $d);                              # round integer
   return ($d);
}


sub getmydate {                # setzt datumsvariablen
###########
 ($sec,$min,$hour,$mday,$mon,$year,$wday,$ydat,$isdst) = localtime(time);
 $mon++;
 if ($sec < 10)  { $sec = "0$sec"; }
 if ($min < 10)  { $min = "0$min"; }
 if ($hour < 10) { $hour = "0$hour"; }
 if ($mday < 10) { $mday = "0$mday"; }
 if ($mon < 10)  { $mon = "0$mon"; }
 $year=1900+$year;
 $date_and_time = "$mday\.$mon\.$year, $hour:$min $sec s";
 $date = "$mday\.$mon\.$year";
 $date_us= "$mon\.$mday\.$year";
 $date_all="$year\.$mon\.$mday\.$hour\.$min\.$sec";
}

############# dow of day
sub getdow {
#day of week (tt.mm.jjjj)
 my $day="$mday.$mon.$year";

 my @day = ("sun","mon","tue","wed","thu","fri","sat");
 my @d = (0,3,2,5,0,3,5,1,4,6,2,4);
 my @tmp;
 @tmp=split(/\./,$day);
 my $t=$tmp[0];
 my $m=$tmp[1];
 my $j=$tmp[2];
 if ($j < 100) { $j =2000+$j; }
 if ($m < 3) { $j-- ; }
 $tmp[3] = $day[($j+int($j/4)-int($j/100)+int($j/400)+$d[$m-1]+$t) % 7];
 return ($tmp[3]);
}

###############################################
#end of my script, do not delete the line below
1;
