#!/usr/bin/perl
# (c) napp-it  2012-2017
# 11. may  17

# datamover agent
# last modification:

# debug ex:
# /var/web-gui/data/napp-it/zfsos/_lib/illumos/agents/datamover.pl test_mover01.cfg debug


use strict;                                                    # disable for syntax check
use Fcntl qw(:flock);                                          # filelocking
$|=1;                                                          # keine Pufferung, no buffering

use vars qw($r $par $id $action $f $t @t $p @p %p @w $w $find $date $tm $log $err $dir %sys $ok $src $dst $debug);

#GNU find
$find="/usr/gnu/bin/find";
if (!(-f "/usr/gnu/bin/find")) {
     print "datamover.pl 18: missing GNU find\n";
     exit;
}

$par=$ARGV[0];
 if (!(defined($par))) {
     print "datamover.pl 24: missing parameter ex run_movername\n";
     exit;
 }

 #date
 &my_date;
 $date=$ARGV[1];                              # sys{'date_all'}
 if ($date eq "debug") { $debug="1"; }

 if (!($date=~/\d+\./)) { $date="$sys{'date_all'}"; }                # debug mode ex 123

 #par: run_job date
 $par=~s/ .*//;
 chomp($par);
 if ($debug) { print "\n\n#################################\nparameter: $par $date\n"; }

 if (!($par=~/^run\_\w|^test\_\w/)) {
     &my_log_ok("datamover.pl 47: wrong parameter, must be run_movername or test_movername");
     exit;
 }

 $id=$par;
 $id=~s/^run\_//;
 $id=~s/^test\_//;       # movername
 $id=~s/\.cfg$//;        # name or name.cfg
 $id.=".cfg";            # configfile ex mover1.cfg
 ##################################################

 if (!(-f "/var/web-gui/_log/datamover/$id")) {
     &my_log_ok("datamover.pl 59: config file ../_log/datamover/$id is missing");
     exit;
 }

 # do not start this job twice
     $t=`ps axw | grep datamover.pl | grep _$id | grep -v grep | grep -v sudo`;
     if ($t=~/datamover/s) {
          @t=split(/\n/,$t);
          if (@t>1) {
            &my_log_ok("datamover.pl 68: datamover $id is running, try again later");
            exit;
          }
     }

#give 5s to cancel/kill     
sleep(5);

# logfiles
  $log="/var/web-gui/_log/datamover/$id.$date.findlog";
  $err="/var/web-gui/_log/datamover/$id.$date.errlog";
  $ok="/var/web-gui/_log/datamover/$id.$date.oklog";

 # read data
  @t=();
  open (F,"/var/web-gui/_log/datamover/$id");
      flock(F, LOCK_EX);
      @t=<F>;
  close (F);

  foreach my $l (@t) {
      @w=();
      @w=split(/\t/,$l);
      $t="$w[0]";
      $p{$t}=$w[1];                             # all data in %p
  }


  # check method
  if (!($p{'autoselect_method'}=~/move|copy|cpio|filesync|rsync|cmd/)) {
      &my_log_ok("datamover.pl 90: datamover $id method $p{'autoselect_method'} not known");
      exit;
  }

  ##########################
  ###### run cmd  ##########
  ##########################
  if ($p{'autoselect_method'} eq "cmd")      {
      if ($debug) { print "$p{'value_opt_cmd'}\n"; }

      $r=`echo "cmd: $p{'value_opt_cmd'} " >> $log`;
      $r=`$p{'value_opt_cmd'}`;                             # run cmd

      if ($debug) { print "$r\n"; }

      if ($r ne "") {
           $r=~s/\n/<br>/gs;
           $r=`echo "$r" >> $err`;
      }

      &my_log_ok("101: cmd end");
      exit;
  }
  #########################


  $src="'"."/$p{'select_src_zfs'}$p{'value_src_folder'}"."'";
  $dst="\'/$p{'select_dst_zfs'}$p{'value_dst_folder'}\'";

  # check source
  if (!($p{'select_src_zfs'}=~/\w/)) {
      &my_log_ok("datamover.pl 99, no source fs given");
      exit;
  }

  $t=`/usr/sbin/zfs get -H name $p{'select_src_zfs'} 2>&1`;
  if ($t=~/cannot /s) {
      &my_log_ok("datamover.pl 105: datamover cannot open sourcefolder ZFS $p{'select_src_zfs'}");
      exit;
  }

  #check source
  $t=`/usr/gnu/bin/find $src -maxdepth 0 -type d 2>&1`;
  if ($t=~/No such file/is)  {
      &my_log_ok("datamover.pl 110: datamover cannot open sourcefolder ZFS/folder -$src-");
      exit;
  }

  # check destination
  if (!($p{'select_dst_zfs'}=~/\w/)) {
     &my_log_ok("datamover.pl 116, no destination fs given");
     exit;
  }

  $t=`/usr/sbin/zfs get -H name $p{'select_dst_zfs'} 2>&1`;
  if ($t=~/cannot /s) {
      &my_log_ok("datamover.pl 122: datamover cannot open destination ZFS $p{'select_dst_zfs'}");
      exit;
  }

  $t=`sudo /usr/gnu/bin/find $dst -maxdepth 0 -type d 2>&1`;
  # step 1, try to create target
  if ($t=~/No such file/is)  {
     `mkdir $dst`;
  }

  $t=`sudo /usr/gnu/bin/find $dst -maxdepth 0 -type d 2>&1`;
  # step 2, exit. dolder not available
  if ($t=~/No such file/is)  {
     `mkdir $dst`;
      &my_log_ok("datamover.pl 126: datamover cannot open destination ZFS/folder $dst");
      exit;
  }

  #find, currently supported: cpio, move, cmd
  if ($debug) { print "method: $p{'autoselect_method'}\n#################################\n\n"; }

  $p{'autoselect_method'}=~s/\s//gs;
  if ($p{'autoselect_method'} eq "copy")     { &my_find_copy;     }
  if ($p{'autoselect_method'} eq "cpio")     { &my_find_cpio;     }
  if ($p{'autoselect_method'} eq "move")     { &my_find_move;     }
  if ($p{'autoselect_method'} eq "filesync") { &my_find_filesync; }
  if ($p{'autoselect_method'} eq "rsync")    { &my_rsync;         }
  if ($debug) { print "no method!"; }

#############
sub my_isdir {
#############
  my $f=$_[0];
  my $t=`sudo /usr/gnu/bin/find $f -maxdepth 0 -type d 2>&1`;
  # step 1, try to create target
  if ($t=~/No such file/is)  {
     return(0);
  }
  return(1);
}

#############
sub my_find {
############
  my ($name,$cmd,$sf);
  # $find="/usr/gnu/bin/find";                # see top of script

  # create/use snap ex tank/data@datamover_test.cfg_date_all ?
  if (($p{'select_use_snap'} eq "yes") || ($p{'select_src_snap'} eq "yes")) {
       $name=$p{'select_src_zfs'}."\@datamover_$id"."_$date";
       `echo "create src snap $name" >> $log`;
       &my_snap($name);
  }

  # create/use dst snap ex tank/data@datamover_test.cfg_date_all ?
  if ($p{'select_dst_snap'} eq "yes") {
       $name=$p{'select_dst_zfs'}."\@datamover_$id"."_$date";
       `echo "create dst snap $name" >> $log`;
       &my_snap($name);
  }

  #use src snap
  $t="0";
  if ($p{'select_use_snap'}   eq "yes")    { $t="1"; }
  if ($p{'autoselect_method'} eq "move")   { $t="0"; }
  if (($p{'autoselect_method'} eq "filesync") && ($p{'value_opt_filesync'}=~/f/)) { $t="0"; }  # filesync two-way

  if ($t) {
     $sf=$name;
     $sf=~s/.*\@//;         # snapname -> folder below .zfs/snapshot

     # was $src="'"."/$p{'select_src_zfs'}$p{'value_src_folder'}"."'";
     $src="\'/$p{'select_src_zfs'}/.zfs/snapshot/$sf$p{'value_src_folder'}\'";
     `echo "use src snap as source: $src" >> $log`;

     #check new source
     $t=`/usr/gnu/bin/find $src -maxdepth 0 -type d 2>&1`;
     if ($t=~/No such file/is)  {
       &my_log_ok("datamover.pl 185: datamover cannot open snap sourcefolder ZFS/folder: $src");
       exit;
     }
  }

  # use relative find with . after a cd
  $cmd="cd $src && $find . ";       # use relative paths from src

  #or abs like /pool/fs/data
  if ($p{'select_abspath'} eq "abs") {
     $cmd=" find $src ";
  }

  #add find depth as first option!
  if ($p{'value_depth'} =~/\-m..depth \+*\d/) {
      $cmd.="$p{'value_depth'} ";
  }

  #add other find opt
  if ($p{'value_find'}=~/\-\w+ /i) {
      $cmd.="$p{'value_find'} ";
  }

  #add find path
  if ($p{'value_path'}=~/\-path /) {

      $cmd.="$p{'value_path'} ";
  }

  #add find name
  if ($p{'value_fname'} =~/\-name /) {
      $cmd.="$p{'value_fname'} ";
  }

  #add find size
  if ($p{'value_size'} =~/\-size [+-]*\d/) {
      $cmd.="$p{'value_size'} ";
  }

  #add find age
  if ($p{'value_age'} =~/\-mtime [+-]*\d/) {
      $cmd.="$p{'value_age'} ";
  }

  if ($debug) { print "find command\n$cmd\n"; }
  return($cmd);
}


#############
sub my_find_cpio {                      # find . -print -depth | cpio -Ppumdav  filesystem2
#############                           # ex /usr/gnu/bin/find '/b1/data' -path */Sonstiges/* -size +100k -size -500k

# commands example
# echo "/usr/gnu/bin/find '/b1/data' -path */path/* | tee cfg.log | cpio -Ppumdav /b1/test  2>&1 >> err.log" > cfg.log
# find '/b1/data' -path */path/* | tee /var/web-gui/_log/datamover/test.time.cfg.log | cpio -Ppumdav /b1/test  2>> err.log

  my $cmd=&my_find;                 # find command

  #add tee
  $cmd.=" | tee $log";

  if ($par=~/run/) {
    #add cpio
    $cmd.=" | /usr/bin/cpio $p{'value_opt_cpio'} $dst";  # cpio to destination
  }

  # add errorlog
  $cmd.=" 2>>$err";

  # find must not contain ""
  $cmd=~s/\"/'/g;

  #write command to logs
  `echo "$par: $cmd" > $log`;

  # execute cmd (test or cpio)
  `$cmd`;

  &my_date;
  &my_log_ok("finished $sys{'date_all'}");
  exit;

}

#############           #
sub my_find_copy {      # optional
#############
      print "optional"; exit;
      &my_find;
}

#############
sub my_find_move {
#############
  my ($cmd,$cmd1,$cmd2,@d,$d,$sf,$df0,$df2,$df,$df3);

  $sf=$src;          # '/pool/zfs/folder'
  $sf=~s/\'//g;      # /pool/zfs/folder

  $df=$dst;          # '/pool/zfs/folder'
  $df=~s/\'//g;      # /pool/zfs/folder

  # move moves target folder, cpio content, get folder
  $df2=$sf;
  $df2=~s/.*\///;     # zfs/folder -> folder

  $sf="$sf/";        # source folder/
  $sf=~s/\/+/\//;

  $d="$df/$df2/";    # '/pool/zfs/folder/lastsrcfolder'
  $d="'$d"."'";

  # copy ../folder to /backup/folder where backup and folder are not there
  $t=`sudo /usr/gnu/bin/find $d -maxdepth 0 -type d 2>&1`;
  # step 1, try to create target
  if ($t=~/No such file/is)  {
     `mkdir $d`;
  }

  # dest folder
  $df=~s/\/+/\//g;
  $df="'".$df."'";   # 'dstfolder'

  ## options: ##
  # $p{'select_opt2_move'} -> dirmove, filemove + copy folder structure, filemove +del empty src folder
  ###############################

  # folder move /usr/bin/mv -f /b1/test/backup/data/ /b1/rtest/backup/


  # find <moved-from> -name file\* -type f -mtime +1 -exec mv {} <moved-to> \;

  # move is done in two steps
  # 1. cpio directory structure only
  # 2. move (faster than cpio if its done within same fs)

  $cmd=&my_find;                                         # find command t find files and directories

  ##################################
  # step 1, cpio directory structure from find
  ##################################

  $cmd1=$cmd;
  $cmd1.=" -type d | tee $log";                          # directories only, logging

  # copy is /zfs/source to /dir (creates sourcefolder)
  if ($par=~/run/) {
    #add cpio
    $cmd1.=" | /usr/bin/cpio $p{'value_opt_cpio'} $d";  # cpio dir to destination/srcfolder
  }

  # add errorlog
  $cmd1.=" 2>>$err";

  # find must not contain ""
  $cmd1=~s/\"/'/g;

  #write command to logs
  `echo "" > $log`;                                       # create file
  `echo "$par: $cmd1" >> $log`;                           # add cmd1

  # execute cmd (test or cpio)
  if ($debug) { print "\nstart dircopy with cmd1: \n$cmd1\n"; }
  `$cmd1`;

  #print log
  if ($debug) { print "\nFindlog of dircopy to $dst:\n" .`cat $log` ."\n\n"; }


  ##################
  # step 2a, dirmove       # folder move /usr/bin/mv -f /b1/test/backup/data/ /b1/rtest/backup/
  ##################


  if ($p{'select_opt2_move'}=~/dirmove/) {
       if ($debug) { print "\nmove method: dirmove\n"; }
       if (-f "$log") {
          $d=`cat $log`;

          @d=split(/\n/,$d);
          @d=sort(@d);                  # toplevel first
          foreach my $d (@d) {

             if (!($d=~/^\.|^\//))     { next; }

             # if rel path
             if ($d=~/^\./) {
                   $d=~s/\./$sf/;
                   $df3=$df."/";
             }  else {
                   # use abs path for destination
                   $df3="$df"."/$df2";

                   $df0=$d;          # parent of destination for move
                   $t=rindex($df0,"/");
                   if ($t > 0) {
                      $df0=substr($df0,0,$t);
                   }
                   $df0.="/";
                   $df0=~s/\/+/\//;

                   $df3.="$df0/";
             }

             $df3=~s/\/+/\//g;
             $df3=~s/\'//g;
             $df3="'".$df3."'";

#print "\n$df3\n"; exit;

             $d.="/";
             $d=~s/\/+/\//g;
             $d="'".$d."'";

             if (&my_isdir($d)) {
                 if ($par=~/run/) {
                    if ($debug) { print "move dir: $d to $df3\n"; }
                    `echo "mv $p{'value_opt_move'} $d $df3" >> $err`;
                    `/usr/bin/mv $p{'value_opt_move'} $d $df3 2>>$err`;
                  } else {
                    if ($debug) { print "would move dir: $d to $df3\n"; }
                  }
             } else {
               if ($debug) { print "move dir: $d not available - already moved\n"; }
             }

          }

       } else {
          &my_log_ok("341 logfile $log with dirstructure not found");
          exit;
       }
       # end of dirmove
  }


  #print log
  if ($debug) { print "\n\nErrorlog\n".`cat $err`; }

exit;
  ########################
  # step 2 create filelist
  ########################
  $cmd2=$cmd;
  $cmd2.=" -type f | tee $log";                          # files only, logging

  #  filelog marker
  `echo "--filelog--" >> $log`;

  # add errorlog
  $cmd2.=" 1>>$log 2>>$err";

  # find must not contain ""
  $cmd2=~s/\"/'/g;

  # get files
  `$cmd2`;

  if ($debug) { print "\n\nstep 2, log of find files:\n" .`cat $log` .`cat $err`; }
  exit;

  ##############
  # step 3, move
  ##############
  $cmd.="-type f";           # files only

  if ($par=~/run/) {
    # mo find <moved-from> -name file\* -type f -mtime +1 -exec mv {} <moved-to> \;
    $cmd.=" -print0 | xargs -0 '/usr/bin/mv $p{'value_opt_move'}' {} $dst";  # mv to destination
  }

  # add errorlog
  $cmd.=" 1>>$log 2>>$err";

  # find must not contain ""
  $cmd=~s/\"/'/g;

  #write command to logs
  `echo "$par: $cmd" > $log`;
  `echo "$par: $cmd" > $log`;

  # execute cmd (test or cpio)
    if ($debug) { print "\nstart mover with cmd: \n$cmd\n"; }
  `$cmd`;

  &my_date;
  &my_log_ok("finished $sys{'date_all'}");
  exit;
}
#############
sub my_find_filesync {
#############
    print "optional"; exit;
}

#############
sub my_rsync {
#############
   print "optional"; exit;
}


###########        # create a ZFS snap
sub my_snap {
###########
  # create a snap
  my ($snap,$r,$fs);

  $snap=$_[0];     # ex zfs snapshot tank/data@datamover_test.cfg_date_all

  if (!($snap=~/^[a-zA-Z0-9\-\_\.\/]+\@[a-zA-Z0-9\-\_\.]+/)) {
    &my_log_ok("my_snap 272: wrong snap name $snap");
    exit;
  }

  $fs=$snap;
  $fs=~s/\@.*//; # fs name
  $r=`/usr/sbin/zfs get -H -o value readonly $fs 2>&1`;
  if (!($r=~/off/s)) {
      &my_log_ok("my_snap 314: could not create $snap (fs $fs is readonly)");
      exit;
  }

  $r=`/usr/sbin/zfs snapshot $snap 2>&1`;
  if ($r=~/cannot |usage:/) {
      &my_log_ok("my_snap 272: could not create snap $snap (error: $r\)");
      exit;
  }

  $r=`/usr/sbin/zfs list -t snapshot | grep $snap`;
  if ($r eq "") {
     &my_log_ok("my_snap 314: tried to create snap $snap but without success or errormessage!");
     exit;
  }

}

###########
sub my_date {                # setzt datumsvariablen
###########
 my ($min,$hour,$mday,$sec,$mon,$year,$wday,$ydat,$isdat,$isdst);
 ($sec,$min,$hour,$mday,$mon,$year,$wday,$ydat,$isdst) = localtime(time);
 $mon++;
 if ($sec < 10) { $sec = "0$sec"; }
 if ($min < 10) { $min = "0$min"; }
 if ($hour < 10) { $hour = "0$hour"; }
 if ($mday < 10) { $mday = "0$mday"; }
 if ($mon < 10) { $mon = "0$mon"; }
 $year=1900+$year;

 $sys{'sec'}=$sec;
 $sys{'min'}=$min;
 $sys{'hour'}=$hour;
 $sys{'mday'}=$mday;
 $sys{'mon'}=$mon;
 $sys{'year'}=$year;
 $sys{'date_and_time'} = "$mday\.$mon\.$year, $hour:$min $sec s";
 $sys{'date'} = "$mday\.$mon\.$year";
 $sys{'date_us'}= "$mon\.$mday\.$year";
 $sys{'date_all'}="$year\.$mon\.$mday\.$hour\.$min\.$sec";
}

#############
sub my_log_ok {         # last command prior exit!!
##############
  my $mess=$_[0];
  `echo "$mess" >> $ok`;
   if ($debug) {
        print "exit message: $mess\n\n";
        print "log\n";
        print `cat $log`;

        print "\n\nerrorlog\n";
        print `cat $err`;
        print "\n\n";
   }
  exit;
}


##


#eof
1;
