#!/usr/bin/perl
# library for os dependent monitor functions
# (c) Gnther Alka 2014-2017

# datafiles ex /tmp/iostat.log are generated from stat.pl as service


############
sub monitor {
############ default: grey.png
 # title requires \r
 # alert requires \r\n

 my ($m,$t,$r,@l,$l,$c,$w,@w,$i,$pools,@pools,$state,$img,$busy,@v);
 $t=time();

 # get poolstat
 ##############
    if (-f "/tmp/poolstat.log") {
        #$r=`cat /tmp/poolstat.log`;
        @w=();
        open (F,"</tmp/poolstat.log");
             flock(F, LOCK_EX);
             @w=<F>;
        close (F);
        $r=join("",@w);

        $i="";    # info
        foreach my $v (@w) {
           chomp($v);
           $v=~s/^ +//;
           if ($v=~/pool:/)     { $i.="\r$v\r"; $pools.="$v,"; next; }
           if ($v=~/state:/)    { $i.="$v\r";   next; }
           if ($v=~/scan:/)     { $i.="$v\r";   next; }
           if ($v=~/errors:/)   { $i.="$v\r";   next; }
        }

        $c=$t-$w[0];              # interval max 70s
        if ($c < 70) {

            $t=$i; $t=~s/\r/\\r\\n/gs;
            if ($r=~/OFFLINE|DEGRADED|UNAVAIL|FAULTED|REMOVED/s) {
                $l="<a href='#' onclick=\"javascript:alert(\'$t\');\" title=\"Pool state: error\r$i\"> Pool <img src='/_doc/red.png' alt=''>&nbsp;</a>";
            } else {
                $l="<a href='#' onclick=\"javascript:alert(\'$t\');\" title=\"Pool state: ok\r$i\"> Pool <img src='/_doc/green.png' alt=''>&nbsp;</a>";
            }
        } else {
                    $l="<a href='#' title=\"Pool state: logfile too old ($c s)\"> Pool <img src='/_doc/grey.png' alt=''>&nbsp;</a>";
        }
    } else {
        $l='<a href="#" title="Pool state: no logfile"> Pool <img src="/_doc/grey.png" alt="">&nbsp;</a>';
    }
    $m="  $l\n";


 #add pool capacity
 ##################
    $pools=~s/,$//; $pools=~s/pool: //g;
    @pools=();
    @pools=split(/,/,$pools);
    $i="ZFS pool capacity $pools\r";
    $state="";

    # NAME                                             TYPE        CREATION                USED  AVAIL  REFER
    # $r=`/usr/sbin/zfs list -H -o all`;

    if (-f "/tmp/zfsstat.log") {
        #$r=`cat /tmp/zfsstat.log`;
        @l=();
        open (F,"</tmp/zfsstat.log");
             flock(F, LOCK_EX);
             @l=<F>;
        close (F);
        $c=$t-$l[0];              # interval max 70s

        if ($c < 70) {
           foreach my $pool (@pools) {
                 foreach my $l (@l) {
                    $l=~s/  +/\t/g;
                    if (!($l=~/^$pool\t/)) { next; }
                    chomp($l);

                    @w=();
                    @w=split(/\t/,$l);

                    $r=&my_percent($w[3],$w[4]);           # used, avail
                    if ($r < 10) { $r=~s/^0//; $r="0$r"; }

                    if ($r > 69) { $state.="G"; }          # greenyellow
                    if ($r > 79) { $state.="y"; }          # yellow
                    if ($r > 84) { $state.="Y"; }          # yellowred
                    if ($r > 89) { $state.="r"; }

                    $i.="\rCap $r %, Pool: $pool";
                 }
           }


           # add tmpfs
           # Filesystem     1K-blocks  Used Available Use% Mounted on
           # swap             7499612  4776   7494836   1% /tmp

             $r=`df -k /tmp`;
             chomp($r); $r=~s/.*swap/swap/s;
             $r=~s/ +/ /gs;
             @v=();
             @v=split(/ /,$r);

           $i.="\r\rCap $v[4], tmpfs: /tmp";
           $i.="\rtmpfs $v[2]/$v[3]";

           $v[4]=~s/\%//;
            if ($v[4]> 40) { $state.="y"; }
            if ($v[4]> 50) { $state.="Y"; }
            if ($v[4]> 70) { $state.="R"; }

           #img
           $img="green.png";
           if ($state=~/G/) { $img="greeny.png";  }
           if ($state=~/y/) { $img="yellow.png";  }
           if ($state=~/Y/) { $img="yellowr.png";  }
           if ($state=~/r/) { $img="red.png";  }

           $t=$i; $t=~s/\r/\\r\\n/gs;
           $l="&nbsp;<a href='#' onclick=\"javascript:alert(\'$t\');\" title=\"$i\"> Cap <img src=\"/_doc/$img\" alt=''>&nbsp;</a>";
        }  else {
            $l="&nbsp;<a href='#' title='ZFS pool capacity: logfile too old'> Cap <img src='/_doc/grey.png' alt='Capacity'>&nbsp;</a>";
        }
    } else {
            $l="&nbsp;<a href='#' title='ZFS pool capacity: no logfile'> Cap <img src='/_doc/grey.png' alt='Capacity'>&nbsp;</a>";
    }
    $m.="  $l\n";


 #add disk activity
 ##################
    # show disk and average read, write, wait
    $state="";
    if (-f "/tmp/iostat.log") {
        #$r=`cat /tmp/iostat.log`;

        @l=();
        open (F,"</tmp/iostat.log");
             flock(F, LOCK_EX);
             @l=<F>;
        close (F);
        $c=$t-$l[0];              # interval max 70s

        $i="Disk I/O and activity last 10s\rDevice (rd/s  wr/s  w%  b%)\r";
        if ($c < 70) {
                                         # lines logfile
             foreach my $l (@l) {

                $l=~s/ +/ /g;
                chomp($l);

                #busy
                if ($l=~/cpu_busy/) {
                  $busy=$l; $busy=~s/.*\) //; # current/ av10s
                }

                if (!($l=~/\./)) { next; }                                # no dataline

                $l=~s/^ +//;                                              # no leading spaces
                @w=();
                @w=split(/ /,$l);                                         # 10=device, 11= read10, 12=write10, 14=wait10%, 15=busy10%
                $t=$w[10]; $t.=' ' x 20; $t=substr($t,0,20);
                $i.="\r$t";             # device,
                $i.="\t\t r: $w[11]";     # r,
                $i.=", wr: $w[12]";     # w,

                $i.=", w: $w[14]\%";    # w%,
                $i.=", b: $w[15]\%";    # b%,
                $i.="";

                #state
                if ($w[10]=~/c\d+t\d+/i) {                                # disk not pool
                     if  ($w[14] > 5)   { $state.="G"; }                  # wait
                     if  ($w[14] > 10)  { $state.="y"; }
                     if  ($w[14] > 20)  { $state.="Y"; }
                     if  ($w[14] > 50)  { $state.="r"; }

                     if  ($w[15] > 10)   { $state.="G"; }                 # busy
                     if  ($w[15] > 40)  { $state.="y"; }
                     if  ($w[15] > 60)  { $state.="Y"; }
                     if  ($w[15] > 80)  { $state.="r"; }
                } else {
                    # pool
                    if  ($w[15] > 10)  { $state.="y"; }
                    if  ($w[15] > 50)  { $state.="Y"; }
                    if  ($w[15] > 90)  { $state.="r"; }
                }
             }

             $img="green.png";
             if ($state=~/G/) { $img="greeny.png";  }
             if ($state=~/y/) { $img="yellow.png";  }
             if ($state=~/Y/) { $img="yellowr.png";  }
             if ($state=~/r/) { $img="red.png";  }

             $t=$i; $t=~s/\r/\\r\\n/gs;
             $t=~s/\t\t/\\r\\n  /gs;                                              # data new line

             $l="&nbsp;<a a href='#' onclick=\"javascript:alert(\'$t\');\" title=\"$i\"> Disk <img src='/_doc/$img' alt=''>&nbsp;</a>";
        } else {
             $l="&nbsp;<a title=\"ZFS disk activity: logfile too old\"> Disk <img src='/_doc/grey.png' alt=''>&nbsp;</a>";
        }
    } else {
        $l="&nbsp;<a title=\"ZFS disk activity: no logfile\"> Disk <img src='/_doc/grey.png' alt=''>&nbsp;</a>";
    }
    $m.="  $l\n";


    #nic activity
    $state="";
    if (-f "/tmp/nicstat-raw.log") {
        #$r=`cat /tmp/nicstat-raw.log`;
        @l=();
        open (F,"</tmp/nicstat-raw.log");
             flock(F, LOCK_EX);
             @l=<F>;
        close (F);
        $c=$t-$l[0];              # interval max 70s

        $i="Network activity\rLink \t\t\twMbit/s \t\trMbit/s \t\tutil%\r";
        if ($c < 70) {

             foreach my $l (@l) {
                if (!($l=~/\./)) { next; }                                # no dataline

                $l=~s/^\s+//;                                             # no leading spaces
                @w=();
                @w=split(/\s+/,$l);                                       # 1=link, 2=wMbs, 3=rMbs, 8=%util

                $t=$w[1]; $t.=" "x20; $t=substr($t,0,20);
                $i.="\r$t \t$w[2] \t\t$w[3] \t\t$w[8]\%";
                if ($w[8] > 5)  { $state.="G"; }
                if ($w[8] > 30) { $state.="y"; }
                if ($w[8] > 50) { $state.="Y"; }
                if ($w[8] > 70) { $state.="r"; }
             }

             $img="green.png";
             if ($state=~/G/) { $img="greeny.png";  }
             if ($state=~/y/) { $img="yellow.png";  }
             if ($state=~/Y/) { $img="yellowr.png";  }
             if ($state=~/r/) { $img="red.png";  }

             $i=~s/\n/\r/gs;
             $t=$i; $t=~s/\r/\\r\\n/gs;        # alert

             $l="&nbsp;<a a href='#' onclick=\"javascript:alert(\'$t\');\" title=\"$i\"> Net <img src='/_doc/$img' alt=''>&nbsp;</a>";
        } else {
             $l="&nbsp;<a title='Network activity: logfile too old'> Net <img src='/_doc/grey.png' alt=''>&nbsp;</a>";
        }
    } else {
        $l="&nbsp;<a title='Network activity: no logfile'> Net <img src='/_doc/grey.png' alt=''>&nbsp;</a>";
    }
    $m.="  $l\n";



    #cpu/ps  activity
    $state="";
    if (-f "/tmp/prstat-raw.log") {
        #$r=`cat /tmp/prstat-raw.log`;

        @l=();
        open (F,"</tmp/prstat-raw.log");
             flock(F, LOCK_EX);
             @l=<F>;
        close (F);
        $c=$t-$l[0];              # interval max 70s

        $i="CPU\tProcess\r";
        if ($c < 70) {

             foreach my $l (@l) {
                if (!($l=~/\%|\:/)) { next; }                             # no dataline
                $l=~s/^\s+//;                                             # no leading spaces
                $l=~s/\s+/ /g;
                @w=();
                @w=split(/ /,$l);

                if ($l=~/load averages/) {
                     $i.="\r\raverage load 1, 5, 15 min: \r";

                     $l=~s/.*: //;                                         # keep averages
                     $l=~s/,/\%,/g;
                     $l.='%';
                     $l=~s/,/\t/g;
                     $i.=$l;
                     if ($busy ne "") {
                        $i.="\r\rbusy (iostat): now, last 10s\r";
                        $t=$busy;
                        $t=~s/ +/\t/; $t.='%'; $t=~s/\t/\%  \t/g;           # format busy like: 2 5
                        $i.="$t";
                     }

                     $l=~s/,.*//;    # keep 1min
                     if ($line > 5)  { $state.="G"; }
                     if ($line > 30) { $state.="y"; }
                     if ($line > 50) { $state.="Y"; }
                     if ($line > 80) { $state.="r"; }
                } else {
                      # add  cpu and process
                      $i.="\r$w[8]\t$w[9]";
                }

             }
             $img="green.png";
             if ($state=~/G/) { $img="greeny.png";  }
             if ($state=~/y/) { $img="yellow.png";  }
             if ($state=~/Y/) { $img="yellowr.png";  }
             if ($state=~/r/) { $img="red.png";  }

             $i=~s/\n/\r/gs;
             $t=$i; $t=~s/\r/\\r\\n/gs;        # alert

             $l="&nbsp;<a a href='#' onclick=\"javascript:alert(\'$t\');\" title=\"$i\"> CPU <img src='/_doc/$img' alt=''>&nbsp;</a>";
        } else {
          $l="&nbsp;<a title='CPU activity: logfile too old'> Net <img src='/_doc/grey.png' alt=''>&nbsp;</a>";
        }
    } else {
        $l="&nbsp;<a title='CPU activity: no logfile'> CPU <img src='/_doc/grey.png' alt=''>&nbsp;</a>";
    }
    $m.="  $l\n";

   # job activity
           # jobinfo, running=y, error=r
           $state="";

           $r= `ls /var/web-gui/_log/jobs/`;
           $r=~s/\s+/\t/gs;
           my @files=split(/\t/,$r);
           my $err="0";
           my $running="ok";

           $i="Job state\r";

           $r= `cat /var/web-gui/_log/jobs/jobs.log`;
           if ($r=~/error;/) {
                $state.="y";
                $i.="\rError in last 50 job-log";
           }

           foreach my $t (@files) {

                      if (!($t=~/\.log$/))  { next; }
                      if ($t eq "jobs.log") { next; }

                      #read /var/web-gui/data/napp-it/_log/jobs/.log
                      $r= `cat /var/web-gui/_log/jobs/$t`;
                      if ($r=~/^error;/) {

                           # check if active
                           $t=~s/log/par/;
                           $r= `cat /var/web-gui/_log/jobs/$t`;
                           if (!($r=~/status=active/s))   { next; }

                           $state.="r";
                           $i.="\rError in job $t";  $i=~s/\.par//;
                      }
           }
           $img="green.png";
           if ($state=~/y/) { $img="yellow.png";  }
           if ($state=~/r/) { $img="red.png";  }

           $t=$i; $t=~s/\r/\\r\\n/gs;
           $m.="&nbsp;<a a href='#' onclick=\"javascript:alert(\'$t\');\" title=\"$i\"> Job <img src='/_doc/$img' alt='Job state'>&nbsp;</a>";

  return ($m);
}

############
sub fsstat {
###########

  my @l=();

  # raw data
  if ($_[0] eq "raw") {
      if (-f "/tmp/fsstat-raw.log") {
            open (PF, "</tmp/fsstat-raw.log");
                   flock(PF, LOCK_EX);
                   @l = <PF>;
            close (PF);
            my $r=join("\n",@l);
            return($r);
      } else {
             return ("no data");
      }
  }

  if (-f "/tmp/fsstat.log") {
        open (PF, "</tmp/fsstat.log");
               flock(PF, LOCK_EX);
               @l = <PF>;
        close (PF);
  }  else {
     return ("(xna);0;100;fsstat no data");
  }

  # single line
  chomp($l[0]);
  #print "## lib 102: $l[0]\n\n";

  return ("$l[0]"."fsstat ops;");


}

################
sub arcstat {
################
  @l=();
  my ($t,@w,$w);
  if (-f "/tmp/arcstat.log") {
        open (PF, "</tmp/arcstat.log");
               flock(PF, LOCK_EX);
               @l = <PF>;
        close (PF);
  } else {
         return ("n.a.;arcstat no data;10;");
  }

  if ($_[0] eq "raw") {
     $t=join("",@l);
     return($t);
  }

  $t=pop(@l);    # last
  chomp($t);
  $t=~s/ +/,/g;
  @w=split(/,/,$t);

  $t="$w[4],$w[7],$w[12],$w[13],"; #arc-hit, l2hit,  aver. l2aver
  return($t);

}

################
sub zilstat {
################
  @l=();
  my ($t,@w,$w);
  if (-f "/tmp/zilstat.log") {
        open (PF, "</tmp/zilstat.log");
               flock(PF, LOCK_EX);
               @l = <PF>;
        close (PF);
  } else {
         return ("n.a.;zilstat no data;10;");
  }

  if ($_[0] eq "raw") {
     $t=join("",@l);
     return($t);
  }

  #always raw
     $t=join("",@l);
     return($t);
}

################
sub nicstat {
################
  @l=();
  # raw data
  if ($_[0] eq "raw") {
      if (-f "/tmp/nicstat-raw.log") {
            open (PF, "</tmp/nicstat-raw.log");
                   flock(PF, LOCK_EX);
                   @l = <PF>;
            close (PF);
            my $r=join("\n",@l);
            $l[0]=~s/ s/s/gs;
            return($r);
      } else {
             return ("no data");
      }
  }

  #graph data
  if (-f "/tmp/nicstat.log") {
        open (PF, "</tmp/nicstat.log");
               flock(PF, LOCK_EX);
               @l = <PF>;
        close (PF);
  } else {
         return ("n.a.;nicstat no data;10;");
  }

  chomp($l[0]);


  #return ("a,b;1,2:1,2;10;nicstat disabled");
  return ("$l[0]"."load in %;");

}


#############
sub prstat {
#############
 my (@l);
 # data form 'stat.pl prstat' service

  @l=();

   # raw data
  if ($_[0] eq "raw") {
      if (-f "/tmp/prstat-raw.log") {
            open (PF, "</tmp/prstat-raw.log");
                   flock(PF, LOCK_EX);
                   @l = <PF>;
            close (PF);
            my $r=join("\n",@l);
            return($r);
      } else {
             return ("no data");
      }
  }

  if (-f "/tmp/prstat.log") {
        open (PF, "</tmp/prstat.log");
               flock(PF, LOCK_EX);
               @l = <PF>;
        close (PF);
  } else {
         return ("(n.a.);0;prstat no data");
  }

  # single line
  # configd,perl,bash,nscd,VBoxSVC,perl,prstat,1min,5min,15min;0.1,0.1,0.0,0.0,0.0,0.0,0.0,0.2,0.1,0.1;
  chomp($l[0]);

  #print "## lib 521: $l[0]\n\n";
  return ("$l[0]"."load in %;");
}

######## 60s iostat -> 60min, dow
sub long_iostat {
##################

  # data  typically 6 graphs
     # busy cpu graph1 = title; ytext; y-maxunit; data (60,commalist);          # 4 values 0-3
     # w%       graph2.= title; ytext; y-maxunit; data (60,commalist);          # 4 values 4-7
     # b%       graph3                                                          # 4 values 8-11
     # read io  graph4                                                          # 4 values 12-15
     # write io graph5                                                          # 4 values 16-19
     # wait io  graph6                                                          # 4 values 20-23

     my (@w,$w,@l,$l,$t,$message,$list,$prop,$dev,$y,$m,$data,@data,$tm,$cpu,$time);
     $data[0]='b% (disk is busy in % of time) // w% (transactions are waiting in % of time) // cpu% (busy)';
     $data[1]='in %';
     $data[2]='100';

     $data[4]='percent of time there are transactions waiting';
     $data[5]='w%';
     $data[6]='100';

     $data[8]='percent of time the disk is busy';
     $data[9]='b%';
     $data[10]='100';

     $data[12]='read per sec';
     $data[13]='IO/s';
     $data[14]='1000';

     $data[16]='IO writes per sec';
     $data[17]='IO/s';
     $data[18]='1000';

     $data[20]='IO waiting transactions';
     $data[21]='IO/s';
     $data[22]='1000';

     $tm=time();
     @l=();

     if (-f "/var/web-gui/_log/monitor/iostat.$in{'prop2'}.$in{'prop'}.log") {            # iostat.day.hour.log
            open (PF, "</var/web-gui/_log/monitor/iostat.$in{'prop2'}.$in{'prop'}.log");
                   flock(PF, LOCK_EX);
                   @l = <PF>;
            close (PF);
     }

     # which data?
     #1400241899
     # cpu;2,1,1,1,3,3,1,1,3,1,3,1,1,1,1,1,1,3,1,1,1,1,5,1,1,3,1,1,1,3 .. (60 values)
     # b1;read60s;0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 ..
     # b1;write60s;0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 ..
     # b1;wait60s;0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 ..
     # b1;av60s_wperc;0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 ..
     # b1;av60s_bperc;0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ..

     if (!($l[1]=/\w/)) { # no or no valid data
           push (@l,"cpu;idle;0");
           push (@l,"rpool;read60s;0");
           push (@l,"rpool;write60s;0");
           push (@l,"rpool;wait60s;0");
           push (@l,"rpool;av60s_wperc;0");
           push (@l,"rpool;av60s_bperc;0");
     }

     $cpu=$l[1]; $cpu=~s/;$//; $cpu=~s/^.*;//;    # cpu busy data
     $data[3]=$cpu;

     $message=$_[0];
     if ($message=~/list=:/)  {                   # no device, use first device
         $dev=$l[2]; $dev=~s/;.*//;
         chomp($dev);
     } else {
         $dev=$message;
         $dev=~s/.*list=//;  $dev=~s/:.*//;
     }

     $y=100;
     foreach $l (@l) {
         if (!($l=~/^$dev\;/)) { next; }          # search device

         # ok found
         @w=();
         @w=split(/;/,$l);

         #read60
         if ($w[1]=~/read/) {                     # read60
              #get max
              $data[14]=&get_mymax($w[2]);        # max value
             # if ($data[14] >999) { $data[13].="  x 1000"; }
              $data[15]=$w[2];                    # data
         }

         #write
         if ($w[1]=~/write/) {                    # write
              #get max
              $data[18]=&get_mymax($w[2]);        # max value
              #if ($data[18] >999) { $data[17].="  x 1000";  }
              $data[19]=$w[2];                    # data
         }

         #wait
         if ($w[1]=~/wait/) {                     # wait io
              #get max
              $data[22]=&get_mymax($w[2]);        # max value
           #   if ($data[22] >999) { $data[21].="  x 1000";   }
              $data[23]=$w[2];                    # data
         }

         #w%
         if ($w[1]=~/wperc/) {                    # wait%
              $data[7]=$w[2];                     # data
         }

         #b%
         if ($w[1]=~/bperc/) {                    # busy%
              $data[11]=$w[2];                    # data
         }
     }

     $t=substr($data[15],0,70);
     $data=join(";",@data);
     if ($data[15] eq "") {
        return("long_iostat $l[0]","$data");
     } else {
         return("long_iostat $l[0] device $dev read -> $t ..","$data");
     }


}

######## 60s iostat
sub short_iostat {
##################

  # data  typically 6 graphs
     # busy cpu graph1 = title; ytext; y-maxunit; data (60,commalist);          # 4 values 0-3
     # w%       graph2.= title; ytext; y-maxunit; data (60,commalist);          # 4 values 4-7
     # b%       graph3                                                          # 4 values 8-11
     # read io  graph4                                                          # 4 values 12-15
     # write io graph5                                                          # 4 values 16-19
     # wait io  graph6                                                          # 4 values 20-23

     my (@w,$w,@l,$l,$t,$message,$list,$prop,$dev,$y,$m,$data,@data,$tm,$cpu);
     $data[0]='b% (disk is busy in % of time) // w% (transactions are waiting in % of time) // cpu% (busy)';
     $data[1]='in %';
     $data[2]='100';

     $data[4]='percent of time there are transactions waiting';
     $data[5]='w%';
     $data[6]='100';

     $data[8]='percent of time the disk is busy';
     $data[9]='b%';
     $data[10]='100';

     $data[12]='read per sec';
     $data[13]='IO/s';
     $data[14]='1000';

     $data[16]='IO writes per sec';
     $data[17]='IO/s';
     $data[18]='1000';

     $data[20]='IO waiting transactions';
     $data[21]='IO/s';
     $data[22]='1000';


     $tm=time();
     @l=();
     if (-f "/tmp/iostat_60s.log") {
            open (PF, "</tmp/iostat_60s.log");
                   flock(PF, LOCK_EX);
                   @l = <PF>;
            close (PF);
     } else {
            $data[0]="iostat 60s - no datafile";
            $data=join(";",@data);
            return("short_iostat $tm","$r");
     }

     # which data?
     #1400241899
     # cpu;2,1,1,1,3,3,1,1,3,1,3,1,1,1,1,1,1,3,1,1,1,1,5,1,1,3,1,1,1,3 .. (60 values)
     # b1;read60s;0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 ..
     # b1;write60s;0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 ..
     # b1;wait60s;0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 ..
     # b1;av60s_wperc;0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 ..
     # b1;av60s_bperc;0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ..

     $w=$t-$l[0];
     if ($w > 70) {
          $data[0]="iostat 60s - data too old";
          $data=join(";",@data);
     }

     $cpu=$l[1]; $cpu=~s/;$//; $cpu=~s/^.*;//;    # cpu busy data
     $data[3]=$cpu;

     $message=$_[0];
     if ($message=~/list=:/)  {                   # no device, use first device
         $dev=$l[2]; $dev=~s/;.*//;
         chomp($dev);
     } else {
         $dev=$message;
         $dev=~s/.*list=//;  $dev=~s/:.*//;
     }

     $y=100;
     foreach $l (@l) {
         if (!($l=~/^$dev\;/)) { next; }          # search device

         # ok found
         @w=();
         @w=split(/;/,$l);

         #read60
         if ($w[1]=~/read/) {                     # read60
              #get max
              $data[14]=&get_mymax($w[2]);        # max value
        #      if ($data[14] >999) { $data[13].="  x 1000"; }
              $data[15]=$w[2];                    # data
         }

         #write
         if ($w[1]=~/write/) {                    # write
              #get max
              $data[18]=&get_mymax($w[2]);        # max value
         #     if ($data[18] >999) { $data[17].="  x 1000";  }
              $data[19]=$w[2];                    # data
         }

         #wait
         if ($w[1]=~/wait/) {                     # wait io
              #get max
              $data[22]=&get_mymax($w[2]);        # max value
          #    if ($data[22] >999) { $data[21].="  x 1000";   }
              $data[23]=$w[2];                    # data
         }

         #w%
         if ($w[1]=~/wperc/) {                    # wait%
              $data[7]=$w[2];                     # data
         }

         #b%
         if ($w[1]=~/bperc/) {                    # busy%
              $data[11]=$w[2];                    # data
         }
     }

     $t=substr($data[15],0,70);
     $data=join(";",@data);
     if ($data[15] eq "") {
        return("short_iostat $tm","$data");
     } else {
         return("short_iostat $tm device $dev read -> $t ..","$data");
     }


}

#############  #max value from a commalist
sub get_mymax {
#############
  my @max=split(/,/,$_[0]);
  my $max="0";
  foreach my $l (@max) {
      if ($l > $max) { $max=$l; }
  }

  $max=$max*1.3;
  if ($max <= 100) { $max=100; }
  if (($max > 100)  && ($max <= 500 ))   { $max=500; }
  if (($max > 500)  && ($max <=1000 ))   { $max=1000; }

  if (($max > 1000)  && ($max <= 5000))  { $max=5000; }
  if (($max > 1000)  && ($max <= 10000)) { $max=10000; }
  if (($max > 10000) && ($max <= 15000)) { $max=15000; }
  if (($max > 15000) && ($max <= 20000)) { $max=20000; }
  if (($max > 20000) && ($max <= 50000)) { $max=50000; }
  if ($max > 50000) { $max=$max/1000;  $max=sprintf ("%.0f", $max); $max=$max * 1000; }   # ganzzahl

  $max=sprintf ("%.0f", $max);
  return($max);
}

########    disk and pool activity, par: pool, data
sub iostat   {
########
# data form 'stat.pl iostat' service


#cpu_busy%(now/av10s) 2  2
#    r/s    w/s   kr/s   kw/s wait actv wsvc_t asvc_t  %w  %b device  av10s_read  av10s_write  av10s_wait
#    0.0    0.0    0.0    0.0  0.0  0.0    0.0    0.0   0   0 rpool  0  13  0
#    0.0    0.0    0.0    0.0  0.0  0.0    0.0    0.0   0   0 c2t0d0  0  14  0
#    0.0    0.0    0.0    0.0  0.0  0.0    0.0    0.0   0   0 c2t1d0  0  0  0
#    0.0    0.0    0.0    0.0  0.0  0.0    0.0    0.0   0   0 c1t0d0  0  0  0
#    0.0    0.0    0.0    0.0  0.0  0.0    0.0    0.0   0   0 tank  0  0  0

 my (@w,$w,@l,$l,$k,$d,$t,@a,$a,$log,$x,$c,@data,@max,$label);

     $data[0]="iostat,$hour"."h,$min"."m,$sec"."s";       # info

     $max[7]="0";
     $max[8]="0";
     $max[9]="0";                                          # min 10

     @l=();
     if (-f "/tmp/iostat.log") {
            open (PF, "</tmp/iostat.log");
                   flock(PF, LOCK_EX);
                   @l = <PF>;
            close (PF);
     } else {
            # default string: text; x-unit1; labels1(disk1,disk22); key1(r,w,wt); data1(0,0,0:1,3,2); x-unit2; labels2(disks); key2 (%w,%b); data2 (1,2,3,4:1,..);idle;idle_10s;

            $data[1]="1000";                          # x-unit1
            $data[2]="(n.a.)";                        # y-labels (c0t1,c0t2)
            $data[3]="iostat: no data";               # keys1    (read,write,wait)
            $data[4]="0";                             # data1
            $data[5]="100";                           # x-unit2
            $data[6]="(n.a.)";                        # y-labels2 (c0t1,c0t2)
            $data[7]="no data";                       # keys2     (%b,%w)
            $data[8]="0";                             # data2
            $data[9]="-";                             # cpu busy
            $data[10]="-";                            # cpu busy av10s

            $d=join(";",@data);

            if ($_[0] eq "data") {     return ('no data'); }  # pure data
            return("$d");
     }

     if ($_[0] eq "data") {   $d=join("",@l);  return ($d); } # pure data


     $x=1000;                                    # default x units
     $data[3]='read,read last 10s,write,write last 10s';
     $data[7]='wait,wait last 10s,w%,b%';

     my ($p1,$p2);                               # count devices
     foreach $l (@l) {                           # lines of iostat, include time(1st line), fd and pools

              chomp($l);
              if ($l=~/^\d+$/) {  next; }        # 1st line=time()

              $t=time();
       # print "## 230: $t -> $l \n";            # !! print to /tmp when socketserver in background

              $l=~s/\.\d+//gs;                   # no comma values
              $l=~s/^ +//;                       # start with values
              $l=~s/ +/ /g;

              @w=();                             # r/s,  w/s,  kr/s,  kw/s,  wait,  actv,  wsvc_t,  asvc_t,  %w,  %b,  device,   avread, avwrite, avwait
              @w=split(/ /,$l);

              #devices and pool only
              if ($w[10] =~/^fd\d$/) { next; }   # fd0

              #cpu
              if ($w[0] =~/^cpu/)  {             # busy
                  $data[9]=$w[1];
                  $data[10]=$w[2]                # busy 10s
              }

              #skip header
              if (!($l=~/^\d+/))    { next; }    # not data, ex header

              $label=$w[10];
             if (length($w[10]) > 20) { $label="."; $label.=substr($w[10],15); }        # shorten long labels


              $p1="1";
              $p2="1";
              if ($_[0] eq "pools") {             # pools only?

                  # get average/ max read,write,wait over all active disks
                  if (($w[10]=~/c\d+t\d+/) || ($w[10]=~/c\d+d\d/)) { # ex wwn or c2t1d0 or c2d0 or c1d1

                      # only for disks
                      if ($w[0] > 0) { $max[0].="$w[0],";   }       # 0=av reads
                      if ($w[1] > 0) { $max[1].="$w[1],";   }       # 1=av write

                      if ($w[4] > 0) { $max[2].="$w[4],";   }       # 2=av wait
                      if ($w[8] > 0) { $max[3].="$w[8],";   }       # 3=av w%
                      if ($w[9] > 0) { $max[4].="$w[9],";   }       # 4=av b%

                      if ($w[0] > 0) {
                           if ($max[5] eq "")   { $max[5]="$w[0]"; } # base
                           if ($w[0] < $max[5]) { $max[5]=$w[0]; }   # 5= min reads
                      }
                      if ($w[1] > 0) {
                          if ($max[6] eq "")   { $max[6]="$w[1]"; }  # base
                          if ($w[1] < $max[6]) { $max[6]=$w[1]; }    # 6= min write
                      }

                      if ($w[4] > $max[7]) { $max[7]=$w[4]; }       # 7= max wait
                      if ($w[8] > $max[8]) { $max[8]=$w[8]; }       # 8= max w%
                      if ($w[9] > $max[9]) { $max[9]=$w[9]; }       # 9= max b%

                      #print "## 193lib $w[10]\n$l\nw4,8,9=$w[4],$w[8],$w[9]\n7=$max[7]\n8=$max[8]\n9=$max[9]\n\n";

                      $p1="0";

                   } else {        # add label

                       # add all non-disk as labels like rpool

                       $data[2].="$label,";           # devicelist (y-label-1)
                       $data[6].="$label,";           # devicelist (y-label-2)
                       $p2="0";                       # do not re-add devices in 207
                   }

              }

              if ($p1) { # add all values

                  if ($w[10]=~/c\d+t\d+/) {        # real values
                               # add all dev as labels like c0t0d0,rpool
                               if ($p2) {
                                    $data[2].="$label,";           # devicelist (y-label-1)
                                    $data[6].="$label,";           # devicelist (y-label-2)
                               }

                               # add read to data1
                                    $data[4].="$w[0],";
                                    while ( $w[0] > $x ) { $x=$x*2  }   # set new x units

                               # average read last 10s
                                  $data[4].="$w[11],";
                                  while ( $w[11] > $x ) { $x=$x*2  }    # set new x units

                               # write
                                    $data[4].="$w[1],";
                                    while ( $w[1] > $x ) { $x=$x*2  }   # set new x units

                               # average write_data last 10s
                                $data[4].="$w[12]:";
                                while ( $w[12] > $x ) { $x=$x*2  }      # set new x units
                  } else {
                              #10% values for pools (show real disk)
                               # add all dev as labels like c0t0d0,rpool
                               if ($p2) {
                                    $data[2].="$label (10%),";           # devicelist (y-label-1)
                                    $data[6].="$label,";           # devicelist (y-label-2)
                               }

                               # add read to data1
                                    $data[4].=$w[0]/10;    $data[4].=",";
                                    while ( $w[0] > $x*10 ) { $x=$x*2  }   # set new x units

                               # average read last 10s
                                  $data[4].=$w[11]/10;    $data[4].=",";
                                  while ( $w[11] > $x*10 ) { $x=$x*2  }    # set new x units

                               # write
                                    $data[4].=$w[1]/10;   $data[4].=",";
                                    while ( $w[1] > $x*10 ) { $x=$x*2  }   # set new x units

                               # average write_data last 10s
                                $data[4].=$w[12]/10;  $data[4].=":";
                                while ( $w[12] > $x*10 ) { $x=$x*2  }      # set new x units
                  }

                  ########## graph 2 ##

                  # wait
                  $data[8].="$w[4],";

                  #average wait last 10s
                  $data[8].="$w[13],";

                  # w%
                  $data[8].="$w[8],";

                  # busy %b
                  $data[8].="$w[9]:";

              }
     }


  $data[1]="$x";

  #remove trailing , or :
  foreach my $d (@data) {
       $d=~s/^,+|^:+|,+$|:+$//g;
  }

  if ($_[0] eq "pools") {                     # pools only, add average disk vs max disks
     # get average

     $max[0]=&get_avrg($max[0]);              # read
     $max[1]=&get_avrg($max[1]);              # write
     $max[2]=&get_avrg($max[2]);              # wait
     $max[3]=&get_avrg($max[3]);              # %w
     $max[4]=&get_avrg($max[4]);              # %b

     foreach my $m (@max) {                   # defaults
         if (!($m >0)) { $m=0; }
     }

     # scale min/avr read vs x or output is to minimal
     if (!($max[0] > 0)) {$max[0]=$x;  }
     if (!($max[1] > 0)) {$max[1]=$x;  }
     if (!($max[5] > 0)) {$max[6]=$x;  }
     if (!($max[6] > 0)) {$max[6]=$x;  }

     while ( ($max[0] < $x/2) && ($max[1] < $x/2) && ($max[5] < $x/2) && ($max[6] < $x/2) ) {
         $max[0]=$max[0]*1.5;  $max[1]=$max[1]*1.5; $max[5]=$max[5]*1.5; $max[6]=$max[6]*1.5;
         $max[0]=sprintf ("%.0f", $max[0]); $max[1]=sprintf ("%.0f", $max[1]);
         $max[5]=sprintf ("%.0f", $max[5]); $max[6]=sprintf ("%.0f", $max[6]);
     }

     #add av devices to data
     $data[2].=",rel_avr_dsk,worst_dsk";
     $data[6].=",avrg_disk,worst_disk";
     $data[4].=":$max[0],0,$max[1],0";         # average read/write
     $data[8].=":$max[2],0,$max[3],$max[4]";   # average wait, $w,%b

     #add min/max device to data
     $data[4].=":$max[5],0,$max[6],0";         # min read/write
     $data[8].=":$max[7],0,$max[8],$max[9]";   # max wait, $w,%b


  }


  # datastring
  $d=join(";",@data);
  $d=~s/\s+//g;
                                # data[11]

  #print "304 lib ### \n$d\n";

  $log="$data[0];$data[1];$data[2];Data1;$data[4];Data2;$data[8]";
  $log=substr($log,0,300);
  $log=~s/;/;\&nbsp;\&nbsp;\&nbsp;/g;
  $log=~s/:/:\&nbsp;\&nbsp;/g;

  #debug
  #print "\n\n$d\n";

  return ("$log","$d");
}



###############  parameter: commastring
sub get_avrg  {
###############

  my $data=$_[0];           # 7,3,5..
  my $i;

  $data=~s/,+$|^,+|\s+//g;
  my @data=split(/,/,$data);

  $n=@data;
  if (($n==0) || ($data eq "")) { return (0); }


  # sum all data
  $data="0";
  foreach my $d (@data) {
        $data=$data+$d;
  }
  $data=$data/$n;                                # average
  $data=sprintf ("%.0f", $data);                 # round integer

  #return (0);
  # print "### 964 -$data-\n";

  return ($data);
}

################
 sub stat2table {
 ################
   my ($l,@l,$tc,$c,$table,@td,$td,$i,$opt,$o,$color,$r,$w,$lines,@t,$font,$n,@w,@tc,@c,@snap);
   $l=$_[0];

   #width 100% auto
   $l=~s/ +/\t/gs;

   @l=split(/\n/,$l);
   $lines=@l;

   # get max colum number
   foreach $l (@l) {
        $l=~s/^\t//gs;                # spaces at the beginning
        $c=($l=~tr/\t//);             # count tabs in line
        if ($c > $tc) { $tc=$c; }
   }

   # set min 10 characters in each column:

   $r="\n";
   for ($i=0;$i<=$tc;$i++) {
         $r.='_' x10;
         $r.="\t";
   }
   $r=~s/\t+$//;
   push(@l,$r);


   # remove last value (fill the rest)
   pop(@tc);

   #create table
   $table="<table class='list2table' border='0' style='width:1000px; padding: 0; margin-left:-2px; padding-right: 6px'>\n";
   $r="2";

   $font="font-family: 'SourceCodeProRegular'; size:9px";
   foreach $l (@l) {

           $table.="\t<tr>\n";
           @td=split(/\t/,$l);
           $i="0";
           $o=$opt;

           if ($r/2 == int($r/2))  {                  #gerade/ungerade reihe
                    $color="td1_bg";
           } else {
                    $color="td2_bg";
           }


           $n="1";                                    #td counter
           for ($i=0;$i<=$tc;$i++) {
                $td=$td[$i];
                if ($td eq "") { $td="\&nbsp;"; }
                $table.="\t\t<td class=\"$color\" nowrap style=\"$bold\">\&nbsp;$td\&nbsp</td>\n";
                #option;
                $o=~s/=$i=/$td/g;                     #ersetze option z.b. =0= mit wert[0]
                  $n++;
           }
           #add option
           if ($opt ne "") {
             $table.="\t\t<td nowrap>$o</td>\n";
           }
           $table.="\t</tr>\n";
           $r++;
   }
   $table.="</table>\n";

   #debug
   #$table.="<hr>widths: $w<hr> snapp: $w2<hr>allposition: $current{'list2table'}<hr>used snap: $positions<hr>";
   return ($table);
 }

# get percent n1 of n2 (value in K,M,G,T,P)
sub my_percent {
    my $used=&my_kbytes($_[0]);
    my $avail=&my_kbytes($_[1]);

    my $total=$used+$avail;
    my $p;
    if (!($total > 0)) { return ("0"); }
    $p=$used*100/$total;    # filled  in percent

    $p = sprintf ("%.0f", $p);
    return($p);
}

sub my_kbytes {
    my $b=$_[0];
    if ($b=~/K/) { $b=~s/K//; return($b); }
    if ($b=~/M/) { $b=~s/M//; $b=$b*1000; return($b); }
    if ($b=~/G/) { $b=~s/G//; $b=$b*1000000; return($b); }
    if ($b=~/T/) { $b=~s/T//; $b=$b*1000000000; return($b); }
    if ($b=~/P/) { $b=~s/P//; $b=$b*1000000000000; return($b); }
}


#
1;
