#!
# get-disk-sas2.pl
#
# (c) Gnther Alka 2010-2020
# 30.5.16  mpt_sas detection
# 8.5.17, check opt sas adress as wwn
# 03 apr 18, lsiutil
# 18. apr 18 disk location
# 10. dec 2018 background option
# 29. dec 2018 problem with a mix of sas2 and sas3 hbas, depends on sas3ircu
# 18.aug 2020 permission /tmp/saslist.ircu

# wwn detection of lsi hba is done in tw0 steps
# 1. detect all via driver debugging
# 2. overwrite the values via sas2ircu/ sas3ircu

# reason
# mdb is faster and supports all hba's
# sasircu is slower but gives more infos and gives sometimes other (correct) results
# sometimes values are completely different ex 0 vs 2, soetimes up/down order is different
# On Omni mostly they give the same info, on Solaris it seems better to only use sasircu
# on Omni sas3ircu is a problem as is only working on Solaris (or must be patched what can affect stabilit)
# if you want ses detection, you need the sasircu values

# to solve this, first run is mdb and second sasircu to overwrite values

use strict;

$disk{'all_lsi'}="";
$disk{'all_nlsi'}="";

# step 1: get all lsi hba
&get_mylsi_all_hba;

# get lsi disks, slot and target
&get_mylsi_dsks_from_driver;

#debug
#&print_hash(%disk);

# get all lsi disks from sasircu, overwrite values from mdb as sasircu is more accurate
if ($disk{'force_mpt_sas_detection'} ne "1") {
  &get_mydisk_sasircu;
}

#debug
#&print_hash(%disk);

# get usage (all_lsi_used etc)
&get_my_lsi_usage;

#debug
#&print_hash(%disk);

# fill %disk with disk sas2/3ircu infos
#obsolet &get_mydisk_sas2;           # load values on require  in menu sas2 extension



############### menu disk location and map, via sasircu
sub my_enc_blink {
###############
    my ($out, $ant, @l, $l, @c, $c, $n, $t, $d, @w, $w, $k, $i, $sn, $u, $con, $enc, $slot, $prot, @l, @v, $s);

    print <<EoF;
   <script type="text/javascript">
      \$("#mylog_close").animate({height: 'hide', opacity: 'hide'}, 'fast');
      \$("#mylog_cmd").animate({height: 'hide', opacity: 'hide'}, 'fast');
   </script>
EoF

    if ($disk{'force_mpt_sas_detection'} eq "1") {
      print "<script language='javascript'>document.getElementById('messages').innerHTML=\'<b class=\"font_darkred\">Only available with sasircu detection, not with mpt_sas disk detection</b>\'</script>\n";
      sleep(3);
      $in{'list'}="";
      &reload;
    }

    #set led to on
         @w=split(/:/,$in{'prop'});
         $con=$w[1]; $con=~s/c//;
         $enc=$w[0]; $enc=~s/e//;

         $l="$in{'disk'}\_proto";
         $prot=$disk{$l};

         $slot=$w[2]; $slot=~s/s//;
         print "<b><br><br>This will activate the red alert on supported backplanes for 20s. <hr>";
         print "Disk: $in{'disk'} ";
         $l="$in{'disk'}\_product";       print "product: $disk{$l} ";
         $l="$in{'disk'}\_cap";           print "cap: $disk{$l} ";
         $l="$in{'disk'}\_pool";          print "pool: $disk{$l} ";
         $l="$in{'disk'}\_vdev";          print "vdev: $disk{$l} <br><br>";

         print "Prot: $prot <br>";
         print "Cont: $con <br>";
         print "Encl: $enc <br>";
         print "Slot: $slot </b>";

         ### SAS2/3 ###
         $l="$in{'disk'}\_proto";

         if ($disk{$l}=~/SAS2/i) {

               $t="$in{'disk'}"."_sas_slot";
               $t=$disk{$t};

               $s="$in{'disk'}"."_slot";
               $s=$disk{$s};

               if (($t eq "") || ($s ne $t)) {
                 print "<script language='javascript'>document.getElementById('messages').innerHTML=\'<b class=\"font_darkred\">Slotinfo not available</b>\'</script>\n";
                 sleep(2);
                 $in{'list'}="";
                 &reload;
               }

               $t=~s/[a-z]//g;  # remain 1:1:4  (e:c:s)

               @t=(); @t=split(/:/,$t);
               $enc=$t[0]; $con=$t[1]; $slot=$t[2];

               $t="$in{'disk'}"."_sas_slot";
               print "<script language='javascript'>document.getElementById('messages').innerHTML=\'Activate alert LED on backplane <b style=\"color:red\">$disk{$t}</b> - please <b style=\"color:red\">verify via dd</b><br>as sas2ircu enables wrong led on some HBAs, please wait about 10s\'</script>\n";
               &exe("/var/web-gui/_my/tools/sas2ircu/sas2ircu $con LOCATE $enc\:$slot ON");

               sleep (10);
               print "<script language='javascript'>document.getElementById('messages').innerHTML='Switch LED off, please wait for a page refresh!'</script>\n";
               &exe("/var/web-gui/_my/tools/sas2ircu/sas2ircu $con LOCATE $enc\:$slot Off");

               $in{'list'}="";
               &reload;

         }
         if ($disk{$l}=~/SAS3/i) {

               $t="$in{'disk'}"."_sas_slot";
               $t=$disk{$t};

               $s="$in{'disk'}"."_slot";
               $s=$disk{$s};

               if (($t eq "") || ($s ne $t)) {
                 print "<script language='javascript'>document.getElementById('messages').innerHTML=\'<b style=\"color:red\">Slotinfo not available</b>\'</script>\n";
                 sleep(2);
                 $in{'list'}="";
                 &reload;
               }

               $t=~s/[a-z]//g;  # remain 1:1:4  (e:c:s)

               @t=(); @t=split(/:/,$t);
               $enc=$t[0]; $con=$t[1]; $slot=$t[2];

               $t="$in{'disk'}"."_sas_slot";
               print "<script language='javascript'>document.getElementById('messages').innerHTML=\'Activate alert LED on backplane <b style=\"color:red\">$disk{$t}</b> - please <b style=\"color:red\">verify via dd</b><br>as sas3ircu enables wrong led on some HBAs, please wait about 10s\'</script>\n";
               &exe("/var/web-gui/_my/tools/sas3ircu/sas3ircu $con LOCATE $enc\:$slot ON");

               sleep (10);
               print "<script language='javascript'>document.getElementById('messages').innerHTML='Switch LED off, please wait for a page refresh!'</script>\n";
               &exe("/var/web-gui/_my/tools/sas3ircu/sas3ircu $con LOCATE $enc\:$slot Off");
               $in{'list'}="";
               &reload;
         }
         &log_end;
         print "<script language='javascript'>document.getElementById('messages').innerHTML='No SAS Controller with Enclosure support found'</script>\n";
         print "<br><br>No SAS Controller with Enclosure support found";
         exit;
}




############### menu disk location and map, end via reload menu
sub my_dd_blink {
###############
   my ($out, $ant, @l, $l, @c, $c, $n, $t, $d, @w, $w, $k, $i, $sn, $u, $con, $enc, $slot, $prot, @l, @v);

   my ($ask,$close,$form,$info,$t,$opt,$r,$p,$d,$z,$f);
  # $close="<a href=\"/cgi-bin/admin.pl?id=$in{'id'}\&amp;action=\&amp;l1=$in{'l1'}\&amp;l2=$in{'l2'}\&amp;l3=$in{'l3'}\&amp;list=$in{'list'}\&amp;zfs=$in{'zfs'}\"><b>\&nbsp;cancel\&nbsp;/close\&nbsp;</b></a>";

   print <<EoF;
   <script type="text/javascript">
      \$("#mylog_close").animate({height: 'hide', opacity: 'hide'}, 'fast');
      \$("#mylog_cmd").animate({height: 'hide', opacity: 'hide'}, 'fast');
   </script>
EoF
         $txt{'disk_dd_mess'}=~s/=d=/$in{'prop'}/g;
         print "$txt{'disk_dd_mess'}";
         print "<b><br><br>Locate disk by dd-traffic, selected disk should blink <hr>";
         print "Disk: $in{'disk'} ";
         $l="$in{'disk'}\_product";
         print "<br>product: $disk{$l} ";

         $l="$in{'disk'}\_cap";           print "cap: $disk{$l} ";
         $l="$in{'disk'}\_pool";          print "pool: $disk{$l} ";
         $l="$in{'disk'}\_vdev";          print "vdev: $disk{$l} <br><br>";
         print "<font style='color: red'> - klick on menu \'Disk Location\' to cancel -</font>";
         $in{'list'}="";


$t=<<EoF;
  dd detection for disk <b style='color:red'>$in{'prop'}</b> will be enabled for a few seconds. <br>Please wait or select another disk/menu.
EoF
&messages_box($t);
&messages_fadein;

         print "<br><br><br><br><br><br>";

         ### form ###
         #$l="$in{'disk'}\_product";
         #print &ask_pop("disk disk activity led is on (100%):<br><br>$in{'disk'}<br>$disk{$l}<br>","<br><font style='color: red'> - select another disk or close to cancel detection -</font>","");

         &exe("dd if=/dev/dsk/$in{'prop'} of=/dev/null bs=10480 count=500000");
         &reload;

}

####################    # get used, unused etc
sub get_my_lsi_usage {
####################
my ($r,$c,@c,@w,$k,$n,@v,@d,$enc);

#default lsi 2008/3008
  $disk{'all_lsi_used'}="";          # part of a pool
  $disk{'all_lsi_unused'}="";

# sata and other
  $disk{'all_nlsi'};
  $disk{'all_nlsi_used'}="";
  $disk{'all_nlsi_unused'}="";

  @d=split(/\t/,$disk{'all_dsk'});                # all disks
  foreach my $d (@d) {

      #used (member of a pool)
      $k="$d"."_pool";

      # nlsi?
      if (!($disk{'all_lsi'}=~/$d/)) {
         if (!($disk{'all_nlsi'}=~/$d/)) {
            $disk{'all_nlsi'}.="\t$d";
         }

         if ($disk{$k}=~/\w/) {                # poolmember
            if (!($disk{'all_nlsi_used'}=~/$d/)) {
               $disk{'all_nlsi_used'}.="\t$d";
            }
         } else {
            if (!($disk{'all_nlsi_unused'}=~/$d/)) {
              $disk{'all_nlsi_unused'}.="\t$d";
            }
         }
         next;
      }

      # wwn
      if ($disk{$k}=~/\w/) {                # poolmember
          if (!($disk{'all_lsi_used'}=~/$d/)) {
            $disk{'all_lsi_used'}.="\t$d";
          }
      } else {
          if (!($disk{'all_lsi_unused'}=~/$d/)) {
            $disk{'all_lsi_unused'}.="\t$d";
          }
      }

  }
  $disk{'all_lsi'}=~s/^\t+//;
  $disk{'all_lsi_used'}=~s/^\t+//;
  $disk{'all_lsi_unused'}=~s/^\t+//;

  $disk{'all_nlsi'}=~s/^\t+//;
  $disk{'all_nlsi_used'}=~s/^\t+//;
  $disk{'all_nlsi_unused'}=~s/^\t+//;

}

###################     disks, target and slot
sub get_mylsi_dsks_from_driver {
###################
  my ($r,$c,@c,@w,$k,$n,$n2,$n3,@v,@d,$enc,$type,$wwn);

  $enc="1";                                       # default enclosure
  @d=split(/\t/,$disk{'all_dsk'});                # all disks

  if ($sys{'auto_os'}=~/Solaris/i) {
       #
       if ($disk{'lsi_drvs'}=~/mpt/) {
         $r.="mpt_sas\n";
         $r.=`echo "::mptsas -t" | sudo mdb -k`;
       }

       if ($disk{'lsi_drvs'}=~/lsc/) {            # Solaris 3008 HBA
         $r.="lsc_sas\n";
         $r.=`echo "::lsc -t" | sudo mdb -k`;
       }

#&mess("311 <br>$r");
       # wwn same line as sas and slots
       #$r=~s/\n5/  5/gs;

       @w=();
       @w=split(/\n/,$r);

       # sas 2
       # TGT                             SA  TYPE #P STATUS    DEVHDL
       # DEVNAME                         UA NCMDS #T DEVINFO
       # ffffa1c0468e2b00  4433221107000000  SATA  7 Transient 9
       # xx(00 or wwn)     w55cd2e414dbdfc5d     0  0

       # sas3
       # TGT               SA                 DEVHDL    TYPE  #Port STATUS FLAGS
       # DEVNAME                         UA  DEVINFO NCMDS #T

       # ffffa1c01716ba00  4433221101000000        9    SATA   3    Stable
       # 50014ee25906bcb2 w50014ee25906bcb2      881     0  0

       # port/slot is either pos3 (sas2) or pos4 (sas3)
       # target    is either pos5 (sas2) or pos2 (sas3)

       $n2="-1";     # controller nr sas2
       $n3="-1";     # controller nr sas3
       for ($i=0;$i<=@w; $i++) {
            #sas2/3 type

            if ($w[$i]=~/^mpt_sas/) {           #switch type
              $type="SAS2"; next;
            }
            if ($w[$i]=~/^lsc_sas/) {
              $type="SAS3"; next;
            }

            if ($w[$i]=~/^TGT/) {                       # begin controller
                if ($type eq "SAS2") { $n2++; next; }   # count 0,1,2..
                if ($type eq "SAS3") { $n3++; next; }
            }

            # not a data line
            if (!($w[$i]=~/^ffff/))  { next; }
            if (!($w[$i+1]=~/ w5/))  { next; }      # no wwn in next line

            $wwn=$w[$i+1]; $wwn=~s/.* w5/5/;        # keep wwn 500...
            $wwn=~s/ .*//;

            @v=();
            @v=split(/ +/,$w[$i]);

            # for all disks
            foreach my $d (@d) {

                if (length($d) < 15)       { next; }   # not a wwn
                if (!($d=~/$wwn/i))        { next; }   # not this disk

                # mdb proto
                $k="$d"."_mdb_proto";
                $disk{$k}=$type;                  # sas controller type for this disk SAS2 or SAS3

                $k="$d"."_proto";
                $disk{$k}="$type (mdb)";                  # can be overwritten by sasircu

                # mdb slot enclosure: controller : slot
                $k="$d"."_mdb_slot";
                if ($type eq "SAS2") {
                  $n=$n2;
                  $disk{$k}="e$enc".":m$n".":s$v[3]";   # ex  e1:m1:s04

                  $k="$d"."_mdb_hba";
                  $disk{$k}="m$n";

                } else {
                  $n=$n3;
                  $disk{$k}="e$enc".":l$n".":s$v[4]";   # ex  e1:l1:s04

                  $k="$d"."_mdb_hba";
                  $disk{$k}="l$n";
                }


                # slot enclosure: controller : slot, can be overwritten by sasirci
                $k="$d"."_slot";
                if ($type eq "SAS2") {
                  $n=$n2;
                  $disk{$k}="e$enc".":m$n".":s$v[3]";   # ex  e1:m1:s04
                } else {
                  $n=$n3;
                  $disk{$k}="e$enc".":l$n".":s$v[4]";   # ex  e1:l1:s04       # Solaris
                }

                #slot_disk (reverse assignement for map)        ex e1:m1:s4_disk -> c0t500...
                if ($type eq "SAS2") {
                   $k="e$enc".":m$n".":s$v[3]"."_mdb_disk";
                } else {
                   $k="e$enc".":l$n".":s$v[4]"."_mdb_disk";
                }
                $disk{$k}=$d;

                # target/dev
                $k="$d"."_mdb_t";
                if ($type eq "SAS2") {
                  $disk{$k}=$v[5];
                } else {
                  $disk{$k}=$v[2];
                }

                # mpt_sas
                $k="$d"."_mdb_sas";
                $disk{$k}=$v[1];

                # mpt_guid
                $k="$d"."_mdb_guid";
                $disk{$k}=$wwn;

                #$disk{'all_lsi'}
                if (!($disk{'all_lsi'}=~/$d/)) {
                  $disk{'all_lsi'}.="\t$d";
                }
            }
       }
#&print_hash(%disk);
       #############
  } else { # Illumos
       #############

        $r=`echo "::mptsas -t" | sudo mdb -k`;
#&mess("311 <br>$r");

        @w=();
        @w=split(/\n/,$r);
        $n="-1";
        foreach my $w (@w) {
             $w=~s/^ +//;
             if ($w=~/==/) { $n++; }        # ctrl number start with 0
             if (!($w=~/ 5/))   { next; }   # not a disk with a wwn

             @v=();
             @v=split(/ +/,$w);             # ffffc1252ed26480    4    9   5001b44c4bdd4884   0     MAX
                                            # addr                slot dev wwn
             if (length($v[3]) < 15)        { next; }     # wwn

             # for all disks
             foreach my $d (@d) {
                if (!($d=~/$v[3]/i))       { next; }   # not this disk
                if (length($d) < 15)       { next; }   # not a wwn

                # info ex
                # c1t5000CCA23DF2EEA4d0_sas_protocol    ->        'SAS2-SATA'   oder 'SAS2'
                # c1t5000CCA23DF2EEA4d0_sas_slot        ->        'c1:e1:s4'
                # c1t5000CCA23DF2EEA4d0_target          ->        '10'               (dev)
                # c1t5000CCA23DF2EEA4d0_mpt_sas         ->        'ffffc1252ed26480'

                # lsi_hba_1_sas        ->        'SAS3' # from n

                # mdb proto
                $k="$d"."_mdb_proto";
                $t="lsi_hba_$n"."_sas";
                $disk{$k}=$disk{$t};                  # sas controller type for this disk

                # proto can be overwritten with sasircu
                $k="$d"."_proto";
                $t="lsi_hba_$n"."_sas";
                $disk{$k}="$disk{$t} (mdb)";                  # sas controller type for this disk

                # mdb slot enclosure: controller : slot
                $k="$d"."_mdb_slot";
                $disk{$k}="e$enc".":m$n".":s$v[1]";   # ex  e1:m1:s04           war m2:e1:s4

                # slot enclosure: controller : slot, can be overwritten by sasircu
                $k="$d"."_slot";
                $disk{$k}="e$enc".":m$n".":s$v[1]";   # ex  e1:m1:s04           war m2:e1:s4

                #slot_disk (reverse assignement for map)        ex e1:m1:s4_disk -> c0t500...
                $k="e$enc".":m$n".":s$v[1]"."_disk";
                $disk{$k}=$d;

                #controller number ex m0
                $k="$d"."_mdb_hba";
                $disk{$k}="m$n";

                # target/dev
                $k="$d"."_mdb_t";
                $disk{$k}=$v[2];

                # mpt_sas
                $k="$d"."_mdb_sas";
                $disk{$k}=$v[0];

                # mpt_guid
                $k="$d"."_mdb_guid";
                $disk{$k}=$v[3];

                # preset sn (can be overwritten by sasircu with sas sn)
                $k="$d"."_sas_sn";
                $t="$d"."_sn";
                $disk{$k}="$disk{$t} (sn)";

                #$disk{'all_lsi'}
                if (!($disk{'all_lsi'}=~/$d/)) {
                  $disk{'all_lsi'}.="\t$d";
                }
             }
        }


       # &mess($r);
  }
$disk{'all_lsi'}=~s/^\t+//;

#debug
#&print_hash(%disk);
}


###################         # controller info (not disks)
sub get_mylsi_all_hba {
###################
  my ($r,$c,@c,@w,$k);
  $c='(echo 0;) | sudo /var/web-gui/data/tools/other/lsiutil.i386';
  $r=`$c`;

  #3 MPT Ports found

  #   Port Name         Chip Vendor/Type/Rev    MPT Rev  Firmware Rev  IOC
  # 1.  mpt_sas0          LSI Logic SAS3008 C0      205      0f000300     0   # on OmniOS   200 means 2008
  # 2.  mpt_sas2          LSI Logic 00c4 01         206      0f000000     0
  # 3.  mpt_sas20         LSI Logic SAS2008 B2      200      0d003900     0   # on Solaris and 3008 controller instead lsc20
  # 0   1                 2                         3        4            4   # @w

  @c=split(/\n/,$r);

  $disk{'lsi_hbas'}="0";
  $disk{'lsi2_hbas'}="0";
  $disk{'lsi3_hbas'}="0";
  foreach my $c (@c) {
         $c=~s/^ +//;
         if (!($c=~/^\d+\. /))   { next; }              # is a controller
         #print "$c<br>";
         @w=();
         @w=split(/  +/,$c);

         if ($w[3] <200)         { next; }              # ex esxi contr

         #count
         $disk{'lsi_hbas'}++;                           # number of lsi hba

         # drv name l or m                              # lsc is Solaris only for lsi 3008
         $k="lsi_hba_$disk{'lsi_hbas'}"."_drv";         # lsc or mpt
         $disk{$k}=substr($w[1],0,3);

         #drivers
         if (!($disk{'lsi_drvs'}=~/$disk{$k}/)) {
            $disk{'lsi_drvs'}.="\t$disk{$k}";           # ex: mpt \t lsc
            $disk{'lsi_drvs'}=~s/^\t+//;
         }

         #sas_protocol SAS2-SAS or SAS3-SAS             # like in sas2/3ircu
         $t="SAS3";                                     # default
         if ($w[3] <200)         { $t=""; }             # ex esxi contr
         if ($w[3]=~/200/)       { $t="SAS2"; }         # rev 200
         if ($w[2]=~/2008|2308/) { $t="SAS2"; }         # use sas2ircu

         $k="lsi_hba_$disk{'lsi_hbas'}"."_sas";         # SAS2 or SAS3
         $disk{$k}=$t;

         $k="lsi_hba_list";
         $disk{$k}.="\t$c";
         $disk{$k}=~s/^\t+//;

         if ($t eq "SAS2") {
            $disk{'lsi2_hbas'}++;
         }
         if ($t eq "SAS3")  {
            $disk{'lsi3_hbas'}++;
         }
  }


  # If you use ksi2 hba and lsi3 hba together, you must either use mpt_sas detection
  # or you need sas2ircu and/or sas3ircu
  if (($disk{'lsi2_hbas'} > 0) && ($disk{'lsi3_hbas'} > 0)) {
     $t="";
     if (!(-f "/var/web-gui/_my/tools/sas2ircu/sas2ircu")) { $t.="0"; }
     if (!(-f "/var/web-gui/_my/tools/sas3ircu/sas3ircu")) { $t.="0"; }
     if ($t=~/0/) {
       $disk{'force_mpt_sas_detection'}="1";
     }
  }
  if ($disk{'lsi2_hbas'} > 0)  {
     if (!(-f "/var/web-gui/_my/tools/sas2ircu/sas2ircu")) {
        $disk{'force_mpt_sas_detection'}="1";
     }
  }
  if ($disk{'lsi3_hbas'} > 0)  {
     if (!(-f "/var/web-gui/_my/tools/sas3ircu/sas3ircu")) {
        $disk{'force_mpt_sas_detection'}="1";
     }
  }

  #debug
  #&print_hash(%disk);
  #&print_array(@c);
  #&mess($r);

}


######################       # enc,  port from sas2/3ircu
sub get_mydisk_sasircu  {
#######################      # parameter full (= with additional v7 info GUID and SAS Adress)

   # each disk has a controller, encolosure and slot info with properties like
   # c0e1s23_Serial No -> abcd
   my ($out, $ant, $ant2, $ant3, @l, $l, @c, $c, $n, $t, @d, $d, @w, $w, $k, $i, $sn, $u, $con, $enc, $slot, $id, $ver, $ctrl);

   @w=();  @w=split(/\t/,$disk{'all_dsk'}); if (!($cfg{'ver'}=~/c|m/))   { print "$txt{'me'}"; sleep(2); &log_end; exit; }


   # add basic  like poolmember
   &load_lib('disklib.pl');
   &get_mydsk_stat;               # update unused and replaceable  aus disklib.pl

   # disk location use either detection via mpt_sas driver or via sasircu

   # v2 or 3
   if ($disk{'lsi2_hbas'} > 0) {
          if (-f '/var/web-gui/_my/tools/sas2ircu/sas2ircu') {
            $t=&exe('chmod 755 /var/web-gui/_my/tools/sas2ircu/sas2ircu');             # executable, displays all disks but no GUID
            $ant2=&exe('/var/web-gui/_my/tools/sas2ircu/sas2ircu LIST','','31 get-disk-sas2.pl');
            $ver=$ant2; $ver=~s/.*Version //s; $ver=~s/ .*//s;
            $disk{'sasircu'}="sas2ircu $ver";
          }
   }
   if ($disk{'lsi3_hbas'} > 0) {
          if (-f '/var/web-gui/_my/tools/sas3ircu/sas3ircu') {
            $t=&exe('chmod 755 /var/web-gui/_my/tools/sas3ircu/sas3ircu');             # executable, displays all disks but no GUID
            $ant3=&exe('/var/web-gui/_my/tools/sas3ircu/sas3ircu LIST 2>&1','','31 get-disk-sas2.pl');
            # segmentation fault: ant3="";
            if ($ant3 eq "") {
              if ($current{'callby.acc'}) { return; }           #
              &mess("disk detection for LSI SAS3 HBAs via sas3ircu failed<br>Try a different sas3ircu or disable sas3ircu in menu Disks > Disks Location > select sas3ircu");

            } else {

              $ver=$ant3; $ver=~s/.*Version //s; $ver=~s/ .*//s;
              $disk{'sasircu'}.=" sas3ircu $ver";
            }
          }
   }


   if (! (($ant2=~/successfully/is) || ($ant3=~/successfully/is))  )   {
              #  - No supported SAS2/3 controllers found

              $disk{'all_nlsi2'}=$disk{'all_dsk'};
              $disk{'all_nlsi2_used'}=$disk{'all_inuse'};
              $disk{'all_nlsi2_unused'}=$disk{'all_unused'};

              if (($ant2=~/error/is) || ($ant3=~/error/i)) { &mess("sasircu reports an error, check SAS2/3 controller or try a reboot:<hr>$ant2<br>$ant3"); }
              return;
   }


# to increase performance, slotinfo is only loaded when alldisk has changed
     my ($disk1,$disk2,$last);


##################
# del old logfile
$t=`sudo test -f "/tmp/nappit/saslist.ircu" && echo 1 || echo 2`;  chomp($t);
if ($t eq 1) {
  `sudo rm /tmp/nappit/saslist.ircu 2>/dev/null`;
  `touch /tmp/nappit/saslist.ircu`;
  `sudo /bin/chown napp-it /tmp/nappit/saslist.ircu  2> /dev/null`;
}


# read last
     $t=`sudo test -f "/tmp/nappit/saslist_last.ircu" && echo 1 || echo 2`;  chomp($t);
     if ($t eq "1")  {
      `sudo /bin/chown napp-it /tmp/nappit/saslist_last.ircu  2> /dev/null`;
       $last=&exe("cat /tmp/nappit/saslist_last.ircu");
     }

     my $diff=time();



#acc
# sas_last.ircu    (time)   # created/update when acc runs last (by acc)
# access.last  (time)       # create/update when user calls a menu (admin.pl)

# acc: del old files
if (!( $current{'callby.acc'} )) {                      # sub called by accelerator

  $t=`sudo test -f "/tmp/nappit/sas_last.ircu" && echo 1 || echo 2`;  chomp($t);
  if ($t eq "1")  {
     `sudo /bin/chown napp-it /tmp/nappit/sas_last.ircu  2> /dev/null`;

     # last time get_disk-sas2 is called by napp-it (not aa)
     #
     open (F,"</tmp/nappit/sas_last.ircu");
        flock(F, LOCK_EX);
        @t=<F>;
     close (F);

     $t=time;
     if ($t-$t[0] > 1200) {                              # acc files older 20 min
       `sudo rm /tmp/nappit/*.ircu`;                     # too old, rm all ircu files
     }
  }
}

# List of SAS2 controller -> @c
   @l=split(/\n/,$ant2);

   foreach $l (@l) {
           $l=~s/^\s+//;
           $l=~s/\s+/\t/g;
           if ($l=~/^\d/) {                # index
                push (@c,"2-$l");
                $l=~s/\t$//;
                @w=();
                @w=split(/\t/,$l);
                $disk{'controller_sas2'}.="$w[0]\t";
                $t="controller_sas2_$w[0]";
                $disk{$t}=$l;

                ## add sas.txt
                open (PF, ">>/tmp/nappit/saslist.ircu");
                  flock(PF, LOCK_EX);
                  print PF "$t|$l|\n";
                close (PF);

           }
   }

   # list of sas2 controller
   #&print_array(@c);

   $disk{'controller_sas2'}=~s/\t$//;                                           # ex 0\t1
   open (PF, ">>/tmp/nappit/saslist.ircu");
     flock(PF, LOCK_EX);
     print PF "controller_sas2|$disk{'controller_sas2'}|\n";
   close (PF);


# add List of SAS3 controller to @c
###############################
   @l=();
   @l=split(/\n/,$ant3);

   foreach $l (@l) {
           $l=~s/^\s+//;
           $l=~s/\s+/\t/g;
           if ($l=~/^\d/) {
                push (@c,"3-$l");
                $l=~s/\t$//;
                @w=();
                @w=split(/\t/,$l);
                $disk{'controller_sas3'}.="$w[0]\t";
                $t="controller_sas3_$w[0]";
                $disk{$t}=$l;

                ## add to sas2.txt
                open (PF, ">>/tmp/nappit/saslist.ircu");
                   flock(PF, LOCK_EX);
                   print PF "$t|$l|\n";
                close (PF);
           }
   }

   # list of sas3 controller
   #&print_array(@l);

   # list of all controller
   #&print_array(@c);

   $disk{'controller_sas3'}=~s/\t$//;
   ## add to sas2.txt
   open (PF, ">>/tmp/nappit/saslist.ircu");
     flock(PF, LOCK_EX);
     print PF "controller_sas3|$disk{'controller_sas3'}|\n";
   close (PF);


   #debug controller
   #&mess($disk{'controller_sas3'});
   #&print_array(@c);


   ##############
   # SAS2 details
   ##############

   my $controllerinfo;
   my $irvolinfo;
   my ($enclosureinfo,$enclosure);
   my ($physinfo,$phys);
   my $devtype;
   my %dsk;
   my $sas;
   my $nrl=0;  # number of lsc
   my $nrm=0;  # nr of mrsas device

   $ant="";

   &log_add("\n##########\nall_dsk\n##########\n$disk{'all_dsk'}");    # add to log

   # each sas2 and sas3 controller
   #&print_array(@c);

   foreach $c (@c) {      # each controller
         chomp($c);
         $t=$c; $t=~s/\t.*//;

         $ant="";                                           #  disk list

if ($current{'callby.acc'}) { print "acc controller: $t\n"; }

         #############
         #### step 1 get all disks but guid is not displayed  (v7 shows guid but not all disks)
         #############

         #########
         #sas2 hba
         #########

         if ($t=~/2\-/) {             # number only like 0,1,2..
              $t=~s/2\-//;

              if ((-f "/tmp/nappit/sas2ircu_$t\_display.ircu") && (!($current{'callby.acc'})) )  {
if ($current{'callby.acc'}) { print "read sas2ircu_$t\_display.ircu\n"; }

                open (F,"</tmp/nappit/sas2ircu_$t\_display.ircu");
                      flock(F, LOCK_EX);
                      @t=<F>;
                close (F);
                $ant=join("",@t);
                &exe("cat /tmp/nappit/sas2ircu_$t\_display.ircu");                   #log
                $current{'acc_mess'}="acc";

              } else {

if ($current{'callby.acc'}) { print "write sas2ircu_$t\_display.ircu\n"; }
                 $ant=&exe("/var/web-gui/_my/tools/sas2ircu/sas2ircu $t display");
                 if ($current{'callby.acc'}) {                                           # sub called by accelerator
                     open (PF, ">/tmp/nappit/sas2ircu_$t\_display.ircu");
                           flock(PF, LOCK_EX);
                           print PF "$ant";
                     close (PF);
                 }
              }

              $sas="2";
              $con="m$nrm";     # Solaris and Illumos use mpt_sas driver
              $nrm++;
         }

         #########
         #sas3 hba
         #########

         if ($t=~/3\-/) {
             $t=~s/3\-//;

             if ((-f "/tmp/nappit/sas3ircu_$t\_display.ircu")   && (!($current{'callby.acc'})) ) {
                open (F,"</tmp/nappit/sas3ircu_$t\_display.ircu");
                      flock(F, LOCK_EX);
                      @t=<F>;
                close (F);
                $ant=join("",@t);
                &exe("cat /tmp/nappit/sas3ircu_$t\_display.ircu");                   #log
                $current{'acc_mess'}="acc";

             } else {

                if (-f '/var/web-gui/_my/tools/sas3ircu/sas3ircu') {
                  $ant=&exe("/var/web-gui/_my/tools/sas3ircu/sas3ircu $t display");    # displays all disks
                  if ($current{'callby.acc'}) {                                        # sub called by accelerator
                     open (PF, ">/tmp/nappit/sas3ircu_$t\_display.ircu");
                           flock(PF, LOCK_EX);
                           print PF "$ant";
                     close (PF);
                   }
                }
             }
             $sas="3";

              if ($sys{'auto_os'}=~/Solaris/i) {
                $con="l$nrl";                              # Solaris use lsc driver
                $nrl++;
              } else {
                $con="m$nrm";
                $nrm++;
              }
#&mess("828 debug $ant<br>$con");

         }

         #debug
         #print "Content-type: text/html\n\n<pre>$ant"; exit;

         if ($ant=~/successfully/is) {

               $physinfo=$ant;
               $physinfo=~s/.*Physical device information\n\-*\n*//is;
               $physinfo=~s/\-\-\-+.*//s;
               @t=();
               @t=split(/Device is a/,$physinfo);
               $disk{'devices'}=@t;
               $disk{'devices'}--;

               ## add sas.txt
               open (PF, ">>/tmp/nappit/saslist.ircu");
                 flock(PF, LOCK_EX);
                 print PF "devices|$disk{'devices'}|\n";
               close (PF);

               #debug devices/disks
               # print "Content-type: text/html\n\n$#t";
               # &print_array(@t);  exit;

               # get keys/ values from all devices
               # line 1: Device is a Hard disk
               # line 2: Enclosure #   :  2
               # line 3: Slot #        :  23


               # disks
               for ($i=1;$i<@t; $i++) {
                        #
                      $devtype="";
                      $enc="";
                      $slot="";
                      %dsk=();
                      $dsk{'con'}=$con;

                      @l=();
                      @l=split(/\n/,$t[$i]);

                      # debug lines of a device
                      # if ($t[$i]=~/3LN4TW4X/s) {       # sn
                      #   print "Content-type: text/html\n\n";
#&print_array(@l);
                      # }

                      # lines of a device
                      foreach my $l (@l) {

                                           $l=~s/^ +//;

                                           # device type, line1, ex "Hard disk"
                                           if (!($l=~/:/)) {
                                               $dsk{'sas_devtype'}=$l;
                                               next;
                                           }
                                           if (!($l=~/:/)) { next; }

                                           @w=();
                                           @w=split(/ *: */,$l);

                                           if ($w[0] eq "") { next; }

                                           # Enclosure, line 2
                                           if ($w[0]=~/^Enclosure/) {
                                               $dsk{'enc'}=$w[1];
                                               next;
                                           }

                                           # slot, line 3
                                           if ($w[0]=~/^Slot/) {
                                               $dsk{'slot'}=$w[1];
                                               next;
                                           }


                                           #sas adress = 5003048-0-0181-dafd -> WWN ?
                                           if ($w[0]=~/SAS Address/i) {
                                               $dsk{'sasaddr'}=$w[1];
                                               $dsk{'sasaddr'}=~tr/a-z/A-Z/;
                                               $dsk{'sasaddr'}=~s/[^A-Z0-9]//g;       #normalize  /could be diff with '-'
                                               next;
                                           }

                                           # Sn
                                           if ($w[0]=~/Serial/i) {
                                               $dsk{'sn'}=$w[1];
                                               $dsk{'sn'}=~tr/a-z/A-Z/;
                                               $dsk{'sn'}=~s/[^A-Z0-9]//g;            #normalize  /could be diff with '-'
                                               next;
                                           }

                                           # State
                                           if ($w[0]=~/^State/) {
                                               $dsk{'state'}=$w[1];
                                               next;
                                           }

                                           # GUID
                                           if ($w[0]=~/^GUID/) {
                                               $dsk{'guid'}=$w[1];
                                               next;
                                           }


                                           # Protocol
                                           if ($w[0]=~/^Protocol/) {
                                               $dsk{'proto'}="SAS$sas-$w[1]";
                                               next;
                                           }
                                           # Drive type
                                           if ($w[0]=~/^Drive Type/) {
                                               $dsk{'drive_type'}=$w[1];
                                               next;
                                           }


                      } # end lines of device
                      #
                      # debug
                      #  print "Content-type: text/html\n\n<pre>sn=$dsk{'sn'}\n";
                      #  &print_array(@l);  exit;
                      # &print_hash(%dsk); exit;

                      # get id from guid (same without d0 at the end)
                      @w=();
                      @w=split(/\t/,$disk{'all_dsk'});
                      $id="";
                      foreach $w (@w) {
                           if ($dsk{'guid'} ne "") {
                               if ($w=~/$dsk{'guid'}/i) { $id=$w; last; }
                           }
                      }

                      # in some cases, sas2ircu displays a differnt GUID than iostat (different to WWN)
                      # but sas address is identical to wwn beside some '-'
                      if ($id eq "") {
                         foreach $w (@w) {
                              $t=$dsk{'sasaddr'};
                              if ($t ne "") {
                                 if ($w=~/$t/i) { $id=$w; last; }
                              }
                         }
                      }

                      # in some cases, sas2ircu displays a differnt GUID than iostat (not included in WWN)
                      # the difference was that sas2ircu supress the last number in GUID compared to wwn
                      if ($id eq "") {
                         foreach $w (@w) {
                              $t=$dsk{'guid'};
                              chop($t);
                              if ($t ne "") {
                                 if ($w=~/$t/i) { $id=$w; last; }
                              }
                         }
                      }

                      # write to %disk
                      if ($id ne "") {

                           ## add sas2.txt
                           open (PF, ">>/tmp/nappit/saslist.ircu");
                                                      flock(PF, LOCK_EX);

                                                      $t="$id\_sas_slot";
                                                      $disk{$t}="e$dsk{'enc'}\:$con\:s$dsk{'slot'}";
                                                      print PF "$t|$disk{$t}|\n";

                                                      $t="$id\_slot";             # overwrite mdb
                                                      $disk{$t}="e$dsk{'enc'}\:$con\:s$dsk{'slot'}";

                                                      # reverse slot -> wwn
                                                      $t="e$dsk{'enc'}\:$con\:s$dsk{'slot'}\_disk";
                                                      $disk{$t}=$id;
                                                      print PF "$t|$id|\n";

                                                      $t="$id\_sas_guid";
                                                      $disk{$t}="$dsk{'guid'}";
                                                      print PF "$t|$disk{$t}|\n";

                                                      $t="$id\_sas_state";
                                                      $disk{$t}="$dsk{'state'}";
                                                      print PF "$t|$disk{$t}|\n";

                                                      $t="$id\_sas_proto";
                                                      $disk{$t}="$dsk{'proto'}";
                                                      print PF "$t|$disk{$t}|\n";

                                                      $t="$id\_proto";
                                                      $disk{$t}="$dsk{'proto'}";     # overwrite mdb

                                                      $t="$id\_sas_drive_type";
                                                      $disk{$t}="$dsk{'drive_type'}";
                                                      print PF "$t|$disk{$t}|\n";

                                                      $t="$id\_sas_sn";
                                                      $disk{$t}="$dsk{'sn'}";
                                                      print PF "$t|$disk{$t}|\n";
                           close (PF);

                    }


                    ###############
                    # end of step 2
                    ################

                    # add
                     if (!($disk{'all_lsi'}=~/$id/)) {
                       $disk{'all_lsi'}.="$id\t";
                     }

                     #lsi (un)used?
                     if ($disk{'all_inuse'}=~/\b$id\b/) {
                             $disk{'all_lsi_used'}.="$id\t";
                     } else {
                             $disk{'all_lsi_unused'}.="$id\t";
                     }


               }  # /devices

               # get non lsi
               ##############

               # debug
               #   print "Content-type: text/html\n\n<pre>227 \n";
               #   &print_hash(%disk); exit;

               # 3. get enclosureinfo
               ######################
                  $enclosureinfo=$ant;
                  $enclosureinfo=~s/.*Enclosure information\n\-*\n*//is;
                  $enclosureinfo=~s/\-\-\-+.*//s;
                  @t=();
                  @t=split(/Enclosure#/,$enclosureinfo);
                  $disk{'enclosures'}=@t;
                  $disk{'enclosures'}--;


                  ## add sas2.txt
                  open (PF, ">>/tmp/nappit/saslist.ircu");
                    flock(PF, LOCK_EX);
                    print PF "enclosures|$disk{'enclosures'}|\n";
                  close (PF);

                  # debug
                  #print "Content-type: text/html\n\n<pre>$ant";
                  #&print_array(@t);  exit;

                  # get keys/values from all enclosures

                  for (my $i=1;$i<@t; $i++) {
                        #
                        $enc="";

                        @l=();
                        @l=split(/\n/,$t[$i]);

                        foreach my $l (@l) {
                             #
                             $l=~s/^ +//;

                             @w=();
                             @w=split(/ *: */,$l);
                             $w[0]=~s/^ +//;

                             # first line= Enc-nr
                             if ($w[0] eq "") {
                                  $enc=$w[1];
                                  next;
                             }                 # enclosure number only

                             if (!($disk{'enclosure_keys'}=~/$w[0]/))   {         # new key
                                 $disk{'enclosure_keys'}.="$w[0]\t";
                             }
                             $t="enclosure$enc"."_$w[0]";
                             $w[1]=~s/\s+/ /g;
                             $disk{$t}=$w[1];
                        }
                  }
                  $disk{'enclosure_keys'}=~s/\t$//;


         } else { #/ if success, otherwise ex no sas3ircu


         }

  } #/@c
  #######################################################################

  $disk{'all_lsi'}=~s/\t$//;
  $disk{'all_lsi_used'}=~s/\t$//;
  $disk{'all_lsi_unused'}=~s/\t$//;

  ## add sas2.txt
  open (PF, ">>/tmp/nappit/saslist.ircu");
           flock(PF, LOCK_EX);
           print PF "all_lsi|$disk{'all_lsi'}|\n";
           print PF "all_lsi_used|$disk{'all_lsi_used'}|\n";
           print PF "all_lsi_unused|$disk{'all_lsi_unused'}|\n";
           print PF "enclosure_keys|$disk{'enclosure_keys'}|\n";
  close (PF);


  # get non lsi2
  ##############
  @w=();
  @w=split(/\t/,$disk{'all_dsk'});
  foreach $w (@w) {
     if (!($disk{'all_lsi'}=~/\b$w\b/)) {

          $disk{'all_nlsi'}.="$w\t";

         #nlsi (un)used?
         if ($disk{'all_inuse'}=~/\b$w\b/) {
              $disk{'all_nlsi_used'}.="$w\t";
         } else {
              $disk{'all_nlsi_unused'}.="$w\t";
         }

     }
  }
  $disk{'all_nlsi'}=~s/\t$//;
  $disk{'all_nlsi_used'}=~s/\t$//;
  $disk{'all_nlsi_unused'}=~s/\t$//;

   ## add sas2.txt
     open (PF, ">>/tmp/nappit/saslist.ircu");
           flock(PF, LOCK_EX);
           print PF "all_nlsi|$disk{'all_nlsi'}|\n";
 #          print PF "all_nlsi_used|$disk{'all_nlsi_used'}|\n";       # only sas disks detected here!
 #          print PF "all_nlsi_unused|$disk{'all_nlsi_unused'}|\n";
           print PF "sasircu|$disk{'sasircu'}|\n";
     close (PF);

  #

  @t=();  @t=split(/\t/,$disk{'all_lsi'}); $t="";


  $t=time();
  open (PF, ">/tmp/nappit/saslist_last.ircu");                                    # write time(),
          flock(PF, LOCK_EX);
          print PF "$t\n";
  close (PF);

  #debug
  #&print_hash(%disk);

}



###################### target to wwn translation to get disk of log errors like timeout on mpt_sas0, Target9
sub xxxget_mydisk_target {
######################
   #######
   my ($r,$t,@l,$l,$w,$m,$w,@w,@d,$d,$k);

   # not on Solaris echo command not working
   if ($sys{'auto_os'}=~/Solaris/) { return; }

   $r=`echo "::mptsas -t" | sudo mdb -k`;
   @l=split(/\n/,$r);
   foreach $l (@l) {
                $l=~s/^ +//;
                $l=~s/ +$//;
                $l=~s/ +/\t/g;
   }

   #  mptsas_target_t slot devhdl wwn ncmds throttle dr_f
   #  lag dups
   # ---------------------------------------------------------------
   #  ffffff05f826d680 3 9 5001b44c4bdd48c2 0 MAX
   #  INACTIVE 0
   #  ffffff05f79f2b80 2 10 5001b44c4bdd4899 0 MAX
   #  INACTIVE 0
   #  ..

   @d=();
   @d=split(/\t/,$disk{'all_dsk'});         #  c4t0d0 c5t0d0 c5t1d0 c1t5000CCA22BC92D3Ed0

   #debug
   # push (@l,@d);
   # &print_array(@l);
   # %disk=();

   # test all disks for slot and devhdl
   foreach $d (@d) {
        foreach $l (@l) {
                 @w=();
                 @w=split(/\t/,$l);

                 if (!($w[2] >= 0))   { next; }  # devhd (target) missing

                 #this disk?
                 if (!($w[3]=~/^500/)) { next; }  # wwn
                 if (!($d=~/$w[3]/i))  { next; }

                 $k=$d."_target";
                 $disk{$k}=$w[2];

                 $k=$d."_mpt_sas";
                 $disk{$k}=$w[0];
                 # $disk{$k}="$w[0] ($w[1])";

                 $k=$d."_mpt_slot";
                 $disk{$k}=$w[1];


                 #controller from c1t500...
                 $k=$d."_mpt_ctrl";
                 $disk{$k}=$d;
                 $disk{$k}=~s/t.*//;       # ex c1

        }
   }

   #debug
   #&print_hash(%disk);


}




# eof
1;
