 # get-disk.pl
#
# (c) napp-it 2010-23
# 11.1.2018 all_mirror
# fill %disk with disk infos from iostat -Enrs
# 12.6.19 blkdev, show vendor and procuct, not removed
# 13.nov 2023 iostat

 use strict;

 my ($diff,$last,$disk2);


 if (-f "/var/web-gui/_log/partitions.enabled") {   # read partition only when enabled (parted has problems with some configs)
    &log_add("\nAttention, If reading partition data via parted hangs on new disks: Run Disks - Initialize via Rollovermenu!\n");
    &get_mydisk_parted;             # get partitions from parted, use format in case of errors

    if ($disk{'parted_error'}) {
      &get_mydisk_format;           # problem with parted, run detection with format addionally (shows disks only not partitions)
    }
 }  else {
     &get_mydisk_format;
 }

 #&print_hash(%disk);                     #debug
 &get_mydisk;                       # add disk{'all'} from iostat

 #&print_hash(%disk);                     #debug

 # preload only in submenus under disk
 if ($in{'l1'} ne "") {
     &load_lib('disklib.pl');
     &get_mydsk_stat;               # update unused and replaceable  aus disklib.pl
 }


 # load lofi
 if ($disk{'all_dsk_lofi'} ne "") {
      &get_mydisk_lofi;
 }

#debug
#  &print_var($disk{'all_dsk'});
#  &print_var($disk{'all'});

 ######################## encrypted disks
 sub get_mydisk_lofi {
 ########################
     my ($k,$r,$t,@v);
     $r="";
     $r=`sudo $sys{'sbinpath'}lofiadm`;
     if (!($r=~/lofi/s)) { return; }
     my @t=split(/\n/,$r);
     foreach my $l (@t) {
         if (!($l=~/lofi/s)) { next; }
         $l=~s/\s+/ /g;
         @v=();
         @v=split(/ /,$l);        # lofidev, file or device, encrypted

         $t=$v[0];                # ex /dev/lofi/1
         if ($t =~/lofi/) {
              $k="$t";
              $disk{$k}=$v[1];

              $k="$t"."_options";
              $disk{$k}=$v[2];
         }
     }

 }

 ######################## if parted gives an error, use format
 sub get_mydisk_format {
 ########################
     my ($k,$r,$t,@data,$nr,$id,$last,$last0);
     $r="";

     # check accelerator
     if ((-f "/tmp/nappit/format_last.dsk") && ($cfg{'ver'} ne "")) {
          open (F,"</tmp/nappit/format_last.dsk");
                flock(F, LOCK_EX);
                @data=<F>;
          close (F);
          $t=time();

          # read acc
          if (($t-$data[0] < 1200) && (-f "/tmp/nappit/format.dsk"))   { # max 20min
               @data=();
               open (F,"</tmp/nappit/format.dsk");
                  flock(F, LOCK_EX);
                  @data=<F>;
               close (F);
               $r=join("",@data);
               &exe("echo read_acc_format");
                $current{'acc_mess'}="acc";
          } else {
               &exe("rm /tmp/nappit/format.dsk");
               &exe("rm /tmp/nappit/format_last.dsk");
          }

          if (!($r=~/AVAILABLE/s)) { $r=""; }
     }

#debug
#&mess($r);
     #####

     if ($r eq "") {
          &log_add("exe(get-disk.pl 86): format\n");
          $r=`echo | sudo $sys{'sbinpath'}format`;
          &log_add("get-disk.pl 90: detect disks via format: $r");
     }


#AVAILABLE DISK SELECTIONS:
#       0. c1t0d0 <VMware-Virtual SATA Hard Drive-00000001-30.00GB>
#          /pci@0,0/pci15ad,790@11/pci15ad,7e0@3/disk@0,0
#       1. c2t0d0 <VMware-Virtual disk-2.0-10.00GB>
#          /pci@0,0/pci15ad,7a0@15/pci15ad,1976@0/sd@0,0
#       2. c3t1d0 <drive type unknown>
#          /pci@0,0/pci15ad,7a0@16/pci8086,3900@0/blkdev@1,0


     my @t=split(/\n/,$r);
     foreach my $t (@t) {
             $t=~s/^ +//;

 #            if ($t=~/<drive type unknown>/) { next; }          # skip irregular entries  but keep blkdev

             if (!($t=~/^\d+\./)) {                             # 1. c1t1d0

                      # get id
                      if ($t=~/\//) {                           # /pci..
                              $id=$t; $id=~s/.*\///;

                              if ($last ne "") {
                                     $k="$last"."_format_id";
                                     $disk{$k}=$t;

                                     # add disk{'all'} from format as iostat does not add nvme disks

                                     if ($t=~/blkdev/)  {
                                         if (!($disk{'all'}=~/$last\b/)) {
                                            $disk{'all'}.="\t$last";
                                         }
                                         $k="$last"."_vendor";
                                         $disk{$k}='blkdev';

                                         # add (may not work with every blkdev)
                                         if ($last0=~/\wB>/) {          # -200 GB>
                                            $r=$last0;
                                            $r=~s/.*-//;
                                            $r=~s/>//;
                                            $k="$last"."_partcap";
                                            $disk{$k}=$r;
                                         }
                                     }
                              }

                      }
                      next;
             }

             $last0=$t;                    # remember for nvme size
             $nr=$t;
             $nr=~s/\..*//;                # nr of disk in format ex 2.

             $t=~s/^\d+\. +//;             # remove nr

             $id=$t;
             $id=~s/ .*//;                 # keeps id

             $last=$id;


             # lofi
             if ($t=~/lofi/)  {
                  $t=~s/ .*//;            # keeps id
                  $disk{'all_dsk_lofi'}.="\t$id";
             }


             #format nr (number from format ex 2.)
             $k="$id"."_format_nr";
             $disk{$k}=$nr;

             $k="$id"."_stat";
             $disk{$k}='ok';

             $k="$id"."_part";
             if ($disk{$k} eq "") {
                $disk{$k}='single';                           # not parted
             }

             if (!($disk{'all_dsk'}=~/\b$id\b/))  {           # do not add twice (from parted)
                    $disk{'all_dsk'}.="\t$id";
                    $disk{'all_dsk_count'}++;
                    $disk{'all_part'}.="\t$id";
             }

     }

     $disk{'all'} =~s/^\t+//;
     $disk{'all_dsk'} =~s/^\t+//;
     $disk{'all_part'}=~s/^\t+//;
     $disk{'all_dsk_lofi'}=~s/^\t+//;

# &print_var($disk{'all_dsk'});
#&print_hash(%disk);

 }



 #######################  use parted to read partitions
 sub get_mydisk_parted {
 #######################
       my ($r,$v,$k,@w,$w,$id,$p,$d,@d,$t,$id,@l,$l,@data);

       $disk{'all_dsk'}="";
       $disk{'all_dsk_count'}="0";

       $disk{'all_part'}="";
       $disk{'all_part_count'}="0";
       $disk{'all_dsk_nopart'}="";

       # check accelerator
       if ((-f "/tmp/nappit/parted_last.dsk") && ($cfg{'ver'} ne "")) {
          open (F,"</tmp/nappit/parted_last.dsk");
                flock(F, LOCK_EX);
                @data=<F>;
          close (F);
          $t=time();

          # read acc
          if (($t-$data[0] < 1200) && (-f "/tmp/nappit/parted_lm.dsk"))   { # max 20min
               @data=();
               open (F,"</tmp/nappit/parted_lm.dsk");
                  flock(F, LOCK_EX);
                  @data=<F>;
               close (F);
               $r=join("",@data);
               $current{'acc_mess'}="acc";
               &exe("echo read_acc_parted_lm");
          } else {
               &exe("rm /tmp/nappit/parted_lm.dsk");
               &exe("rm /tmp/nappit/parted_last.dsk");
          }
          if (!($r=~/dev/s)) { $r=""; }
       }
#debug
#&mess($r);

     #####

     if ($r eq "") {
         $r=&exe("parted -lms",'','get-disk.pl 28');
     }

     #use format in case of errors
       if ( (!($r=~/:;/s)) || ($r=~/Error:/s)) {
          $disk{'parted_error'}="1";
       }


       # BYT
       # po:    device:cap:model2:logical:physical:parttable:model1      # model1=Generic IDE,  model2=ide
       # p1..n  nr:start:end:size:fs:name:flag
       #
       # BYT

       # or without a partition table
       # Error: /dev/dsk/c5t1d0p0: unrecognised disk label


       @l=split(/\n/,$r);

       # problem: name id like c1t1d1 if only one partition (1)
       # name it like c1t1d1p1 only if there are more partitions otherwise the part may not be accessable with id+p1
       # so in a first step, count partitions

       foreach $l (@l) {
                if (!($l=~/:/))   {  next; }
                $l=~s/^\s+//s;  # important on label error

                @w=();
                @w=split(/:/,$l);
                if ($w[0]=~/\//) {                           # new device
                      $id=$w[0];
                      $id=~s/^.*\///;
                      $id=~s/p0//;
                      next;
                }
                if ($w[0]=~/^\d/) {                         # partition 1..n
                     if ($w[0] > 4)  { next; }              # part 1..4 are supported
                     $k="$id"."_part";
                     $disk{$k}.=",$w[0]";
                     $disk{$k}=~s/^,//;
                }

                # Error: /dev/dsk/c4t1d0p0: unrecognised disk label
                if (($w[0] =~/Error/) && ($w[2] =~/unrecognised disk label/)) {
#&print_array(@w);
                      $id=$w[1];
                      $id=~s/^.*\///;
                      $id=~s/p0//;
                      $k="$id"."_part";
                      $disk{$k}="1";

                      $k="$id"."_parted_error";
                      $disk{$k}="label error";

                      $k="$id"."_stat";
                      $disk{$k}='re-partition';

                      #$disk{'all_unused'}.="\t$id";

                      $disk{'all_dsk_nopart'}.="\t$id";
                      $disk{'all_dsk'}.="\t$id";
                      $disk{'all_dsk_count'}++;

                      $disk{'all_part'}.="\t$id$p";             # use no part as disk in unused
                      $disk{'all_part'}=~s/^\t//;

                      next;
                }
       }
# $disk{'all_unused'}=~s/^\t//;

#&print_array(@l);

#&mess($r);
#&print_hash(%disk);

       # now get all infos
       foreach $l (@l) {
                if (!($l=~/:/)) {  next; }

                $l=~s/\;$//;
                @w=();
                @w=split(/:/,$l);

                if ($w[0]=~/\//) {                           # new device

                             $id=$w[0];
                             $id=~s/^.*\///;
                             $id=~s/p0//;

                             $disk{'all_dsk'}.="\t$id";
                             $disk{'all_dsk_count'}++;

                             #stat
                             $k="$id"."_stat";
                             $disk{$k}='ok';

                             $k="$id"."_diskcap";                # disk capacity
                             $w[1]=~s/KB/ KB/; $w[1]=~s/MB/ MB/; $w[1]=~s/GB/ GB/; $w[1]=~s/TB/ TB/; $w[1]=~s/PB/ PB/;
                             $disk{$k}=$w[1];

                             $k="$id"."_ptable";
                             $disk{$k}=$w[5];

                             $k="$id"."_pmodel";
                             $disk{$k}=$w[6];
                             next;
                 }

                 if ($w[0]=~/^\d/) {                        # partition 1..n
                                         if ($w[0] > 4)  { next; }               # part 1..4 are supported

                                         $disk{'all_part_count'}++;

                                         # use part-nr only when there are more than 1 part
                                         $p="";
                                         $k="$id\_part";
                                         if ($disk{$k}=~/,/) { $p="p$w[0]";}    # add part nr

                                         $disk{'all_part'}.="\t$id$p";
                                         $disk{'all_part'}=~s/^\t//;

                                         $k="$id\p$w[0]\_pstart";
                                         $disk{$k}="$w[1]";

                                         $k="$id\p$w[0]\_pend";
                                         $disk{$k}="$w[2]";

                                         $k="$id\p$w[0]\_partcap";
                                         if ($w[3] eq "0") {$w[3]="-"; }
                                         $disk{$k}="$w[3]";


                                         $k="$id\p$w[0]\_ptype";
                                         $disk{$k}="$w[4]";

                                         $k="$id\p$w[0]\_pfs";
                                         $disk{$k}="$w[5]";

                                         if ($w[0] ==1)  {
                                                   $k="$id\_pfs";              # if only one part
                                                   $disk{$k}="$w[5]";
                                         }

                                         $k="$id\p$w[0]\_pflags";
                                         $disk{$k}="$w[6]";

                                         &get_mydisk_bcap("$id$p",$w[3]);      # cap and bcap

                                        # add disk{'all'} from parted as iostat does not add nvme disks
                                        if (!($disk{'all'}=~/$id\b/)) {
                                           $disk{'all'}.="\t$id";
                                        }
                 }
       }
       $disk{'all_dsk'}=~s/\t+$//;
       $disk{'all'}=~s/^\t+//;


#debug
#&mess("-$disk{'all'}-");
#&print_var($disk{'all'});

#&print_var($disk{'all_part'});
# &print_var($disk{'all_dsk'});

       #check if there are disks without partition
       @d=();
       @d=split(/\t/,$disk{'all_dsk'});
       foreach my $d (@d) {
             $t="$d\_part";
             if (!($disk{$t}=~/\d/)) {
                  $disk{$t}="0";
                  $disk{'all_dsk_nopart'}.="$d\t";
             }
       }
       $disk{'all_dsk_nopart'}=~s/\t+$//;
       #&mess($r);

 }



#####################          !!
sub get_mydisk_bcap { #
###################
  my $id=$_[0];                           # diskid
  my $bytes=$_[1];                        # bytes
  my ($ib,$iu,$b,$u,$k);

 # &mess($bytes);
# &print_hash(%disk);
  my $dez=&b_convert("$bytes","b","s");
  my $bin=&b_convert("$bytes","b","is");

  $k="$id\_partcap";
  $disk{$k}="$dez";
  if (!($dez =~/\d/)) { $disk{$k}="-"; }


  $k="$id\_partcap2";
  $disk{$k}="$bin";
  if (!($bin =~/\d/)) { $disk{$k}="-"; }

}




##############
sub get_mydisk  {
##############
  my (@w,$w,$key,$k,$n,@p,$p,$r,$i,$v,@v,$d,@d,$id,$id2,$key2,$guid,$phys,$devid,$j,$ids,$out,$device,$u,$lun);


  # local lu
  $r=&exe("stmfadm list-lu");
  @t=split(/\n/,$r);
  $disk{'all_local_lu'}="";
  my @local_lu=();
  foreach my $t (@t) {
     if ($t=~/LU Name: /) {
        $t=~s/LU Name: //;
        $disk{'all_local_lu'}.="$t\t";
        push(@local_lu,$t);                             # to check against sn
     }
  }
  $disk{'all_local_lu'}=~s/\t+$//;


  ######################################################
  $out="\n\n2. Diskinfo via iostat -Ensr \n";
  ######################################################



  $r=&exe('iostat -Ensr','','get-disk.pl 289');         # c1t0d0   ,Soft Errors: 0 ,Hard Errors: 0 ,Transport Errors: 0
                                                        # Vendor: VMware   ,Product: Virtual disk     ,Revision: 1.0  ,Serial No:
                                                        # Size: 4.29GB <4294967296 bytes>
                                                        # ,Media Error: 0 ,Device Not Ready: 0 ,No Device: 0 ,Recoverable: 0
                                                        # Illegal Request: 512 ,Predictive Failure Analysis: 0



  $r=~s/\n/,/gs;
  @w=();
  @w=split(/\,/,$r);
#&print_array(@w);

  # defaults
  $disk{'all_unused'}="";                                                  # add later in sub get_mydsk_stat
  $disk{'all_na'}="";                                                      # disks not avail
  $disk{'all'}.="\t";                                                      # from format

#&print_hash(%disk);
  foreach $w (@w) {
        if (!($w=~/\w+/)) { next; }                                        # empty line
        $w=~s/\s+$//;
        if (($w=~/^[c]\d+.*d/) && (!($w=~/Serial/))) {                     # id  and not a serial
                $id=$w;

                $out.="\nid $w, ";

                #all
                if (!($disk{'all'}=~/$id\b/)) {                             # all disks, nvme are missing
                   $disk{'all'}.="$id\t";
                }

                #not avail
                if (!($disk{'all_dsk'}=~/$id/)) {
                   $disk{'all_na'}.="$id\t";

                   $key="$id\_stat";
                   $disk{$key}="removed";

                   # or parted error
                   $t="$id\_parted_error";
                   if ($disk{$t} ne "") {
                       $disk{$key}=$disk{$t};
                   }

                }

                #all_id
                if (length($id) < 9) {                                      # c2t0d0s0
                                if (!($disk{'all_id'}=~/$id\b/)) {
                                   $disk{'all_id'}.="$id\t";
                                }
                }  else {
                                #all_guid                                    #c8t5000000000000103d0
                                if (!($disk{'all_guid'}=~/$id\b/)) {
                                     $disk{'all_guid'}.="$id\t";
                                }
                }
                next;
        }

        if (!($w=~/\:/)) { next; }                        #empty line

        $w=~s/\: /:/;
        $w=~s/ +$//;
        @p=split(/:/,$w);                                # Size: 4GB <400000 byte>

        $key="$id\_error";
        if ($disk{$key} eq "") { $disk{$key}="Error: "; }
        if ($p[0] eq "Soft Errors") {
                $disk{$key}=" S:$p[1] ";          next;
        }
        if ($p[0] eq "Hard Errors") {
                $disk{$key}.=" H:$p[1] ";         next;
        }
        if ($p[0] eq "Transport Errors") {
                $disk{$key}.=" T:$p[1] ";
        }

        #remove error on removed disks optional
        #$t="$id\_stat";
        #if ($disk{$t} ne 'ok') { $disk{$key}='-';}

        $disk{$key}=~s/ $//;
        $disk{$key}=~s/ +/ /g;

        $key="$id\_vendor";
        if ($p[0] eq "Vendor") {
                $disk{$key}="$p[1]";
                $out.="Vendor: $p[1]  ";
                next;
        }

        #model
        $key="$id\_model";
        if ($p[0] eq "Model") {
                $disk{$key}="$p[1]";

                # use model as venfor when missing
                $key="$id\_product";
                $disk{$key}="$p[1]";

                $out.="Model: $p[1]  ";
        }

        $key="$id\_product";
        if ($p[0] eq "Product") {
                $p[1]=~s/ +/ /g;
                $disk{$key}="$p[1]";
                $out.="Product: $p[1]  ";

                #now check if Prroduct is COMSTAR and LU is local, add target alias later
                if ($disk{$key} eq "COMSTAR") {
                     $key="$id\_sn";
                     foreach my $lu (@local_lu) {
                        if ($id=~/$lu/i) {
                            $disk{$key}="loc_lu_";
                        }
                     }
                     # not a local_lu get info from target
                     if ($disk{$key} ne "loc_lu_") {
                         $disk{$key}="target:";
                     }
                     $t="$id\_stat";
                     if ($disk{$t} eq 'ok') {
                        $disk{'all_dsk_comstar'}.="$id\t";
                     }  else {
                        $disk{'all_dsk_comstar_removed'}.="$id\t";
                     }
                }


        }

        $key="$id\_revision";
        if ($p[0] eq "Revision") {
                $disk{$key}="$p[1]";
        }

        $key="$id\_sn";
        if ($p[0] eq "Serial No") {
                if ($disk{$key} eq "") {
                  $disk{$key}="$p[1]";
                  $disk{$key}=~tr/a-z/A-Z/;                                 # normalize wg lsi sas2
                  $disk{$key}=~s/[^A-Z0-9]//g;

                  $out.="SN: $p[1]   ";
                } else {
                  $out.="SN: $disk{$key}, ";
                }
        }

        $key="$id\_diskcap";                                                # parted adds partition cap
        if ($p[0] eq "Size") {

                $p[1]=~s/.*\<//;
                $p[1]=~s/ bytes>//;       # size in bytes

                &get_mydisk_bcap($id,$p[1]);
                $out.="Size: $p[1]   ";
        }

        $key="$id\_ill_request";
        if ($p[0] eq "Illegal Request") {
                $p[1]=~s/ <.*//;
                $disk{$key}="$p[1]";
        }

        $key="$id\_recoverable";
        if ($p[0] eq "Recoverable") {
                $p[1]=~s/ <.*//;
                $disk{$key}="$p[1]";
        }

        $key="$id\_pred_fail_anal";
        if ($p[0] eq "Predictive Failure Analysis") {
                $p[1]=~s/ <.*//;
                $disk{$key}="$p[1]";
        }

  }

  # remote lu with sn from target alias
  $r=&exe("iscsiadm list target -S");

  # output with sendtarget discovery
  # Target: iqn.2010-09.org.napp-it:neu
  #      Alias: node1
  #      TPGT: 1
  #      ISID: 4000002a0000
  #      Connections: 1
  #      LUN: 0
  #           Vendor:  SUN
  #           Product: COMSTAR
  #           OS Device Name: /dev/rdsk/c3t600144F08278020F000057A222020001d0s2
  #
  # Target: iqn.2010-09.org.napp-it:1469471015
  # ..


  # output with static discovery
  #Target: iqn.2010-09.org.napp-it:tank2:netraid-1-lu2
  #      Alias: tank2/netraid-1-lu2
  #      ISID: 4000002a0000
  #      Connections: 0

  #Target: iqn.2010-09.org.napp-it:tank:netraid-1-lu1
  #      Alias: tank/netraid-1-lu1
  #      ISID: 4000002a0000
  #      Connections: 0


  # but
  # c11t25d0_format_id        ->         '/iscsi/disk@0000iqn.2010-09.org.napp-it%3Atank2%3Anetraid-1-lu20001,0'

  # care only about sendtarget discovery

  @t=split(/Target: /,$r);
  foreach my $t (@t) {
     if ($t eq "") { next; }
     @w=();
     @w=split(/\n/,$t);
     $n="";
     $lun="";

     foreach my $w (@w) {
         if ($w=~/Alias: /i) {  $n=$w; $n=~s/.*Alias:\s//; next; }         # aliasname
         if ($w=~/OS Device Name:/i) {
             $key=$w;
             $key=~s/.*dsk\///;       # OS Device Name: /dev/rdsk/c0t600144F070CC440000004C1539849395d0s2
             $key=~s/s\d+$//;

             # add alias to sn
             $key.="_sn";
             if (!($disk{$key}=~/target\:\w/)) { # only first alias
                $disk{$key}.="$n";         # add target alias to sn
             }
         }
     }
  }

  #debug
  #&print_array(@t);
  #&print_hash(%disk);

  # remove CD/DVD (no partcap, state ne "ONLINE" no errors)
  @t=();
  @t=split(/\t/,$disk{'all'});
  foreach my $t (@t) {
         $r="";

         if ($t eq "") { next; }
         $key="$t\_partcap";
         if ($disk{$key}=~/\d/)         { $r.="0"; } else { $r.="1";  }

         $key="$t\_state";
         if ($disk{$key} =~/[a-z]/i)    { $r.="0"; } else { $r.="1";  }

         $key="$t\_error";
         if ($disk{$key} =~/:/)         { $r.="0"; } else { $r.="1";  }

         $key="$t\_format_id";          # nvme aus format (iostat nicht sichtbar )
         if ($disk{$key}=~/blkdev/)               { $r.="0"; } else { $r.="1";  }

         $key="$t\_part";               # aus parted
         if ($disk{$key})               { $r.="0"; } else { $r.="1";  }

         # all false, remove from all, all_id, all_na
         if (!($r=~/0/)) {
             $disk{'all'}=~s/$t\t*//;
             $disk{'all_id'}=~s/$t\t*//;
             $disk{'all_na'}=~s/$t\t*//;
         }

        # if ($t eq "c4t0d0") { &mess("$t->$r"); }
  }
  ########### /cd

  $disk{'all_dsk'}=~s/\t$//;
  $disk{'all'}=~s/^\t+|\t$//g;
  $disk{'all_id'}=~s/\t$//;
  $disk{'all_guid'}=~s/\t$//;

 # $disk{'all_unused'}=~s/\t+$//;
  $disk{'all_replaceable'}=~s/\t$//;

  $disk{'all_dsk_comstar'}=~s/\t+$//;
  $disk{'all_dsk_comstar_removed'}=~s/\t+$//;


#debug
#&print_hash(%disk);


  #list of used controller ex c1
       @w=();
       @w=split(/\t/,$disk{'all'});
       @w=sort(@w);
       foreach $t (@w) {
              $t=substr($t,0,2);
              if ($disk{'controller'}=~/$t\t/) { next; }
              $disk{'controller'}.="$t\t";
       }
       $disk{'controller'}=~s/\t$//;

  #list of used controller ex c1d
       @w=();
       @w=split(/\t/,$disk{'all'});
       @w=sort(@w);
       foreach $t (@w) {
              $t=substr($t,0,3);
              if ($disk{'controller2'}=~/$t\t/) { next; }
              $disk{'controller2'}.="$t\t";
       }
       $disk{'controller2'}=~s/\t$//;

  # add smartvalues if they are up to date (acc)
      if (-f "/tmp/nappit/smart_last.smart")  {
          $r=&read_file("/tmp/nappit/smart_last.smart");
          $t=time();

          $r=$t-$r;           # no older than 30min=1800s
          if ($r<1800) {
               if (-f "/tmp/nappit/smart_data.smart")  {
                   &read_ahfile("/tmp/nappit/smart_data.smart","disk","h+");
                   #&print_hash(%disk);
               }
          }
      }
}
# eof
1;
