#!/usr/bin/perl

## agent_autolock.pl
## used for user enc switching by movin a switcher file to a watched folder

## (c) Gnther Alka  2017-2020
## feb 13 2020

## debug: start 'perl /var/web-gui/data/napp-it/zfsos/_lib/scripts/agents/agent_autolock.pl d' manually from console

# 1. this service creates a ZFS filesystem /pool/UserEncrypt on the first pool with encrypted filesystems
# 2. the script creates watched folders for every encrypted filesystem with userencrypt=on in settings
#    If you set userencrypt to off you must optionally remove the watched folders manually to keep permissions otherwise intact

# 3. you must enable smb sharing manually ex via a sharename like userencrypt and set permissions to
#    the share and to the watched folders accordingly to the users you want to allow for switching.

#define global var and settings
use strict;
use vars qw(@tt @al $par $date_ymd %sys $date_all $ydat $date_us $date_and_time $isdst $sec $min $hour $mday $mon $year $wday $dow $date $t @t $f @f %data $D %p %cfg %in);

#load lib
require "/var/web-gui/data/napp-it/zfsos/_lib/illumos/zfslib.pl";
require  "/var/web-gui/data/napp-it/zfsos/_lib/scripts/agents/agent_autolock.lib";
require "/var/web-gui/data/napp-it/zfsos/_lib/illumos/keyserverlib.pl";

#os
$t=`uname -a`;
@t=split(/ /,$t);
if ($t[3]=~/^11/)      {                                                                         # Oracle Solaris
   $sys{'auto_os'}="Solaris 11";
   $sys{'auto_os_type'}="sol_s";
   $sys{'auto_os_ver'}=$t[3];
}


$par=$ARGV[0];
if (!(defined($par))) { $par="-"; }

use Fcntl qw(:flock);                                                    # filelocking
$|=1;

use strict;
use warnings;

use lib "/var/web-gui/data/napp-it/CGI";
use Expect;   # for unlocking

print "035 start autolock service\n";

#Debug=0/1
$D="0";
if ($par=~/d/i) { $D="1"; }

$p{'last_autolock'}="";


$t=`ps axw | grep agent_autolock.pl | grep -v grep`;     # all agents ex for par = " zfs"
chomp ($t);
@t=split(/\n/,$t);
$f="0";
foreach my $t (@t) {
  if ($t=~/sudo perl/) { next; }
  $f++;
}


# only one instance
if ($f > 1) {          # service and sudo
       if ($D) { print "053 autolock agent is already running\n$t\n"; }
       exit;
}


#initial read config
&my_read_config;

# initial check
&my_read_enc_fs;

# initial create missing zfs UserEncrypt or watched folders below when fs_userlock = yes
&my_check_folders;

#keysource -> key
&my_keys;

#debug, file/watched folders now ready
#&print_hash(%p);

# start service
&my_autolock_service;

exit;

# read exclusive
#  open (F,"<$log");
#        flock(F, LOCK_EX);
#        @data=<F>;
#  close (F);                                                                        # keine Pufferung, no buffering

# write ex
# open (PF, ">$log");
#    flock(PF, LOCK_EX);
#    print PF "$t";
#    print PF @data;
# close (PF);


#############
sub my_autolock_service {
##############
   #
   my ($p,$f,$r,$t,$ok,$w,$k,@r,@w,@e);

   while(1) {
     #
     #date and time
     &my_getdate;            # update time

     # process autolock (0,15,30,46 min)
     &my_auto_switching;

     #process watched folders (manual switching)
     &my_manual_switching;

     #debug
     #&print_hash(%p);

     #wait 3s
     sleep(3);

     #redo read config
     &my_read_config;

     # redo check
     &my_read_enc_fs;

     # redo create missing zfs UserEncrypt or watched folders below when fs_userlock = yes
     &my_check_folders;
   }
}


######################
sub my_auto_switching {     # via watched folder
######################
    my ($p,$f,$r,$t,$i,$ok,$w,$k,@r,@w,@e,$tt,$status,$z);

    #debug only
    #$hour="18"; $min="45";
    #$date_ymd="2018.02.14";
    #&print_hash(%p);

    if ($D) {  print "\n   ### 140 ### autolock=$hour\:$min  last($p{'last_autolock'})\n"; }

    # every 15 min only
    if (!("00,15,30,45"=~/$min/)) {  return; }

    if ($p{'last_autolock'} eq "$hour\:$min") {
         if ($D) {print "   ### 146 ### $date_ymd $hour\:$min already processed: skip\n\n"; }
         return;
    }
    $p{'last_autolock'}="$hour\:$min";

    @e=();
    @e=split(/\t/,$p{'enc_fs'});

    $i="0";
    foreach my $fs (@e) {

            #debug
            #$i++;  if ($i > 3) { exit; }
            #if ($fs eq "tank/eins")  {  print "159 exit\n"; exit; }

            $p{'exit'}="0";      # reset for every fs

            if ($D) {  print "\n   ### 162 ### process auto for $date_ymd $fs\n   ################################################\n"; }

            $k="$fs\_autolock";
            if (!(defined($p{$k}))) { $p{$k}=""; }
            if ($p{$k} ne "yes") { next; }

            #get tt
            $k="$fs\_timetable";

            # check tt
            if (!($p{$k}=~/\w/))  { next; }

            if ($D) {  print "\n   ### 173 ### check tt $p{$k} for date $date_ymd $min min\n"; }
            &my_a_parse_date($p{$k});    #tt for date_ymd  (jjjj.mm.tt)

            # check lock/unlock hour
            if ($D) {  print "   ### 177 ### check autolock $date_ymd $hour\:$min for fs $fs\:  unlock=$p{'unlock_hour'}, lock=$p{'lock_hour'}\n";  }

            # (un)lock fs?
            $t="1";
            if ($p{'unlock_hour'}=~/\d:\d/) { $t="0"; }
            if ($p{'lock_hour'}=~/\d:\d/)   { $t="0"; }
            if ($t) {
                 if ($D) {  print "  ### 184 ### no valid unlock/lock time\n"; }
                 next;
            }

            # pool
            $p=$fs;
            $p=~s/\/.*//;

            #zfs fs
            $z=$fs;
            $z=~s/.*\///;

            $f="/$p/UserEncrypt/$z";                  # ex /tank/UserEncrypt/data
            $r=`zfs get -H -o value keystatus $fs`;   # available, unavailable
            chomp($r);

            if ($D) {  print "   ### 200 ### keystatus $fs=$r => unlock at $p{'unlock_hour'}, lock at $p{'unlock_hour'}, current=$hour\:$min\n"; }

            if ($p{'unlock_hour'}=~/\d:\d/) {         # check unlock now
              if ($r eq "unavailable")   {
                 if ($p{'unlock_hour'}=~/$hour\:$min/) {    # allow multiple timer
                      if ($D) {  print "   ### 180 ### ### unlock $fs for date $date_ymd $min min\n\n"; }
                      &my_log("auto-unlock fs $fs");
                      &my_unlock($fs,$r);
                 }
               }
            }

            if ($p{'lock_hour'}=~/\d:\d/) {           # check lock now
              if ($r eq "available")   {
                 if ($p{'lock_hour'}=~/$hour\:$min/) {    # allow multiple hours
                      if ($D) {  print "   ### 206 ### ### lock $fs for date $date_ymd $min min\n\n"; }
                      &my_log("auto-lock fs $fs");
                      &my_lock($fs,$r);
                 }
               }
            }

            #print new status
            $r=`zfs get -H -o value keystatus $fs`;   # available, unavailable
            chomp($r);
            if ($r eq "available")   { $status="unlocked"; }
            if ($r eq "unavailable") { $status="locked"; }

            # set new state
            if (-d "$f") {
                if ($D) {  print "   ### 229 ### ### update $f\n"; }
               `rm -f $f/service-state.*`;         # service state file
               `touch $f/service-state.online_$status`;
            }
            #debug
            #exit; # sleep(5);
    }
}


######################
sub my_manual_switching {     # via watched folder
######################
    my ($p,$f,$r,$t,$z,$ok,$w,$k,@r,@w,@e,$status);

    @e=split(/\t/,$p{'enc_fs'});
    foreach my $e (@e) {

         # pool
         $p=$e;
         $p=~s/\/.*//;

         #zfs fs
         $z=$e;
         $z=~s/.*\///;

         $f="/$p/UserEncrypt/$z";             # ex /tank/UserEncrypt/data

#debug
#&print_hash(%p);

         if (-d "$f") {

             $k="$e\_userlock";
             if ($p{$k} =~/yes|smbkey/) {
                 #
                 #status
                 $status="unknown";
                 $r=`zfs get -H -o value keystatus $e`;
                 chomp($r);
                 if ($D) { print "156 keystatus $e=$r\n"; }
                 if ($r eq "available")   { $status="unlocked"; }
                 if ($r eq "unavailable") { $status="locked"; }

                 # set state to online
                 if (-d $f) {
                   `rm -f $f/service-state.*`;         # service state file
                   `touch $f/service-state.online_$status`;
                 }

                if ($D) { print "164 check watched lock folder for $e\n"; }
                # check lock

                $cfg{'user_smbkey'}="";
                 if (-f "$f/=lock/=switcher.move") {
                    &my_log("user-lock fs $e");
                    &my_lock($e,$r);
                 }
#print "305: $f\n";
                # check unlock
                 if (-f "$f/=unlock/=switcher.move") {
                    $in{'user_smbkey'}=`cat $f/=unlock/=switcher.move`;          # unlock can require smbkey
                    $in{'user_smbkey'}=~s/\s+//gs;
                    &my_log("user-unlock fs $e");
                    &my_unlock($e,$r);
                 }
              } else {
                 # set state to disabled
                 if (-d $f) {
                   `rm -f $f/service-state.*`;         # service state file
                   `touch $f/service-state.disabled`
                 }
              }

              #delete mover
              if (-d $f) {
                 `rm -f $f/=lock/*`;
                 `rm -f $f/=unlock/*`;
                 `touch $f/=switcher.move`;
              }
        }
    }
}


##############        check if key is available
sub my_keys {
##############
   my ($p,$f,$r,$s,$t,$ok,$w,$k,@r,@w,@e);
   @e=split(/\t/,$p{'enc_fs'});
   foreach my $e (@e) {
       $t="$e\_keysource";
       if (!(defined($p{$t}))) { $p{$t}=""; }

       $s=$p{$t};              # source like /pool/file or http(s)://url

       # get filekey
       if ($s=~/^\//) {
             if ($D) { print "314 keysource for $e = $s\n"; }

             if (-f "$s") {
                #
                open (F,"<$s");
                  flock(F, LOCK_EX);
                  @r=<F>;
                close (F);
                if ($r[0]=~/\w+/) {
                   chomp($r[0]);
                   $k="$e\_key";
                   $p{$k}=$r[0];               # set/update key
                   if ($D) { print "326 update key from $s\n"; }
                }
             }
       }
       # get webkey
       if ($s=~/^http/) {
                if ($D) { print "332 keysource for $e = $s\n"; }
                $r=`wget -O - $s 2>/dev/null`;
                if ($r[0]=~/\w+/) {
                   chomp($r[0]);
                   $k="$e\_key";
                   $p{$k}=$r[0];               # set/update key
                   if ($D) { print "338 update key from $s\n"; }
                }
       }
   }
}

################
sub my_check_folders {
################
   my ($p,$f,$r,$t,$ok,$w,$k,@r,@w,@e);
   @e=split(/\t/,$p{'enc_fs'});

   #debug
   #&print_array(@e);

   foreach my $e (@e) {
      # check only when userlock eq yes
      $k="$e\_userlock";

      if ($D) { print "357 check for ZFS UserEncrypt and watched folders for $e\n"; }

      if (!(defined($p{$k}))) { $p{$k}=""; }
      if (!($p{$k} =~/yes|smbkey/)) { next; }

      # check fs pool/UserEncrypt
      $p=$e;
      $p=~s/\/.*//;   # pool

      $k="$p\_UserEncrypt";

      if ($p{$k} eq "no") {
         if ($D) { print "369 create ZFS $p/UserEncrypt\n"; }
         `zfs create $p/UserEncrypt`;           # SMB share must be enabled manually
         `echo 'if user-locking is enabled for encrypted folders, move the switcher file to the lock or unlock subfolder' > "/$p/UserEncrypt/_use this watched folder for user crypt switching only.txt"`;
         `/usr/bin/chmod A=user:root:full_set:file_inherit/dir_inherit:allow /$p/UserEncrypt`;     # root only
         $p{$k}="yes";
      }

      # check watched folder below UserEncrypt
      $k="$e\_userlock";
      if ($p{$k} =~/yes|smbkey/) {
            #
            if ($D) { print "380 check enc filesystem folder for $e\n"; }
            $f=$e;
            $f=~s/.*\///;      # filesystem

            if (!(-d "/$p/UserEncrypt/$f")) {
               if ($D) { print "385 create folder /$p/UserEncrypt/$f\n"; }
               mkdir("/$p/UserEncrypt/$f");
               `/usr/bin/chmod A=user:root:full_set:file_inherit/dir_inherit:allow /$p/UserEncrypt/$f`;     # root only
            }

            # check watched switch folder for fs
            if (!(-d "/$p/UserEncrypt/$f/=unlock")) {
               if ($D) { print "392 create watched folder /$p/UserEncrypt/$f/=unlock\n"; }
                mkdir("/$p/UserEncrypt/$f/=unlock");
               `/usr/bin/chmod A=user:root:full_set:file_inherit/dir_inherit:allow /$p/UserEncrypt/$f/=unlock)`;     # root only
            }
            if (!(-d "/$p/UserEncrypt/$f/=lock")) {
               if ($D) { print "397 create watched folder /$p/UserEncrypt/$f/=lock\n"; }
                mkdir("/$p/UserEncrypt/$f/=lock");
               `/usr/bin/chmod A=user:root:full_set:file_inherit/dir_inherit:allow /$p/UserEncrypt/$f/=lock)`;     # root only
            }
            if (!(-f "/$p/UserEncrypt/$f/=switcher.move")) {
               if ($D) { print "402 create switcher /$p/UserEncrypt/$f/=switcher.move\n"; }
               `touch /$p/UserEncrypt/$f/=switcher.move`;
            }
            if (!(-f "/$p/UserEncrypt/$f/service-state.initialize")) {
               if ($D) { print "406 create service-state /$p/UserEncrypt/$f/service-state.initialize\n"; }
               `touch /$p/UserEncrypt/$f/service-state.initialize`;
            }
      }
   }
}







###############
sub print_hash { #parameter %
###############
 my %h = @_;
 my ($k,$v,@tmp,$w,$a);

      while (($k,$v) = each %h) {
           $w=$k." "."." x 40;
           $w=substr($w,0,41);
           push (@tmp,"$w -> \'$v\' \n");
      }
      @tmp=sort(@tmp);
      print "\n\&print_hash\n\n";
      foreach my $t (@tmp) {
        print $t;
      }
      exit;
}


###############
sub print_array { #parameter @
###############
  my @t=@_;
  my $t;
  # @t=sort(@t); # no

      print "\n\&print_array\n\n";

      for (my $i=0;$i<@t; $i++) {
           print "[$i] -> '$t[$i]\'\n";
      }
      exit;
}
###############################################
#end of my script, do not delete the line below
1;
