#!/usr/bin/perl
#
# zraid agent
# (c) Guenther Alka may 28, 2019

# service running on one cluster control only (together with failover head)
# when failover mode is set to auto
# service can be run from console for debugging or
# as a service initiated by scripts/agent/bootinit.pl

use strict;
use vars qw(%current $mess @h1 @h2 @jobs $jobid $jobtype $par $state $auto_os $os $path $cgipath $logpath $log $t $f $n @t @m $r %zfs %sys %p %cfg %in $last);

use Fcntl qw(:flock);                                                               # opt filelocking possible
$|=1;

my ($interval,$hn,$ok,$r,$save,$h1,$h2);
my $autolog="0";    # 1 gives extra logging in cluster auto log

#wait between checks
$interval="5";                                                                       # 5s (double check: failover 10s)
$save="0";    # save user on check

# read lib
    require ("/var/web-gui/data/wwwroot/cgi-bin/admin-lib.pl");
    require ("/var/web-gui/data/napp-it/zfsos/_lib/illumos/cluster-lib.pl");
    require ("/var/web-gui/data/napp-it/zfsos/_lib/illumos/grouplib.pl");

$t=`ps axw | grep agent_zraid_ssf.pl | grep -v grep`;
     @t=();
     @t=split(/\n/,$t);
     if (@t > 1) {
       print "agent already running\n";
       my_zlog("agent already running\n");
       exit;
     }

    &my_read;  # read cluster settings

# service, run forever
$jobid=time();

$ok=0;
my_zlog("start service at time: $t\n");

while (1) {

   &my_read;  # allow re-configure of cluster settings

   #debug
   #$p{'select_c1_failover_mode'}="auto";
   #&print_hash(%p);

   print "\n";

   # manual failover in progress
   if (-f "/tmp/block_autofailover") {
          print "manual failover in progress, recheck in 40 s\n\n";
          &my_zlog("manual failover in progress, recheck in 40 s");
          sleep (40);      # sleep long enough to be sure failover is completed
          next;
   }

   if (-f "/tmp/block_manfailover") {             # remove blocking of manual failover
      `sudo rm /tmp/block_manfailover`;
   }


   # man ip set?
   if (!( ($p{'value_ip_mh1'} =~/\d/) || ($p{'value_ip_mh2'} =~/\d/)) ) {
          print "man ip h1 and h2 not set\n\n";
          &my_zlog("man ip h1 and h2 not set");
          sleep ($interval);
          next;
   }

   # check if ha ip is pingable
   if ($p{'value_ip_ha'}=~/\d+\.\d+\.\d+\.\d+/) {
       $r=`/usr/sbin/ping $p{'value_ip_ha'} 1`;
       if ($r=~/alive/s) {
         print "ha ip $p{'value_ip_ha'} is up, extra wait 5 s\n";
         sleep (5);                                  # wait  extra
       }
   }

   # check if heads are pingable
   $r=""; $h1=""; $h2="";
   $t=`/usr/sbin/ping $p{'value_ip_mh1'} 1`;
   if ($t=~/alive/s) { $h1="alive"; $r="h1=alive, "; } else { $r="h1=offline, "; }

   $t=`/usr/sbin/ping $p{'value_ip_mh2'} 1`;
   if ($t=~/alive/s) { $h2="alive"; $r.="h2=alive"; } else { $r.="h2=offline"; }

   #&my_zlog("status: $r");
   $mess="status: $r";

  ################
  # state of head1  @ requestmode=, hostname=, zlist=, importable=, failovermode=, servermode=, status=
  ################
  @h1=();
  if ($h1 eq "alive") {
      #
      @h1=&my_get_state("get_cl_control_state","$p{'value_ip_mh1'}","h1",$autolog);    # clusterlib

      # debug
      if ($autolog) {
        $t=$h1[2]; if ($t=~/zraid\-1/) { $t="zraid-1 mounted"; } else { $t="zraid-1 not mounted"; } if ($h2[2] eq "") { $t=""; }
        &my_zlog("array h1 $p{'value_ip_mh1'}\: $h1[0],$h1[1],$t,$h1[3],$h1[4],$h1[5],$h1[6]");
      }

      # h1 has zraid-mounted: next
      if ($h1[1]=~/hostname=/) {                # online
         if ($h1[2]=~/zraid\-\d+\s/) {
             print "h1 has zraid mounted, save=$save, recheck in $interval s\n";
             &my_zlog("$mess, h1 has zraid mounted, recheck in $interval s");

             #opt save user and comstar every 2nd run
             if ($save>0) {
               if ($p{'select_c1_agent_syncuser'} eq "yes") {
                  $p{'netdata'}="usersave=$p{'select_userrange'};_";
                  $r=&my_get_state("sync_cluster_user","$p{'value_ip_mh1'}","h1");
                  print "save cluster user on h1, $r\n";
               }

               if ($p{'select_c1_agent_synccomstar'} eq "yes") {
                  $p{'netdata'}="comstarsave=yes;_";
                  $r=&my_get_state("sync_cluster_comstar","$p{'value_ip_mh1'}","h1");
                  print "save comstar on h1, $r\n";
               }
               $save="0";
             } else {
               $save++;
             }

             sleep ($interval);                                  # wait for faiover missing
             next;
         }
      }
  }

  ################
  # state of head2  @ requestmode=, hostname=, zlist=, importable=, failovermode=, servermode=, status=
  ################
  @h2=();
  if ($h2 eq "alive") {
      #
      @h2=&my_get_state("get_c1_control_state","$p{'value_ip_mh2'}","h2",$autolog);      # clusterlib , last par=1: autolog (debug)

      # debug
      if ($autolog) {
        $t=$h2[2];
        if ($t=~/zraid\-1/) { $t="zraid-1 mounted"; } else { $t="zraid-1 not mounted"; } if ($h2[2] eq "") { $t=""; }
        &my_zlog("array h2 $p{'value_ip_mh2'}\: $h2[0],$h2[1],$t,$h2[3],$h2[4],$h2[5],$h2[6]");
      }


      # h2 has zraid-mounted: next
      if ($h2[1]=~/hostname=/) {                # online
         if ($h2[2]=~/zraid\-\d+\s/) {
             print "h2 has a zraid mounted, save=$save, recheck in $interval s\n";
             &my_zlog("$mess, h2 has a zraid mounted, recheck in $interval s");

             #opt save user and comstar every 2nd run
             if ($save>0) {
               if ($p{'select_c1_agent_syncuser'} eq "yes") {
                  $p{'netdata'}="usersave=$p{'select_userrange'};_";
                  $r=&my_get_state("sync_cluster_user","$p{'value_ip_mh2'}","h2");
                  print "save cluster user on h2, $r\n";
               }

               if ($p{'select_c1_agent_synccomstar'} eq "yes") {
                  $p{'netdata'}="comstarsave=yes;_";
                  $r=&my_get_state("sync_cluster_comstar","$p{'value_ip_mh2'}","h2");
                  print "save comstar on h2, $r\n";
               }
               $save="0";
             } else {
               $save++;
             }

             sleep ($interval);                                  # wait for faiover missing
             next;
         }
      }
  }


  # check failovemode (manual allows user sync when service is enabled)
  if ($p{'select_c1_failover_mode'}  ne "auto") {             # this control only in auto mode
          print "mode: $p{'select_c1_failover_mode'}, recheck in $interval s\n\n";
          &my_zlog("$mess, mode: $p{'select_c1_failover_mode'}, recheck in $interval s");
          sleep ($interval);
          next;
  }

  # it seems we have no master, try to switch on second failure
  if ($ok eq "0") {
     $ok++;
     print "\nno master, recheck once in $interval s\n";
     &my_zlog("$mess, no master, recheck, once in $interval s");
     sleep ($interval);                                     # wait for faiover missing
     next;
  } else {
     $ok="0";
  }

  # no active head
  if (($h1 eq "") && ($h2 eq ""))    {
     sleep ($interval);
     next;
  }

  # block manual failover
  `touch /tmp/block_manfailover`;


   # manual failover in progress, recheck now
   if (-f "/tmp/block_autofailover") {
          print "manual failover in progress2, recheck in 40 s\n\n";
          &my_zlog("manual failover in progress2, recheck in 40 s");
          sleep (40);      # sleep long enough to be sure failover is completed
          next;
   }

  print "initiate switch to master (node 1/2 mode: $h1[5], $h2[5])\n";
  &my_zlog("initiate switch to master (node 1/2 mode: $h1[5], $h2[5])");

  #################################
  # try to make h1 a master, kill 2
  #################################

    if (($h1[5]=~/servermode=cluster\-failover/) && ($h1[3]=~/zraid\-\d/)) {        # allow failover and zraid importable
      # kill 2
      $p{'netaction'}="clusternode_down";                   # node down
      $p{'netdata'}="";
      $r= &grouplib_ask_remote_zraid($p{'value_ip_mh2'},'','switch 131','1');

      print "initiate switch h1 to master: kill h2\n$r\n\n";
      &my_zlog("initiate switch h1 to master: kill h2: $r");

      # opt Stonith 2
      if ($p{'select_stonith'}=~/yes/)  {
         $r=`$p{'value_st2'}`;
         print "stonith h2: $r \n\n";
         &my_zlog("stonith h2: $r");
      }

      # up
      $p{'netaction'}="clusternode_up";       # ex set head 1 to master
      $p{'netdata'}="";

      # restore users ?
      if ($p{'select_c1_agent_restoreuser'}=~/yes/)     {
        $p{'netdata'}.="userrestore=$p{'select_userrange'};_";
      }

      # switch comstar?
      if ($p{'select_c1_agent_restorecomstar'}=~/yes/)  {
        $p{'netdata'}.="comstarrestore=yes;_";
      }

      # restore jobs?
      if ($p{'select_jobs'} eq "yes") {
        $p{'netdata'}.="jobs=yes;_";          # values between ;_
      }

      #add ha link
      $p{'netdata'}.="ha-link=$p{'value_link_ha1'};_";

      #add ha ip
      $t="value_ip_ha";
      $p{'netdata'}.="ha-ip=$p{'value_ip_ha'};_";

      #add ha netmask
      $p{'netdata'}.="ha-netmask=$p{'value_netmask_ha'};_";

      # add post?
      print "\nselect_post=$p{'select_post'}\n";
      if (($p{'text_post'}=~/\w/) && ($p{'select_post'} =~/yes/)) {
        $t=$p{'text_post'}; $t=~s/text_post : //; # \n is <n>
        $p{'netdata'}.="post=$t;_";

        $t=~s/<n>/\n/gs;
        print "post=$t\n\n";
      }

      # up h1
      $p{'netdata'}=~s/\n/<n>/gs;

      # security check: h2 down
      $t=`/usr/sbin/ping $p{'value_ip_mh2'} 1 2&>1`;      chomp ($t);
      if ($t=~/alive/) {
         print "error!! head 2 alive while trying to failover to 1\n\n";
         &my_zlog("error!! head 2 alive while trying to failover to 1");
      } else {
         $r=&grouplib_ask_remote_zraid($p{'value_ip_mh1'},'','switch 163');            # h1 up
         print "initiate switch h1 to master: up h1\n$r\n\n";
         &my_zlog("initiate switch h1 to master: up h1: $r");
      }
      $ok="0";

      if (-f "/tmp/nappit/cluster_last.clu") {  # delete acc files
        `sudo rm /tmp/nappit/*.clu`;
      }

      sleep(10);
      next;
    }

  #################################
  # try to make h2 a master, kill 1
  #################################

    if (($h2[5]=~/servermode=cluster\-failover/) && ($h2[3]=~/zraid\-\d/)) {        # allow failover and zraid importable
      # kill 2
      $p{'netaction'}="clusternode_down";                   # node down
      $p{'netdata'}="";
      $r= &grouplib_ask_remote_zraid($p{'value_ip_mh1'},'','switch 131','1');

      print "initiate switch h2 to master: kill h1\n$r\n\n";
      &my_zlog("initiate switch h2 to master: kill h1: $r");

      # opt Stonith1
      if ($p{'select_stonith'}=~/yes/)  {
         $r=`$p{'value_st1'}`;
         print "stonith h1: $r \n\n";
         &my_zlog("stonith h1: $r");
      }

      # up
      $p{'netaction'}="clusternode_up";       # ex set head 1 to master
      $p{'netdata'}="";

      # switch users ?
      if ($p{'select_userrestore'}=~/yes/)  {
        $p{'netdata'}.="userrestore=$p{'select_userrange'};_";
      }

      # switch comstar ?
      if ($p{'select_comstarrestore'}=~/yes/)  {
        $p{'netdata'}.="comstarrestore=yes;_";
      }

      # restore jobs?
      if ($p{'select_jobs'} eq "yes") {
        $p{'netdata'}.="jobs=yes;_";          # values between ;_
      }

      #add ha link
      $p{'netdata'}.="ha-link=$p{'value_link_ha2'};_";

      #add ha ip
      $t="value_ip_ha";
      $p{'netdata'}.="ha-ip=$p{'value_ip_ha'};_";

      #add ha netmask
      $p{'netdata'}.="ha-netmask=$p{'value_netmask_ha'};_";

      # add post?

      print "\nselect_post=$p{'select_post'}\n";
      if (($p{'text_post'}=~/\w/) && ($p{'select_post'} =~/yes/)) {
        $t=$p{'text_post'}; $t=~s/text_post : //; # \n is <n>
        $p{'netdata'}.="post=$t;_";

        $t=~s/<n>/\n/gs;
        print "post=$t\n\n";
      }

      # up h1
      $p{'netdata'}=~s/\n/<n>/gs;

      # security check: h1 down
      $t=`/usr/sbin/ping $p{'value_ip_mh1'} 1 2&>1`;      chomp ($t);
      if ($t=~/alive/) {
          print "error!! head 1 alive while trying to failover to 2\n\n";
          &my_zlog("error!! head 1 alive while trying to failover to 2");
      } else {
          print "initiate switch h2 to master: up h2\n$r\n\n";
          &my_zlog("initiate switch h2 to master: up h2: $r");
          $r=&grouplib_ask_remote_zraid($p{'value_ip_mh2'},'','switch 163');           # h2 up
      }

      if (-f "/tmp/nappit/cluster_last.clu") {  # delete acc files
        `sudo rm /tmp/nappit/*.clu`;
      }

      $ok="0";
      sleep(10);
      next;

    }

   $ok=0;                                                  # reset err counter

   # wait
   sleep ($interval);


}

#############
sub my_read {
############
    $cfg{'ver'}="c";                 # bootup only
    &my_read_settings ;              # read data  (lib)

    #reload targets
    print "\nnext; reload iscsi luns";
    `/usr/sbin/devfsadm -i iscsi`;
}




#
1;
