#!/usr/bin/perl
#(c) napp-it 2023

# script to analyze and/or filter files based on criteria like blocksize, size, lastedit or type
# call it via perl ../analyze.pl filesystem


use utf8;              # filenames
use strict;
use vars qw($inode %bs $blocksize $lastedit $filesize $ext @l @p $bs $fs $t);

$fs  = $ARGV[0];          # filesystem
$ext = $ARGV[1];          # file extensions only ex pdf,avi        optional

if ($fs eq "") { print "filesystem missing\n"; exit; }

# calculate in Bytes
$t="4K"; $t=&convert($t,"i",'b');   print "4K=".$t."\n";   $bs{'4K'}=$t;
$t="8K"; $t=&convert($t,"i",'b');   print "8K=".$t."\n";   $bs{'8K'}=$t;
$t="16K"; $t=&convert($t,"i",'b');  print "16K=".$t."\n";  $bs{'16K'}=$t;
$t="32K"; $t=&convert($t,"i",'b');  print "32K=".$t."\n";  $bs{'32K'}=$t;
$t="64K"; $t=&convert($t,"i",'b');  print "64K=".$t."\n";  $bs{'64K'}=$t;
$t="128K"; $t=&convert($t,"i",'b'); print "128K=".$t."\n"; $bs{'128K'}=$t;


# &print_hash(%bs);
# process
&zdb;


########## zdb stat
sub zdb {
##########
print "process zdb $fs | grep plain\ninode,blocksize,filesize\n";
# zdb filesystem:


   my ($line,@p);
   open (T,"-|","zdb $fs | grep plain");                   # request zdb filesystem for plain files
         while (my $line=<T>)
         {
            #         inode             blocksize                   filesize
            #         60    1   128K    75K           44K    512    75K      100.00  ZFS plain file
            #chomp($line);

            $line=~s/^ +//;         # remove leading spaces
            chomp($line);
            @p=split(/ +/,$line);   # parameter

            $inode=$p[0];
            $blocksize=&convert($p[3],"i","b");               # in Bytes
            $filesize=&convert($p[6],"i","b");                # in Bytes
            if ($filesize<4096) { next; }                     # min 4K

            print "$inode,$blocksize,$filesize\n";

         }
   close (T);
   exit;
}

########## KB-KiB-byte conerter ex &convert($data,"i","b");
sub convert {
##########
   #
   my $data=$_[0];     # data

   my $in=$_[1];       # in data [bid]:  (b)ytes = default, b(i)nary base like KiB, (d)eziman base like KB
   my $out=$_[2];      # out data [bsi(n)]: (b)ytes = default, (s)short like KB, b(i)nary base like KiB, n=number of dezimals

   my $dez=$out;  $dez=~s/[^0-9]//g;
   if ($dez eq "")     { $dez="2"; }  # default ex 10.15 MB
   if ($out eq "")     { $out="b"; }  # default exact bytes

   my $base="1000";
   if ($in=~/i/i)      {  $base="1024"; }  # or
   if ($data=~/iB/i)   {  $base="1024"; }

   $data=~s/,/./g;         # german 1,1 = 1.1
   my $bytes=$data; $bytes=~s/[^0-9\.\,]//g;
#&mess($bytes);

   if (($bytes eq "") || ($bytes eq "0"))     {  return("0"); }

   # 1. calculate bytes
   if ($data=~/k/i)     {  $bytes=$bytes * $base; }                                  # KB
   if ($data=~/m/i)     {  $bytes=$bytes * $base * $base; }                          # MB
   if ($data=~/g/i)     {  $bytes=$bytes * $base * $base * $base; }                  # GB
   if ($data=~/t/i)     {  $bytes=$bytes * $base * $base * $base * $base; }          # TB
   if ($data=~/p/i)     {  $bytes=$bytes * $base * $base * $base * $base * $base; }  # PB
   if ($data=~/e/i)     {  $bytes=$bytes * $base * $base * $base * $base * $base * $base; }  # EB

#&mess($bytes);
   if ($out=~/b/i)     {  return($bytes); }           # return exact number of Bytes

   # calculate output
   $base="1000";
   if ($out=~/i/i)          {  $base="1024";   }

   my $unit="";
   if ($bytes < 1000 )      {  return($bytes); }      # return exact number of Bytes

   $unit="K";
   if ($bytes >= 1000 )   {  $bytes=$bytes/$base; $unit="K"; } # < 1 MB (KB)
   if ($bytes >= 1000 )   {  $bytes=$bytes/$base; $unit="M"; } # < 1 GB (MB)
   if ($bytes >= 1000 )   {  $bytes=$bytes/$base; $unit="G"; } # < 1 TB (GB)
   if ($bytes >= 1000 )   {  $bytes=$bytes/$base; $unit="T"; } # < 1 PB (TB)
   if ($bytes >= 1000 )   {  $bytes=$bytes/$base; $unit="P"; } # < 1 EB (PB)
   if ($bytes >= 1000 )   {  $bytes=$bytes/$base; $unit="E"; } # < 1 ZB (EB)

   # calc dezimal
   $dez.="f";
   $bytes = sprintf ("%.$dez", $bytes);    # round  ex $bytes = sprintf ("%.1f", $bytes);
   $bytes=~s/\.0+$//;

   $bytes.=" $unit";
   if ($out=~/i/i) {  $bytes.="iB"; } else {  $bytes.="B";  }
   return($bytes);

}

###############
sub print_hash { #parameter %
###############
 my %h = @_;
 my ($k,$v,@tmp,$w,$a);


 while (($k,$v) = each %h) {
           $w=$k." "."." x 40;
           $w=substr($w,0,41);
           push (@tmp,"$w -> \'$v\' \n");
 }
 @tmp=sort(@tmp);
 print "\&print_hash\n\n";
 foreach my $t (@tmp) {
        print $t;
 }
 print "\n";
 exit;
}

###############
sub print_array { #parameter @
###############
  my @t=@_;
  my $t;
  # @t=sort(@t); # no


  print "\&print_array\n\n";

  for (my $i=0;$i<@t; $i++) {
           print "[$i] -> '$t[$i]\'\n";
  }
  print "\n";

  exit;

}

#  eof
1;
