#!/usr/bin/perl
# (c) napp-it 2012-2023
# create|remove esxi snaps via soap/ssh
# apr 24, 2023

# debug: run from cli    perl /var/web-gui/data/napp-it/zfsos/_lib/scripts/esxi/job-esxisnap.pl create_id
#                     or perl /var/web-gui/data/napp-it/zfsos/_lib/scripts/esxi/job-esxisnap.pl remove_id
# jobid can be a replicate or autosnap job with a setting esxi_ip=n.n.n.n,i
# n.n.n.n is the ip of an ESXi server
# ,i is wait (optional) in sec between ESXi snap create or delete commands (default is 5)
# pre: create ESXi snaps for all VMs on NFS
# post: destroy last ESXi snap for all VMs on NFS
# Setup in menu System > ESXi > NFS datastore (ip root pw NFS_datastores_commalist) for SOAP or ip root ssh (for SSH)


use strict;
use Fcntl qw(:flock);                                          # filelocking
$|=1;                                                          # keine Pufferung, no buffering

use vars qw($par $id $action $f $t @t $p @p %sys %in %cfg %current $error $info $message $text $subject $opt $ref $ip $fs);

 $par=$ARGV[0];                                                # action_$job time, for debugging purpose, you can call manually via cli with #par: run_id action
                                                               # ex:  perl job-snap.pl run_1357655102 pre
 # common lib
 use lib "/var/web-gui/data/napp-it/CGI";                      # add to perl search path for libs ex. Expect.pm and others

 require "/var/web-gui/data/wwwroot/cgi-bin/admin-lib.pl";
 require "/var/web-gui/data/napp-it/zfsos/_lib/scripts/esxi/soap-lib.pl";
 require "/var/web-gui/data/napp-it/zfsos/_lib/scripts/esxi/ssh-lib.pl";      # use when pw=ssh

 $f=`uname -a`;
 if ($f=~/Linux/){
       $f="linux";

 } else {
       $f="illumos";
       $sys{'sbinpath'}='/usr/sbin/';
 }


 if (!(defined($par))) {
     &log("job-esxisnap:  missing option: run_jobid");
     print "job-esxisnap/log: missing par: create|remove_id\n";
     exit;
 }

 ############

 #par: action_job timestamp
 $par=~s/ .*//;

 # parameter: action_$job time
 if (!($par =~/\w+\_\d+$/)) {
     print "job-esxisnap: wrong par $par\n";
     exit;
 }

 @t=();
 @t=split(/\_/,$par);    # action job
 $action=$t[0];
 $action=~s/[^a-z]//g;

 $id=$t[1];
 &my_gettime;

 # get par -> %current
 $f="/var/web-gui/_log/jobs/$id\.par";
 $ip=`cat $f | grep esxi_ip`;   chomp($ip);      # esxi_ip=n.n.n.n,10mpq (m=mem, q=quiesce, p=soap (ssh=default)
 print "job: $ip\n";

 if ($ip=~/,/) { $opt=$ip; $opt=~s/.*,//; }    # 1.1.1.1,10mq
 $ip=~s/,.*//;
 $ip=~s/.*=//;
 $in{'autoselect_host'}=$ip;
 $in{'select_host'}=$ip;

 $in{'opt'}=$opt;                               # p=soap, mem/quiesce via ssh

 # get fs
     # repli: source=nvme/nfs
     #  snap: pool=nvme/nfs
 $fs=`cat $f | grep pool=`;   chomp($fs);             # snapjob  ex nvme/nfs
 if ($fs eq "") {
      $fs=`cat $f | grep source=`;   chomp($fs);      # replijob
 }
 $fs=~s/.*=//;
 $in{'select_fs'}="/$fs";                             # /pool/nvme

 print "start; job-esxisnap.pl $ARGV[0] ip=$ip fs=$in{'select_fs'} opt=$opt\n";
 &my_log("start; job-esxisnap.pl $ARGV[0] ip=$ip fs=$in{'select_fs'} opt=$opt");
 #############################################


 # action
 if ($action eq "")                   {  &my_log("error; job-esxisnap.pl missing action"); print "77: missing  action - must be not empty\n";      exit; }
 if (!("create,remove"=~/$action/))   {  &my_log("error; job-esxisnap.pl wrong action"); print "78: wrong action - must be create or remove: $action \n";  exit; }

 if ($in{'opt'} =~/p/) {    # use soap
     #
     print "method: soap (opt with p)\n";
     &my_read_serverlist;                 # read user, pw for host ->   $in{'autoselect_host'} $in{'value_user'} $in{'value_pass'} $in{'moids'}

     if (($in{'autoselect_host'} eq "") || ($in{'value_user'} eq "") || ($in{'value_pass'} eq "") || ($in{'moids'} eq "")) {
        print "error; job-esxisnap.pl missing ip or user or soap pw or moid/datastores; setup in menu System > ESXi > NFS datastore\n";
        &my_log("error; job-esxisnap.pl missing ip or user or soap pw or moid/datastores; setup in menu System > ESXi > NFS datastore");
        exit;
     }

 } else {   # use ssh

     $in{'autoselect_host'}=$ip;
     $in{'value_user'}="root";
     #
     print "method: ssh (opt without p)\n";
     if (($in{'autoselect_host'} eq "") || ($in{'value_user'} ne "root")) {
        print "error; job-esxisnap.pl missing ip root; setup in menu System > ESXi > Serverlist\n";
        &my_log("error; job-esxisnap.pl missing ip root; setup in menu System > ESXi > Serverlist");
        exit;
     }
 }

 if ($action eq "create") {
    &my_snap_create;                # on all VMs on this NFS $in{'fs'}
 } else {
    &my_snap_remove;                # on all VMs on this NFS $in{'fs'} with desc=autosnap
 }
 print "end\n";
 exit;




#############           pre job
sub my_snap_create  {
##############
     my ($t1,$t2,$r,$f,$t);
     $t1=time();
     if ($in{'opt'} =~/p/) {     # soap
       print "mass_create_snaps_moids\n";
       $r=&my_mass_create_snaps_moids;
     } else {
       $r=&ssh_create;
     }
     $t2=time();
     $t=$t2-$t1;

     $f=&my_result($r);
     &my_log("create; ESXi snaps time $t s;$f");
     print "time $t s\n$r\n$f\n";

}

#############        post job
sub my_snap_remove  {
##############
     #
     my ($t1,$t2,$r,$f);
     $t1=time();
     if ($in{'opt'} =~/p/) {     # soap
        print "soap_remove_current_snaps\n";                  # last (current) only
        $r=&my_mass_remove_current_snaps;
        #print "mass_remove_snaps_moid\n";                    # opt all
        #$r=&my_mass_remove_snaps_moids;
     } else {
        print "ssh remove all autosnaps\n";                   #
        $r=&ssh_autojob_remove;                               # desc=autosnap only
        #$r=&ssh_last_remove;                                 # opt last only
        #$r=&ssh_remove;                                      # opt all
     }


     $t2=time();
     $t=$t2-$t1;


     $f=&my_result($r);
     &my_log("delete; autojob ESXi snaps time $t s;$f");
     print "time $t s\n$r\n";
     exit;

}

#############
sub my_result {
############
   if (!($in{'opt'} =~/p/)) { return(); }     # ssh
   my $i=0;
   my $s=0;
   my $r=$_[0];
   print "result:\n$r\n";

   my @l=split(/\n/,$r);
   foreach my $l (@l) {
      if (!($l=~/moid/)) { next; }
      $i++;
      if ($l=~/Success/i) {
        $s++;
      }
   }
   return ("ESXi soap snaps processed: $i ; Successfull: $s; opt: $opt");
}

#############
sub list2table {                  # to avoid errors
  my $r=$_[0];
  return($r);
}

###########  ## global log
sub my_log {
###########

  # print "job-repli 548 - call log id=$id action=$action\n";

  # line 1: lastrun = ok or err
  # line 2: time of last run
  # line 3+: info

  &my_gettime;                                             # $current{'timestamp'}, $current{'logtime'}
  my (@v,$v,$n,$l,$f1,$f2,@data);
  $info=$_[0];

  $l="ok";
  if ($info=~/error/) { $l="error";}


 #stage 1: log jobtype.log
 ###########################
  my $log="/var/web-gui/_log/jobs/$id\.log";
  #
  open (F,"<$log");
          flock(F, LOCK_EX);
          @data=<F>;
  close (F);

  #unshift
  unshift(@data,"$l;$info; time: $current{'timestamp'}; \n");

  if (@data > 50) {
            $t=@data-50;
            splice(@data,50,$t);
  }

  $n=join("",@data);
  open (PF, ">$log");
      flock(PF, LOCK_EX);
      print PF "$n";
  close (PF);

}



sub my_gettime {
###########
 my ($sec,$min,$hour,$mday,$mon,$year,$wday,$ydat,$isdst) = localtime(time);
 my @m=('','jan','feb','mar','apr','may','jun','jul','aug','sep','oct','nov','dec');
 $mon++;
 if ($sec < 10) { $sec = "0$sec"; }
 if ($min < 10) { $min = "0$min"; }
 if ($hour < 10) { $hour = "0$hour"; }
 if ($mday < 10) { $mday = "0$mday"; }
 if ($mon < 10) { $mon = "0$mon"; }
 $year=1900+$year;
 $current{'logtime'}="$mday\.$m[$mon]\_$hour\_$min";
 $current{'timestamp'}="$year\.$mon\.$mday\.$hour\.$min\.$sec";

}

##
1;
