#!/usr/bin/perl
# (c) napp-it
# 20.mai.2019
# 10.oct.2021, backup napp-it.cfg, bonjour.sh

# save smb groups
# request backup job
# skip backup when pool not mounted
# debug with jobid:  perl /var/web-gui/data/napp-it/zfsos/_lib/scripts/job-backup.pl run_1557502420


use strict;
use Fcntl qw(:flock);                                          # filelocking
$|=1;                                                          # keine Pufferung, no buffering

use vars qw($ref $par $id $action $f $t @t $p @p %current $error $info $r $ZFS $os $do @user %sys %in %cfg);

 # common lib
 require "/var/web-gui/data/napp-it/zfsos/_lib/scripts/task-lib.pl";        #
 require "/var/web-gui/data/wwwroot/cgi-bin/admin-lib.pl";
 require "/var/web-gui/data/napp-it/zfsos/_lib/scripts/backup-lib.pl";

$par=$ARGV[0];
 if (!(defined($par))) {
     &log("job-backup:  missing parameter");
     exit;
 }


 # common lib
 $f=`uname -a`;
 if ($f=~/Linux/){
       $os="linux";
       $ZFS="/sbin/zfs";
 } else {
       $os="illumos";
       $ZFS="/usr/sbin/zfs";
 }


 ############

 #par: action_job timestamp
 $par=~s/ .*//;

 # parameter: action_$job time
 if (!($par =~/\w+\_\d+$/)) {
     print "job-backup 20: wrong par $par";
     exit;
 }
 @t=();
 @t=split(/\_/,$par);    # action job
 $action=$t[0];
 $action=~s/[^a-z]//g;

 $id=$t[1];
 #############################################

 # get par -> %current
 &task_read_jobpar($id);


# run?
if ($action eq "")                     {  print "job-backup 42: wrong action - must be not empty\n";      exit; }
if (!("run,cancel,delete"=~/$action/)) {  print "job-backup 43: wrong action - must be run: $action \n";  exit; }

#cancel or delete - optional
if ($action eq "delete") { &my_delete; $info="delete job"; exit; }

#debug
# print "job-scrub 38: run id=$id par $par\n";

 $error=&my_run;
 &my_log;
 exit;


  ## delete job
 sub my_delete {
 ##############
 my ($r);
      if ($id ne "") {
        $r=`sudo rm -f /var/web-gui/_log/jobs/$id.*`;
        print "##### job-backup 68: delete $id ######  $r\n";
      } else {
        print "job-backup delete, no id !! ########### \n";
      }
 }

  ## run job (child process)
 sub my_run {
 ############

# print "job-backup 67\n";



    &task_gettime;                                             # $current{'timestamp'}, $current{'logtime'}

    my ($t,$r,@new,@old,@l,@w,$new,$ti);

    $t=$id."_pool";
    my $pool         = $current{$t};          # poolname

    $t=$id."_fs";
    my $fs           = $current{$t};          # filesystem

    $t=$id."_nappit";
    my $nappit      = $current{$t};          # appliance files (_log or all)

    $t=$id."_userfiles";
    my $userfiles    = $current{$t};          # user files

    $t=$id."_smbgroups";
    my $smbgroups    = $current{$t};          # smbgroups: smbadm show

    $t=$id."_idmappings";
    my $idmappings    = $current{$t};         # idmappings: idmap list

    $t=$id."_userrange";
    my $userrange    = $current{$t};          # userrange uid ex 100-60000

    $t=$id."_comstar";
    my $comstar    = $current{$t};            # Comstar settings

    $t=$id."_cluster";
    my $cluster    = $current{$t};            # Cluster settings

    $t=$id."_addscript";
    my $addscript    = $current{$t};          # add script

    # check if pool is mounted
    $r=`$ZFS list -H -o name | grep -v /`;    # pool

    if (!($r=~/\b$pool\b/)) {
      $info="Pool $pool currently not mounted, cancel backup job"; return;
    }

    # check if backup filesystem is available, opt create and secure (adminlib.pl)
    $t=&bl_check_appl_backup("$pool");
    if ($t ne "") {
       $info=$t;
       print "check $pool -> return\n";
       return;
    }

    # napp-it always
    $nappit="yes";                             # always, some folders in _log
    $fs="/$pool/backup_appliance";

    #log
    `echo $current{'timestamp'} > /$fs/lastrun.log`;
    `echo napp-it_$nappit >> /$fs/lastrun.log`;
    `echo user_$userfiles >> /$fs/lastrun.log`;
    `echo user_$userrange >> /$fs/lastrun.log`;
    `echo smbgroups_$smbgroups  >> /$fs/lastrun.log`;
    `echo idmappings_$idmappings  >> /$fs/lastrun.log`;
    `echo cluster_$cluster  >> /$fs/lastrun.log`;
    `echo addscript_$addscript  >> /$fs/lastrun.log`; ;

    # copy files via /usr/bin/cp
    $r="";

    #########
    # napp-it
    #########
    if ($nappit eq "yes") {           # backup core napp-it files
         print "167 sub bl_save_nappit($pool)\n";
         $r.=&bl_save_nappit($pool);
    }

    ######
    # user
    ######
    @user=();
    if ($userfiles eq "yes") {      # user between 100 and 30000
        $r.=&bl_save_user($pool,$userrange);  # save user to pool in range ex 100-30000 (preserve >)
    }

    # settings

    ###########
    #idmappings
    ###########
    if (($idmappings eq "yes") && ($os eq "illumos")) {
         $r.=`sudo /usr/sbin/idmap list > /$pool/backup_appliance/settings/idmap.log`;
    }

    # comstar config
    if ($comstar eq "yes") {
         &bl_iscsi_save_base("$pool");       # base backup
         &bl_iscsi_backup_full("$pool");     # full backup
    }

    # cluster config (controlserver only)
    if ($cluster eq "yes") {
       if (-f "/var/web-gui/_log/cluster/settings.txt") {
         `sudo /usr/bin/cp -p /var/web-gui/_log/cluster/settings.txt /$pool/backup_appliance/cluster/settings.txt`;
       }
       if (-f "/var/web-gui/_log/cluster/failover.txt") {
         `sudo /usr/bin/cp -p /var/web-gui/_log/cluster/failover.txt /$pool/backup_appliance/cluster/failover.txt`;
       }
    }

    #script
    if ($addscript eq "yes") {
      if (-f "/var/web-gui/_my/backup.sh") {
         $r.=`sh /var/web-gui/_my/backup.sh`;
      }
    }

    # print "$r\n";

    # return
    if ($r eq "")      { return (); }        # is noerror
    return ($r);                             # is error

 }



###########  ## log last run in jobfile /_log/jobs/id...job
sub my_log {
###########

  # print "job-repli 548 - call log id=$id action=$action\n";

  # line 1: lastrun = ok or err
  # line 2: time of last run
  # line 3+: info

  &task_gettime;                                             # $current{'timestamp'}, $current{'logtime'}

  my (@v,$v,$n,$l,$f1,$f2,@data);

  if ($info eq "") { $info="finished"; }

  $l=$error;
  if ($l eq "") { $l="ok"; } else { $l="error"; $info="$info $error"; }

  $l=~s/\n/ /gs;
  $info=~s/\n.*//s;                                        # first line


 #stage 1: log general jobs
 ###########################
  my $log="/var/web-gui/_log/jobs/jobs.log";
  my $lock="/var/web-gui/_log/jobs/jobs.lock";
  open (X,"<$lock");
  flock(X, LOCK_EX);
          open (F,"<$log");
              flock(F, LOCK_EX);
              @data=<F>;
          close (F);

        #unshift   $l=ok/error
        unshift(@data,"$l; backup $ref $id; time: $current{'timestamp'}; info: $info\n");

        if (@data > 50) {
                   $t=@data-50;
                   splice(@data,50,$t);
        }

       $t=join("",@data);
       open (PF, ">$log");
          flock(PF, LOCK_EX);
          print PF "$t";
       close (PF);
 close (X);

 if ($info=~/^delete /) { exit; }


 #stage 2 update job.par  $current{'logtime'};
  $log="/var/web-gui/_log/jobs/$id.log";

  $t="$id"."_last";                                              # update last
  $current{$t}=$current{'logtime'};
  &task_update_par($id);                                         # update last


  # stage 3 id.log newest at top
  ###################
  &my_logid("$l","info: $info");

  # update .par
  &task_gettime;                                             # $current{'timestamp'}, $current{'logtime'}
  $t="$id"."_last";                                              # update last
  $current{$t}=$current{'logtime'};
  &task_update_par($id);

 exit;  # log always last action

}


##############  log to id.log
sub my_logid {
#############
my $err =$_[0];
my $data=$_[1];
$data=~s/ +/ /g;
my @data;
&task_gettime;

my $log="/var/web-gui/_log/jobs/$id.log";
  open (F,"<$log");
        flock(F, LOCK_EX);
        @data=<F>;
  close (F);

  #unshift   $l=ok/error
  unshift(@data,"$err; time: $current{'timestamp'}; $data\n");

  if (@data > 50) {
             $t=@data-50;
             splice(@data,50,$t);                 # delete from 50+
  }

 $t=join("",@data);
 open (PF, ">$log");
    flock(PF, LOCK_EX);
    print PF "$t";
 close (PF);


}



##
1;
