#!/usr/bin/perl
#
# (c) Guenther Alka 2017-2020
# last 18.8.2020
# report discstatus based on smartvalues (temp, check)
# this report can be started every min/5min/15min
# at hour=22, min=30 a selftest of all disks is initiated
# at any hour at min=0, a smartstatus or alert can be initiated

# run alerts with any min interval setting and status at min=0
# if you want to run the smartcheck now at cli, add parameter now
# if you want to run the selftest now at cli, add parameter selftest

# you can only use either now or test
# perl /var/web-gui/data/napp-it/zfsos/_lib/scripts/report/r05#smart#temp_and_shortcheck#SI#AS.pl now

# if you only want to update smartvalues, use update as parameter

###############################
# part 1 - same for all reports
###############################

use strict;
use vars qw($par %in %disk %current %zfs %sys %cfg @par @t $t $f $r $os $id $alertlist $alertlistf $alertlog $statlog $alertsend $statsend $alertmess $info $alert $message $text $subject $binpath $lpath $spath);

use Fcntl qw(:flock); # filelocking
$|=1;                 # keine Pufferung, no buffering

  my $smt="/usr/sbin/smartctl";     # compiled
  $t=`find /opt -name smartctl`;    $t=~s/\n.*//s;  # first entry
  if ($t=~/smartctl/) { $smt=$t; }

if (!(-f "$smt")) {
    print "Smartmontools are not installed\n";
    exit;
}

require "/var/web-gui/data/wwwroot/cgi-bin/admin-lib.pl";
&getdate;
         # $sys{'min'}=$min;
         # $sys{'hour'}=$hour;
         # $sys{'mday'}=$mday;
         # $sys{'mon'}=$mon;
         # $sys{'year'}=$year;


# smartcheck:  start a selftest once per day
# check once per hour,

$par=$ARGV[0];
$par=~s/ +//g;

#optional
#if ($par eq "") {
#  `touch /tmp/nappit/smart_report.log`;
#   $t=`cat /tmp/nappit/smart_report.log`;   chomp($t);
#   if ($sys{'hour'} eq $t) {
#     print "end: run only once an hour\n";
#     exit;
#  }
#}
#`echo $sys{'hour'} > /tmp/nappit/smart_report.log`;

require "/var/web-gui/data/napp-it/zfsos/_lib/illumos/disklib.pl";
require "/var/web-gui/data/napp-it/zfsos/_lib/illumos/get-disk.pl";                         # load basic disk infos

#process report and return results
require "/var/web-gui/data/napp-it/zfsos/_lib/scripts/report/report.lib";                                          # load my_report, set message, text and alertmess
print "message_begin=$message"."message_end\nalert_begin=$alertmess"."alert_end\nstatus_begin=$text"."status_end\n";   # end


###################
# end common part 1
###################


###########################
# part 2, modify per report



###############   executed via require report.lib
sub my_report {
###############
 my ($last,$diff,$smart,@l,@w,$a,$d,@d,$r,$why,$disks);

#debug, show a hash at console
#&print_myhash(%disk);        exit;

# initiate short test at hour=0, min=0
# /usr/sbin/smartctl -t short -d sat,12 -T permissive /dev/rdsk/c3t2d0s0

if ($par eq "update") {
       print "### skip: update smartvalues only\n";
       $r=`/usr/bin/perl /var/web-gui/data/napp-it/zfsos/_lib/illumos/get-disk-smart-bg.pl`;
       exit;
}

# once a day $sys{'mday'};
`touch /tmp/nappit/smartshort_report.log`;
$t=`cat /tmp/nappit/smartshort_report.log`;   chomp($t);

if (($sys{'mday'} ne $t) || ($par eq "selftest")) {
     print "do selftest for day=$sys{'mday'} \n";

     @d=();
     @d=split(/\t/,$disk{'all_dsk'});
     foreach my $d (@d) {
       if (!($d=~/^c/))  {  next; }
       $t="/dev/rdsk/$d"."s0";

       $r=`/usr/sbin/smartctl -t short -d sat,12 -T permissive $t`;
       if ($r=~/error/s) {
         print "######### short selftest: smartctl -t short -d scsi -T permissive $t\n";
         $r=`/usr/sbin/smartctl -t short -d scsi -T permissive $t`;
       } else {
         print "######### short selftest: smartctl -t short -d sat,12 -T permissive $t\n";
       }
       print "$r\n";
     }
     `echo $sys{'mday'} > /tmp/nappit/smartshort_report.log`;
     exit;
}  else {
     print "skip selftest for day=$sys{'mday'} \n";
}


# otherwise any hour, min=0 smartcheck
# load smart

     if ((-f '/tmp/nappit/smart_last.smart') && (-f '/tmp/nappit/smart_data.smart')) {
        $last=&exe("cat /tmp/nappit/smart_last.smart");
     }
     $diff=time();
     if ($last >0) { $diff=$diff-$last; }

     # maxold=61min = 3660s
     if ($diff > 3660) {
         print "Smartvalues are too old, re-read smartvalues from disk\n";
         $r=`/usr/bin/perl /var/web-gui/data/napp-it/zfsos/_lib/illumos/get-disk-smart-bg.pl`;

     } else {
         $r=$last;        #
     }

     $smart=&exe("cat /tmp/nappit/smart_data.smart");     # all smart values

     #debug
#print $smart;
#exit;
     #print `cat /tmp/nappit/napp-it_smart.txt | grep B87`;   exit;

     @l=split(/\n/,$smart);
     $text="";
     $alertmess="";
     $alert="0";

     foreach my $l (@l) {
             if (!($l=~/_health|_selftest|_temp|overview2/)) { next; }

             $why="";
             @w=();
             @w=split(/\|/,$l);     #c3t2d0_smart_health|PASSED|

             $a="0";

             #health
             if ($w[0]=~/_health/) {  # PASSED?
               if ($w[1] =~/a-z/i) {
                 if (!($w[1]=~/PASSED/i)) {
                    $why.="health ";
                    $a="1";
                 }
               }
             }

             #self
             if ($w[0]=~/_selftest/) {  # without error    or --
               if ($w[1] =~/a-z/i) {
                 if ($w[1] ne "without error") {
                    $why.="selftest ";
                    $a="1";
                 }
               }
             }

             #overview2
             if ($w[0]=~/_overview2/) {  # problem only when increasing?
               if ($w[1] =~/problem/i) {
                    $a="1";
                    $w[1]="SMART problem or warning";
               }
             }

             #temp
             if ($w[0]=~/temp/) {
                 $w[1]=~s/\D//g;

                 # alert
                 if ($w[1]>59) {                       # alert >59 C
                   $a="1";
                 }

                 $w[1].=" degree Celsius";
             }

             $w[0].=" "x40; $w[0]=substr($w[0],0,40);
             $text.= "$w[0] ->   $w[1]\n";              # use for stat

             # alert
             if ($a) {
               $alertmess.= "$w[0] ->   $w[1]\n";
               $alert="1";
             }
     }

     $subject="ok";
     if ($alert) { $subject.=" Smarterror/problem"; }

     # c3t4d0_smart_selftest                    ->   without error
     # c3t4d0_temp                              ->   29 degree Celsius
     # c3t4d0_smart_health                      ->   PASSED

     #debug
     print "smart 222: $subject\n";

     return ($subject);          # use it ex for email subject
}




###############
sub print_myhash { #parameter %
################
 my %h = @_;
 my ($k,$v,@tmp,$w);
 `clear`;
 while (($k,$v) = each %h) {
      $k.=" "x 30;
      $k=substr($k,0,30);
      print "$k \t->\t \'$v\'\n";
 }
 exit;
}

# eof 2
1;