#!
# zfs lib
###############################
# zfs functions for napp-it    Copyright (c) 2010-2022 napp-it
################################
# functions are autoloaded if action contains &thislib_xx() ex &zfslib_list() will autoload zfslib via require

# 23.2. 2016hide services like afp
# 14.3.2016 destroy zfs shows clone origin
# 3.5. rsyncd fix
# 11.9 line 1048,1104 acl -> ACL in l2
# 21.3.17
# feb 06 2018, unlock
# enc prompt or file
# mar 2, pool >used = used + avail - fres
# mar 09, nfs, solaris enc, thanks martin
# mar 30: pool 5000
# jun.07 pool readonly
# jun 11, allow fd on rpool
# sep 14 18 snap hold, release
# dec 11 18 acc
# dec 23 zfslist inc. rpool/userfs
# dec 29 red = #ba2f28
# jan 11 19, modify zfs: del acc,
# mar 19, clone not a valid snap property in Solaris
# jun 05, show smb share settings in overview
# jun 27 illumos-enc
# sep 17 autotrim
# okt 15, 2019 special_small_blocks
# dec 21, 2019 show properties even when enc/locked
# jan 24, 2020 enc based on 2 files and web
# feb 10, 2020 unlock via web
# mar 24, 2020 minIO
# mar 28,2020 nested enc filesystems
#             minio config folder and reset
# apr 20, 2020 nested enc filesytems, smb sharing of parent/client
# may 08, 2020 minio omnios 151034
# may 23, 2020 bulk get (chris)
# jun 16, 2020 do not reshare on bootup with name=off
# sep 04, 2020 s3 show region
# jul 07, 2021 all snaps cloned
# sep 15, 2021 chris, bulk optimization
# okt 7, 2021 mount after unlock with smb=off
# nov 09, 2021 new S3 settings MINIO_ROOT_USER, MINIO_ROOT_PASSWORD (replaces KEY and SECRET)
# Dec, 16, remember name,abe for unlock
# jan 24, 2022 chris: speedup svcs services l 315ff
# mar 2022, disable www sharing as Apache is internal
# nov 22, dez 15 2022 remember smb share acl
# sep 18. 2023 show rpool snaps in list_datasnaps


#########
sub zfslib {  #
###########################
}


 ###############
 sub zfslib_acl2set {  #hide:    rwxp--aARWcCos
 ###############
      my $a=$_[0];
      $a=~s/rwxp--a-R-c--s/default_create/gs;
      $a=~s/rwxpdDaARWcCos/full_set/gs;
      $a=~s/rwxpdDaARWc--s/modify_set/gs;
      $a=~s/rwxp-DaARWc--s/modify_set_subfolder/gs;
      $a=~s/-w-p---A-W----/write_set/gs;
      $a=~s/r-----a-R-c---/read_set/gs;
      $a=~s/r-----a-R-c--s/reads_set/gs;
      $a=~s/-wxp---A-W----/writex_set/gs;
      $a=~s/r-x---a-R-c---/readx_set/gs;
      $a=~s/r-x---a-R-c--s/readxs_set/gs;
      $a=~s/------a-R-c--s/read_attr_set/gs;
      $a=~s/rwxp--aARWcCos/owner_default_set/gs;
      $a=~s/-w----a-R-c--s/create_file_set/gs;
      $a=~s/-w-p--a-R-c--s/create_file_folder_set/gs;
      $a=~s/---p--a-R-c--s/create_folder_set/gs;
      $a=~s/--------------/none/gs;
      return ($a);
 }
 ###############
 sub zfslib_set2acl {  #hide:    rwxp--aARWcCos
 ###############
      my $a=$_[0];
      $a=~s/default_create/rwxp--a-R-c--s/gs;
      $a=~s/full_set/rwxpdDaARWcCos/gs;
      $a=~s/modify_set\b/rwxpdDaARWc--s/gs;
      $a=~s/modify_set_subfolder/rwxp-DaARWc--s/gs;
      $a=~s/write_set/-w-p---A-W----/gs;
      $a=~s/read_set/r-----a-R-c---/gs;
      $a=~s/reads_set/r-----a-R-c--s/gs;
      $a=~s/writex_set/-wxp---A-W----/gs;
      $a=~s/readx_set/r-x---a-R-c---/gs;
      $a=~s/readxs_set/r-x---a-R-c--s/gs;
      $a=~s/read_attr_set/------a-R-c--s/gs;
      $a=~s/owner_default_set/rwxp--aARWcCos/gs;
      $a=~s/create_file_set/-w----a-R-c--s/gs;
      $a=~s/create_file_folder_set/-w-p--a-R-c--s/gs;
      $a=~s/create_folder_set/---p--a-R-c--s/gs;
      $a=~s/none/--------------/gs;
      return ($a);
 }

 ###############
 sub zfslib_val2kb  {  #hide:
 ###############

      my $w1=$_[0];
      $w1=~s/,/./;
      if ($w1=~/K/) { $w1=~s/K//;  }
      if ($w1=~/M/) { $w1=~s/M//; $w1=$w1*1000; }
      if ($w1=~/G/) { $w1=~s/G//; $w1=$w1*1000000; }
      if ($w1=~/T/) { $w1=~s/T//; $w1=$w1*1000000000; }
      if ($w1=~/P/) { $w1=~s/P//; $w1=$w1*1000000000000; }
      return ($w1);
 }

  ###############
 sub zfslib_val2xb  {  #hide:  kb -> gb,tb..
 ###############

      my $w1=$_[0];
      if ($w1 < 1000) { $w1.="K"; return($w1);  }

      $w1=$w1/1000;
      $w1=sprintf ("%.1f", $w1);
      if ($w1 < 1000) { $w1.="M"; return($w1);  }

      $w1=$w1/1000;
      $w1=sprintf ("%.1f", $w1);
      if ($w1 < 1000) { $w1.="G"; return($w1);  }

      $w1=$w1/1000;
      $w1=sprintf ("%.1f", $w1);
      if ($w1 < 1000) { $w1.="T"; return($w1);  }

      $w1=$w1/1000;
      $w1=sprintf ("%.1f", $w1);
      if ($w1 < 1000) { $w1.="P"; return($w1);  }

      return ($w1);
 }

 ###############
 sub zfslib_val2proz  {  #hide:  total, used return % value
 ###############


      my $total=$_[0];    # total
      my $used=$_[1];     # used
      if (($total eq "") || ($total eq "0")) { return ""; }

      my $p=($total-$used)/$total*100;       #avail in% =(total - used)/total * 100
      $p=sprintf ("%.0f", $p);               #runden
      return ($p);
 }

 ###############
 sub zfslib_pool2used  {  #hide:
 ###############  return list of pools with used in %
      my $r=&exe("/usr/sbin/zfs list -H -o name,used,avail,mountpoint", "n");
      my @zfs=split(/\n/,$r);
      $r="NAME\tUSED\tAVAIL\tMOUNTPOINT\t%\n";
      my @w=();
      my ($used,$total,$t);
      foreach my $z (@zfs) {

               @w=split(/\t/,$z);

               if ($w[0]=~/\//) { next; }        # pool only
               $r.=$z;                           # poolname,used,avail,mp

     # &mess($z);

               $used=&zfslib_val2kb($w[1]);
               $total=$used + &zfslib_val2kb($w[2]);  #total=used+avail

              $t=&zfslib_val2proz($total,$used);
              if ($t <=15) { $t.='%!'; } else { $t.='%'; }         # alert  15%!
              $r.="\t$t\n";

      }
      return ($r);
 }



###################
sub zfslib_create_datasnap {          #create snap vol@date
#####################
    my ($zfs,$out,$r);

    if ($in{'select_snapzfs'} ne "") {

    $r=$in{'checkbox_recursiv'};
       if ($r=~/on/i) {
          $r=" -r";
       } else {
          $r="";
       }


       $in{'value_comment'}=~s/ /_/g; $in{'value_comment'}=~s/[^a-zA-Z0-9\_]//g;
       if ($in{'value_comment'} ne "") { $in{'value_comment'}="_$in{'value_comment'}"; }
       $out=&exe("$sys{'sbinpath'}zfs snapshot$r \"$in{'select_snapzfs'}\"\@$sys{'date_all'}$in{'value_comment'}");
       if ($out ne "") { &mess($out); }
       print "$sys{'sbinpath'}zfs snapshot$r \"$in{'select_snapzfs'}\"\@$sys{'date_all'}  done.. <br><br>";
       &zfslib_list_datasnaps;
       return;
    }

    $zfs=&zfslib_list('comma');         #commalist

    print "Name: ZFS\@date<br><br>\n";
    &ask('begin');
    &ask('select_snapzfs','ZFS',$zfs);
    &ask('checkbox_recursiv','Recursiv','on','on');
    &ask('value_comment','Comment:','','m40');
    &ask('end');

    #&print_array(@pools);
}



#############         ##########################
sub zfslib_list {        #  list zfs datasets par: 'source (filesystem,volume,snapshot,syspool,comma,nl,br,array,header,mountpoint)' , 'below zfs'
#############         ##########################
                      # comma,nl,br,array -> form of return value
                      # header -> incl header, mount(point) -> returns mountpoint instead of name
                      # sys | all   all pools inkl sys

   my ($r,$z,@n,$h,$par);

  if ($_[0]=~/header/) { $h=""; } else { $h="-H "; }

  if ($_[0]=~/filesystem/) {
          # zfs acc


          $r=&exe("$sys{'sbinpath'}zfs list $h-t filesystem");
  } elsif  ($_[0]=~/volume/)   {
          $r=&exe("$sys{'sbinpath'}zfs list $h-t volume");
  } elsif  ($_[0]=~/snapshot/) {
          $r=&exe("$sys{'sbinpath'}zfs list $h-t snapshot");
  } else {
          $r=&exe("$sys{'sbinpath'}zfs list $h");
  }

  @t=();
  @t=split(/\n/,$r);
  foreach $z (@t) {
           if (($z=~/^$zfs{'syspool'}\s/) && (!($_[0]=~/sys|all/)))          { next; }        #no syspool itself

           if ($z=~/^$zfs{'syspool'}\/\w/)     {                             # allow user fs
             if($z=~/\/ROOT|\/dump|\/swap|\/export|VARSHARE/) { next; }
           }

           if (($_[1] ne "") && (!($z=~/^$_[1]/))) { next; }                 # list only zfs below

           if ($_[0]=~/mount/) {
                   $z=~s/^.*\s//;
                   if (($z eq "-") || ($z eq "legacy"))                     { next; }  # no mountpoint
           } else { $z=~s/\s.*//; }

           push (@n,$z);
  }

  $r=join("\n",@n);
  #rows
  if ($_[0]=~/nl/)  {
        if ($_[0]=~/br/)  {  $r=~s/\n/<br>\n/gs;    }
        $par="zfslib_list_$_[0]_nl"; $current{$par}=$r;
        return ($r);
  }

  #array
  if ($_[0]=~/array/)  {
         $par="zfslib_list_$_[0]_array"; $current{$par}=$r;
          return (@n); }

  #kommalist
  if ($_[1]=~/comma/)  {
        $r=join(",",@n);
        $par="zfslib_list_$_[0]_comma";
        $current{$par}=$r;
        return ($r);
  }

  #default tablist
  $par="zfslib_list_$_[0]_tab";
  $current{$par}=$r;
  $r=join("\t",@n); return ($r);
}



#################
 sub zfslib_overview {            # folder settings
 ##############                   # print overview if user zfs
    my (@zfs,$zfs,@z,$z,$url,$url2,$u,$n,$p,$r,@w,@prop,$prop,$name,$acl,$w1,$w2,$l,$sharename,$ip,$title,$perm,@shares_www,@shares_ftp,$guest,$ssb,$rs,$ps);
    my (%ssbcache);

    if ($in{'list'}  eq "details") { &zfslib_details; exit; }

    #change zfs property
    if ($in{'answered'})           { &zfslib_change_property; }


    # uid-> Winname
    if ($in{'prop'} eq "") {  # skip when editing (performance)
      require "/var/web-gui/data/napp-it/zfsos/_lib/illumos/acllib2.pl";
      &id2sid;
    }

#&print_hash(%in);

    # detect service state once
    ###########################
    my @svcs=split /\n/, &exe("svcs", "n");

    # afp
     #$t=&exe("svcs | grep netatalk", "n");      $t=~s/ .*|\n//gs;
     ($t) = grep {/netatalk/} @svcs;            $t=~s/ .*|\n//gs;
     if ($t=~/online/s)              { $current{'txt_afp'}="AFP"; } else { $current{'txt_afp'}="<font class='font_orange'>AFP</font>"; }

    # nfs
     #$t=&exe("svcs | grep nfs/server", "n");    $t=~s/ .*|\n//gs;
     ($t) = grep {m`nfs/server`} @svcs;         $t=~s/ .*|\n//gs;
     if ($t=~/online/s)              { $current{'txt_nfs'}="NFS"; } else { $current{'txt_nfs'}="<font class='font_orange'>NFS</font>"; }

    # smb
     #$t=&exe("svcs | grep smb/server", "n");    $t=~s/ .*|\n//gs;
     ($t) = grep {m`smb/server`} @svcs;         $t=~s/ .*|\n//gs;
     if ($t=~/online/s)              { $current{'txt_smb'}="SMB"; } else { $current{'txt_smb'}="<font class='font_orange'>SMB</font>"; }

   # rsync
    #$t=&exe("svcs | grep rsyncd", "n");         $t=~s/ .*|\n//gs;
    ($t) = grep {/rsyncd/} @svcs;               $t=~s/ .*|\n//gs;
    if ($t=~/online/s)               { $current{'txt_rsync'}="RSYNC"; } else { $current{'txt_rsync'}="<font class='font_orange'>RSYNC</font>"; }

   # www
    #$t=&exe("svcs | grep apache", "n");         $t=~s/ .*|\n//gs;
    ($t) = grep {/apache/} @svcs;               $t=~s/ .*|\n//gs;
    if ($t=~/online/s)               { $current{'txt_www'}="WWW"; } else { $current{'txt_www'}="<font class='font_orange'>WWW</font>"; }

   # ftp
    #$t=&exe("svcs | grep proftpd", "n");        $t=~s/ .*|\n//gs;
    ($t) = grep {/proftpd/} @svcs;              $t=~s/ .*|\n//gs;
    if ($t=~/online/s)               { $current{'txt_ftp'}="FTP"; } else { $current{'txt_ftp'}="<font class='font_orange'>FTP</font>"; }

   # iscsi
    #$t=&exe("svcs | grep stmf", "n");           $t=~s/ .*|\n//gs;
    ($t) = grep {/stmf/} @svcs;                 $t=~s/ .*|\n//gs;
    if ($t=~/online/s)               { $current{'txt_iscsi'}="iSCSI"; } else { $current{'txt_iscsi'}="<font class='font_orange'>iSCSI</font>"; }


    # s3 minIO
    $ps="";
    if (-f "/opt/ooce/minio/bin/minio") {
       $ps=&exe("ps axw | grep bin/minio | grep -v grep", "n");
       @t=split(/\n/,$ps);
       foreach my $l (@t) {
         #$l=~s/\/S3\_data//;
         $l=~s/.*minio server +\///;    # remain pool .....
         $l=~s/ +\-\-address +/,/;      # remove --address
         $l=~s/ +\-\-config\-dir.*//;   # remove --config-dir.*
         $l=~s/.*minio server +\///;    #
       }
       $ps=join(";",@t);
       $current{'txt_s3'}="S3cloud"; } else { $current{'txt_s3'}="<font class='font_orange'>S3cloud</font>";
    }
#&mess($ps);

    #ib,fc
    #$t=&exe("svcs | grep ibsrp/target", "n");   $t=~s/ .*|\n//gs;
    ($t) = grep {m`ibsrp/target`} @svcs;        $t=~s/ .*|\n//gs;
    if ($t=~/online/s)               { $current{'txt_ib'}="IB"; } else { $current{'txt_ib'}="<font class='font_orange'>IB</font>"; }


    #fc
    $current{'txt_fc'}="<font class='font_orange'>FC</font>";

    ######################
    # get needed iSCSI properties for overview: iSCSI share is on when there is a view enabled to /dev/zvol/rdsk/zfs/iscsi_volume_id
    $t=&exe("/usr/sbin/sbdadm list-lu");
    @t=(); @t=split(/\n/,$t);

    foreach my $t (@t) {
            if (!($t=~/^\d+/)) { next; }
            $t=~s/ +/ /g;
            @w=(); @w=split(/ /,$t);    # ==view, 1=size, 2=source

            if (($w[2]=~/\/zvol\/rdsk.*\/iscsi_\d+/) && ($w[0]=~/^600/))  {   # view to zfs zvol
                  $w=$w[2]; $w=~s/.*\/zvol\/rdsk\///; $w=~s/\/iscsi_.*//;     # keep zfs

                  $p=$w[1];
                  $p=&zfslib_val2xb($p/1000);           # size in bytes
                  $t="$w"."_iscsi"; $current{$t}="$p";                                            # Data Size  to current when on

                  $p="$w"."\_iscsi_guid";
                  $current{$p}="$w[0]";
            }
    }


# &print_hash(%current);


    # get rsync properties from /pool/rsync_shares like pool/zfs1\npool/zfs2
    #######################
    if (!($txt{'app_hide_zfs_services'}=~/rsync/i)) {
        @t=(); @t=split(/\t/,$zfs{'datapools'});
        foreach my $s (@t) {
            if (-f "/$s/.rsyncd_shares") {
                $t=&exe("cat /$s/.rsyncd_shares", "n");
                $current{'shares_rsync'}.="$t";
            }
        }
        $current{'shares_rsync'}=~s/\n/\t/gs; $current{'shares_rsync'}=~s/\t+/\t/gs;
    }

    # get www properties from /pool/www_shares like pool/zfs1\npool/zfs2
    #######################
    if (!($txt{'app_hide_zfs_services'}=~/www/i)) {
        @t=(); @t=split(/\t/,$zfs{'datapools'});
        foreach my $s (@t) {
            if (-f "/$s/.www_shares") {
                $t=&exe("cat /$s/.www_shares", "n");
                $current{'shares_www'}.="$t";
            }
        }
        $current{'shares_www'}=~s/\n/\t/gs; $current{'shares_www'}=~s/\t+/\t/gs;
        @shares_www=split(/\t/,$current{'shares_www'});
#&print_array(@shares_www);
    }


    # get ftp properties from /pool/ftp_shares like pool/zfs1\npool/zfs2
    #######################
    if (!($txt{'app_hide_zfs_services'}=~/ftp/i)) {
        @t=(); @t=split(/\t/,$zfs{'datapools'});
        foreach my $s (@t) {
            if (-f "/$s/.ftp_shares") {
                $t=&exe("cat /$s/.ftp_shares", "n");
                $current{'shares_ftp'}.="$t";
            }
        }
        $current{'shares_ftp'}=~s/\n/\t/gs; $current{'shares_ftp'}=~s/\t+/\t/gs;
        @shares_ftp=split(/\t/,$current{'shares_ftp'});
    }

    #get all afp-properties, v3 get values from get-zfs -> %sys
    ########################
    if (!($txt{'app_hide_zfs_services'}=~/afp/i)) {
        if ($sys{'netatalk3'} eq "") {
            if (-f "/etc/netatalk/AppleVolumes.default") {
                  $r=&exe("cat /etc/netatalk/AppleVolumes.default");
                  @z=();
                  @z=split(/\n/,$r);
                  foreach $z (@z) {
                     if (!($z=~/^\//)) { next; }                        # ex /tank/data data options:acls allow:@staff
                     $z=~s/^\///;                                       # zfs poolname  /name -> name
                     $z=~s/ +/ /g;
                     @w=();
                     @w=split(/ /,$z);
                     $t=$w[0]."_shareafp";
                     $current{$t}=$w[1];
                     if ($z=~/options:/) {
                          $z=~s/.*options://;
                          $t=$w[0]."_shareafp_options";
                          $current{$t}=$z;
                          if ($current{$t}=~/tm/) {                    # Time Machine ?
                                   $t=$w[0]."_shareafp_tm";
                                    $current{$t}="(T)";
                          }
                     }
                  }
            }
        }
    }


    #url
    $url="<a title=\"=title=\" class=\"=class=\" href=\"/cgi-bin/admin.pl?id=$in{'id'}\&amp;action=\&amp;l1=$in{'l1'}\&amp;l2=$in{'l2'}\&amp;l3=$in{'l3'}\&amp;list==list=\&amp;zfs==zfs=\&amp;prop==prop=\&amp;prop2==prop2=\&amp;old==old=\">=text=</a>";

    #url2=goto
    $url2="<a title=\"=title=\" class=\"=class=\" href=\"/cgi-bin/admin.pl?id=$in{'id'}\&amp;l1==l1=\&amp;l2==l2=\&amp;l3==l3=\&amp;zfs==zfs=\">=text=</a>";


    # list of filesystems, headline

    $t="ZFS (all properties)";
    if ($zfs{'all_clones'}=~/\@/) {
         $t.="\tCL";                                                        # Clones when needed
    }
    $t.="\t$current{'txt_smb'}&nbsp;\t$current{'txt_nfs'}";                 #smb and nfs always

    if (!($txt{'app_hide_zfs_services'}=~/www/i))   { $t.="\t$current{'txt_www'}&nbsp;&nbsp;";  }  #add www
    if (!($txt{'app_hide_zfs_services'}=~/ftp/i))   { $t.="\t$current{'txt_ftp'}&nbsp;&nbsp;";  }  #add ftp
    if (!($txt{'app_hide_zfs_services'}=~/rsync/i)) { $t.="\t$current{'txt_rsync'}&nbsp;&nbsp;";}  #add rsync
    if (!($txt{'app_hide_zfs_services'}=~/afp/i))   { $t.="\t$current{'txt_afp'}&nbsp;&nbsp;";  }  #add afp



    # Header
    $o='&nbsp;'x 2;
    @zfs=("$t\t$current{'txt_fc'},$current{'txt_ib'},$current{'txt_iscsi'}\t$current{'txt_s3'}\tNBMAND\tREC\tAVAILABLE$o\tUSED\tRES\tRFRES\tQUO\tRFQU\tSBS\tSYNC\tCOMPR\tDEDUP\tCRYPT\tSOURCE\tFOLDER\&nbsp;ACL\tSHARE\&nbsp;ACL$o\tPERM\tRDONLY");
    @t=();
    @t=split(/\t/,$zfs{'allfs'});
    @t=sort(@t);
    push(@zfs,@t);

#&print_array(@t);
#&print_hash(%current);
#&print_array(@shares_www);


     ## generate list
     #  ex NAME,SHARESMB,SHARENFS,..
     foreach $zfs (@zfs) {                          # list of zfs names

         # Headers
         if ($zfs=~/^ZFS \(/) {  next;  }           # headers (tablist)

         $name=$zfs;

         if (!($name=~/\//))  {                     # pool itself
           $o=" (pool)";
         } else {
           $o="";
         }


         # phase 1: get values for each ZFS

         # encrypted without key ??
         ############################
           $r="1";
           $t=$name."_encryption";
           $t=$zfs{$t};
           if ($t=~/on|\d+/) {
                         $t=$name."_keystatus";
                         $t=$zfs{$t};
                         if ($t ne "available") {
                            $r="0";
                         }
           }

           ## ZFS name
           $r="1";          # optional, offer option to display all fs properties even on enc filesystems
           if ($r) {
                            $u=$url;
                            $t="$name$o";
                            if (length($t) > 22) {
                               $u=~s/=title=/$t/g;
                               $t=substr($t,0,22);
                               $t.=" ...";
                            } else {
                               $u=~s/=title=/$t/g;
                            }

                            $u=~s/=text=/$t/g;         $u=~s/=zfs=/$name/g;     $u=~s/=style=//g;           # =zfs=     -> show details of zfs
                            $u=~s/=prop=//g;           $u=~s/=list=/details/g;  $u=~s/=old=/$name/g;   # =zfs= -> edit =prop=    (property)
                            $zfs="$u";
           } else {
                            $zfs=$name;
          }
          if ($o ne "") { $zfs="\n$zfs"; }          # pools after newline


         # get properties off all ZFS, must be same like headline above

         @prop=('origin','sharesmb','sharenfs','www','ftp','rsync','afp','iscsi','S3','nbmand','recsize','avail','used','reservation','refreservation','quota','refquota','ssb','sync','compression','dedup','encryption','folder-','share-','perm','readonly');
         foreach $prop (@prop)   {                  # for all properties
             # NAME of ZFS (always first)
             if ($prop eq "NAME") {
                next;
             }

             #hide/ skip
             if (($txt{'app_hide_zfs_services'}=~/afp/i)        && ($prop eq "afp"))        { next; }
             if (($txt{'app_hide_zfs_services'}=~/ftp/i)        && ($prop eq "ftp"))        { next; }
             if (($txt{'app_hide_zfs_services'}=~/www/i)        && ($prop eq "www"))        { next; }
             if (($txt{'app_hide_zfs_services'}=~/rsync/i)      && ($prop eq "rsync"))      { next; }

             $prop=lc($prop);                       #lowercase

             ##origin (clone)
             if ($prop =~/origin/i) {
                  if ($zfs{'all_clones'}=~/\@/) {
                      $t=$name."_origin";
                      $t=$zfs{$t};

                      $zfs.="\t";

                      $u="";
                      # check if fs contains a iscsi_clone below
                      # not on pool itself
                      if ($name=~/\//)  {
                           if ($zfs{'all_clones'}=~/$name\/iscsi_\d+/) {  # a iscsi clone is located below
                              $u=$zfs{'all_clones'};
                              $u=~s/.*$name\/iscsi_\d+//;
                              $u=~s/\t.*//;                                                   # source snap
                              $u="<a title=\"iscsi volume clone from snap: $u\">vol&nbsp;</a>";
                           }
                      }

                      if ($t=~/\@/) {
                          $u.="<a title=\"clone from snap: $t\">@&nbsp;</a>";            # clone from snap
                      }
                      if ($u eq "") {
                          $u="-";
                      }
                      $zfs.="$u";

                      next;
                  }   else {
                      next;
                  }
             }

             ##########
             # SHARESMB
             ##########


             if ($prop =~/sharesmb/i) {


                  if (!($name=~/\//))  {             #not availabe on sys or pool itself
                        $zfs.="-\t";
                  } else {

                      # encrypted without key then unavailable ??
                      ###########################
                      $t=$name."_encryption";
                      $t=$zfs{$t};
                      if ($t=~/on|\d+/) {

                         $t=$name."_keystatus";
                         $t=$zfs{$t};
                         if ($t ne "available") {
                            $zfs.="\t<b class='font_darkred'>***</b>";
                            next;
                         }
                      }


                       #share smb on/off (not syspool)
                       ################################
                       # solaris 11
                       #############
                       if ($sys{'auto_os'} eq "Solaris 11") {
                            # list all shares of this zfs
                            $t=&exe("$sys{'sbinpath'}zfs get -H -o value share $name | grep prot=smb", "n");     # opt guestok=true
                            $guest="";
                            if ($t=~/guestok=true/) {  $guest=",g";}
                            if ($t=~/abe=true/)     {  $guest.=",a";}

                            $t=~s/,*prot=smb//gs;

                            @z=();
                            @z=split(/\n/,$t);
                            $n="";
                            $r=""; # results
                            foreach my $z (@z) {        # sol 11 can have multiple shares          !! problem, would need redo of share acl (multiple files)
                                                        $u=$url;
                                                        $t=$z;                                     # name=..,path=.., ro=..
                                                        $t=~s/,path=.*//;                          # get name
                                                        $t=~s/name=//;                             # name=abc


                                                        $p=$z; $p=~s/.*path=//; $p=~s/,.*//;       # path

                                                        if (length($t) > 12) {
                                                            $u=~s/=title=/$t/g;
                                                            $t=substr($t,0,12);
                                                            $t.="..";

                                                        } else {
                                                            $u=~s/=title=/$t/g;
                                                        }

                                                        $u=~s/=text=/$t$guest/g;
                                                        $u=~s/=title=/$t$guest/g;            $u=~s/=zfs=/$name/g;      $u=~s/=style=//g;     # =zfs= -> show details of zfs
                                                        $u=~s/=prop=/sharesmb ($t)/g; $u=~s/=list=/overview/g;  $u=~s/=old=/$t/g;     # =zfs= -> edit =prop=    (property)
                                                        $u=~s/=prop2=/$t/g;     #name
                                                        $n.="$u ";
                                                        $r.="$t";

                            }


                            # add new entry
#&mess($r);
                            if ($r eq "") {
                              $u=$url;
                              $u=~s/=text=/off/g;            $u=~s/=zfs=/$name/g;      $u=~s/=style=//g;   $u=~s/=title=/smb off/g;       # =zfs=     -> show details of zfs
                              $u=~s/=prop=/sharesmb (add)/g; $u=~s/=list=/overview/g;  $u=~s/=old=/$t/g;          # =zfs= -> edit =prop=    (property)
                              $n.="$u<br>";

                            }
                            $zfs.="\t$n";

                       } else {

                            $u=$url;
                            $t=$name."_sharesmb";
                            $t=$zfs{$t};
                            $t=~s/name=//;

                            # show abe or guestok
                            $t=~s/,//g;
                            if ($t=~/abe=true/)     { $t=~s/abe=true//; $t.=",a"; }
                            if ($t=~/guestok=true/) { $t=~s/guestok=true//; $t.=",g"; }

                            if (length($t) > 18) {
                                     $u=~s/=title=/$t/g;
                                     $t=substr($t,0,18);
                                     $t.="..";
                            } else {
                                     $u=~s/=title=/$t/g;
                            }

                            $u=~s/=title=/$t/g;
                            $u=~s/=text=/$t/g;       $u=~s/=zfs=/$name/g;      $u=~s/=style=//g;          # =zfs=     -> show details of zfs
                            $u=~s/=prop=/sharesmb/g; $u=~s/=list=/overview/g;  $u=~s/=old=/$t/g;          # =zfs= -> edit =prop=    (property)
                            $zfs.="\t$u";
                       }
                  }
                  next;
            }


            #################
            #share nfs on/off
            #################

            if ($prop =~/sharenfs/) {
                 if (!($name=~/\//))  {                          #not availabe on sys or pool itself
                        $zfs.="\t-";
                 } else {

                      # encrypted without key ??
                      $t=$name."_encryption";
                      $t=$zfs{$t};
                      if ($t=~/on|\d+/) {
                         $t=$name."_keystatus";
                         $t=$zfs{$t};
                         if ($t ne "available") {
                            $zfs.="\t<b class='font_darkred'>***</b>";
                            next;
                         }
                      }


                      # solaris 11
                      #############
                      if ($sys{'auto_os'} eq "Solaris 11") {
                            # list all shares of this zfs
                            $t=&exe("$sys{'sbinpath'}zfs get -H -o value share $name | grep prot=nfs", "n");
                            chomp($t);
                            $t=~s/,*prot=nfs//gs;

                           if ($t eq "") { $t="off"; } else { $t="on"; }


                           $u=$url;
                           $u=~s/=title=/$t/g;
                           $u=~s/=text=/$t/g;       $u=~s/=zfs=/$name/g;     $u=~s/=style=//g;          # =zfs= -> show details of zfs
                           $u=~s/=prop=/sharenfs/g; $u=~s/=list=/overview/g; $u=~s/=old=/$t/g;          # =zfs= -> edit =prop= (property)
                           #$n.="$u<br>";

                           $zfs.="\t$u";

                      } else {

                            $u=$url;
                            $t=$name."_sharenfs";
                            $t=$zfs{$t};
                            $u=~s/=title=/$t/g;
                            if (length($t) > 3) { $t=substr($t,0,4); $t.=".."; }
                            $u=~s/=text=/$t/g;        $u=~s/=zfs=/$name/g;     $u=~s/=style=//g;
                            $u=~s/=prop=/sharenfs/g;  $u=~s/=list=/overview/g; $u=~s/=old=/$t/;
                            $zfs.="\t$u";
                      }
                }
                next;
            }

            ###########
            # share www        not available since Apache is internal (configure manually)
            ###########


            if ($prop =~/www/) {



                 # encrypted without key ??
                      $t=$name."_encryption";
                      $t=$zfs{$t};
                      if ($t=~/on|\d+/) {
                         $t=$name."_keystatus";
                         $t=$zfs{$t};
                         if ($t ne "available") {
                            $zfs.="\t<b class='font_darkred'>***</b>";
                            next;
                         }
                      }
                 $u=$url;
                 $u=~s/=zfs=/$name/g;       $u=~s/=style=//g;
                 $u=~s/=prop=/share_www/g;  $u=~s/=list=/overview/g;

                 if (!($name=~/\//))  {             #not availabe on sys or pool itself
                        $u="-";

                 } else {

                       #get par (ip, name)
                       $t=""; @t=();
#&mess("$prop $name");
#&print_array(@shares_www);
                       foreach $p (@shares_www) {
                           if ($p=~/^$name,/i) {
                                $u=~s/=title=/$p/g;
                                $t=$p;
                                @t=split(/,/,$t);
                           }
                       }
                       if ($t ne "") {

                           $t=$t[1];
                           if ($t=~/:.*:/) {       # ip6
                              $t=~s/.*\://;
                           } else {
                              $t=~s/.*\.//;
                           }

                           if ($t eq "") {
                                $t="on";
                           }

                           $u=~s/=text=/$t/g;
                           $u=~s/=title=/on/g;  # fallback
                           $u=~s/=old=/on/;
                       } else {
                           $u=~s/=title=/www off/g;
                           $u=~s/=text=/off/g;
                           $u=~s/=old=/off/;
                       }
                 }
                 $zfs.="\t$u";
                 next;

            }

            ###########
            # share ftp
            ###########

            if ($prop =~/ftp/) {
                # encrypted without key ??
                      $t=$name."_encryption";
                      $t=$zfs{$t};
                      if ($t=~/on|\d+/) {
                         $t=$name."_keystatus";
                         $t=$zfs{$t};
                         if ($t ne "available") {
                            $zfs.="\t<b class='font_darkred'>***</b>";
                            next;
                         }
                      }

                 $u=$url;
                 $u=~s/=zfs=/$name/g;          $u=~s/=style=//g;
                 $u=~s/=prop=/share_ftp/g;     $u=~s/=list=/overview/g;

                 if (!($name=~/\//))  {             #not availabe on sys or pool itself
                        $u="-";

                 } else {
                       #get par (ip:port)
                       $t=""; @t=();
#&mess($name);
#&print_array(@shares_ftp);
                       foreach $p (@shares_ftp) {
                           if ($p=~/^$name,/) {
                                $u=~s/=title=/$p/g;
                                $t=$p;
                                @t=split(/,/,$t);
                           }
                       }
                       if ($t ne "") {
                           $t=$t[1];
                           if ($t=~/:.*:/) {       # ip6
                              $t=~s/.*\://;
                           } else {
                              $t=~s/.*\.//;
                           }

                           if ($t eq "") {
                                $t="on";
                           }
                            $u=~s/=text=/$t/g;
                       }
#&mess($current{'shares_ftp'});
                       if ($current{'shares_ftp'}=~/\b$name,|\b$name$/) {
                           $u=~s/=title=/ftp on/g;
                           $u=~s/=text=/on/g;
                           $u=~s/=old=/on/;
                       } else {
                           $u=~s/=title=/ftp off/g;
                           $u=~s/=text=/off/g;
                           $u=~s/=old=/off/;
                       }
                 }
                 $zfs.="\t$u";
                 next;
            }

            #############
            # share rsync
            #############

            if ($prop =~/rsync/) {

                  # encrypted without key ??
                      $t=$name."_encryption";
                      $t=$zfs{$t};
                      if ($t=~/on|\d+/) {
                         $t=$name."_keystatus";
                         $t=$zfs{$t};
                         if ($t ne "available") {
                            $zfs.="\t<b class='font_darkred'>***</b>";
                            next;
                         }
                      }
                 $u=$url;
                 $u=~s/=zfs=/$name/g;         $u=~s/=style=//g;
                 $u=~s/=prop=/share_rsync/g;  $u=~s/=list=/overview/g;

                 if (!($name=~/\//))  {             #not availabe on sys or pool itself
                        $u="-";

                 } else {
                       if ($current{'shares_rsync'}=~/\b$name\b/) {
                           $u=~s/=title=/rsync on/g;
                           $u=~s/=text=/on/g;
                           $u=~s/=old=/on/;
                       } else {
                           $u=~s/=title=/rsync off/g;
                           $u=~s/=text=/off/g;
                           $u=~s/=old=/off/;
                       }
                 }
                 $zfs.="\t$u";
                 next;
            }

            #############
            # share iscsi
            #############
            # only zvol: off (vol)                 # zvol from %zfs   zfs_iscsi_vol
            # zvol + LU without view: off (hide)   # lu from %zfs     zfs_iscsi_LU
            # zvol + LU + view: size               # view from %zfs   zfs_iscsi_LU_view
            # size   $current{'zfs_iscsi'}
            #
#debug
#&print_var($name);
#&print_hash(%current);
#&print_hash(%zfs);
            if ($prop =~/iscsi/) {

                 # encrypted without key ??
                      $t=$name."_encryption";
                      $t=$zfs{$t};
                      if ($t=~/on|\d+/) {
                         $t=$name."_keystatus";
                         $t=$zfs{$t};
                         if ($t ne "available") {
                            $zfs.="\t<b class='font_darkred'>***</b>";
                            next;
                         }
                      }

                 $u=$url;
                 $u=~s/=zfs=/$name/g;         $u=~s/=style=//g;
                 $u=~s/=prop=/share_iscsi/g;  $u=~s/=list=/overview/g;

                 if (!($name=~/\//))  {             #not available on sys or pool itself
                        $u="-";

                 } else {
                       # view=ON?
                       $t="$name\_iscsi_LU_view";
                       if ($zfs{$t} ne "") {
                           $t="$name\_iscsi";
                           $u=~s/=text=/$current{$t}/g;  # size
                           $p="$name\_iscsi_LU";
                           $u=~s/=title=/GUID=$zfs{$p}, size=$current{$t}/g;
                           $u=~s/=old=/on/; $u=~s/=prop2=/$zfs{$p}/;
                       } else {
                           $w="zfs unset";
                           $p="zfs unset (zvol is missing)";

                           #vol?
                           $t="$name\_iscsi_vol";
                           if ($zfs{$t} ne "") { $p="off (volume exist, LU is unset)"; $w="off (vol)"; }

                           #LU?
                           $t="$name\_iscsi_LU";
                           if ($zfs{$t} ne "") { $p="off (hide, LU exist, view is unset)"; $w="off (hide)"; }
                           $u=~s/=title=/iscsi $p/g;

                           $u=~s/=text=/$w/g;
                           $u=~s/=old=/off/;

                           $p="$name\_iscsi_LU";
                           $u=~s/=prop2=/$zfs{$p}/;
                       }
                 }
                 $zfs.="\t$u";
                 next;
            }

            ##########
            ##S3 minIO
            ##########

            if ($prop =~/S3/i) {

                 # encrypted without key ??
                      $t=$name."_encryption";
                      $t=$zfs{$t};
                      if ($t=~/on|\d+/) {
                         $t=$name."_keystatus";
                         $t=$zfs{$t};
                         if ($t ne "available") {
                            $zfs.="\t<b class='font_darkred'>***</b>";
                            next;
                         }
                      }

                 if (!($name=~/\//))  {             #not availabe on sys or pool itself
                        $zfs.="\t-";

                 } else {

                     $t=$name."_hasS3";
                     $t=$zfs{$t};
                     $n = ($t) ? "off" : "unset";
#&mess($ps);
                     if ($ps=~/\b$name\/s3_data/is) {
                        @t=split(/;/,$ps);

                        foreach my $l (@t) {
                          if ($l=~/\b$name\/s3_data/is) {
                            $t=$l; $t=~s/.*,//;
                            last;
                          }
                        }
                        if ($t=~/:\d/) {
                          $t=~s/ .*//;
                          $n=$t;
                        } else {
                          $n="on";
                        }
                     }

                     $u=$url;
                     $u=~s/=text=/$n/g;        $u=~s/=zfs=/$name/g;     $u=~s/=title=/$name/g;
                     $u=~s/=prop=/shareS3/g;   $u=~s/=list=/overview/g; $u=~s/=old=/$n/;
                     $zfs.="\t$u";
                  }
                  next;
            }


            ##########
            # share afp
            ###########

            if ($prop =~/afp/) {

               # encrypted without key ??
                      $t=$name."_encryption";
                      $t=$zfs{$t};
                      if ($t=~/on|\d+/) {
                         $t=$name."_keystatus";
                         $t=$zfs{$t};
                         if ($t ne "available") {
                            $zfs.="\t<b class='font_darkred'>***</b>";
                            next;
                         }
                      }

               if (!($name=~/\//))  {             #not availabe on sys or pool itself
                        $zfs.="\t-";
               } else {

                      # encrypted without key ??
                      $t=$name."_encryption";
                      $t=$zfs{$t};
                      if ($t eq "on") {
                         $t=$name."_keystatus";
                         $t=$zfs{$t};
                         if ($t ne "available") {
                            $zfs.="\t<b class='font_darkred'>***</b>";
                            next;
                         }
                      }

                      #share afp on/off (not syspool)


                     if (! ((-f "/usr/local/etc/afp.conf") || (-f "/opt/netatalk/etc/afp.conf") || (-f "/etc/netatalk/AppleVolumes.default")) ) {
                                                       $zfs.="\tn.a.";
                     } else {
                                                      $n=$name."_shareafp";
                                                      $n=$zfs{$n};
                                                      if ($n eq "") { $n="off"; }                    # sharename

                                                      if ($sys{'netatalk3'} ne "") {                 #v3 settings
                                                                $t=$name."_afp_time_machine";
                                                                $t=$zfs{$t};
                                                                if ($t eq "yes") { $n.=" (T)"; }                        # options
                                                      } else { #v2

                                                                $t=$name."_shareafp_options";
                                                                $t=$zfs{$t};                         # options

                                                                if ($t=~/^tm/) { $n.="(T)"; }
                                                                if ($t =~/ro/) { $n.="(RO)"; }
                                                      }

                                                      $u=$url; $u=~s/=text=/$n/g;     $u=~s/=zfs=/$name/g;     $u=~s/=title=/$t/g;
                                                      $u=~s/=prop=/shareafp/g;        $u=~s/=list=/overview/g; $u=~s/=old=/$n/;
                                                      $zfs.="\t$u";
                      }
                      # /afp

                 }
                 next;
             }


             #crypt
             #########
             if ($prop =~/crypt/) {
### martingo 20180306 >>> Wenn keysource von einem Parent geerbt wurde, enthaelt dieser Key den verschluesselten Parent (kann auch ein Parent weiter oben sein)
### die sonstige Logik (URL, Schrift, ...) wird übersprungen
                   if ($sys{'auto_os'} eq "Solaris 11") {
                           if ( $zfs{$name.'_keysource_inherited_from'} ) {
                                # $zfs .= "\t<span title=\"inherited from $zfs{$name.'_keysource_inherited_from'}\">auto</span>"; # scheinbar werden die Zeichen gezählt und daraus die Spaltenbreite berechnet. Die Spalte wird so zu breit
                                $zfs .= "\tauto\t-";
                                next;
                            }
                   } else {
                           if ( ($zfs{$name.'_encryptionroot'} =~/\//) && ( $zfs{$name.'_encryptionroot'} ne $name )) {
                                # $zfs .= "\t<span title=\"inherited from $zfs{$name.'_keysource_inherited_from'}\">auto</span>"; # scheinbar werden die Zeichen gezählt und daraus die Spaltenbreite berechnet. Die Spalte wird so zu breit
                                $zfs .= "\tauto\t-";
                                next;
                            }
                   }
### <<<
                    $u=$url;
                    $t=$name."_encryption";
                    $t=$zfs{$t};
                    if ($t eq "") {
                                 $zfs.="\tn.a.\t-";
                    } else {

                        #keysource (user prop)
                        #$p=&exe("zfs get -H -o value enc_split: $name");  chomp($p);
                        $p=$zfs{$name."_enc_split:"};

                        if (!($p=~/[LW]\d:[LW]\d/)) { $p='&nbsp;'; }


                        if ($t=~/on|\d+/) {
                           $t=$name."_keystatus";
                           $t=$zfs{$t};

                           # lock/ unlock
                           $u=$url;
                           $u=~s/=zfs=/$name/;

                           if ($t eq 'available') {
                              $u=~s/=title=/click to lock/g;
                              $t="<b class='font_darkblue'>unlocked</b>";
                              $u=~s/=prop=/lock/;
                           } else {
                              $u=~s/=title=/click to unlock/g;
                              $t="<b class='font_darkred'>locked</b>";
                              $u=~s/=prop=/unlock/;
                           }


                           $u=~s/=text=/$t/;    $u=~s/=style=//g;
                           $u=~s/=title=/$t/g;
                           $u=~s/=old=/on/;
                           $u=~s/=list=/overview/;
                           $zfs.="\t$u\t$p";
                        } else {  #off
                           $zfs.="\toff\t$p";
                        }
                   }
                next;
             }

            #############
            # ssb
            #############

            if ($prop =~/ssb/) {

                #small allocation classes
                $t=$name; $t=~s/\/.*//;
                if (exists $ssbcache{$t}) {
                   $t=$ssbcache{$t};
                } else {
                   my $pool = $t;
                   $t=&exe("/sbin/zpool status $t", "n");
                   $ssbcache{$pool} = $t;
                }
                $ssb="-";
                if ($t=~/\tspecial\t/s) {
                    $ssb=&zfslib_get_property('special_small_blocks',$name);       #&mess($name);
                }

                 $u=$url;
                 $u=~s/=zfs=/$name/g;         $u=~s/=style=//g;
                 $u=~s/=prop=/SSB/g;          $u=~s/=list=/special small blocks/g;

                 if ($ssb eq "-")  {
                        $u="-";

                 } else {
                        $u=~s/=title=/SSB/g;
                        $u=~s/=text=/$ssb/g;
                        $u=~s/=old=/$ssb/;

                 }
                 $zfs.="\t$u";
                 next;
            }

             ############
             # folder acl
             ####################
             if ($prop =~/^folder-/) {
                if ($in{'prop'} ne "") {  # skip when editing (performance)
                  $u="-";
                }  else {
                     #

                   # encrypted without key ??
                        $t=$name."_encryption";
                        $t=$zfs{$t};
                        if ($t eq "on") {
                           $t=$name."_keystatus";
                           $t=$zfs{$t};
                           if ($t ne "available") {
                              $zfs.="\t<b class='font_darkred'>***</b>";
                              next;
                           }
                        }


                  $t="$name\_dv";
                  if ($zfs{$t} ne "") {
                     $r=$zfs{$t}; $r=~s/<n>/\n/gs;
                  } else {
                     $r= &exe("$sys{'binpath'}ls -dV /$name", "n");
                  }

                  #without ACL beside basic trivials
                  $o="";
                  $t=$r; $t=~s/\d.*//;      # number of files > 999: drwxrwx---+1925 nas ->  drwxrwx---+ (or + in name may produce problems)
                  $title="";
                  if (!($t=~/\+/))  {
                                 $o="default ACL";
                                 $title="default";

                  } else {

                                 $r=&zfslib_acl2set($r);            # show sets
                                 $title=$r;
                                 $title=~s/  +/ /gs;
                                 $title=~s/.*\d \//\//s;            # acl only, no perm

                                 $title=&uid2winname($title);       # uid to winname
                                 $title=~s/\n/\r/gs;

                                  #show everyone@..ex  owner@:rwxp--aARWcCos:-------:allow'

                                  @t=split(/\n/,$r);
                                  foreach $t (@t) {
                                          $t=~s/^\s+//;
                                  }
                                  @t=sort(@t); @t=reverse(@t);
                                  $r="";
                                  foreach $t (@t) {
                                          if (($t=~/\@/) && ($t=~/allow/))  {
                                               $t=~s/owner\@:/o=/;
                                               $t=~s/group\@:/g=/;
                                               $t=~s/everyone\@:/e=/;
                                               $t=~s/_set//;
                                               $t=~s/default_//;
                                               $t=~s/:.*//;

                                               if ($t=~/-/) {
                                                 $t=~s/=.*/=special/;
                                               }
                                               $t=~s/modify/modi/;
                                               $t=~s/create/crea/;
                                               $r.="$t, ";
                                          }
                                  }
                                  $r=~s/, $//;
                                  if ($r eq "") { $r="root only";}
                  }

                  $u=$url2;
                  $u=~s/=zfs=/$name/g;
                  $u=~s/=l1=/$in{'l1'}/g;    $u=~s/=l2=/15_ACL_extension=-lin/g;       $u=~s/=l3=/01_ACL on folders/;

                  if ($o ne "") {
                                   $u=~s/=text=/$o/g;
                                   $u=~s/=title=/$title/g;
                  } else {
                                   if ($r eq "e=modi") {
                                      $u=~s/=text=/every\@=mod/g;
                                      $u=~s/=title=/$title/g;
                                   } else {
                                      $u=~s/=text=/special/g;
                                      $u=~s/=title=/$title/g;
                                   }
                  }
                }
                $zfs.="\t$u";
                next;
             }

             # shareacl everyone
             ####################

             if ($prop =~/^share-/) {

                if ($in{'prop'} ne "") {  # skip when editing (performance)
                  $zfs.="\t-";
                }  else {

                  $title="";

                  $t="$name\_sharesmb";

                  # if ($r ne "") {
                  if ( $zfs{$t} ne "off") {    # smb share
                            #      if (defined $shacl) {
                            #         $r=$shacl;
                            #      } else {
                            $t=$name;       #ex tank/data
                            $t=~s/.*\///;   # ex data
                            $r= &exe("$sys{'binpath'}ls -dV \'/$name/.zfs/shares/$t\'", "n");   chomp($r);
                            $r=~s/ +//gs;           # remove spaces
                            @t=split(/\n/,$r);

                            splice @t, 0, 1;        # remove first entry (filename)



                                  $title=$r;
                                  $title=~s/  +/ /gs;
                                  $title=~s/.*\d \//\//s;            # acl only, no perm
                                  $title=~s/.*\.zfs\///s;

                                  $title=&uid2winname($title);       # uid to winname
                                  $title=~s/\n/\r/gs;

                                  $p="special";
                                  foreach my $t (@t) {

                                     if ($t=~/everyone\@/) {
                                       #&mess($t);
                                       if ($t eq 'everyone@:rwxpdDaARWcCos:-------:allow') { $p='every@=full'; }
                                       if ($t eq 'everyone@:rwxpdDaARWc--s:-------:allow') { $p='every@=mod'; }
                                       if ($t eq 'everyone@:r-----a-R-c---:fd-----:allow') { $p='every@=read'; }
                                       last;
                                     }
                                  }

                                  $u=$url2;
                                  $u=~s/=zfs=/$name/g;
                                  $u=~s/=l1=/$in{'l1'}/g;    $u=~s/=l2=/15_ACL_extension=-lin/g;       $u=~s/=l3=/02_ACL on SMB shares/;

                                  $u=~s/=text=/$p/g;
                                  $u=~s/=title=/$title/g;

                                  $zfs.="\t$u";
                   } else {

                                  $zfs.="\t-";
                   }
                }
                next;
              }

              # perm
              ########
              if ($prop =~/perm/) {
                if ($in{'prop'} ne "") {  # skip when editing (performance)
                  $r="-";
                } else {

                   $t="$name\_ls";
                   if ($zfs{$t} ne "") {
                      $r=$zfs{$t}; $r=~s/<n>/\n/gs;
                   } else {
                      $r=&exe("$sys{'binpath'}ls -l -d /$name");
                  }
#&mess($perm);

                  if (!($r=~/^d/)) { $r="-"; $perm=""; }
                  $r=~s/^d//;
                  $r=~s/rwx/7/g;  $r=~s/rw-/6/g; $r=~s/r-x/5/g; $r=~s/r--/4/g;   $r=~s/---/0/g;

                  $perm=$r;
                  $perm=~s/  +/ /gs;
                  $perm=~s/\n.*//s;                  # permissions
                  $perm=~s/\//\r\//;


                  $r=~s/ .*//;

                  if ($r=~/\+/) {
                       $r="<a href=\"#\" title=\"$perm\">ACL</a>";
                  }  else {
                       $r="<a href=\"#\" title=\"$perm\" class=\"font_darkred\" ><b>$r</b></a>";
                  }
                 }
                 $zfs.="\t$r";
                 next;
             }


            # all other regular zfs properties
            ####################################

                    $u=$url;
                    $t=$name."_$prop";
                    $t=$zfs{$t};
                    if ($t eq "") {
                                 $zfs.="\tn.a.";
                    } else {
                            # encrypted without key ??
                            $o=$name."_encryption";
                            $o=$zfs{$o};
                            if ($o eq "on") {
                                 $o=$name."_keystatus";
                                  $o=$zfs{$o};
                                  if ($o ne "available") {
                                       $zfs.="\t$t";
                                       next;
                                  }
                             }

                             #avail total = avail + used -> in %
                             if ($prop eq "avail") {
                                  $w1=$t;
                                  $w1=&zfslib_val2kb($w1);


                                  $w2=$name."_used";
                                  $w2=$zfs{$w2};
                                  $w2=&zfslib_val2kb($w2);

                                  if ($w2>0) {
                                         $w2=&zfslib_val2proz($w1+$w2,$w2);              # total, used -> prozent value

                                         # pool: show percent, filesystem show iB

                                         if (!($name=~/\//)) {          # pool. add percent
                                                     if ($w2 < 26) {
                                                          if ($w2 < 16) {              #alert!
                                                               $t.="<font color='red'> [$w2%]</font>";
                                                          } else {
                                                               $t.="<font class='font_orange'> [$w2%]</font>";
                                                          }
                                                     }  else {
                                                          $t.=" [$w2%]";
                                                     }
                                         }
                                  }
                             }


                             # read-only-value
                             if ("type,creation,available,referenced,compressratio,mounted,avail,"=~/$prop,/i) {
                                # used in %

                                $zfs.="\t$t";
                                next;
                             }

                             $u=$url;
                             # readonly zfs red
                             if (($prop eq "readonly") && ($t eq "on")) {
                                  $o="<font class=\"font_darkred\">$t</font>";
                             } else {
                                  $o=$t;
                             }

                             if ($prop eq "used") {
                                   $u=~s/=title=/User and Group-Quotas/g;
                             }


                             $u=~s/=text=/$o/g;         $u=~s/=zfs=/$name/g;     $u=~s/=title=//g;
                             $u=~s/=prop=/$prop/g;      $u=~s/=list=/overview/g; $u=~s/=old=/$t/;
                             $zfs.="\t$u";
                    }
        }
     }

#&print_array(@zfs);
#&print_hash(%current);

    $zfs="";
    $zfs=join("\n",@zfs);

    #show as table
    if (!($zfs =~/\d/s)) { $zfs.="-none-"; }      # ex available

    $zfs=~s/  +/\t/gs;
    $zfs=&list2table($zfs);
    $zfs="<!--wsarea-->\n<div id='wsarea'>\n$zfs\n</div>\n<!--wsarea_end-->\n";    # realtime update of list only

    print "$zfs<br>$txt{'zfslist_footer'} <br>Size example 1T means 1 TiB";

    # add user and group quotas
    if ($in{'prop'} =~/^use/) {
         &zfslib_userquotas;

         if ($in{'prop'} ne "used")  {
              print &zfslib_ask_property;              # edit user
              print "\n</body></html>";
              return;
         } else {
              $in{'prop'}="";
         }
    }

    #ask property
    if (($in{'prop'} ne "") && ($in{'answered'} ne "1")) { print &zfslib_ask_property; }


    print "\n</body></html>";
    return;

 }


 ################
 sub zfslib_details {                 #hide:
 ################    show list of current zfs details  or show list of available zfs properties

    my ($zfs,$r,$url,$url2,$f,$l,@l,$acl);


   if ($in{'list'}  eq "overview") { &zfslib_overview; exit; }


if (!($in{'zfs'} eq $zfs{'syspool'})) {
   #########################

   #change zfs property
    if ($in{'answered'}) { &zfslib_change_property; }

   #overview
   $url="<a href=\"/cgi-bin/admin.pl?id=$in{'id'}\&amp;action=\&amp;l1=$in{'l1'}\&amp;l2=$in{'l2'}\&amp;l3=$in{'l3'}\&amp;list=overview\&amp;zfs=\">Overview</a>";

   #list of  properies
   $url2=" <a href=\"/cgi-bin/admin.pl?id=$in{'id'}\&amp;action=\&amp;l1=$in{'l1'}\&amp;l2=$in{'l2'}\&amp;l3=$in{'l3'}\&amp;zfs=$in{'zfs'}\&amp;list=options\">Properties</a>";

   print "$url &nbsp; $url2\n";


   print "\n<br><br><b>ACL + Permissions /$in{'zfs'}:</b><br>\n";

   ### get folder acl ######################
   ###################
   $f= &exe("ls -dV /$in{'zfs'}");

   #print set
   $r=&zfslib_acl2set($f);            # show sets
   $r=~s/  +/\t/gs;
   print &list2table($r);


   #get share acl
   #################

   $f=$in{'zfs'}; $f=~s/^.*\///;               # -> zfs name
   $f="/$in{'zfs'}/.zfs/shares/$f";

   $f= &exe("ls -dV $f");

   $f=&zfslib_acl2set($f);                            # show sets
   $acl=$f;

   $r=$f;

   #######################
   #print "<br>\n";
   print "\n<br><br><b> Share ACL /$in{'zfs'} for everybody\@</b> \n";

   $r=~s/  +/\t/gs;
   print &list2table($r);


   #ask property
    if (($in{'prop'} ne "") && ($in{'answered'} ne "1")) { print &zfslib_ask_property; }
   #################################

   # !!

 } #/syspool

   ##print overview of "zfs"-folder
   ##################################
   print "<br><br>\n<b>ZFS Properties $in{'zfs'}</b><br>zfs set PROPERTY=VALUE NAME<br>";



   if ($in{'list'} eq "options") {
      $l=&exe("$sys{'sbinpath'}zfs set");                  #list options
      @l=split(/\n/,$l);  $l="";
      foreach $t (@l) {
        if ($t=~/^missing|^usage:|^The |^Sizes |^User-|^a user|^\s+set /) { next; }
        if (!($t=~/\w/)) { next; }
        $t=~s/^ +|^\t+//;
        $l.="$t\n";
      }
   } else {
     $l=&exe("$sys{'sbinpath'}zfs get all $in{'zfs'}");
   }

   #insert ask to table
   $l=&parse_date($l);
   $l=~s/  +/\t/gs;
   print &list2table($l);
   ##################################

   ## solaris 11
   if ($sys{'auto_os'} eq "Solaris 11") {
     print "<br><b>Solaris 11 share settings: zfs get -o name,value share</b>";

     $l=&exe("zfs get -o name,value share");
     $l=~s/,/\t/gs;
     $l=~s/  +/\t/gs;
     print &list2table($l); ;
   }


    print "\n</body></html>";

   return;

 }

 ##user and groupquotas
 sub zfslib_userquotas {
 ######################
     #url
     my $url="<a title=\"=title=\" href=\"/cgi-bin/admin.pl?id=$in{'id'}\&amp;l1=$in{'l1'}\&amp;l2=$in{'l2'}\&amp;l3=$in{'l3'}\&amp;list==list=\&amp;zfs==zfs=\&amp;prop==prop=\&amp;prop2==prop2=\&amp;old==old=\">=text=</a>";
     my (@l,$l,$t,@w,$w,$u);

     ##user
     ######

     my $header="SOURCE\tTYPE\tNAME\tUSED\tSET QUOTA (acl-extension only)\n";     # type, name, used, quota
     print "\n<br><br><br><b>User-Quota settings /$in{'zfs'}:</b><br>\n";

     my $r=&exe("$sys{'sbinpath'}zfs userspace -H \'$in{'zfs'}\'");

     @l=split(/\n/,$r);
     foreach $l (@l) {
                   $l=~s/^\s+//g;
                   $l=~s/\s+/\t/g;
                   @w=();
                   @w=split(/\t/,$l);

                   # set in c4
                   $t=$w[4];
                   $w[4]=$url;
                   $w[4]=~s/=text=/$t/; $w[4]=~s/=zfs=/$in{'zfs'}/; $w[4]=~s/=prop=/userquota:$w[2]/;
                   $w[4]=~s/=list=/$w[2]/; $w[4]=~s/=prop2=/userquota/; $w[4]=~s/=title=/set userquota of $w[2]/;  if (!($cfg{'ver'}=~/a|c/)) { $w[4]=$t; }

                   $l=join("\t",@w);

     }
     $r=join("\n",@l);

     $r="$header\n$r";
     print &list2table($r);


     # group
     #######

     $header="SOURCE\tTYPE\tNAME &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;\tUSED\tSET QUOTA (acl-extension only)\n";
     print "\n<br><br><br><b>Group-Quota settings /$in{'zfs'}:</b><br>\n";

     $r=&exe("$sys{'sbinpath'}zfs groupspace -H \'$in{'zfs'}\'");

     @l=split(/\n/,$r);
     foreach $l (@l) {
                   $l=~s/^\s+//g;
                   $l=~s/\s+/\t/g;
                   @w=();
                   @w=split(/\t/,$l);

                   # set in c4
                   $t=$w[4];
                   $w[4]=$url;
                   $w[4]=~s/=text=/$t/; $w[4]=~s/=zfs=/$in{'zfs'}/; $w[4]=~s/=prop=/usergquota:$w[2]/;
                   $w[4]=~s/=list=/$w[2]/; $w[4]=~s/=prop2=/usergquota/; $w[4]=~s/=title=/set groupquota of $w[2]/;  if (!($cfg{'ver'}=~/a|c/)) { $w[4]=$t; }

                   $l=join("\t",@w);

     }
     $r=join("\n",@l);

     $r="$header\n$r";
     print &list2table($r);

     &log_end;
     print "<br><br><br><br>";
     return;
 }

 #######################
 sub zfslib_refresh_svc  {                   #hide: refresh or reload all share services (smb,afp,nfs,www) - needed for ex if you lock/ unlock encrypted zfs
 #######################
  my ($smb,$nfs,$apache,$afp,$r);
  my $s=&exe("svcs");
  my @s=split(/\n/,$s);
  foreach $t (@s) {
           # smb
           if (($s=~/online/) && ($s=~/\/smb\/server/) && ($smb ne "1")) {
              $smb="1";
              $t=$in{'zfs'}."_sharesmb";
              if ($zfs{$t}=~/on|\=/) {
               $r=`sudo /usr/sbin/svcadm disable smb/server && sudo /usr/sbin/svcadm enable -r smb/server | at -s now`;
              }
           }
           # nfs
           if (($s=~/online/) && ($s=~/\/nfs\/server/) && ($nfs ne "1")) {
              $nfs="1";
              $t=$in{'zfs'}."_sharenfs";
              if ($zfs{$t}=~/on/) {
                `sudo /usr/sbin/svcadm refresh nfs/server | at -s now | > /dev/null`;
              }
           }
           # apache2 (nexenta)
           if (($s=~/online/) && ($s=~/apache2$/) && ($apache ne "1")) {
                 $apache="1";
                 `sudo /usr/sbin/svcadm refresh apache2 | at -s now | > /dev/null`;
           }
           # apache22 (solaris)
           if (($s=~/online/) && ($s=~/apache22$/) && ($apache ne "1")) {
                 $apache="1";
                `sudo /usr/sbin/svcadm refresh apache22 | at -s now | > /dev/null`;;
           }
           #afp
           $t=&exe("ps ax");
           if (($t=~/afpd/s) && ($afp ne "1")) {
                $afp="1";
                #print "restart AFPd";
                `sudo /etc/init.d/netatalk stop`;
                `sudo /etc/init.d/netatalk start`;
           }
  }
  #exit;

 }

 ##########################
 sub zfslib_update_rsync {      #hide: update /var/web-gui/_log/rsyncd.conf
 ##########################
    my (@pools,$r,@s,$f,$sn,$n,$cf,$pf,$t);
    # read shares, verify if ZFS is available
    # for all shares, add settings to /var/web-gui/_log/rsyncd.conf

    `sudo touch /var/web-gui/_log/rsyncd.conf`;
    `sudo chmod 666 /var/web-gui/_log/rsyncd.conf`;

    $n=`cat /var/web-gui/_log/rsyncd.conf`;
    $n=~s/\[.*//s;   # keep header

    # part 1: verify shares
    @pools=split(/\t/,$zfs{'datapools'});
    foreach my $p (@pools) {
            # read shares

             $f="/$p/.rsyncd_shares";

             `sudo touch $f`;
             `sudo chmod 666 $f`;

              $r=`cat \'$f\'`;

              @s=();
              @s=split(/\n/,$r);                           # rsync shares

              open (PF, ">$f");                            # open share config file
              foreach my $s (@s) {

                  $sn=$s; $sn=~s/\//./g;                    # tank/data -> tank.data
                  $cf="/$p/.rsyncd_shares_config_$sn";      # share config file
                  $pf="/$p/.rsyncd_shares_scrt_$sn";        # share pw file

                  if (-d "/$s") {
                       print PF "$s\n";                     # write zfs filesystem

                       # add to n(ew)
                       if (-f "$cf") {
                          $t=`cat $cf`;
                          $n.=$t;
                          $n.="\n";

                          if ((-f "$pf") && ($t=~/auth users *= *\w+/)) {
                               $n.="secrets file = $pf\n";
                          }
                          $n.="\n";
                       }

                  } else {

                       #if (-f "$cf")  { `sudo rm $cf`; }     # config file, optional
                       #if (-f "$pf")  { `sudo rm $pf`; }     # pw file
                  }
              }
              close (PF);

              # update configfile
              $n=~s/\n +/\n/gs;
              open (PF, ">/var/web-gui/_log/rsyncd.conf");
                       print PF "$n\n";
              close (PF);

    }

 }
######
sub zfslib_get_ftp_par {  # hide: parameter: configfile, ParameterList -> par to %current
######
 my ($t,$c,$p,$o);
 my $config=$_[0];

 #  $par="VirtualHost,ServerName,Port";
 my @c=split(/\n/,$config);

 foreach $c (@c) {
         if ($c=~/<VirtualHost/) {
              $o="vftp_ip";
              $current{$o}=$c;    $current{$o}=~s/.*VirtualHost +|\>//g;    #ip
              if ($current{$o} eq "") { $current{$o}="127.0.0.1"; }
         }
         if ($c=~/Port/) {
              $o="vftp_port";
              $current{$o}=$c;    $current{$o}=~s/.*Port *//g;              #port
              if ($current{$o} eq "") { $current{$o}="21"; }
         }
         if ($c=~/<Anonymous/) {
              $o="vftp_dir";
              $current{$o}=$c;    $current{$o}=~s/.*Anonynmous *|>//g;        #dir
              if ($current{$o} eq "") { $current{$o}="/$in{'zfs'}"; }
         }
 }
 $current{'vftp_host'}="$current{'vftp_ip'}:$current{'vftp_port'}";

 #debug
  #&print_array(@c);
  #&print_hash(%current);

}

######
sub zfslib_get_vhost_par {  # hide: parameter: configfile, ParameterList -> par to %current
######
 my ($t,$c,$p,$o);
 my $config=$_[0];
 my $par=$_[1];
 if ($par eq "") {  $par="VirtualHost,DocumentRoot,ErrorLog,CustomLog,ServerAdmin,ServerName,Listen"; }
 my @par=split(/,/,$par);
 my @c=split(/\n/,$config);

 foreach $c (@c) {                       # every line of config
     foreach $p (@par) {                 # every par

         $c=~s/<|>//g;                   # ex <VirtualHost
         if (!($c=~/^\s*$p/i)) { next; } # ex DocumentRoot /path

         chomp($c);
         $c=~s/^\s*$p\s+//;


         $t="vhost_$p";

         # 'path with spaces'
         if ($c=~/\'$|\"$/) {
           $c=~s/\'|\"//g;
           if ($c=~/ /) { $c="\"$c\""; }   # mask
         }

         if ($p eq "VirtualHost") {
              #port
              if ($c=~/:/) {
                   $o="vhost_port";
                   $current{$o}=$c;
                   $current{$o}=~s/.*://;    # port only

                   $o="vhost_ip";
                   $current{$o}=$c;
                   $current{$o}=~s/:.*//;    # port only
              }

         }
         $current{$t}=$c;

     }
 }

 #debug
 # &print_array(@c);
 # &print_hash(%current);
}


 ##########################
 sub zfslib_update_ftp {      #hide: update   /opt/local/etc/proftpd.conf
 ##########################
    my (@pools,$r,@s,$f,$sn,$n,$cf,$pf,$t,$l,$c);
    # read shares, verify if ZFS is available
    # for all shares, add settings to /var/web-gui/_log/rsyncd.conf

    if ($sys{'auto_os_type'} eq "sol_s") {
          $c="/etc/proftpd.conf";                        # Solaris
    } else {
          $c="/opt/local/etc/proftpd.conf";
    }

    `sudo touch $c`;
    `sudo chmod 666 $c`;

    if (!(-f $c)) { &mess("FTP configuration $c not found"); }

    # generate new config
    $n=`cat \'$c\'`;
    $n=~s/ *<VirtualHost.*/\n#VirtualHosts: created from Menu ZFS folders, do not edit manually/s;

    # add virtual ftp from all pools
    # part 1: verify shares
    @pools=split(/\t/,$zfs{'datapools'});
    foreach my $p (@pools) {
            # read shares

             $f="/$p/.ftp_shares";

             `sudo touch $f`;
             `sudo chmod 666 $f`;

              $r=`cat \'$f\'`;

              @s=();
              @s=split(/\n/,$r);                        # www shares

              open (PF, ">$f");
              foreach my $s (@s) {
                  $l=$s;                                # keep line
                  $s=~s/,.*//;                          # remove ip and name
                  $sn=$s; $sn=~s/\//./g;                # tank/data -> tank.data
                  $cf="/$p/.ftp_shares_config_$sn";     # share config file

                  if (-d "/$s") {
                       print PF "$l\n";

                       # add to n(ew)
                       if (-f "$cf") {

                          $t=`cat $cf`;
                          if ($t eq "") { next; }

                          $n.="\n\n################# vftp: /$in{'zfs'} ##################\n";
                          $n.=$t;
                          $n.="\n";
                       }

                  } else {

                       if (-f "$cf")  { `sudo rm $cf`; }     # config file
                  }
              }
              close (PF);
    }
    # update configfile
              $n=~s/\n +/\n/gs;
              open (PF, ">$c");
                       print PF "$n\n";
              close (PF);

              #refresh
              &exe("svcadm clear proftpd");
              &exe("svcadm refresh proftpd"); sleep(2);

              print "service state: in case of problems, check menu Services-Apache<hr><pre>";
              print `svcs proftpd`;
              print "</pre>";
              sleep(2)
}

 ##########################
 sub zfslib_update_www {      #hide: update
 ##########################
    my (@pools,$r,@s,$f,$sn,$n,$cf,$pf,$t,$l);
    # read shares, verify if ZFS is available
    # for all shares, add settings to /var/web-gui/_log/rsyncd.conf

    `sudo touch /opt/local/etc/httpd/httpd-vhosts.conf`;
    `sudo chmod 666 /opt/local/etc/httpd/httpd-vhosts.conf`;

    # generate /opt/local/etc/httpd/httpd-vhosts.conf
    $n=<<EoF;
# Virtual Hosts
#
# Required modules: mod_log_config

# If you want to maintain multiple domains/hostnames on your
# machine you can setup VirtualHost containers for them. Most configurations
# use only name-based virtual hosts so the server doesn't need to worry about
# IP addresses. This is indicated by the asterisks in the directives below.
#
# Please see the documentation at
# <URL:http://httpd.apache.org/docs/2.4/vhosts/>
# for further details before you try to setup virtual hosts.
#
# You may use the command line option '-S' to verify your virtual host
# configuration.

#
# VirtualHost example:
# Almost any Apache directive may go into a VirtualHost container.
# The first VirtualHost section is used for all requests that do not
# match a ServerName or ServerAlias in any <VirtualHost> block.
#
# This file is autogenerated from zfslib.pl module zfslib_update_www
# do not edit manually, edit in napp-it menu zfs folder
#

EoF

    # part 1: verify shares
    @pools=split(/\t/,$zfs{'datapools'});
    foreach my $p (@pools) {
            # read shares

             $f="/$p/.www_shares";

             `sudo touch $f`;
             `sudo chmod 666 $f`;

              $r=`cat \'$f\'`;

              @s=();
              @s=split(/\n/,$r);                        # www shares

              open (PF, ">$f");
              foreach my $s (@s) {
                  $l=$s;                                # keep line
                  $s=~s/,.*//;                          # remove ip and name
                  $sn=$s; $sn=~s/\//./g;                # tank/data -> tank.data
                  $cf="/$p/.www_shares_config_$sn";     # share config file

                  if (-d "/$s") {
                       print PF "$l\n";

                       # add to n(ew)
                       if (-f "$cf") {

                          $t=`cat $cf`;
                          if ($t eq "") { next; }

                          $n.="\n\n################# vhost: /$in{'zfs'} ##################\n";
                          $n.=$t;
                          $n.="\n";
                       }

                  } else {

                       if (-f "$cf")  { `sudo rm $cf`; }     # config file
                  }
              }
              close (PF);
        }

        # update configfile
              $n=~s/\n +/\n/gs;
              open (PF, ">/opt/local/etc/httpd/httpd-vhosts.conf");
                       print PF "$n\n";
              close (PF);

              #refresh
              if ($sys{'auto_os_type'} eq "sol_s") {
                   $t="apache2";                       # Solaris
              } else {
                   $t="apache";
              }
              &exe("svcadm clear $t");
              &exe("svcadm refresh $t"); sleep(2);

              print "service state: in case of problems, check menu Services-Apache<hr><pre>";
              print `svcs $t`;
              print "</pre>";
              sleep(2);
 }

 ######################
 sub zfslib_change_property {                 #hide:
 #######################    set zfs details
 my ($e,$r,$s,$t,@l,$l,@n,$i,$o,$p,$z,$f,$ok,$pool,$k);


   ##### execute lock
   ##################
   if ($in{'prop'} eq "lock") {
        print "please wait...\n";


        #remember smb
        if ($in{'remember'} eq "last") {

            # get smb share value
            if ($sys{'auto_os'} eq "Solaris 11") {
                $r=`sudo /sbin/zfs get -H -o name,value share $in{'zfs'} | grep prot=smb`; chomp($r); # name=geheim6,path=/tank/geheim6,prot=smb,guestok=true
            } else {
                $r=`sudo /sbin/zfs get -H -o value sharesmb $in{'zfs'}`;
            }
            chomp($r);

            $r=`sudo /sbin/zfs set enc_smbshare:=$r $in{'zfs'}`;
        }

        #unshare
        if ($in{'remember'} eq "off") {
            $r=`sudo /sbin/zfs set sharesmb=off $in{'zfs'}`;
            $r=&exe("zfs inherit -r enc_smbshare: $in{'zfs'}");
            $r=&exe("zfs inherit -r sharesmb $in{'zfs'}");
        }

        if ($sys{'auto_os'} =~/Solaris/) {   # Solaris

            ### martingo 20180306 >>> Solaris Keys sicherheitshalber rekursiv entladen; vermutlich nicht notwendig, schadet aber nicht
            # ex $r= &exe("$sys{'sbinpath'}zfs key -u -f $in{'zfs'}");

            $r=`sudo /sbin/zfs key -u -r -f $in{'zfs'}`;
            if ($r ne "") {  &mess($r); }
            ### <<<

        } else { # Open-ZFS

             $r=`sudo /sbin/zfs unmount -f $in{'zfs'}`;      # key unload require unmounted fs
             $r=`sudo /sbin/zfs unload-key -r $in{'zfs'}`;
             if (!($r =~/ success/)) {  &mess($r); }

        }

        #if ($in{'smbrefresh'} eq "yes") {  # opt SMB refresh
        #  print "<br>refresh services";
        #  &zfslib_refresh_svc;
        #}
   }


   ##### unlock
   ###########
   if ($in{'prop'} eq "unlock") {
        print "please wait ...\n";

        if (($in{'call'} =~/agent|boot/) && ($in{'pass'}=~/[LW]\d:[LW]\d/)) { print "Split=$in{'pass'}"; }

        $k=$in{'pass'};
        if ($in{'pass'}=~/[LW]\d:[LW]\d/) {  # ex L1:L2
             $in{'pass'}=~s/[^LW\d:]//g;
             require "/var/web-gui/data/napp-it/zfsos/_lib/illumos/keyserverlib.pl";
             $k=&key_get_lw($in{'zfs'},$in{'pass'});

        }

        if (!($k=~/\w+/)) {
          if ($in{'call'} eq "agent") { return "error, missing key"; }
          &mess("invalid or missing key");
        }

        #debug
        if ($in{'call'} eq "agent")   {            # auto/userunlock
           print "\ncall=$in{'call'}\n";
           $t=&key_get_smbak($k);   # text only
           print "\n\n#######user smbkey: $in{'user_smbkey'}\nsmbkey: $t\n######\n";   #$in{'user_smbkey'} from auto_lock.pl line 309
           if ($t ne "$in{'user_smbkey'}") {
               print "####### wrong smbkey in =switcher.move\n";
               return("error, wrong or missing smbkey");
           }
        }

        if ($sys{'auto_os'} eq "Solaris 11") {
             # not prompt
             $r=`sudo /sbin/zfs get -H -o value keysource $in{'zfs'}`;
             chomp($r);

             if ($r=~/,prompt/) {
                #
                use Expect;
                my $exp; #->raw_pty(1);
     ### martingo 20180306 >>> mount hat die Option -r fuer rekursives mounten nicht; key fuehrt ebenfalls den mount durch (mit -r inklusive aller UnterZFS)
                #$exp = Expect->spawn("sudo $sys{'sbinpath'}zfs mount $in{'zfs'}") or die "Couldn't mount: $!\n";
                $exp = Expect->spawn("sudo /sbin/zfs key -l -r $in{'zfs'}") or die "Couldn't mount: $!\n";
     ### <<<

                $exp->expect(1,
                        [ 'Enter' =>
                                 sub {
                                     $exp->send("$k\r");
                                     print "<br><br>\n";
                                     exp_continue;
                                 }
                        ]
                 );
                 $exp->soft_close();


             } else {
                 print "<br>";
                 print `sudo /sbin/zfs key -l $in{'zfs'} 2>&1`; # unlock
             }

             # on success
             $r=`sudo /sbin/zfs get -H -o value keystatus $in{'zfs'}`; chomp($r);
             if ($r eq "available") {
                  #

                  # restore smb shares?
                  $r=`sudo /sbin/zfs get -H -o name,value share $in{'zfs'} | grep prot=smb`; chomp($r); # name=geheim6,path=/tank/geheim6,prot=smb,guestok=true

                  if ($r eq "") {  # no share currently activated
                       $r=`sudo /sbin/zfs get -H -o value enc_smbshare: $in{'zfs'}`; chomp($r);

                       if ($r=~/[a-z]/) {                      # sharesmb=name=data,guestok=true
                              #reshare
                              $t=$r;
                              $t=~s/.*name=//; $t=~s/\+.*//;   # keep name

                              if ($t ne "off") {              # do not share with name=off
                                #
                                if ($t eq "on") {
                                  $t="=on";
                                } else {
                                  $t="=name=$t";
                                }

                                $t="share$t,path=/$in{'zfs'},prot=smb";
                                if ($r=~/\+guest/) { $t.=',guestok=true'; }
                                if ($r=~/\+abe/)   { $t.=',abe=true'; }

                                $t.=" $in{'zfs'}";

                                $r=`sudo /sbin/zfs set $t`;
                                $r=`sudo /sbin/zfs set sharesmb=on $in{'zfs'}`;
                              }
                       }
                  }

                  if ($in{'pass'}=~/^[LW]\d:[LW]\d$/) {  # ex L1:L2
                     $r=`sudo /sbin/zfs set enc_split:=$in{'pass'} $in{'zfs'}`; # remember last enc_split:
                  }  else {
                     if ($in{'call'} =~/agent|boot/) { return "error, split info missing"; }
                  }


              } else {
                 if ($in{'call'} =~/agent|boot/) { return "error, wrong key"; }
                 &mess("wrong key, unlock not possible");
              }

        } else { # Illumos

             $r=`sudo /sbin/zfs get -H -o value keylocation $in{'zfs'}`;
             chomp($r);

             if ($r=~/prompt/) {
                #
                use Expect;
                my $exp; #->raw_pty(1);
     ### martingo 20180306 >>> mount hat die Option -r für rekursives mounten nicht; key führt ebenfalls den mount durch (mit -r inklusive aller UnterZFS)
                #$exp = Expect->spawn("sudo $sys{'sbinpath'}zfs mount $in{'zfs'}") or die "Couldn't mount: $!\n";
                $exp = Expect->spawn("sudo /sbin/zfs load-key $in{'zfs'}") or die "Couldn't load key: $!\n";
     ### <<<

                $exp->expect(1,
                        [ 'Enter' =>
                                 sub {
                                     $exp->send("$k\r");
                                     print "<br><br>\n";
                                     exp_continue;
                                 }
                        ]
                 );
                 $exp->soft_close();

                 print `sudo /sbin/zfs mount $in{'zfs'} 2>&1`; # mount, key is available

             } else {      # file
                 print "<br>";
                 print `sudo /sbin/zfs load-key $in{'zfs'} 2>&1`; # unlock and mount, key is available as file
             }


             # on success (Ill)
             $r=`sudo /sbin/zfs get -H -o value keystatus $in{'zfs'}`; chomp($r);
             if ($r eq "available") {

               #on success, mount subfilesystems 7.10.2021, not mounted with smb=off on parent
               $r=`sudo /sbin/zfs list -H -o name -t filesystem | grep $in{'zfs'}/`; chomp($r);
               @t=split(/\n/,$r);
               foreach my $t (@t) {
                    print `sudo /sbin/zfs mount $t 2>&1`; # mount sub, key is available
                    print "mount $t<br>";
               }
               #&print_array(@t);

               ###
               $r=`sudo /sbin/zfs set sharesmb=off $in{'zfs'}`;    #  off as default


               if ($in{'smbrefresh'} eq "off") {

                  $r=&exe("zfs inherit -r sharesmb $in{'zfs'}");   # inherit off

               } else {         #last

                  # restore smb shares?

                  $r=`sudo /sbin/zfs get -H -o value enc_smbshare: $in{'zfs'}`; chomp($r);

                  if ($r=~/[a-z]/) {                      # sharesmb=name=data,guestok=true
                              #reshare
                              $t=$r;
                              $t=~s/.*name=//; $t=~s/,.*//;  #keep name

                              if ($t ne "off") {           # do not share with name=off
                                #
                                if ($t eq "on") {
                                  $t="on";
                                } else {
                                  $t="name=$t";
                                }

                                if ($r=~/guest/) { $t.=',guestok=true'; }
                                if ($r=~/abe/)   { $t.=',abe=true'; }

                                $r=`sudo /sbin/zfs set sharesmb=$t $in{'zfs'}`;
#&mess("zfs set sharesmb=$t $in{'zfs'}<br>$r");
                              }
                  }

                  # (re)set local zfs prop (ex missing after replication)
                  $t=&key_get_smbak($in{'pass'});
                  $r=`sudo /sbin/zfs set enc_smbak:=$t $in{'zfs'}`;           # enc_smbak:

                  if ($in{'pass'}=~/^[LW]\d:[LW]\d$/) {  # ex L1:L2
                    $r=`sudo /sbin/zfs set enc_split:=$in{'pass'} $in{'zfs'}`; # remember last enc_split:
                  } else {
                     if ($in{'call'} =~/agent|boot/) { return "error, split info missing"; }
                  }
               }

              } else {
                 if ($in{'call'} =~/agent|boot/) { return "error, wrong key"; }
                 &mess("wrong key, unlock not possible");
              }
        }
   }



   ##### s3 minIO ##
   #################
   if ($in{'prop'} =~/shareS3/) {

     if ($in{'share'} eq "off") {
          `sudo pkill -f "minio server /$in{'zfs'}/S3_data"`;
          `sudo /sbin/zfs set s3_auto:=$in{'auto'} $in{'zfs'}`;
     } else {

        $f="/$in{'zfs'}/S3_data";
        $r=`sudo test -d "$f" && echo 1 || echo 2`;  chomp($r);
        if ($r eq "2") {
            print `sudo mkdir $f`;
            print `sudo /usr/bin/chmod A=user:root:full_set:file_inherit/dir_inherit:allow $f`."<br>";  # root full
            print `sudo /usr/bin/chmod A1+everyone\@:modify_set:file_inherit/dir_inherit:allow $f`;     # everyone modify
        }

        $f="/$in{'zfs'}/S3_data/.minio.sys";  # s3 encryped settings
        # clear .minio.sys
        if ($in{'s3clear'} eq "yes") {
             $r=`sudo test -d "$f" && echo 1 || echo 2`;  chomp($r);
             if ($r eq "1") {
               print `sudo rm -r $f`;
            }
        }

        # check .minio.sys
        $r=`sudo test -d "$f" && echo 1 || echo 2`;  chomp($r);
        if ($r eq "2") {
            print `sudo mkdir $f`;
        }
        print `sudo /usr/bin/chmod A=user:root:full_set:file_inherit/dir_inherit:allow $f`."<br>";  # root only (always)


        # get folder S3_config
        if ($in{'s3config'} eq "") { $in{'s3config'}=$in{'zfs'}; }
        if ( $in{'s3config'} eq $in{'zfs'} ) {       # config in zfs folder
           $f="/$in{'zfs'}/S3_config";
        } else {                                      # config in folder from about > settings
           $t=$in{'zfs'}; $t=~s/\//~/g;
           $f="/$in{'s3config'}/S3_keydata/$t/S3_config";

           #check folders
           $r=`sudo test -d "/$in{'s3config'}" && echo 1 || echo 2`;  chomp($r);
           if ($r eq "2") { &mess("S3 keydate folder not found: /$in{'s3config'}"); }

           $r=`sudo test -d "/$in{'s3config'}/S3_keydata" && echo 1 || echo 2`;  chomp($r);
           if ($r eq "2") { print `sudo mkdir /$in{'s3config'}/S3_keydata`;    }

           $r=`sudo test -d "/$in{'s3config'}/S3_keydata/$t" && echo 1 || echo 2`;  chomp($r);
           if ($r eq "2") { print `sudo mkdir /$in{'s3config'}/S3_keydata/$t`;    }

        }
        # $f is configfolder
#&print_hash(%in);
#&mess($f);
        # check folder S3_config
        $r=`sudo test -d "$f" && echo 1 || echo 2`;  chomp($r);
        if ($r eq "2") {
            print `sudo mkdir $f`;
        }

        if ( $in{'s3config'} eq $in{'zfs'} ) {
          print `sudo /usr/bin/chmod A=user:root:full_set:file_inherit/dir_inherit:allow $f`;         # root only (always)
        } else {
          print `sudo /usr/bin/chmod A=user:root:full_set:file_inherit/dir_inherit:allow /$in{'s3config'}/S3_keydata`;         # root only (always)
        }

       # $in{'alt_server'}=~s/\r//gs;   # keep \n only    # obsolete
       # $in{'alt_server'}=~s/\s+$//s;

        # check key and secret
        $in{'key'}=~s/\s+//g;
        $in{'secret'}=~s/\s+//g;
        if (($in{'secret'} eq "") || ($in{'key'} eq "")) {  &mess("KEY (min 3 char) and SECRET (min 8 char) are required"); }
        if (length($in{'secret'}) < 8) {  &mess("ROOT pw length is 8 char minimum"); }
        if (length($in{'key'}) < 3)    {  &mess("ROOT user length is 3 char minimum"); }

        if ($in{'secret_old'} eq "") { $in{'secret_old'}=$in{'secret'} }    # create new
        if ($in{'key_old'} eq "")    { $in{'key_old'}=$in{'key'} }          # create new

        # check address
        $in{'address'}=~s/ +//g;
        if (!($in{'address'}=~/:\d+$/))  {  &mess("Address:port required like :9000 or 192.168.1.1:9000"); }

        # nwe apr 2021, console port
        @tmp=split(/:/,$in{'address'});
        $tmp[1]=$tmp[1]-1000;                    # ex :8000

        $in{'console'}=join(":",@tmp);

        # export key and secret  (new apr 2021)
        $e="export MINIO_ROOT_USER=$in{'key'}; export MINIO_ROOT_PASSWORD=$in{'secret'};";

        # export region
        if ($in{'region'} ne "") {
           $e.=" export MINIO_REGION=$in{'region'};";
        }

       # obsolet apr 2021
       # change creditentials        # ??
       # if ($in{'s3clear'} ne "yes") {
       #      if (($in{'key'} ne $in{'key_old'}) || ($in{'secret'} ne $in{'secret_old'})) {
       #       $e.=" export MINIO_ACCESS_KEY_OLD=$in{'key_old'}; export MINIO_SECRET_KEY_OLD=$in{'secret_old'};";        #obsolet
       #        &perm_write("$f/key_old.txt","$in{'key_old'}",700,'add');
       #        &perm_write("$f/secret_old.txt","$in{'secret_old'}",700,'add');
       #      }
       # }

        #export
        # export MINIO_ACCESS_KEY=mini2;       # depre, ROOT_USER
        # export MINIO_SECRET_KEY=minio134;    # depre, ROOT_PASSWORD
        # export MINIO_ACCESS_KEY_OLD=minio;   #depr
        # export MINIO_SECRET_KEY_OLD=minio133;# depr
        # export MINIO_REGION_NAME=

        #cert  S
       # $r=`sudo test -f "/var/web-gui/_log/minio/public.crt" && echo 1 || echo 2`;  chomp($r);       # old
        $r=`sudo test -f "/var/web-gui/_log/minio/certs/public.crt" && echo 1 || echo 2`;  chomp($r);  # new apr 21
        if ($r eq "1") {
           #$k=" -S /var/web-gui/_log/minio";       # must be tested
           $k=" --certs-dir /var/web-gui/_log/minio/certs";       # new
        } else {
           $k="";
        }

        if ($in{'alt_server'} ne "") {
             @t=split(/\n/,$in{'alt_server'});
             foreach my $t (@t) {
               $t=~s/ +$//;
               if (!($t=~/;$/)) { $t.="; ";}
             }
             $s=join("",@t);  # single line
             $s=~s/; *$//;
             $r="$e /opt/ooce/minio/bin/minio server $s 2> $f/log.txt \&";              # start in background
        } else {
             $in{'server'}=~s/; *$//;

             # add port and console
             $r="$e /opt/ooce/minio/bin/minio server $in{'server'} --address $in{'address'} --console-address $in{'console'}$k 2> $f/log.txt;";  # start in background
        }

        #nice
        if ($in{'nice'} eq "yes") {
           $r=~s/\/opt\/ooce/nice \/opt\/ooce/;
        }
#&mess($r);
        # save values file data perm
        &perm_write("$f/key.txt","$in{'key'}",700);
        &perm_write("$f/secret.txt","$in{'secret'}",700);
        &perm_write("$f/address.txt","$in{'address'}",700);
        &perm_write("$f/region.txt","$in{'region'}",700);

        #set property
        `sudo /sbin/zfs set s3_addr:=$in{'address'} $in{'zfs'}`;
        `sudo /sbin/zfs set s3_auto:=$in{'auto'} $in{'zfs'}`;      # autostart
        `sudo /sbin/zfs set s3_region:=$in{'region'} $in{'zfs'}`;
        `sudo /sbin/zfs set s3_config:=$in{'s3config'} $in{'zfs'}`;

#&print_hash(%in);
#&mess("$s<hr>$k");
#&mess($r);

        # start minio session via taskserver.sh
        $t=time();
        `sudo echo "$r" > /tmp/minio_$t.sh`;

        # `sudo /usr/bin/at -s -f /tmp/minio_$t.sh now +1 min`;  at now has problems with several minio sessions (put the in queue only)

        `sudo cp /tmp/minio_$t.sh $f/minio.sh`;   # just for debugging
        `sudo cp /tmp/minio_$t.sh /var/web-gui/_log/tmp/now.sh`;     # start via taskserver.sh

        `sudo rm /tmp/minio_$t.sh`;                                  # del
        #
      }
   }

   ##### special small blocks
   if ($in{'prop'} eq "SSB") {
        $r= &exe("zfs set special_small_blocks=$in{'ssb'} $in{'zfs'}");
        #&mess("zfs set special_small_blocks=$in{'ssb'} $in{'zfs'}<br>$r");
   }


   ## iSCSI
    if ($in{'prop'} eq "share_iscsi") {
         require "/var/web-gui/data/napp-it/zfsos/_lib/illumos/zfslib-iscsi.pl";
         &zfslib_iscsi_modify;
   }

  #ftp share
  if ($in{'prop'} eq "share_ftp") {
  ################################

       # debug
       # &print_hash(%in);

       $pool=$in{'zfs'}; $pool=~s/\/.*//;  # pool
       $z=$in{'zfs'}; $z=~s/\//./g;        # tank/data -> tank.data (filename)

       #always write config
       $in{'settings'}=~s/\r//gs;          # parse in
       $in{'settings'}=~s/\s+$//s;
       if ($in{'settings'} eq "") {
            $in{'settings'}=&zfslib_get_default_ftp;     #default
       }

       # new ip and port
       $p="21";     # default
       if ($in{'force_ip'}=~/\w+/) {
           $in{'select_ip'}=$in{'force_ip'};
       }

       #port
       if ($in{'select_ip'}=~/:\d/) {
          $p=$in{'select_ip'}; $p=~s/\s//g; $p=~s/.*://;
       }

       #ip
       $i=$in{'select_ip'}; $i=~s/\s//g; $i=~s/:.*//;

       @n=();
       @n=split(/\n/,$in{'settings'});
       foreach my $t (@n) {
           if ($t=~/VirtualHost/) {   $t=~s/<VirtualHost.*/<VirtualHost $i>/;   }      # ip
           if ($t=~/Port/)        {   $t=~s/\d+/$p/;  }                                # port
           if ($t=~/<Anonymous/)  {   $t=~s/Anonymous.*/Anonymous $in{'wwwroot'}>/; }  # dir
       }

       # opt create wwwroot
       if ($in{'wwwroot'}=~/_wwwroot/) {
              # create zfs/_wwwroot
              if (!(-d "$in{'wwwroot'}"))         { `sudo mkdir $in{'wwwroot'}`; }
       }

       $f="/$pool/.ftp_shares_config_$z";      # ftp/zfs config
       $t=join("\n",@n);

       if (!(-f "$f")) { `sudo touch $f`; }
       `sudo chmod 666 $f`;
       open (PF, ">$f");
          print PF "$t\n";
       close (PF);

       # write shares
       if ($in{'share_action'} eq "on")  {
             $f="/$pool/.ftp_shares";
             if (!(-f "$f"))  { `sudo touch $f`; }
             `sudo chmod 666 $f`;
             $t=`cat $f`;
             @t=(); @t=split(/\n/,$t);

             $t="1";
             # edit

             open (PF, ">$f");
                foreach my $l (@t) {
                  if ($l =~/^$in{'zfs'},/) {   # found, edit current settings
                      print PF "$in{'zfs'},$i:$p\n";
                      $t="0";
                  }  else {  # keep
                      print PF "$l\n";
                  }
                }

               # add new
               if ($t) {
                  print PF "$in{'zfs'},$i:$p\n";
               }
             close (PF);
       }

       # off: remove zfs to rsync_shares
       if ($in{'share_action'} eq "off") {

           $f="/$pool/.ftp_shares";
           if (!(-f "$f"))  { `sudo touch $f`; }
           `sudo chmod 666 $f`;
            $t=`cat $f`;
            @t=(); @t=split(/\n/,$t);
            open (PF, ">$f");
                foreach my $l (@t) {
                   if ($l =~/^$in{'zfs'},/) { next; } # remove share
                   print PF "$l\n";
                }
            close (PF);
       }
       &zfslib_update_ftp;  # update .config
  }


  #www share
  ###########
  if ($in{'prop'} eq "share_www") {
  ################################

       # debug
       # &print_hash(%in);

        $p=$in{'zfs'}; $p=~s/\/.*//;  # pool
        $z=$in{'zfs'}; $z=~s/\//./g;  # tank/data -> tank.data (filename)

       #always write config
       $in{'settings'}=~s/\r//gs;     # parse in
       $in{'settings'}=~s/\s+$//s;
       if ($in{'settings'} eq "") {
            $in{'settings'}=&zfslib_get_default_vhost;     #default
       }

       # remove ip, log and document root (force to select values)
       @t=();
       @t=split(/\n/,$in{'settings'});

       @n=();    # set default values
       if ($in{'force_ip'} ne "") {
           if (($in{'force_ip'}=~/:.+:/) || ($in{'force_ip'}=~/^\d+\.\d+\.\d+\.\d+/))  {
                # ok
           } else {
                &mess("Please enter an ip:port value");            # name based vhost not supported
           }
       } else {
           $in{'force_ip'}=$in{'select_ip'};                       # use this
       }

       # create new VirtualHost Config
       push(@n,"Listen $in{'force_ip'}");
       push(@n,"<VirtualHost $in{'select_ip'}>");
       push(@n,"  DocumentRoot \"$in{'wwwroot'}\"");


       if ($in{'wwwroot'}=~/_wwwroot/) {
              # create _wwwlog either under zfs or zfs/_wwwroot
              if (!(-d "$in{'wwwroot'}"))         { `sudo mkdir $in{'wwwroot'}`; }
              if (!(-d "$in{'wwwroot'}/_wwwlog")) { `sudo mkdir $in{'wwwroot'}/_wwwlog`; }
              push(@n,"  ErrorLog \"$in{'wwwroot'}/_wwwlog/error_log\"");
              push(@n,"  CustomLog \"$in{'wwwroot'}/_wwwlog/access_log\" common");
              if (!((-f "/$in{'zfs'}/_wwwroot/index.html") || (-f "/$in{'zfs'}/_wwwroot/index.php"))) {
                  `echo '<html><body>settings:$in{'zfs'}<br><?php phpinfo();?>' > /$in{'zfs'}/_wwwroot/index.php`;
              }
      } else {
              # create _wwwlog either under zfs or zfs/_wwwroot
              if (!(-d "/$in{'zfs'}/_wwwlog")) { `sudo mkdir /$in{'zfs'}/_wwwlog`; }
              if (!((-f "/$in{'zfs'}/index.html") || (-f "/$in{'zfs'}/index.php"))) {
                  `echo '<html><body>settings:$in{'zfs'}<br><?php phpinfo();?>' > /$in{'zfs'}/_wwwroot/index.php`;
              }

              push(@n,"  ErrorLog \"/$in{'zfs'}/_wwwlog/error_log\"");
              push(@n,"  CustomLog \"/$in{'zfs'}/_wwwlog/access_log\" common");
      }
      push(@n,"  ServerName $in{'ServerName'}");
      push(@n,"  ServerAdmin $in{'ServerAdmin'}");


      # add settings
      foreach my $t (@t) {
              if ($t=~/Listen /i)         { next; }
              if ($t=~/VirtualHost/i)     { next; }
              if ($t=~/DocumentRoot/i)    { next; }
              if ($t=~/ServerName|ServerAdmin/i)         { next; }
              if ($t=~/Log /i)            { next; }
              if (!($t=~/\w+/))           { next; }


              $t=~s/^ +//;
              if ($t=~/^</)   {
                     push(@n,"");
                     push(@n,"$t");

              } else {
                      push(@n,"  $t");
              }
      }

      #end
      push(@n,"</VirtualHost>");

      #debug
      #&print_array(@n);

       $f="/$p/.www_shares_config_$z";      # zfs config
       $t=join("\n",@n);

       # replace =document_root=
       $t=~s/=document_root=/$in{wwwroot}/gs;

       if (!(-f "$f")) { `sudo touch $f`; }
       `sudo chmod 666 $f`;
       open (PF, ">$f");
         print PF "$t\n";
       close (PF);


       # on: write zfs to www_shares
       if ($in{'share_action'} eq "on")  {
             $f="/$p/.www_shares";
             if (!(-f "$f"))  { `sudo touch $f`; }
             `sudo chmod 666 $f`;
             $t=`cat $f`;
             @t=(); @t=split(/\n/,$t);

             $t="1";
             # edit

#&print_array(@t);
             open (PF, ">$f");
                foreach my $l (@t) {
                  if ($l =~/^$in{'zfs'},/) {   # found, edit current settings
                      print PF "$in{'zfs'},$in{'force_ip'},$in{'ServerName'}\n";
                      $t="0";
                  }  else {  # keep
                      print PF "$l\n";
                  }
                }

               # add new
               if ($t) {
                  print PF "$in{'zfs'},$in{'force_ip'},$in{'ServerName'}\n";
               }
             close (PF);
       }
  #$t=`cat $f`;
  #&mess($t);


       # off: remove zfs to rsync_shares
       if ($in{'share_action'} eq "off") {

           $f="/$p/.www_shares";
           if (!(-f "$f"))  { `sudo touch $f`; }
           `sudo chmod 666 $f`;
            $t=`cat $f`;
            @t=(); @t=split(/\n/,$t);
            open (PF, ">$f");
                foreach my $l (@t) {
                   if ($l =~/^$in{'zfs'},/) { next; } # remove share
                   print PF "$l\n";
                }
            close (PF);
       }

       &zfslib_update_www;  # update .config

  } #/www

  ############
  #rsync share
  ############
  if ($in{'prop'} eq "share_rsync") {

       # debug
       # &print_hash(%in);

        $p=$in{'zfs'}; $p=~s/\/.*//;  # pool
        $z=$in{'zfs'}; $z=~s/\//./g;  # tank/data -> tank.data (filename)

       #always write config and scrt
       $in{'settings'}=~s/\r//gs;
       $in{'secrets'}=~s/\r//gs;

       # setting must contain path = zfs
       if (!($in{'settings'}=~/\bpath *= *\/$in{'zfs'}\b/s)) { &mess("setting must contain path = /$in{'zfs'} or path = /$in{'zfs'}/somefolder"); }

       $f="/$p/.rsyncd_shares_config_$z";
       `sudo touch $f`;
       `sudo chmod 666 $f`;

       open (PF, ">$f");
         print PF "[$z]\n";              # modulname b7/test -> b1.test
         print PF "$in{'settings'}";
       close (PF);

       $f="/$p/.rsyncd_shares_scrt_$z";
       `sudo touch $f`;
       `sudo chmod 666 $f`;
       open (PF, ">$f");
         print PF "$in{'secrets'}";
       close (PF);
       `sudo chmod 600 $f`;

       # on: write zfs to rsync_shares
       if ($in{'share_action'} eq "on") {
             $f="/$p/.rsyncd_shares";
             `sudo touch $f`;
             `sudo chmod 666 $f`;
             $t=`cat $f`;
             @t=(); @t=split(/\n/,$t);
             $ok="1";
             foreach my $l (@t) {
                  if ($l eq $in{'zfs'}) { $ok="0"; last; } # found
             }
             if ($ok) {      # add
                  open (PF, ">>$f");
                      print PF "$in{'zfs'}\n";
                  close (PF);
             }
       }

       # off: remove zfs to rsync_shares
       if ($in{'share_action'} eq "off") {
           $f="/$p/.rsyncd_shares";
           `sudo touch $f`;
           `sudo chmod 666 $f`;
            $t=`cat $f`;
            @t=(); @t=split(/\n/,$t);
            open (PF, ">$f");
                foreach my $l (@t) {
                   if ($l eq $in{'zfs'}) { next; }
                   print PF "$l\n";
                }
            close (PF);
       }
       &zfslib_update_rsync;  # update .config
  }

  ##########
  #afp-share
  ##########
  if ($in{'prop'} eq "shareafp") {
        $in{'text'}=~s/[^a-zA-Z0-9\-\_]//g;
        if ($in{'text'} eq "") {  &mess('Please enter share-name'); }
        $t="";

        if ($in{'text'} eq "off") {
             if ($sys{'netatalk3'} ne "") { #v3
                          # remove entry from afp.conf
                          $r=&exe("cat $sys{'netatalk3'}");
                          @l=(); @l=split(/\n\[/,$r);              # opt include commented entries

                          @n=();
                          foreach $l (@l) {
                            if (!($l=~/path *= *\/$in{'zfs'}\n/s)) { push(@n,$l); }
                          }

                          #&print_hash(%in);
                          #&print_array(@n);

                          #save new file @n
                          &exe("chmod 666 $sys{'netatalk3'}");
                          $t=join("\n\[",@n);

                          $t=~s/\n+/\n/gs;
                          if (!($t=~/\n$/s)) { $t.="\n"; }    # newline at the end needed or service maintenance error


                          open (PF, ">$sys{'netatalk3'}") || &error("(944) write-error: $file");
                                print PF $t;
                          close (PF);


             } else { #v2
                          &mess('netatalk 2 not supported');
             }

        } else {

                # get volsizelimit
                my $max="";
                if ($in{'max'} ne "unlimited") {
                        if ($in{'max'} eq "10GB")   { $max=" volsizelimit:10000"; }
                        if ($in{'max'} eq "100GB")  { $max=" volsizelimit:100000"; }
                        if ($in{'max'} eq "300GB")  { $max=" volsizelimit:300000"; }
                        if ($in{'max'} eq "600GB")  { $max=" volsizelimit:600000"; }
                        if ($in{'max'} eq "1TB")    { $max=" volsizelimit:1000000"; }
                }

                # share it
                  # use UUID:Tyny in admin.pl - > not allowed here otherwise error in auto.pl
                                my $uuid=""; # create_UUID_as_string(UUID_V1);                #generate UUID

                if ($sys{'netatalk3'} ne "") { #v3

                                #debug
                                #&print_hash(%in);

                                $in{'text'}=~s/\s//g;
                                my $conf="\n[$in{'text'}]\n";     # output string to afp.conf
                                $conf.="path = /$in{'zfs'}\n";

                                if ($in{'max'} ne "unlimited") {
                                    $max=~s/volsizelimit://;
                                    $conf.="vol size limit = $max\n";
                                }

                                #share TimeMachine
                                if ($in{'tm'} eq "1") {
                                    $conf.="time machine = yes\n";

                                        # new avahi file
                                        $r=&exe("cat /etc/avahi/services/adisk.service");        #read it

                                        # ex <txt-record>dk2=adVN=software,adVU=54bc0654-c071-4092-b43c-781c1164bd91</txt-record>
                                        #                  |= lf nr.                          |=lf2 nr + 000 (vierstellig z.b. 1000 2000)

                                        my $rec="<txt-record>dk0=adVF=0xa1,adVN=$in{'text'},adVU=$uuid</txt-record>";
                                        $r=~s/<\/service>/   $rec\n <\/service>/;


                                        #resort entries dk0..dkn
                                        @n=();
                                        @n=split(/\n/,$r);

                                        # mark zfs as supporting Time Machine needed?
                                                #&exe("touch /$in{'zfs'}/.com.apple.timemachine.supported");
                                                #&exe("chmod 644 /$in{'zfs'}/.com.apple.timemachine.supported");


                                }  else {

                                        # new avahi file
                                        $r=&exe("cat /etc/avahi/services/adisk.service");        #read it

                                        # ex <txt-record>dk2=adVN=software,adVU=54bc0654-c071-4092-b43c-781c1164bd91</txt-record>
                                        #                  |= lf nr.                          |=lf2 nr + 000 (vierstellig z.b. 1000 2000)

                                        my $rec="<txt-record>dk0=adVN=$in{'text'},adVU=$uuid</txt-record>"; #
                                        $r=~s/<\/service>/   $rec\n <\/service>/;

                                        #resort entries dk0..dkn
                                        @n=();
                                        @n=split(/\n/,$r);
                                }

                                #read only
                                if ($in{'ro'} ne "") {
                                    $conf.="read only = yes\n";
                                }

                                #volume password
                                if ($in{'vpw'} ne "") {
                                    $conf.="password = $in{'vpw'}\n";
                                }

                                #veto
                                if ($in{'veto'} ne "") {
                                    $conf.="veto files = $in{'vpw'}\n";
                                }
                                #allow user
                                if ($in{'all'} ne "") {
                                    $conf.="valid users = $in{'all'}\n";
                                }
                                #deny user
                                if ($in{'den'} ne "") {
                                    $conf.="invalid users = $in{'den'}\n";
                                }
                                #ro user
                                if ($in{'uro'} ne "") {
                                    $conf.="rolist = $in{'uro'}\n";
                                }
                                #rw user
                                if ($in{'urw'} ne "") {
                                    $conf.="rwlist = $in{'urw'}\n";
                                }

                                &exe("chmod 666 $sys{'netatalk3'}");
                                open (PF, ">>$sys{'netatalk3'}") || &error("(1087) write-error: ");
                                      print PF "\n$conf\n";
                                close (PF);
                                &exe("chmod 644 $sys{'netatalk3'}");


                } else { # v2

                               &mess('netatalk 2 not supported');
                } # end v2
        }

        if ($sys{'netatalk3'} ne "") {
                $r=&exe("svcadm refresh svc:/network/dns/multicast:default");
                $r=&exe("svcadm refresh svc:/system/avahi-bridge-dsd:default");
        } else {
                &mess('netatalk 2 not supported');
        }

        #set aclmode and aclinherit to passhrough
        # &mess($in{'zfs'});
        $r=&exe("$sys{'sbinpath'}zfs set aclmode=passthrough $in{'zfs'}");
        $r=&exe("$sys{'sbinpath'}zfs set aclinherit=passthrough $in{'zfs'}");


        $in{'prop'}="";
        $in{'answered'}="";
        $in{'action'}="";
        $in{'list'}="";
       # &reload;

  }


#&print_hash(%in);
 #### folder acl: smb-optimized,os-default
  if ($in{'prop'} eq "folder-acl") {

    $r.= &exe("chmod A- /$in{'zfs'}");                                      # remove all
    $r.= &exe("$sys{'sbinpath'}zfs set aclinherit=passthrough $in{'zfs'}");       # aclinherit
    $r.= &exe("chmod A=user:root:full_set:file_inherit/dir_inherit:allow /$in{'zfs'}");   # root=full

    if (!($in{'zfs'}=~/\//)) {               # pool itself    readx= r-x---a-R-c---
       $r.= &exe("chmod A+owner\@:full_set:file_inherit/dir_inherit:allow /$in{'zfs'}");
       $r.= &exe("chmod A+everyone\@:r-x---a-R-c---:file_inherit/dir_inherit:allow /$in{'zfs'}");

    } else {

       #owner
       if ($in{'acl-o'} eq "default_create") {
         $in{'acl-o'}="rwxp--a-R-c--s";                   # create only
       }
       if ($in{'acl-o'} eq "none") {
         $in{'acl-o'}="--------------";                   # none
       }
       if ($in{'acl-o'} ne "unset") {
         $r.= &exe("chmod A+owner\@:$in{'acl-o'}:file_inherit/dir_inherit:allow /$in{'zfs'}");
       }

       #group
       if ($in{'acl-g'} eq "default_create") {
         $in{'acl-g'}="rwxp--a-R-c--s";                   # create only
       }
       if ($in{'acl-g'} eq "none") {
         $in{'acl-g'}="--------------";                   # none
       }
       if ($in{'acl-g'} ne "unset") {
          $r.= &exe("chmod A+group\@:$in{'acl-g'}:file_inherit/dir_inherit:allow /$in{'zfs'}");
       }

       #everyone
       if ($in{'acl-e'} eq "default_create") {
         $in{'acl-e'}="rwxp--a-R-c--s";                   # create only
       }
       if ($in{'acl-e'} eq "none") {
         $in{'acl-e'}="--------------";                   # none
       }
       if ($in{'acl-e'} ne "unset") {
          $r.= &exe("chmod A+everyone\@:$in{'acl-e'}:file_inherit/dir_inherit:allow /$in{'zfs'}");
       }

           #if ($in{'acl-o'} eq "smb") {
           #   $r=  &exe("$sys{'sbinpath'}zfs set aclinherit=passthrough $in{'zfs'}");
           #   $r.= &exe("chmod A- /$in{'zfs'}");
           #   $r.= &exe("chmod A=user:root:full_set:file_inherit/dir_inherit:allow /$in{'zfs'}");
           #   $r.= &exe("chmod A+everyone\@:modify_set:file_inherit/dir_inherit:allow /$in{'zfs'}");
           #}

    }
  }



  ##### share acl
  ###############
  if ($in{'prop'} eq "share-acl") {
    $s=$in{'zfs'}; $s=~s/^.*\///;               # -> zfs name
    $s="/$in{'zfs'}/.zfs/shares/$s";            # share-name


    #share level
    if ($in{'share-acl'} eq "none")  {
         #$r.= &exe("chmod A=everyone\@:read_set::allow $s");
         $r= &exe("chmod A=user:root:full_set::allow $s");
    }

    #share level

    if ($in{'share-acl'} eq "read_set")  {
         $r.= &exe("chmod A=everyone\@:read_set::allow $s");
         $r.= &exe("chmod A+user:root:full_set::allow $s");
    }
    if ($in{'share-acl'} eq "modify_set")  {
         $r.= &exe("chmod A=everyone\@:modify_set::allow $s");
         $r.= &exe("chmod A+user:root:full_set::allow $s");
    }
    if ($in{'share-acl'} eq "full_set")  {
         $r= &exe("chmod A=everyone\@:full_set::allow $s");
         $r.= &exe("chmod A+user:root:full_set::allow $s");
    }
    if ($r ne"") {  &mess($r); }
    #reload smb
    $r.= &exe("svcadm restart smb/server");


  } #/acl
  #######

  ###############
  #smb/cif-share
  ##############
  if ($in{'prop'} =~/sharesmb/) {
        $in{'text'}=~s/[^a-zA-Z0-9\-\.\_\$]//g;
        if ($in{'text'} eq "") {  &mess('Please enter share-name'); }


       #&print_hash(%in);

       #############
       ## solaris 11
       #############
       if ($sys{'auto_os'} eq "Solaris 11") {
               $t="";
               # new share command zfs set share=name=abc,path=..,prot=nfs zfs
               if ($in{'text'} eq "off")  {            #  tank/data_smbname

                     #remember share acl
                      &zfslib_remember_shareacl;

                     $r=&exe("$sys{'sbinpath'}zfs set -c share=name=$in{'prop2'} $in{'zfs'}");

                     # no more shares: off
                      $r=&exe("$sys{'sbinpath'}zfs get -o name,value share | grep prot=smb");
                      @n=();
                      @n=split(/\n/,$r);
                      $r="0";
                      foreach my $n (@n) {
#print"-$in{'zfs'}-";
#&print_array(@n);
                         $n=~s/\t.*//;
                         if ($n eq $in{'zfs'}) { $r="1"; last; }
                      }
                      if ($r eq "0") {
                          $r=&exe("$sys{'sbinpath'}zfs set sharesmb=off $in{'zfs'}");
                      }



                      if ($r ne "") { &mess($r);}        #..was removed

               } else {

                     if ($in{'abe'} ne "") { $in{'abe'}=",abe=true";  }
                     if ($in{'path'} eq "") { $in{'path'}="/$in{'zfs'}" ; }    # path optional

                     $t="share=name=$in{'text'},path=$in{'path'},prot=smb$in{'abe'}";

                     if ($in{'guest'} eq "guest") {  $t.=",guestok=true"; }

                     $t.=" $in{'zfs'}";
                     $r=&exe("$sys{'sbinpath'}zfs set $t");
                     if ($r =~/error/) { &mess($r);}        #..was removed

                     $r=&exe("zfs get -H -o value sharesmb $in{'zfs'}");
                     if ($r eq "off") {           # additional: sharesmb=on
                         $r=&exe("$sys{'sbinpath'}zfs set sharesmb=on $in{'zfs'}");
                     }

                     #set shareacl
                     &zfslib_set_shareacl;
               }

               #remember
               $r=&exe("zfs get -H -o value keystatus $in{'zfs'}");     # enc filesystem
               chomp($r);
               if ($r eq "available") {
                   $r=&exe("zfs get -H -o value share $in{'zfs'} | grep prot=smb");
                   $r=~s/,*path=.*prot=smb//;  #name=abc,abe=..
                   $r=~s/=true//g;   #name=abc,abe=.
                   $t=&exe("zfs set enc_smbshare:=$r $in{'zfs'}");    # remember for enc fs

               }

       } else {      # Illumos

               $t="";
               if ($in{'guest'} eq "guest") {
                        $r=&exe("chmod 777 \"/$in{'zfs'}\"");
                        $t=",guestok=true";
               }

               if ($in{'text'} eq "off") {

                        #remember share acl
                        &zfslib_remember_shareacl;

                        $r=&exe("$sys{'sbinpath'}zfs set sharesmb=off $in{'zfs'}");

                        #remember
                        $r=&exe("zfs get -H -o value keystatus $in{'zfs'}");    # enc filesystem
                        chomp($r);
                        if ($r eq "available") {
                           $r=&exe("zfs set enc_smbshare:=off $in{'zfs'}");    # remember for enc fs
                        }



               } else {

                        if ($in{'abe'} ne "") {
                            $r=&exe("$sys{'sbinpath'}zfs set sharesmb=abe=true,name=$in{'text'}$t $in{'zfs'}");
                        } else {
                            $r=&exe("$sys{'sbinpath'}zfs set sharesmb=name=$in{'text'}$t $in{'zfs'}");
                        }
                        if ($r ne "") {  &mess("zfs set sharesmb=name=$in{'text'}$t $in{'zfs'}<br> $r"); }

                        #set shareacl
                        &zfslib_set_shareacl;

                        #remember
                        $r=&exe("zfs get -H -o value keystatus $in{'zfs'}");
                        chomp($r);
                        if ($r eq "available") {
                           $t=&exe("zfs get -H -o value sharesmb $in{'zfs'}");  chomp($t);
                           $t=&exe("zfs set enc_smbshare:=$t $in{'zfs'}")
                        }
               }

      }
  }

  #nfs-share
  ##########
  if ($in{'prop'} eq "sharenfs") {

        $in{'text'}=~s/[^a-zA-Z0-9\:\-\_\@,=\/\.]//g;
        $r="";
        if ($in{'text'} eq "") {  &mess('on or off?'); }
        if ($in{'text'} eq "off") {                           # unshare
             if ($sys{'auto_os'} eq "Solaris 11") {
                $t=$in{'zfs'}; $t=~s/.*\///;                  # dataset
                $t="NFS-$t";
                $r=&exe("$sys{'sbinpath'}zfs set -c share=name=$t $in{'zfs'}");
             }
             $r=&exe("$sys{'sbinpath'}zfs set sharenfs=off $in{'zfs'}");

        } else {
             if ($sys{'auto_os'} eq "Solaris 11") {
                $t=$in{'zfs'}; $t=~s/.*\///; # dataset
                $t="NFS-$t";

                # NFS options?
                $in{'text'}=~s/ +$//;
                if ($in{'text'} ne "on") {
                  $o=",$in{'text'}";
                } else {
                  $o="";
                }
                $o=&exe("$sys{'sbinpath'}zfs set share=name=$t,path=/$in{'zfs'},prot=nfs$o $in{'zfs'}");        # 9.3.18 first setting name, thanks martin
                $r=&exe("$sys{'sbinpath'}zfs set sharenfs=on $in{'zfs'}");                                      # second setting on
             } else {
                 $r=&exe("$sys{'sbinpath'}zfs set sharenfs=$in{'text'} $in{'zfs'}");
             }

       }
     if ($r ne "") {  &mess($r); }
  }

  #compression
  ############
  if ($in{'prop'} eq "compression") {

        # use always lz4
        $t="lz4";

        $in{'text'}=~s/[^a-zA-Z0-9\-\_]//g;
        $r="";
        if ($in{'text'} eq "") {  &mess('on or off?'); }
        if ($in{'text'} eq "off") {  $r=&exe("$sys{'sbinpath'}zfs set compression=off $in{'zfs'}");  }
        if ($in{'text'} ne "off")  {
             # lz4?
             $r=&exe("$sys{'sbinpath'}zfs set compression=$t $in{'zfs'}");
        }
        if ($r ne "") {  &mess($r); }
  }

  #nbmand
  #######
  if ($in{'prop'} eq "nbmand") {
        $in{'text'}=~s/[^a-zA-Z0-9\-\_]//g;
        $r="";
        if ($in{'text'} eq "") {  &mess('on or off?'); }
        if ($in{'text'} eq "off") {  $r=&exe("$sys{'sbinpath'}zfs set nbmand=off $in{'zfs'}");   }
        if ($in{'text'} eq "on")  {  $r=&exe("$sys{'sbinpath'}zfs set nbmand=on $in{'zfs'}");  }

        if ($r ne "") {  &mess($r); }
#&mess("$in{'text'} $r");

        #unmount/mount is needed !!
        $r.=&exe("$sys{'sbinpath'}zfs unmount -f $in{'zfs'}");
        $r.=&exe("$sys{'sbinpath'}zfs mount $in{'zfs'}");

        #or optionally restart service
        $r=&exe("svcs | grep smb/server");
        if ($r ne "") { &exe("svcadm restart smb/server");   }

        $r=&exe("svcs | grep nfs/server");
        if ($r ne "") { &exe("svcadm restart nfs/server");   }

        $r=&exe("svcs | grep netatalk");
        if ($r ne "") { &exe("svcadm restart network/netatalk:default");   }

  }

  #&print_hash(%in);
  #readonly
  if ($in{'prop'} eq "readonly") {

        $in{'text'}=~s/[^a-zA-Z0-9\-\_]//g;
        $r="";
        if ($in{'text'} eq "") {  &mess('on or off?'); }

        #recursiv?
        if ($in{'recursive'}) {

                  # load ZFS and ZFS below
                  my $zfs=`$sys{'sbinpath'}zfs list -H -o name`;   # all ZFS names
                  @t=();
                  @t=split(/\n/,$zfs);

                  #collect ZFS
                  my @zfs=();
                  foreach my $p (@t) {
                                      chomp($p);
                                      if (!($p=~/^$in{'zfs'}\b/)) { next; }            # not below

                                      $type=`$sys{'sbinpath'}zfs list -H -o type`;           # do not post-process volumes
                                      chomp($type);
                                      if ($type eq "volume") { next; }
                                      push (@zfs,$p);
                  }
#&print_array(@zfs);
                  foreach my $p (@zfs) {
                          if ($in{'text'} eq "off") {  $r=&exe("$sys{'sbinpath'}zfs set readonly=off $p");  }
                          if ($in{'text'} eq "on")  {  $r=&exe("$sys{'sbinpath'}zfs set readonly=on $p");  }
                  }

        } else {

          if ($in{'text'} eq "off") {  $r=&exe("$sys{'sbinpath'}zfs set readonly=off $in{'zfs'}");  }
          if ($in{'text'} eq "on")  {  $r=&exe("$sys{'sbinpath'}zfs set readonly=on $in{'zfs'}");  }
          if ($r ne "") {  &mess($r); }
        }
  }

  #unix perm
  ##########
  if ($in{'prop'} eq "unix-permissions") {
        $r=&exe("chmod $in{'perm'} /$in{'zfs'}");

        # set corresponding acl
        $r.= &exe("chmod A- \"/$in{'zfs'}\"");                                  #reset

        if (!($in{'zfs'}=~/\//)) {                                              # pool itself    readx= r-x---a-R-c---

            $r.= &exe("chmod A=owner\@:full_set:file_inherit/dir_inherit:allow /$in{'zfs'}");
            $r.= &exe("chmod A+everyone\@:r-x---a-R-c---:file_inherit/dir_inherit:allow /$in{'zfs'}");          #read execute 755
            $r.= &exe("chmod A+user:root:full_set::allow  /$in{'zfs'}");
        } else {
            if ($in{'perm'} eq "777") {
                 $r.= &exe("chmod A=owner\@:full_set:file_inherit/dir_inherit:allow /$in{'zfs'}");
                 $r.= &exe("chmod A+everyone\@:full_set:file_inherit/dir_inherit:allow /$in{'zfs'}");
                 $r.= &exe("chmod A+user:root:full_set::allow  /$in{'zfs'}");
            }
            if ($in{'perm'} eq "775") {
                 $r.= &exe("chmod A=owner\@:full_set:file_inherit/dir_inherit:allow /$in{'zfs'}");
                 $r.= &exe("chmod A+everyone\@:r-x---a-R-c---:file_inherit/dir_inherit:allow /$in{'zfs'}");          #read execute 755
                 $r.= &exe("chmod A+user:root:full_set::allow  /$in{'zfs'}");
            }
            if ($in{'perm'} eq "770") {
                 $r.= &exe("chmod A=owner\@:full_set:file_inherit/dir_inherit:allow /$in{'zfs'}");
                 $r.= &exe("chmod A+everyone\@:--------------:file_inherit/dir_inherit:allow /$in{'zfs'}");          #none
                 $r.= &exe("chmod A+user:root:full_set::allow  /$in{'zfs'}");
            }
            if ($in{'perm'} eq "755") {
                 $r.= &exe("chmod A=owner\@:full_set:file_inherit/dir_inherit:allow /$in{'zfs'}");
                 $r.= &exe("chmod A+everyone\@:r-x---a-R-c---:file_inherit/dir_inherit:allow /$in{'zfs'}");          #read execute 755
                 $r.= &exe("chmod A+user:root:full_set::allow  /$in{'zfs'}");
            }
            if ($in{'perm'} eq "744") {
                 $r.= &exe("chmod A=owner\@:full_set:file_inherit/dir_inherit:allow /$in{'zfs'}");
                 $r.= &exe("chmod A+everyone\@:read_set:file_inherit/dir_inherit:allow /$in{'zfs'}");                 #read execute 755
                 $r.= &exe("chmod A+user:root:full_set::allow  /$in{'zfs'}");
            }
            if ($in{'perm'} eq "700") {
                 $r.= &exe("chmod A=owner\@:full_set:file_inherit/dir_inherit:allow /$in{'zfs'}");
                 $r.= &exe("chmod A+everyone\@:--------------:file_inherit/dir_inherit:allow /$in{'zfs'}");          #none
                 $r.= &exe("chmod A+group\@:--------------:file_inherit/dir_inherit:allow /$in{'zfs'}");             #none
                 $r.= &exe("chmod A+user:root:full_set::allow  /$in{'zfs'}");
            }

        }
        if ($r ne "") {  &mess($r); }
  }

  #recsize
  if ($in{'prop'} eq "recsize") {
        $in{'text'}=lc($in{'text'});
        $in{'text'}=~s/[^a-z0-9]//g;
        if ($in{'text'} eq '') { $in{'text'}="128k"; }
        $r="";
        $r=&exe("$sys{'sbinpath'}zfs set recsize=$in{'text'} $in{'zfs'}");
        if ($r ne "") {  &mess($r); }
  }

  #dedup
  if ($in{'prop'} eq "dedup") {
        $in{'text'}=~s/[^a-zA-Z0-9\-\_]//g;
        $r="";
        if ($in{'text'} eq "") {  &mess('on or off?'); }
        if ($in{'text'} eq "off") {  $r=&exe("$sys{'sbinpath'}zfs set dedup=off $in{'zfs'}");  }
        if ($in{'text'} eq "on")  {  $r=&exe("$sys{'sbinpath'}zfs set dedup=on $in{'zfs'}");  }
        if ($r ne "") {  &mess($r); }
  }

  #quota
  if ($in{'prop'} eq "quota") {

        if ($in{'text'} eq "") { $in{'text'} = 'none';   }
        if ($in{'text'} ne 'none') {
            if (!($in{'text'}=~/M|G|T/i)) { &mess('MGT (Mega/Giga/Tera)-Byte'); }
            if (!($in{'text'}=~/\d+\w/)) {  &mess('Incorrect size! Enter Quota for example: 10G'); }
        }
        $r="";
        $r=&exe("$sys{'sbinpath'}zfs set quota=$in{'text'} $in{'zfs'}");
        if ($r ne "") {  &mess($r); }
  }

  #refquota
  if ($in{'prop'} eq "refquota") {

        if ($in{'text'} eq "") { $in{'text'} = 'none';   }
        if($in{'text'} ne 'none') {
            if (!($in{'text'}=~/M|G|T/i)) { &mess('MGT (Mega/Giga/Tera)-Byte'); }
            if (!($in{'text'}=~/\d+\w/)) {  &mess('Incorrect size! Enter RefQuota for example: 10G'); }
        }
        $r="";
        $r=&exe("$sys{'sbinpath'}zfs set refquota=$in{'text'} $in{'zfs'}");
        if ($r ne "") {  &mess($r); }
  }

  #reservation
  if ($in{'prop'} eq "reservation") {

        if ($in{'text'} eq "") { $in{'text'} = 'none';   }
        if($in{'text'} ne 'none') {
            if (!($in{'text'}=~/M|G|T/i)) { &mess('MGT (Mega/Giga/Tera)-Byte'); }
            if (!($in{'text'}=~/\d+\w/)) {  &mess('Incorrect size! Enter reservation for example: 10G'); }
        }


         $r="";
        $r=&exe("$sys{'sbinpath'}zfs set reservation=$in{'text'} $in{'zfs'}");
        if ($r ne "") {  &mess($r); }
  }

  #refreservation
  if ($in{'prop'} eq "refreservation") {

        if ($in{'text'} eq "") { $in{'text'} = 'none';   }
        if($in{'text'} ne 'none') {
            if (!($in{'text'}=~/M|G|T/i)) { &mess('MGT (Mega/Giga/Tera)-Byte'); }
            if (!($in{'text'}=~/\d+\w/)) {  &mess('Incorrect size! Enter refreservation for example: 10G'); }
        }


         $r="";
        $r=&exe("$sys{'sbinpath'}zfs set refreservation=$in{'text'} $in{'zfs'}");
        if ($r ne "") {  &mess($r); }
  }

  # userquota: zfs set userquota@student1=10G students/compsci
  if ($in{'prop'} =~/userquota/) {
        $t=$in{'prop'}; $t=~s/userquota://;
        if ($in{'text'} =~/^\d+[KMGTP]$|^none$/) {
           $r=&exe("$sys{'sbinpath'}zfs set userquota\@$t=$in{'text'} $in{'zfs'}");
        if ($r ne "") {  &mess($r); }
        } else {
           &mess("none or a value like 5G");
        }
        $in{'prop'}="used";
       # &reload;
  }

  # groupquota: zfs set groupquota@staff=10G students/compsci
  if ($in{'prop'} =~/usergquota/) {
        $t=$in{'prop'}; $t=~s/usergquota://;
        if ($in{'text'} =~/^\d+[KMGTP]$|^none$/) {
           $r=&exe("$sys{'sbinpath'}zfs set groupquota\@$t=$in{'text'} $in{'zfs'}");
        if ($r ne "") {  &mess($r); }
        } else {
           &mess("none or a value like 5G");
        }
        $in{'prop'}="used";
        #&reload;
  }

  #aclinherit
  if ($in{'prop'} eq "aclinherit") {
        $r=&exe("$sys{'sbinpath'}zfs set aclinherit=$in{'aclinherit'} $in{'zfs'}");
        if ($r ne "") {  &mess($r); }
  }

  #aclmode obsolet in zfs v. 25+
  #if ($in{'prop'} eq "aclmode") {
  #      $r=&exe("$sys{'sbinpath'}zfs set aclmode=$in{'aclmode'} $in{'zfs'}");
  #      if ($r ne "") {  &mess($r); }
  #}

  #sync
  if ($in{'prop'} eq "sync") {
        $r=&exe("$sys{'sbinpath'}zfs set sync=$in{'sync'} $in{'zfs'}");
        if ($r ne "") {  &mess($r); }
  }

 #snapdir
  if ($in{'prop'} eq "snapdir") {
        $in{'text'}=~s/[^a-zA-Z0-9\-\_]//g;
        $r="";
        if ($in{'text'} eq "") {  &mess('snapdir hidden or visible?'); }
        if ($in{'text'} eq "hidden")   {  $r=&exe("$sys{'sbinpath'}zfs set snapdir=hidden $in{'zfs'}");  }
        if ($in{'text'} eq "visible")  {  $r=&exe("$sys{'sbinpath'}zfs set snapdir=visible $in{'zfs'}");  }
        if ($r ne "") {  &mess($r); }
  }

  #readonly
  if ($in{'prop'} eq "readonly") {
        $in{'text'}=~s/[^a-zA-Z0-9\-\_]//g;
        $r="";
        if ($in{'text'} eq "") {  &mess('on or off?'); }
        if ($in{'text'} eq "off") {  $r=&exe("$sys{'sbinpath'}zfs set readonly=off $in{'zfs'}");  }
        if ($in{'text'} eq "on")  {  $r=&exe("$sys{'sbinpath'}zfs set readonly=on $in{'zfs'}");  }
        if ($r ne "") {  &mess($r); }
  }


   $in{'action'}="";
   $in{'prop'}="";
   $in{'zfs'}="";
   if ($in{'call'} eq "agent") { return "ok, $smb"; }

   if (-f "/tmp/nappit/zfs_list.zfs") { `sudo rm /tmp/nappit/*.zfs`;  }   # acc
   &reload;

 }

##############
sub zfslib_get_default_ftp {
##############
my $d= <<EoF;

<VirtualHost =ip=>
  ServerName                     "FTP server"
  Port                           21
  Umask                          022

  <Limit LOGIN>
    DenyAll
  </Limit>

  <Anonymous  =path=>
    User                         nobody
    Group                        nobody
    UserAlias                    anonymous nobody

    <Limit LOGIN>
      AllowAll
    </Limit>

    <Limit WRITE>
      DenyAll
    </Limit>
  </Anonymous>
</VirtualHost>

EoF
return($d)
}


##############
sub zfslib_get_default_vhost {
##############

my $d= <<EoF;

<VirtualHost auto_set>
    ServerAdmin auto_set
    DocumentRoot auto_set
    ServerName auto_set
    ErrorLog auto_set
    CustomLog auto_set
    DirectoryIndex index.php index.html

   # Directory options "/pool/zfs/_wwwroot"
   <Directory "=document_root=">
      Require all granted
      Options Indexes FollowSymLinks
      AllowOverride None
      Order allow,deny
      Allow from all
    </Directory>

   # ScriptAlias ex /cgi-bin/ "/pool/zfs/wwwroot/cgi-bin/"
   ScriptAlias /cgi-bin/ "=document_root=/cgi-bin/"

   #Directory options "/$in{'zfs'}/_wwwroot/cgi-bin"
   <Directory "=document_root=/cgi-bin">
      Require all granted
      AllowOverride None
      Options +ExecCGI
      Order allow,deny
      Allow from all
   </Directory>
</VirtualHost>
EoF
return($d);
 }

 ##############
 sub zfslib_get_iplist {         # to %current
 #############
         # get current ip_list
        $t=`sudo /usr/sbin/ipadm show-addr`;
        my @p=split(/\n/,$t);
        $current{'ip_list'}="";
        foreach my $p (@p) {
              @t=();
              @t=split(/ +/,$p);
              if (!($p=~/ok/))          { next; }
              if ($p=~/::/)             { next; }     # ip6
              if ($p=~/127\.0\.0\.1/)   { next; }     # ip6
              $t[3]=~s/\/.*//;                        # 192.168.1.1/24
             $current{'ip_list'}.="\t$t[3]";
        }
        $current{'ip_list'}=~/\t+$/;
  }

 ##############
 sub zfslib_get_ftpshares {      # to %current
 #############
        my (@ip,@t,$p,$t,$i,@v,$a,$f,@s,@w,$np);

        #get ip
        &zfslib_get_iplist;
        my @t=();
        @t=split(/\t/,$current{'ip_list'});                   # available ip
        foreach my $t (@t) {
           if(!($t=~/\d+/)) { next; }                         # default ftp port 21 only
           push(@ip,$t);
        }

        # read all wwwshares
        my @ftpshares=();

        $p="";
        @t=split(/\t/,$zfs{'datapools'});
        foreach my $t (@t) {
                if (-f "/$t/.ftp_shares") {
                     $p.=`cat /$t/.ftp_shares`;               # ex tank/data,192.168.1.1:21
                     $p.="\n";
                }
        }

        # get avail ip:port
        @s=split(/\n/,$p);

        # get avail adress:ip
        $a=",";

        # step1, determine used ports for every ip
        foreach my $s (@s) {                            # tank/data,192.168.1.1:80,name
                      @w=();
                      @w=split(/,/,$s);

                      $p="";
                      if ($w[1]=~/\]:/) {                       # ip6 [2001:0db8:85a3:08d3:1319:8a2e:0370:7344]:8080
                         $i=$w[1];
                         $i=~s/\].*/\]/;
                         $p=$w[1];
                         $p=~s/.*://;
                         $ip="[$ip]";
                         $current{'vftp_ip'}="$p";
                      } else {                                  # ip4
                         $i=$w[1];
                         $i=~s/:.*//;
                         $p=$w[1];
                         $p=~s/.*://;
                         $current{'vftp_ip'}="$p";
                      }

                      #

                      $t="vftp_ports_$i";
                      if (!($current{$t}=~/\b$p\b/)) {
                         $current{$t}.="$p,";
                      }

                     # &print_hash(%in);
                     #
                      # add used to avail for current share
                      if ($w[0] eq $in{'zfs'})  {
                         $current{'vftp_avail'}.="$i:$p,";
                      }

        }

        # avail port:ip
        my $c="1";
        foreach my $ip (@ip) {                                  # 192.168.1.1 or 2001:0db8:85a3:08d3:1319:8a2e:0370:7344

             # max 1 extra port per ip
             $t="vftp_ports_$ip";

             # no port used
             if ($current{$t} eq "") {
                  $a.="$ip:21,";
             } else {
                  if (!($current{$t} =~/\b21\b/)) { $a.="$ip:21,";   $c++; }  if ($c > 1) { $c="1"; next; }
                  if (!($current{$t} =~/2121/))   { $a.="$ip:2121,"; $c++; }  if ($c > 1) { $c="1"; next; }
             }
        }
        $a=~s/,$//;
        $current{'vftp_avail'}.=$a;
        $current{'vftp_avail'}=~s/,+/,/g;
        $current{'vftp_avail'}=~s/,$|^,//g;
 # debug
 # &print_array(@a);
 # &mess($a);
 # &print_hash(%current);

 }

 ##############
 sub zfslib_get_wwwshares {      # to %current
 #############
        my (@ip,@t,$p,$t,$i,@v,$a,$f,@s,@w,$np);

        #get ip
        &zfslib_get_iplist;
        my @t=();
        @t=split(/\t/,$current{'ip_list'});                   # available ip
        foreach my $t (@t) {
           if(!($t=~/\d+/)) { next; }
           $p="www_share_nextport_$t";
           $current{$p}="80";                                # default www port 80
           push(@ip,$t);
        }

        # read all wwwshares
        my @wwwshares=();

        $p="";
        @t=split(/\t/,$zfs{'datapools'});
        foreach my $t (@t) {
                if (-f "/$t/.www_shares") {
                     $p.=`cat /$t/.www_shares`;               # ex tank/data,192.168.1.1:80
                     $p.="\n";
                }
        }

        # get avail ip:port
        @s=split(/\n/,$p);

        # get avail adress:ip
        $a=",";

        # step1, determine used ports for every ip
        foreach my $s (@s) {                            # tank/data,192.168.1.1:80,name
                      @w=();
                      @w=split(/,/,$s);

                      $p="";
                      if ($w[1]=~/\]:/) {                       # ip6 [2001:0db8:85a3:08d3:1319:8a2e:0370:7344]:8080
                         $i=$w[1];
                         $i=~s/\].*/\]/;
                         $p=$w[1];
                         $p=~s/.*://;
                         $ip="[$ip]";
                         $current{'vhost_VirtualHost'}="[$i]:$p";
                      } else {                                  # ip4
                         $i=$w[1];
                         $i=~s/:.*//;
                         $p=$w[1];
                         $p=~s/.*://;
                         $current{'vhost_VirtualHost'}="$i:$p";
                      }

                      #

                      $t="www_ports_$i";
                      if (!($current{$t}=~/\b$p\b/)) {
                         $current{$t}.="$p,";
                      }

                     # &print_hash(%in);
                     #
                      # add used to avail for current share
                      if ($w[0] eq $in{'zfs'})  {
                         $current{'www_avail'}.="$i:$p,";
                      }

        }

        # avail port:ip
        my $c="1";
        foreach my $ip (@ip) {                                  # 192.168.1.1 or 2001:0db8:85a3:08d3:1319:8a2e:0370:7344

             # max 1 extra per ip
             $t="www_ports_$ip";

             # no port used
             if ($current{$t} eq "") {
                  $a.="$ip:80,$ip:8080,";
             } else {
                  if (!($current{$t} =~/\b80\b/)) { $a.="$ip:80,";   $c++; }  if ($c > 1) { $c="1"; next; }
                  if (!($current{$t} =~/8080/))   { $a.="$ip:8080,"; $c++; }  if ($c > 1) { $c="1"; next; }
                  if (!($current{$t} =~/8081/))   { $a.="$ip:8081,"; $c++; }  if ($c > 1) { $c="1"; next; }
                  if (!($current{$t} =~/8082/))   { $a.="$ip:8082,"; $c++; }  if ($c > 1) { $c="1"; next; }
                  if (!($current{$t} =~/8083/))   { $a.="$ip:8083,"; $c++; }  if ($c > 1) { $c="1"; next; }
             }
        }
        $a=~s/,$//;
        $current{'www_avail'}.=$a;
        $current{'www_avail'}=~s/,+/,/g;
        $current{'www_avail'}=~s/,$|^,//g;
 # debug
 # &print_array(@a);
 # &mess($a);
 # &print_hash(%current);

 }



 #####################
 sub zfslib_ask_property {       #hide: ask for property
 #####################        (div window in front)

    if ($in{'answered'}) { &zfslib_change_property; &zfslib_details; exit; }

    my ($ask,$close,$form,$info,$t,$opt,$r,$p,$d,$z,$f,$r,$l);
    $close="<a href=\"/cgi-bin/admin.pl?id=$in{'id'}\&amp;action=\&amp;l1=$in{'l1'}\&amp;l2=$in{'l2'}\&amp;l3=$in{'l3'}\&amp;list=$in{'list'}\&amp;zfs=$in{'zfs'}\"><b>\&nbsp;cancel\&nbsp;/close\&nbsp;</b></a>";

    #########
    #ask lock
    #########

    if ($in{'prop'} eq "lock") {
        #remember

        $r="";
        if ($sys{'auto_os'} =~/Solaris/) {   # Solaris
           $r=&exe("zfs get -H -o value share $in{'zfs'} | grep prot=smb");
           $r=~s/,*path=.*prot=smb//;  #name=abc,abe=..
           $r=~s/=true//g;   #name=abc,abe=.
        } else {
           $r=&exe("zfs get -H -o value sharesmb $in{'zfs'}");
        }

#&mess("zfs get -H -o name,value share $in{'zfs'} | grep prot=smb<br>$r");
        $r=~s/,/+/g;
        $r.=",off"; $r=~s/^,+//;

        #ad last
        $t=&exe("zfs get -H -o value enc_smbshare: $in{'zfs'}");
        $t=~s/,*path=.*prot=smb//;  #name=abc,abe=..
        $t=~s/=true//g;   #name=abc,abe=.

        $t=~s/,/;/g;
        if ($t ne "-") {          # not set
          $r.=",last: $t";
        } else {
          $r.=",last";
        }
        $r=~s/off,off/off/;       # avoid 2 x off

        $r="last,off";            # always
#&mess($r);

        $form="<table border='1'><tr><td>Encrypted filesystem:\&nbsp;\&nbsp;</td><td><input type='text' class='input' name='text' value='lock it' style='width: 150px' readonly>";
        $form.="</td></tr><tr><td>";
        $form.="Remember SMB share:\&nbsp;</td><td>". &select('remember',$r)."</td></tr></table>";  # smb shares are always disabled
        $info="If you want to reaccess this filesystem, you have to unlock it with your passphrase or a key location <br>like W1:W2 or L1:L1. SMB shares are set to off can be remembered as a user defined ZFS property<br>";
    }

    ###########
    #ask unlock
    ###########

    if ($in{'prop'} eq "unlock") {

        $t=&exe("zfs get -H -o value enc_split: $in{'zfs'}");  chomp($t);           # ex L1:L2
        if (!($t=~/\d:/)) { $t="-"; }

        #same info on all options
        $info="Enter your passphrase or a key location like L1:L1 or W1:W2 to unlock this filesystem. <br>The former state of an SMB share can be restored from a user defined ZFS property.";

        $r="file";
        if ($sys{'auto_os_type'}=~/ill/) {  # Illumos
             $r=&exe("zfs get -H -o value keylocation $in{'zfs'}"); chomp($r);
        }

        if ($sys{'auto_os'}=~/Solaris/) {  # Solaris
             $r=&exe("zfs get -H -o value keysource $in{'zfs'}");   chomp($r);
        }


        if ($r=~/prompt/) {
                 if ($t=~/\d:/) {
                   $form="<table><tr><td>Unlock with Passphrase:\&nbsp;\&nbsp;</td><td><input type='text' name='pass' value=\"$t\" style='width: 150px'> ";
                 } else {
                   $form="<table><tr><td>Unlock with Passphrase:\&nbsp;\&nbsp;</td><td><input type='password' name='pass' value=\"\" style='width: 150px'> ";
                 }
                 $form.="</td></tr><tr><td>";
                 $form.="</td></tr><tr><td>";
                 $form.="Restore SMB share:</td><td>". &select('smbrefresh','last,off')."</td></tr></table>";

        } else {
                 $r=~s/.*,//;
                 $form="<table><tr><td>Unlock from source:\&nbsp;\&nbsp;</td><td><input type='text' name='pass' value=\"$r\" style='width: 250px' readonly> ";
                 $form.="</td></tr><tr><td>";
                 $form.="Restore SMB share:</td><td>". &select('smbrefresh','last,off')."</td></tr></table>";
        }
    }


    ############
    #ask shareS3
    ############
    if ($in{'prop'} =~/shareS3/) {
      &log_end;

       $r=`sudo /sbin/zfs get -H -o value s3_config: $in{'zfs'}`;   chomp($r);
       if ($r=~/\/\w/) {
             $t=$in{'zfs'}; $t=~s/\//~/g;
             $d="$r/S3_keydata/$t";        #pool~data/s3keydata
             if ($r eq $in{'zfs'})  {
                $d="$in{'zfs'}";           # config in filesystem
             }
      } else {
             $d="$in{'zfs'}";
      }

      if ($in{'old'} =~/off|unset/) {

        #read/get values opt from $cfg{'value_key_s3_fs'}/S3_settings/'pool~fd'/S3_config or fs/S3_config

#&mess("/$d/S3_config/key.txt");
        $r=`sudo test -f "/$d/S3_config/key.txt" && echo 1 || echo 2`;  chomp($r);
        if ($r eq "1") {
             $tmp{'key'}=`sudo cat /$d/S3_config/key.txt`;  $tmp{'key_old'}=$tmp{'key'};
        } else {$tmp{'key'}=""; }
#&print_hash(%tmp);
        $r=`sudo test -f "/$d/S3_config/secret.txt" && echo 1 || echo 2`;  chomp($r);
        if ($r eq "1") {
             $tmp{'secret'}=`sudo cat /$d/S3_config/secret.txt`;  $tmp{'secret_old'}=$tmp{'secret'};
        } else {$tmp{'key'}=""; }

        $r=`sudo test -f "/$d/S3_config/address.txt" && echo 1 || echo 2`;  chomp($r);
        if ($r eq "1") { $tmp{'address'}=`sudo cat /$d/S3_config/address.txt`; } else { $tmp{'address'}=""; }

        if ($tmp{'address'} eq "") {       # default :9000
            $t="8999";
            $r=`sudo /sbin/zfs get -H -o value  s3_addr: | grep :`;  # ex n:9000
            @t=split(/\n/,$r);
            foreach $l (@t) { $l=~s/.*://; }  # n:9001 -> 9001
            foreach $l (@t) { if ($l>$t) { $t=$l; }  }               # last highest number
            $t++;
            $tmp{'address'}=":$t";  # next port > 9000
        }

        # verify against already running minio
        $r=`ps axw | grep minio | grep -v grep`;

        while ($r=~/$tmp{'address'}/s)  {
            $t=$tmp{'address'}; $t=~s/.*://;     # port
            $t++;
            $tmp{'address'}=~s/:.*//;            # ip only
            $tmp{'address'}.=":$t";
        }

# &mess("$tmp{'address'}<hr>$r");

        $form=<<EoF;
        <table style='padding:0 5px 0 5px; width:500px'>
        <tr><td>REGION_NAME (opt)</td>   <td><input type='text' class='input' name='region' value="my_location" style='width: 150px'></td></tr>
        <tr><td>ROOT_USER (>2)</td>   <td><input type='text' class='input' name='key' value="$tmp{'key'}" style='width: 150px'></td></tr>
        <tr><td>ROOT_PASSWORD (>7)</td><td><input type='text' class='input' name='secret' value="$tmp{'secret'}" style='width: 150px'></td></tr>
EoF
        $form.=<<EoF;
        <tr><td>Cert folder</td><td><input type='text' class='input' name='cert' value='/var/web-gui/_log/minio/certs' style='width: 300px; background-color: #f7fdf6' readonly></td></tr>
        <tr><td>ADDRESS:PORT</td><td><input type='text' class='input' name='address' value="$tmp{'address'}" style='width: 150px;'></td></tr>
        <tr><td>CONSOLE:PORT</td><td>same as ADRESS but 1000 lower (:8xxxx)</td></tr>

EoF
        # config folder from about > settings
        $t="";
        if ($cfg{'value_key_s3_fs'}=~/\/\w/) {
           $r=`sudo test -d "/$cfg{'value_key_s3_fs'}" && echo 1 || echo 2`;  chomp($r);
           if ($r eq "1") {
             $t="<option selected>$cfg{'value_key_s3_fs'}</option>";
           }
        }
        $t.="<option>$in{'zfs'}</option>";

#&print_hash(%cfg);
         $form.=<<EoF;
         <tr><td>S3 key backupfolder</td><td>
         <select name="s3config" class="input">
          $t
         </select></td></tr>

         <tr><td>Clear/recreate miniIO settings</td><td>
         <select name="s3clear" class="input">
           <option selected>no</option><option>yes</option>
         </select></td></tr>

        <tr><td>Minio server</td><td><input type='text' class='input' name='server' value=\'/$in{'zfs'}/S3_data\' style='width: 298px; background-color: #f7fdf6' readonly></td></tr>

EoF

        # alt server and auto
        $t=`sudo /sbin/zfs get -H -o value s3_auto: $in{'zfs'}`;   chomp($t);
        if (($t=~/yes/) || ($t eq "")) {
           $t="<option selected>yes</option><option>no</option>";
        } else {
           $t="<option selected>no</option><option>yes</option>";
        }

        $form.=<<EoF;
        <tr><td>Alt minio server</td><td><textarea name="alt_server" style="width: 300px; margin-left: 0px; padding: 1px" cols="80" rows="4" class="input" wrap="off"></textarea></td>

        <tr><td>Nice</td><td>
        <select name="nice" class="input">
          <option selected>yes</option><option>no</option>
        </select></td></tr>

        <tr><td>Autostart</td><td>
        <select name="auto" class="input">
          $t
        </select></td>

EoF
        $form.="</tr></table><br>";
        $info=<<EoF;
        <b>S3 cloudservices are provided via minIO</b><br><br>
        You need ROOT_USER and ROOT_PASSWORD (name,password) to access the S3 folder from a client<br>
        via an Internet browser or another S3 client on the server ip:port ex 192.168.1.1:9000<br>
        To access miniO Console, connect to port -1000 ex ex 192.168.1.1:8000<br><br>

        To change a key and secret later, disable the S3 share and re-enable with a new KEY and SECRET.<br><br>
        Per default you share S3 data in a subfolder S3_data with settings in a subfolder S3_config.<br>
        You can override the default minio server call with the alt minio server field.<br><br>
        Per default S3 access is done on any ip via port 9000. You can set the port via :9001 or 192.168.1.1:9003<br>
        If you share several ZFS filesystems, each minIO session needs an unique ip:port. <br><br>
        MINIO_REGION_NAME     (string)    name of the location of the server e.g. "us-west-rack2"<br>
        autostart: on/of after reboot<br><br>
        alt server: set your own server options for minio
        The cert functionality is enabled when you provide /var/web-gui/_log/minio/certs/public.crt.<br><br>
        With nice, you can reduce priority of minio compared to other storage actions.<br>
        Calculate at least additional 400 MB RAM per minIO session. Data of an S3 are stored in a subfolder S3_data.<br>
        or at a default location that you can define in About &gt; settings. Optionally clear and recreate settings and key.

EoF
      } else {        #set off


        #logindata
        $tmp{'na'}="-"; # name
        $r=`sudo test -f "/$d/S3_config/key.txt" && echo 1 || echo 2`;  chomp($r);
        if ($r eq "1") { $tmp{'na'}=`sudo cat /$d/S3_config/key.txt`; }

        $tmp{'pa'}="-"; # secret
        $r=`sudo test -f "/$d/S3_config/secret.txt" && echo 1 || echo 2`;  chomp($r);
        if ($r eq "1") { $tmp{'pa'}=`sudo cat /$d/S3_config/secret.txt`; }

        $tmp{'ap'}="-";
        $r=`sudo test -f "/$d/S3_config/address.txt" && echo 1 || echo 2`;  chomp($r);
        if ($r eq "1") { $tmp{'ap'}=`sudo cat /$d/S3_config/address.txt`; }

        $tmp{'re'}="-";
        $r=`sudo test -f "/$d/S3_config/region.txt" && echo 1 || echo 2`;  chomp($r);
        if ($r eq "1") { $tmp{'re'}=`sudo cat /$d/S3_config/region.txt`; }

        #$form="<a href=\"#\" onclick=\"\$('.info').toggle();\">\&nbsp;\&nbsp;show login data</a><br><br>\n";
        # show name, pass, port
        $form=<<EoF;
        <a href=\"#\" onclick=\"\$('.info').toggle();\">\&nbsp;\&nbsp;Show login</a>

        <div class='info'>
        <table style='padding:0 5px 0 5px; width:500px'>
        <tr>
        <td style='width:110px'>Name</td>
        <td><input type='text' class='input' name='y' value="$tmp{'na'}" style='width: 150px' readonly></td>
        </tr>

        <tr><td>Pass</td>
        <td> <input type='text' class='input' name='z' value="$tmp{'pa'}" style='width: 150px' readonly>
        </td></tr>

        <tr><td>ip:Port</td>
        <td> <input type='text' class='input' name='u' value="$tmp{'ap'}" style='width: 150px' readonly>
        </td></tr>

        <tr><td>Region</td>
        <td> <input type='text' class='input' name='r' value="$tmp{'re'}" style='width: 150px' readonly>
        </td></tr>

        </table>
        </div>
EoF

        $t=`sudo /sbin/zfs get -H -o value s3_auto: $in{'zfs'}`;   chomp($t);
        if (($t=~/yes/) || ($t eq "")) {
           $t="<option selected>yes</option><option>no</option>";
        } else {
           $t="<option selected>no</option><option>yes</option>";
        }

        $form.=<<EoF;
        <table style='padding:0 5px 0 5px; width:500px'>
        <tr><td style='width:110px'>ShareS3</td><td> <input type='text' class='input' name='share' value='off' style='width: 150px' readonly></td>
        <tr><td>autostart</td><td>
        <select name="auto" class="input">
          $t
        </select></td></tr>
        </tr></table>
EoF

        $opt= "onclick=\"return(confirm('unshare S3 sharing of folder $in{'zfs'}?'))\"";
        $info="Disable S3 sharing and set default for reboot";
EoF
      }
    }

    ###############
    #ask share_ftp
    ###############
    if ($in{'prop'} eq "share_ftp") {

        if ($in{'old'} eq "off") {
          $d="<option>on</option><option>preset</option>";
        } else {
          $d="<option>off</option><option>update</option>";
        }

        # table1
        $form="";
        $form.="";
        $form.="";
        $form.=<<EoF;
        <table style='padding:0 5px 0 5px; width:500px'>
        <tr><td style='width:120px;padding-right:5px'> FTP-share&nbsp;</td>
        <td><input type='text' name='text' value=\"$in{'zfs'}\" style='width: 250px' readonly>
        &nbsp; set &nbsp;
        <select name="share_action" class="input">
        $d
        </select>
        <br><br></td>
        </tr></table>
EoF

         # read shares
         &zfslib_get_ftpshares;

        # defaults
        $d=&zfslib_get_default_ftp;

        # read old settings value
        $p=$in{'zfs'}; $p=~s/\/.*//;      # pool
        $z=$in{'zfs'}; $z=~s/\//./g;      # tank/data -> tank.data (filename)


        # get default vhost to use same ip as default
        $t="/$p/.www_shares_config_$z";   # tank/data -> tank.data
        if (-f "$t") {
             $t=`cat \'$t\'`;
             $t=~s/\s+$//s;
             # get values (%current)
             &zfslib_get_vhost_par($t);
        }



        # get ftp settings
        $t="/$p/.ftp_shares_config_$z";   # tank/data -> tank.data
        if (-f "$t") {
             $t=`cat \'$t\'`;
             $t=~s/\s+$//s;
             if ($t ne "") { $t.="\n"; $d=$t; }

             # get values (%current)
             &zfslib_get_ftp_par($t);
        }

        #select possible values
        @t=(); @t=split(/,/,$current{'vftp_avail'});       # list of available ip:port

#debug
#&print_hash(%current);
#&print_array(@t);

        $ip="\n";

        # current first
        if ($current{'vftp_host'} ne "") {
           $ip.="  <option selected>$current{'vftp_host'}</option>\n";
        } else {
           # prefer www-ip
           if ($current{'vhost_ip'} ne "") {
              # 21  avail
              if ($current{'vftp_avail'}=~/$current{'vhost_ip'}:21/) {
                $ip.="  <option selected>$current{'vhost_ip'}:21</option>\n";
                $current{'vftp_ip'}="$current{'vhost_ip'}";
              }
           }
        }

        foreach my $t (@t) {
           if ($t eq "$current{'vftp_host'}") { next; } # current
           $ip.="  <option>$t</option>\n";
        }

         #debug
#&print_hash(%current);
        my $root="";
        if ($current{'vftp_dir'}=~/_wwwroot$/) {
           $root="<option selected>/$in{'zfs'}/_wwwroot</option>\n";
           $root.="<option>/$in{'zfs'}</option>";
        } else {
           $root="<option selected>/$in{'zfs'}</option>\n";
           $root.="<option>/$in{'zfs'}/_wwwroot</option>";
        }

        # toggle details

        $form.=<<EoF;
        <br>&nbsp;&nbsp;Settings: (default=anonymous read access enabled)<hr><br>
        <table style='padding:0 5px 0 5px; width:500px'>
        <tr>
          <td style='width:120px'>vftp IP (avail)</td>
          <td><select name="select_ip" class="input" style='width:260px'>$ip</select> <br>Select unique ip for vftp (port 21)<br><br></td>
        <tr>

        <tr>
          <td style='width:120px'>or force ip:port<br><br></td>
          <td> <input type='text' name='force_ip' value="" style='width: 250px'><br><br></td>
        </tr>

        <tr>
          <td style='width:120px'>DocumentRoot<br><br></td>
          <td><select name="wwwroot" class="input" style='width:260px'>
$root
        </select> <br> Use a subfolder _wwwroot below your ZFS? <br><br><br></td>
        </tr>

        <tr>
        <td style='width:120px; padding-right:5px'>
            <a href='#' class='toggle'>set other manually:</a><br>
            <div class='toggle_content'><br><br>delete form to get defaults<br><br>
            create  vnics in ESXi or napp-it with an IP for every vftp<br><br>
            keep subfolder www for share and logs and www/wwwroot for html files<br><br>IP and Port are forced to select values</div>&nbsp;
        </td>
        <td><div class='toggle_content' style='width: 350px; position:relative'>
        <textarea class='input' name='settings' value='' style='width: 450px; height:250px'>$d</textarea><br>
        </div>
        <br>
        </td>
        </tr>
        </table>

EoF
        $info="<a href='http://www.proftpd.org/docs/' target='_blank'>settings: http://www.proftpd.org/docs/</a>";
    }


    ###############
    #ask share_iscsi
    ###############
    if ($in{'prop'} eq "share_iscsi") {
         require "/var/web-gui/data/napp-it/zfsos/_lib/illumos/zfslib-iscsi.pl";
         ($form,$info)=&zfslib_iscsi_ask();
         #$form="basic iscsi as a filesystem property: to come in current 0.9e1";
    }

    ###############
    #ask share_www
    ###############
    if ($in{'prop'} eq "share_www") {

       &mess("www sharing as a ZFS property is no longer available as Apache is now used for napp-it http/https<br>If you need Apache, you must configure manually in Services > Apache");

        if ($in{'old'} eq "off") {
          $d="<option>on</option><option>preset</option>";
        } else {
          $d="<option>off</option><option>update</option>";
        }

        # table1
        $form="";

        $form.=<<EoF;
        <table style='padding:0 5px 0 5px; width:500px'>
        <tr><td style='width:120px;padding-right:5px'> WWW-share&nbsp;</td>
        <td><input type='text' name='text' value=\"$in{'zfs'}\" style='width: 250px' readonly>
        &nbsp; set &nbsp;
        <select name="share_action" class="input">
        $d
        </select>
        <br><br></td>
        </tr></table>
EoF


         # read shares
         &zfslib_get_wwwshares;

        # defaults
        $d=&zfslib_get_default_vhost;

        # read old settings value
        $p=$in{'zfs'}; $p=~s/\/.*//;      # pool
        $z=$in{'zfs'}; $z=~s/\//./g;      # tank/data -> tank.data (filename)
        $f="/$p/.www_shares_config_$z";   # tank/data -> tank.data
        if (-f "$f") {
             $t=`cat \'$f\'`;
             $t=~s/\s+$//s;
             if ($t ne "") { $t.="\n"; $d=$t; }

             # get values (%current)
             &zfslib_get_vhost_par($t);

        }

        #select possible values
        @t=(); @t=split(/,/,$current{'www_avail'});       # list of available ip:port

#&mess("-$current{'vhost_VirtualHost'}-");
#&print_array(@t);
        $ip="\n";

        # current first
        $ip.="  <option selected>$current{'vhost_VirtualHost'}</option>\n";

        foreach my $t (@t) {
           if ($t eq $current{'vhost_VirtualHost'}) { next; } # current
           $ip.="  <option>$t</option>\n";
        }

         #debug
     # &print_hash(%current);
        my $root="";
        if ($current{'vhost_DocumentRoot'}=~/_wwwroot$/) {
           $root="<option selected>/$in{'zfs'}/_wwwroot</option>\n";
           $root.="<option>/$in{'zfs'}</option>";
        } else {
           $root="<option selected>/$in{'zfs'}</option>\n";
           $root.="<option>/$in{'zfs'}/_wwwroot</option>";
        }

        # toggle details
        if ($current{'vhost_ServerAdmin'} eq "") { $current{'vhost_ServerAdmin'}="webmaster\@thisdomain.com"; }
        if ($current{'vhost_ServerName'}  eq "") { $current{'vhost_ServerName'}="www.thisdomain.com"; }

        $form.=<<EoF;
        <br>&nbsp;&nbsp;Settings:<hr><br>
        <table style='padding:0 5px 0 5px; width:500px'>
        <tr>
          <td style='width:120px'>vhost IP (avail)</td>
          <td><select name="select_ip" class="input" style='width:260px'>$ip</select> <br>Select unique ip:port for vhosts (port 80,8080-8083)<br><br></td>
        <tr>

        <tr>
          <td style='width:120px'>or force ip:port<br><br></td>
          <td> <input type='text' name='force_ip' value="" style='width: 250px'><br><br></td>
        </tr>

        <tr>
          <td style='width:120px'>DocumentRoot<br><br></td>
          <td><select name="wwwroot" class="input" style='width:260px'>
$root
        </select> <br> Use a subfolder _wwwroot below your ZFS? <br><br><br></td>
        </tr>

        <tr>
          <td style='width:120px'>ServerAdmin</td>
          <td> <input type='text' name='ServerAdmin' value=\"$current{'vhost_ServerAdmin'}\" style='width: 250px'> </td>
        </tr>

        <tr>
          <td style='width:120px'>ServerName</td>
          <td> <input type='text' name='ServerName' value=\"$current{'vhost_ServerName'}\" style='width: 250px'> </td>
        </tr>

        <tr>
        <td style='width:120px; padding-right:5px'>
            <a href='#' class='toggle'>set other manually:</a><br>
            <div class='toggle_content'><br><br>delete form to get defaults<br><br>
            create  vnics in ESXi or napp-it with an IP for every vhost<br><br>
            keep subfolder www for share and logs and www/wwwroot for html files<br><br>IP, DocumentRoot and log are forced to select values</div>&nbsp;
        </td>
        <td><div class='toggle_content' style='width: 350px; position:relative'>
        <textarea class='input' name='settings' value='' style='width: 450px; height:250px'>$d</textarea><br>
        </div>
        <br>
        </td>
        </tr>
        </table>

EoF
        $info="<a href='http://httpd.apache.org/docs/2.4/en/' target='_blank'>settings: http://httpd.apache.org/docs/2.4/en/</a>";
    }

    ################
    #ask share_rsync
    ################
    if ($in{'prop'} eq "share_rsync") {
        if ($in{'old'} eq "off") {
          $d="<option>on</option><option>preset</option>";
        } else {
          $d="<option>off</option><option>update</option>";
        }

        $form="<table style='padding:0 5px 0 5px'>";
        $form.='<tr><td> Rsync-share&nbsp;</td>';
        $form.="<td><input type='text' name='text' value=\"$in{'zfs'}\" style='width: 250px' readonly>";
        $form.=<<EoF;
        &nbsp; set &nbsp; <select name="share_action" class="input">
        $d
        </select>
EoF

        $form.= "<br><br></td></tr>";

        # read old settings value
        $p=$in{'zfs'}; $p=~s/\/.*//;  # pool
        $z=$in{'zfs'}; $z=~s/\//./g;  # tank/data -> tank.data (filename)

        $form.="<tr><td>Module</td><td>[$z]</td></tr>\n";

        $f="/$p/.rsyncd_shares_config_$z";    # tank/data -> tank.data

        if (-f "$f") {
             $t=`cat \'$f\'`;
             $t=~s/.*\]\n*//s; # without header
             if ($t eq "") { $t="path = /$in{'zfs'}\n"; }
        } else { $t="path = /$in{'zfs'}\nread only = no\nlist = yes\n"; }

        $form.="<tr><td>add settings ex: <br>comment = Backup<br>auth users = paul, lisa<br>read only = no<br>list=yes<br>uid = user<br>hosts allow = 10.0.0.1/0</td>";
        $form.="<td><textarea name='settings' value='' style='width: 352px; height:150px'>$t</textarea><br>do not manually set: secrets file = filename<br>If you want to rsync local files to this share, set ex:<br>auth user = paul<br>read only = no<br></td></tr>";

        $f="/$p/.rsyncd_shares_scrt_$z";    # tank/data -> tank.data
        `sudo chmod 666 $f`;
        if (-f "$f") {
             $t=`sudo cat \'$f\'`;
             $t=~s/.*\]\n*//s; # without header

        } else { $t=""; }
        `sudo chmod 600 $f`;             # default

        $form.="<tr><td>secets name:pw <br>paul:21W3<br>lisa:71Q4</td>";
        $form.="<td><textarea name='secrets' value='' style='width: 352px; height:70px'>$t</textarea><br>A setting and secrets file is auto-created below /$p<br>";
        $form.="<br>Command to rsync a local folder to this share (set Modulname) ex:<br>rsync -rtv /localfolder paul\@192.168.1.20::$z</td></tr>";

        $form.="</table>";

        $info="<a href='http://rsync.samba.org/ftp/rsync/rsyncd.conf.html' target='_blank'>settings: rsync.samba.org</a>";
    }

    #ask shareafp
    if ($in{'prop'} eq "shareafp") {
      if ($in{'old'} eq "off") {

        # set nbmand to off
        $r=&exe("$sys{'sbinpath'}zfs get -H nbmand $in{'zfs'}");
        if ($r=~/nbmand\s+on/s) {
            &mess("nbmand should be on for filesystem $in{'zfs'} to use SMB: <br><br>Attention: Switching nbmand requires a unmount/mount sequence with a disconnect for all shares!");
        }

        $t=$in{'zfs'}; $t=~s/^.*\///;

        $form="<table style='padding:0 5px 0 5px'>";
        $form.='<tr><td> Afp-share&nbsp;name:&nbsp;</td>';
        $form.="<td><input type='text' name='text' value='$t' style='width: 200px'>";

        $form.= "$txt{'form_afp_ro'}:\&nbsp;<input type='checkbox' name='ro' value='1'> <br><br></td></tr>";

        $form.="<tr><td> Time Machine:</td>";
        $form.="<td><input type='checkbox' name='tm' value='1'>\&nbsp;  max \&nbsp;".&select('max','unlimited,10GB,100GB,300GB,600GB,1TB')."</td></tr>";

        $form.="<tr><td>Hide files:</td>";
        $form.="<td><input type='text' name='veto' style='width: 200px; margin-top: 2px'> $txt{'form_afp_hide'}</td></tr>";

        $form.="<tr><td colspan='2'><br><br>restrict to user,\@groups (commalist ex: user1,user2,\@staff,user3)<br><br></td>\n";

        $form.="<tr><td>login allowed:</td>";
        $form.="<td><input type='text' name='all' value='\@staff' style='width: 350px'> </td>";

        $form.="<tr><td>login denied:</td>";
        $form.="<td><input type='text' name='den' value='' style='width: 350px; margin-top:2px'> </td></tr>";

        $form.="<tr><td>read only:</td>";
        $form.="<td><input type='text' name='uro' value='' style='width: 350px; margin-top:2px'> </td></tr>";

        $form.="<tr><td>write allowed:</td>";
        $form.="<td><input type='text' name='urw' value='' style='width: 350px; margin-top:2px'><br><br></td></tr>";

        $form.="</table>";

        $info="$txt{'form_afp_info'}";
      } else {
        $form="afp share = <input type='text' class='input' name='text' value='off' style='width: 150px' readonly> ";
        $opt= "onclick=\"return(confirm('unshare zfs folder $in{'zfs'}?'))\"";
        $info="stop afp-sharing $in{'zfs'} ";
      }
    }

    #ask sharesmb
    ##########

    if ($in{'prop'} =~/sharesmb/) {
      if (($in{'old'} eq "off") || ($in{'prop'}=~/\(add/)) {
        $t=$in{'zfs'}; $t=~s/^.*\///;

        #old share acl
        $r=`sudo /sbin/zfs get all $in{'zfs'} | grep shareacl_`;
        $r=~s/$in{'zfs'} *//gs;      # remove fs
        @t=split(/\n/,$r); @t=sort(@t);        # sort

        foreach my $i (@t) {
          @p=split(/  +/,$i);   #prop  value  local
          $i=$p[1];
        }
        $r=join("\n<br>",@t);
        $r=&zfslib_acl2set($r);

        #$form="<table><tr>sharename <input type='text' name='text' value='$t' style='width: 150px'> ";
        #$form.=" guest allowed: <input type='checkbox' name='guest' value='guest'>&nbsp;&nbsp;";
       # $form.=" ABE: <input type='checkbox' name='abe' value='abe'>";
        $form=<<Eof;
        <table>
        <tr>
        <td style="padding-top: 5px">share_name:&nbsp;</td>
        <td><input class="input" type='text' name='text' value='$t' style='width: 160px; height:16px'></td>
        <td>&nbsp;&nbsp;guest allowed: <input type='checkbox' name='guest' value='guest'>&nbsp;&nbsp;
          ABE: <input type='checkbox' name='abe' value='abe'></td>

        </tr><tr>
        <td style="padding-top: 6px">share_acl: </td>
        <td>
        <select name="shareacl" class="input" style='width: 166px; height: 20px'>
                <option>root=full, last shareacl</option>
                <option>everyone=full</option>
                <option>root=full, everyone=modify</option>
                <option>root=full, everyone=read</option>
        </select>
        </td>
        <td>&nbsp;</td>
        </tr><tr>
        <td style="padding-top: 9px">last share_acl&nbsp;</td>
        <td colspan="2">
        <div id="" style="overflow:auto; height:70px; padding:5px; width: 320px; font-size: 12px; border:1px solid silver">
        $r</div></td></tr>
        </table><br>
Eof


         #if ($sys{'auto_os'} eq "Solaris 11") {
         # option
         #    $form.="<br>sharepath&nbsp;&nbsp;&nbsp;<input type='text' name='path' value=\'/$in{'zfs'}\' style='width: 150px'> ";
         #}
        $info="$txt{'form_smb_info'}";

      } else {

        $form="Sharesmb: <input type='text' class='input' name='text' value='off' style='width: 150px; margin-left:5px' readonly> ";
        $opt= "onclick=\"return(confirm('unshare zfs folder $in{'zfs'}?'))\"";
        $info="smb-sharing $in{'zfs'} off<br>share acl are remembered";
      }
    }

    #ask sharenfs
    if ($in{'prop'} =~/sharenfs/) {
      if ($in{'old'} eq "off") {
        $form="sharenfs = <input type='text' class='input' name='text' value='on' style='width: 380px'> ";
        $info="set nfs-sharing of $in{'zfs'} to on<br><br>";
        $info.="optional:<br>rw=host1:host2:host3,root=host1:host2:host3,ro=host4 ZFS<br><br>example:<br>\nrw=\@192.168.1.0/24,root=\@192.168.1.0/24";
      } else {
        $form="sharenfs = <input type='text' class='input' name='text' value='off' style='width: 150px' readonly> ";
        $opt= "onclick=\"return(confirm('unshare zfs folder $in{'zfs'}?'))\"";
        $info="nfs-sharing $in{'zfs'} off";
      }
    }

    #use compression LZ4
    if ($in{'prop'} eq "compression") {

      # ask lz4 on pool
      $t=$in{'zfs'}; $t=~s/\/.*//;     # get pool
      $t="(LZ4)";

      if ($in{'old'} eq "off") {
        $form="compression = <input type='text' class='input' name='text' value='on' style='width: 150px' readonly> $t";
        $info="compression $in{'zfs'} on";
      } else {
        $form="compression = <input type='text' class='input' name='text' value='off' style='width: 150px' readonly> ";
        $info="compression $in{'zfs'} off";
      }
    }

    #ask nbmand
    if ($in{'prop'} eq "nbmand") {
      if ($in{'old'} eq "off") {
        $form="nbmand = <input type='text' class='input' name='text' value='on' style='width: 150px' readonly> ";
        $info="nbmand $in{'zfs'} off means: <br>deactivate systemwide mandatory coordination of file-lockings ex between NFS and SMB<br><br>should be on for SMB shares<br>This command needs an unmount/mount cycle!";
      } else {
        $form="nbmand = <input type='text' class='input' name='text' value='off' style='width: 150px' readonly> ";
        $info="nbmand $in{'zfs'} off means: <br>deactivate systemwide mandatory coordination of file-lockings ex between NFS and SMB<br><br>should be on for SMB shares<br>This command needs an unmount/mount cycle!";
      }
    }

    #ask readonly
    if ($in{'prop'} eq "readonly") {
      if ($in{'old'} eq "off") {
        $form="readonly = <input type='text' class='input' name='text' value='on' style='width: 150px' readonly> set recursive: <input type='checkbox' name='recursive' value='1'>";
        $info="readonly of $in{'zfs'} on = read only";
      } else {
        $form="readonly = <input type='text' class='input' name='text' value='off' style='width: 150px' readonly> set recursive: <input type='checkbox' name='recursive' value='1'>";
        $info="readonly $in{'zfs'} off = read/write";
      }
    }


    #ask dedup
    if ($in{'prop'} eq "dedup") {
      if ($in{'old'} eq "off") {
        $form="dedup = <input type='text' class='input' name='text' value='on' style='width: 150px' readonly> ";
        $info="dedup $in{'zfs'} on<br><br>";
        $info.="$txt{'form_dedup_on'}";
      } else {
        $form="dedup = <input type='text' class='input' name='text' value='off' style='width: 150px' readonly> ";
        $info="dedup $in{'zfs'} off<br>$txt{'form_dedup_off'}";
      }
    }

    #ask quota
    if ($in{'prop'} eq "quota") {
        $r=&zfslib_get_property('quota');

        $form="quota = <input type='text' class='input' name='text' value='$r' style='width: 150px'> ";
        $info=$txt{'zfs_info_quota'};
        $info=~s/=zfs=/$in{'zfs'}/gs;
        # ex. $info="set quota of $in{'zfs'} to none or a value in MGT for example 10G<br><br>";
    }

    #ask refquota
    if ($in{'prop'} eq "refquota") {
        $r=&zfslib_get_property('refquota');

        $form="refquota = <input type='text' class='input' name='text' value='$r' style='width: 150px'> ";
        $info=$txt{'zfs_info_refquota'};
        $info=~s/=zfs=/$in{'zfs'}/gs;
        #$info="set refquota of $in{'zfs'} to none or a value in MGT for example 10G";
    }

    #ask userquota
    if ($in{'prop'} =~/userquota/) {
        $t=$in{'prop'}; $t=~s/userquota://;
        $r=&exe("zfs get -H -o value userquota\@$t $in{'zfs'}");

        $form="userquota = <input type='text' class='input' name='text' value='$r' style='width: 150px'> ";
        $info="none, n[KMGT] ex 5G<br>$txt{'zfs_info_userquota'}";
        $info=~s/=zfs=/$in{'zfs'}/gs;
    }

    #ask usergquota (group)
    if ($in{'prop'} =~/usergquota/) {
        $t=$in{'prop'}; $t=~s/usergquota://;
        $r=&exe("zfs get -H -o value groupquota\@$t $in{'zfs'}");

        $form="groupquota = <input type='text' class='input' name='text' value='$r' style='width: 150px'> ";
        $info="none, n[KMGT] ex 5G<br>$txt{'zfs_info_usergquota'}";
        $info=~s/=zfs=/$in{'zfs'}/gs;
    }

    #ask SSB, small special blocks
    if ($in{'prop'} eq "SSB") {
      $info="The ZFS special_small_block property controls the size of small io that goes to the special vdev instead the regular pool. The special vdev must have the same redundancy level than the regular vdevs in the pool. If a special vdev is lost, the pool is lost.<br><br>This value MUST be lower than pool recordsize or all ZFS datablocks not only the smaller ones will try to land on the special vdev. Size=0 will disable using the special vdev. This value is not related to filesysize but ZFS datablock size. This is a filesystem property. The pool setting is the default for child filesystems.";
      $form="Set small special blocks to ". &select('ssb','0,1K,2K,4K,8K,16K,32K,64K,128K',$in{'old'});             #
    }

    #ask recsize
    if ($in{'prop'} eq "recsize") {
        #$form="recsize = <input type='text' class='input' name='text' value='$in{'old'}' style='width: 150px'> ";
        $form="Set recsize to ". &select('text','8K,16K,32K,64K,128K,256K,512K,1M',$in{'old'});
        $info=<<EoF;
         A performance parameter of ZFS is the recordsize that determines the size of filesystem blocks for large files.<br>
         This is also the unit that ZFS validates through checksums. Filesystem blocks are dynamically striped over the vdevs of a pool.<br><br>
         ZFS recordsize can be set for new files with values like 8k, 16k, 32k, 64k, 128k, 256k, 512k or 1m<br>
         The default value is 128K that suits for most workloads. If your applikation is using a very low read/write blocksize, for
         example in an ESXi/ NFS environment for VMs you may reduce latency with a lower value between 16k and 64k. Start with 32k.
EoF
        # ex. $info="set quota of $in{'zfs'} to none or a value in MGT for example 10G<br><br>";
    }

    #ask aclinherit
    if ($in{'prop'} eq "aclinherit") {
        $r=&zfslib_get_property('aclinherit');
        $form="aclinherit = ". &select('aclinherit','discard,noallow,restricted,passthrough,passthrough-x',$r);             # name, values, selected-> return htmlcode   ;
        $info=<<EoF;
<pre>aclinherit — <b>discard</b>: for new objects, no ACL entries are inherited.
<b>noallow</b>: access type of deny is inherited.
<b>restricted</b>: (os default) — write_owner and write_acl permissions are removed.
<b>passthrough</b>: (more win-like) — files are created with inheritable ACEs.
<b>passthrough-x</b>: like passthrough with execute (x) permission.</pre>
EoF
    }


    #ask reservation
    if ($in{'prop'} eq "reservation") {
        $r=&zfslib_get_property('reservation');

        $form="reservation = <input type='text' class='input' name='text' value='$r' style='width: 150px'> ";
        $info=$txt{'zfs_info_res'};
        $info=~s/=zfs=/$in{'zfs'}/gs;
        #$info="set reservation of $in{'zfs'} to none or a value in MGT for example 10G";
    }

    #ask refreservation
    if ($in{'prop'} eq "refreservation") {
        $r=&zfslib_get_property('refreservation');

        $form="refreservation = <input type='text' class='input' name='text' value='$r' style='width: 150px'> ";
        $info=$txt{'zfs_info_refres'};
        $info=~s/=zfs=/$in{'zfs'}/gs;
        #$info="set refreservation of $in{'zfs'} to none or a value in MGT for example 10G";
    }



    #ask snapdir
    if ($in{'prop'} eq "snapdir") {
      if ($in{'old'} eq "hidden") {
        $form="snapdir = <input type='text' class='input' name='text' value='visible' style='width: 150px' readonly> ";
        $info="snapdir $in{'zfs'} = visible";
      } else {
        $form="snapdir = <input type='text' class='input' name='text' value='hidden' style='width: 150px' readonly> ";
        $info="snapdir $in{'zfs'} = hidden";
      }
    }

    #ask sync
    if ($in{'prop'} eq "sync") {
        $r=&zfslib_get_property('sync');
        $form="sync = ". &select('sync','standard,always,disabled',$r);             # name, values, selected-> return htmlcode   ;
        $info=$txt{'form_sync'};
    }


    ### form ###
    $ask=&ask_pop("$form","$info","$opt");
    return ($ask);
 }

 #########################
 sub zfslib_get_property {  # parameter property, return value of current zfs or par2
 ####################
        my $p=$_[0];
        my $zfs=$_[1];
        my $r;
        if ($zfs eq "") { $zfs=$in{'zfs'}; }                   # parameter2 or via %in
        $r=&exe("$sys{'sbinpath'}zfs get -H $p $zfs");
        if ($r=~/bad |suspended/s) { return ""; }              # bad property or io suspended
        $r=~s/^.*\s$p\s+//gs;
        $r=~s/\s.*//s;
#&mess("-$p-<br>-$in{'zfs'}-<br>-$r-");
        return ($r);
 }

 sub zfslib_selectform {   # hide: values: 'property','values,,,','default','r'  (r=really)
 #######################
   my ($r,@v,$opt,$p,$s);
  if ($_[0] eq "") { &mess('zfs_selectform: property is missing'); }
  if ($_[1] eq "") { &mess('zfs_selectform: value is missing'); }

  if ($_[3]=~/r/i) {
   $opt="onclick=\"return(confirm('change/set property ?'))\"";     # js box: really??
  }

   $r=<<EoF;
  <!-- select xxxxx zfs property xxxxx -->
  <form name="ask" method="post" action="/cgi-bin/admin.pl">
  <input type="hidden" name="id" value="$in{'id'}">
  <input type="hidden" name="l1" value="$in{'l1'}">
  <input type="hidden" name="l2" value="$in{'l2'}">
  <input type="hidden" name="l3" value="$in{'l3'}">
  <input type="hidden" name="zfs" value="$in{'zfs'}">
  <input type="hidden" name="prop" value="$_[0]">
  <input type="hidden" name="answered" value="1">
=sel=
 <input type="submit" name="submit" value="set" style='font-size: 10px;'$opt>
  </form>
EoF

   $s=&select($_[0],$_[1],$_[2]);             # name, values, selected-> return htmlcode
   $r=~s/=sel=/$s/s;
   $r=~s/\n|  +|\t//gs;
   return($r);
 }





  ###############
  sub zfslib_delete {               #delete zfs from list
 #################    show_set details
    my ($zfs,@z,$url,$n,@zfs,$r,$cl,$t);


    #show list
    ############

    #del url
       $url="<a style='color:#ba2f28; font-weight:bold' href=\"/cgi-bin/admin.pl?id=$in{'id'}\&amp;action=\&amp;l1=$in{'l1'}\&amp;l2=$in{'l2'}\&amp;l3=$in{'l3'}\&amp;zfs_del=1\&amp;zfs==z=\" onclick=\"return(confirm('destroy zfs filesystem =f= with all snaps and clones?'))\">";

   #del selected
        my $sel="<input type=\"checkbox\" name=\"sel_zfs\" value=\"Z.+\">";

        $zfs=&exe("zfs list -o 'name,readonly,used,avail,creation' -t filesystem | grep -v rpool");
        $zfs.="\n Pool rpool\n";
        $zfs.=&exe("zfs list -t filesystem -o name | grep / | grep rpool | grep -v ROOT | grep -v export");

        # parse date for double spaces
        $zfs=&parse_date($zfs);

        @z=split(/\n/,$zfs);
##

        foreach $t (@z) {

           if ($t eq "$zfs{'syspool'}")    { next; }         #system
           if ($t =~/^$zfs{'syspool'}\/VARSHARE/)            { next; }         #solaris 11

           $n=$t; $n=~s/ .*//;
            if (!($t=~/\w+\//))   {
               if (!($t=~/^NAME/)) {
                   push (@zfs,"");
                   push (@zfs,"<b>Pool $n </b>\t");          #zpool itself
               } else {
                  $t="<b>$t</b>";
                  $t=~s/NAME/NAME\tSEL/;                     # add select
                  $t.="\tCLONE ORIGIN";
               }
           }
           push (@zfs,$t);
         }
#&print_array(@zfs);
         foreach $t (@zfs) {
             if (!($t=~/\w+/))      { next; }               #
             if ($t=~/^<b>NAME/)    { next; }               # no zfs/ header
             if ($t=~/^<b>Pool/)    { next; }
             $n=$t; $n=~s/  +.*//;
             if (!($n=~/\//))       {
                                     $t=~s/  /\t/;         # add column for select
                                     next;                  # poolname
             }

             $zfs=$t; $zfs=~s/ .*//;
             $t=~s/  /<\/a> /;
             $t="$url$t";
             $t=~s/=f=/$n/g;
             #$t=~s/zfs\=/zfs\=$zfs/g;
             $t=~s/=z=/$zfs/g;

             # readonly=red
             $t=~s/on  /<font class='font_darkred'><b>on<\/b><\/font>  /;

             #add select
             $r=$sel;
             $r=~s/Z/$zfs/;
             $t=~s/  /\t$r/;

             # add clone origin
             $cl=$zfs."_origin";
             $t.="\t$zfs{$cl}";

        }

#&print_array(@zfs);

    $t=join("\n",@zfs);
    $t=~s/  +/\t/gs;
    $t=&list2table($t);

    # delete selected
    &ask('begin_selzfs');

    $zfs="<b>ZFS Overview $in{'zfs'}</b>\n\n$t";

    print "<pre class='tt'>$zfs</pre>";
    &ask('confirm','destroy selected','destroy selected filesystems with all snaps and clones?');
    print "</body></html>";

    return;


 }


####################
sub zfslib_list_datasnaps  {          #opt d=inkl. delete option, r=rollback, l=return commalist ex &list_datasnaps('l'), &list_datasnaps('f') with date,<br>&list_datasnaps('d') print with delete
################ show snaps
  # thanks to rick for code-review and bugfix :-))

  my ($s,@snap,$url,$url2,$n,$old,$oldsnap,$new,@w,$z,$r,@data,$clone,@clones,$u);
  $old=""; $new="";

  if (!($_[0]=~/l|f/)) {    #commalist
    print "<b>zfs list -r -t snapshot: </b><br>";

    # not in delete -> l3="" but in Release or Hold
    if (($in{'l2'} eq "") || ($in{'l2'} =~/release|hold/i)) {
      if ($in{'select_sort'} eq "") { $in{'select_sort'}="name+"; }

      &ask('begin','b');
      &ask('value_filter','Filter snaps: ',$in{'value_filter'});
      &ask('value_filter2','and (opt): ',$in{'value_filter2'});
      &ask('select_sort','sort/select snaps by',"name+,name-,creation+,creation-,used-,holded,released,cloned",$in{'select_sort'});
      print "<div style='position: relative; top: -37px; left: 700px'><input type='submit' name='submit_ok' value='submit' style='font-size: 10px;'></div></form>";
    }
  }

  $in{'value_filter'}=~s/^ +//;         # no spaces
  $in{'value_filter2'}=~s/^ +//;

  $in{'value_filter'}=~s/ +$//;         # no trailing spaces
  $in{'value_filter2'}=~s/ +$//;

  if (defined($in{'id'})) {             # only via napp-it, not within auto-jobs

       #del single snap
       $url="<a style='color:#ba2f28; font-weight:bold' href=\"/cgi-bin/admin.pl?id=$in{'id'}\&amp;action=delete\&amp;l1=$in{'l1'}\&amp;l2=$in{'l2'}\&amp;l3=$in{'l3'}\&amp;value_filter=$in{'value_filter'}\&amp;delsnap==f=\" onclick=\"return(confirm('destroy snapshot =f= ?'))\">destroy</a>";

       # rollback
       if ($_[0]=~/r/) {
          $url="<a style='color:#ba2f28; font-weight:bold' href=\"/cgi-bin/admin.pl?id=$in{'id'}\&amp;action=rollback\&amp;l1=$in{'l1'}\&amp;l2=$in{'l2'}\&amp;l3=$in{'l3'}\&amp;value_filter=$in{'value_filter'}\&amp;delsnap==f=\" onclick=\"return(confirm('rollback to snapshot =f= ?'))\">rollback</a>";
       }

       # holded
       if ($_[0]=~/H/) {
          $url="<a style='color:#ba2f28; font-weight:bold' href=\"/cgi-bin/admin.pl?id=$in{'id'}\&amp;action=release\&amp;l1=$in{'l1'}\&amp;l2=$in{'l2'}\&amp;l3=$in{'l3'}\&amp;value_filter=$in{'value_filter'}\&amp;delsnap==f=\" onclick=\"return(confirm('release snapshot =f= and distroy when defer=on?'))\">release</a>";
       }

       # released
       if ($_[0]=~/R/) {
          $url="<a style='color:#ba2f28; font-weight:bold' href=\"/cgi-bin/admin.pl?id=$in{'id'}\&amp;action=hold\&amp;l1=$in{'l1'}\&amp;l2=$in{'l2'}\&amp;l3=$in{'l3'}\&amp;value_filter=$in{'value_filter'}\&amp;delsnap==f=\" onclick=\"return(confirm('hold snapshot =f= ?'))\">hold</a>";
       }

#&print_hash(%in);
       if ($in{'delsnap'}=~/\@/) {
           $in{'delsnap'}=~s/\s//g;

           # delete
           if ($in{'action'} eq "delete") {
             print "delete snapshot $in{'delsnap'}<br>";
             $r=&exe("$sys{'sbinpath'}zfs destroy $in{'delsnap'}");
             print "$r";
             if (-f "/tmp/acc-snap.log") {  &exe("rm /tmp/acc-snap.log"); }
           }
           # rollback
           if ($in{'action'} eq "rollback") {
             print "rollback snapshot $in{'delsnap'}<br>";
             $r=&exe("$sys{'sbinpath'}zfs rollback -f -r -R $in{'delsnap'}");
             print "$r";
             if (-f "/tmp/acc-snap.log") {  &exe("rm /tmp/acc-snap.log"); }
           }

           # HOLDED
           if ($in{'action'} eq "release") {
             print "release snapshot $in{'delsnap'}<br>";
             $r=&exe("$sys{'sbinpath'}zfs release -r keep $in{'delsnap'}");
             print "$r";
             if (-f "/tmp/acc-snap.log") {  &exe("rm /tmp/acc-snap.log"); }
           }

           # RELEASED
           if ($in{'action'} eq "hold") {
             print "hold snapshot $in{'delsnap'}<br>";
             $r=&exe("$sys{'sbinpath'}zfs hold -r keep $in{'delsnap'}");
             print "$r";
             if (-f "/tmp/acc-snap.log") {  &exe("rm /tmp/acc-snap.log"); }
           }
           sleep (2);
           &reload;
       }

  }
  #

  @data=();

  # check accelerator
  if ((-f "/tmp/nappit/zfs_last.zfs") && ($cfg{'ver'} ne "")) {
                open (F,"</tmp/nappit/zfs_last.zfs");
                     flock(F, LOCK_EX);
                     @data=<F>;
                close (F);
                $t=time();

                # read acc
                if (($t-$data[0] < 1200) && (-f "/tmp/nappit/zfs_list_snaps1.zfs"))   { # max 20min
                      @data=();
                      open (F,"</tmp/nappit/zfs_list_snaps1.zfs");
                           flock(F, LOCK_EX);
                           @data=<F>;
                      close (F);
                      $s=join("",@data);
                      &exe("echo acc_read_zfs snaps");
                 } else {
                      &exe("rm /tmp/nappit/*.zfs");
                }

               if (!($s=~/\@/s)) { $s=""; }
 }

 # $s=""; # debug only

 # clones not valid on Solaris
 $t=",clones";
 if ($sys{'auto_os'}=~/Solaris/i) {
   $t="";
 }

 if ($in{'select_sort'} eq "used-") {
        $s=&exe("$sys{'sbinpath'}zfs list -H -r -t snapshot -o name,creation,used,avail,refer,defer_destroy,userrefs$t -S used");
        $s=&parse_date($s);                # Dez 1 -> Dez 01
 } else {

    if ($s eq "") {
        $s=&exe("$sys{'sbinpath'}zfs list -H -r -t snapshot -o name,creation,used,avail,refer,defer_destroy,userrefs$t -S creation");  # default newest first, no header
        $s=&parse_date($s);                # Dez 1 -> Dez 01
    }
  }

  @w=();
  @snap=();
  @w=split(/\n/,$s);


  # clone, print cloned with title
  foreach my $w (@w) {
    if ($w=~/^NAME/) { next; }

    @t=();
    @t=split(/\t/,$w);
#&print_array(@t);
    if (($t[7] eq "-") || ($t[7] eq "")) {
       $t[7]="-";
    } else {
        $t[7]="<a title=\"$t[7]\">cloned</a>";
    }
    $w=join("\t",@t);
  }

 # &print_array(@w);

  # default sort order: newest first
  if ($in{'select_sort'} eq "name+")     { @w=sort(@w); }
  if ($in{'select_sort'} eq "name-")     { @w=reverse sort(@w); }
  if ($in{'select_sort'} eq "creation-") { @w=reverse (@w); }

  # add header
  $t='&nbsp;'x170;
  unshift @w,"NAME$t\t<b<CREATION</b>\t<b>USED</b>\t<b>AVAIL</b>\t<b>REFER</b>\t<b><a title='defer_destroy'>DEFER</b>\t<b>HOLDS</b>\tCLONE";   # add header after sort

  #step 1: filter
  my $count=@w;            # ??
  my $count2='0';

  # count filter
  foreach $t (@w) {
     if (($in{'value_filter'} ne "")  && (!($t=~/$in{'value_filter'}/)))  { next; }
     if (($in{'value_filter2'} ne "") && (!($t=~/$in{'value_filter2'}/))) { next; }
     $count2++;
  }

  foreach $t (@w) {
     #if ($t=~/^$zfs{'syspool'}\//) { next; }          # rpool  ??

     if ($_[0] =~/d|H|R/) {                        # header for datasnaps, Hold and Release

       # add always header
        if ($t=~/^NAME/) {
             $t="<b>ZFS</b>\t<input type='checkbox' style='margin:0 0 0 4px' onclick='toggleChecked(this.checked)'>\t$t";
             $t=~s/NAME/<b>NAME<\/b>/;
             $t=~s/CLONE/<b>CLONE<\/b>\t<b>SNAPS: $count2\/$count<\/b>/;
             push (@snap,$t);
             next;
        }
     }

     #Hold counter > 0: red
     @t=split(/\t/,$t);
#&print_array(@t);
     if ($t[6]>0) {
         $t[6]="<font style='color:#ba2f28'>$t[6]<\/font>";
         $t=join("\t",@t);

         # released only
         if ($in{'select_sort'} eq "released")  {       # released only
                next;
         }
     } else {
         # hold only
         if ($in{'select_sort'} eq "holded")  {         # holded only
                next;
         }
     }

    # cloned
    if ($in{'select_sort'} eq "cloned")  {         # with clones only
          if (!($t=~/>yes</)) { next; }
    }

    if (($in{'value_filter'} ne "")  && (!($t=~/$in{'value_filter'}/)))  { next; }
    if (($in{'value_filter2'} ne "") && (!($t=~/$in{'value_filter2'}/))) { next; }

    push (@snap,$t);
  }

 # &print_array(@snap);

  my $sel="<input type=\"checkbox\" name=\"sel_snaps\" value=\"Z.+\">";
  @clones=();
  @clones=split(/\t/,$zfs{'all_clones'});           # list of clones

  foreach $t (@snap) {

        $t=~s/  1 / 01 /;                            # Oct  1  -> Oct 01
        $t=~s/  2 / 02 /;
        $t=~s/  3 / 03 /;
        $t=~s/  4 / 04 /;
        $t=~s/  5 / 05 /;
        $t=~s/  6 / 06 /;
        $t=~s/  7 / 07 /;
        $t=~s/  8 / 08 /;
        $t=~s/  9 / 09 /;

        $t=~s/  0:/ 00:/;                            # hour   0: -> 00:
        $t=~s/  1:/ 01:/;
        $t=~s/  2:/ 02:/;
        $t=~s/  3:/ 03:/;
        $t=~s/  4:/ 04:/;
        $t=~s/  5:/ 05:/;
        $t=~s/  6:/ 06:/;
        $t=~s/  7:/ 07:/;
        $t=~s/  8:/ 08:/;
        $t=~s/  9:/ 09:/;

        $t=~s/  +/\t/g;                              # default seperator tab

        $n=$t;                                       # name of snap

        $n=~s/\s.*//;
        if (($t=~/\@/) && ($_[0] =~/d|r|H|R/)) {     # delete or rollback or Hold/Release


                    @t=();
                    @t=split(/\t/,$t);

                    $u=$url;                           # default
#&print_array(@t);
                   # has a clone
                   if ($t=~/>cloned</) {

                     $u=$t[7];    # clone as title     <a title="snap">yes</a>
                     $u=~s/\">.*//;
                     $u=~s/.*\"//;

                   }  else {

                     #
                     # link to hold
                     $r=$t[6]; $r=~s/\D//g;
                     if (($r>0) && (!($in{'l2'}=~/release/i)))  {
                       $u="holded";
                     }

                     # link to hold
                     if (($r==0) && ($in{'l2'}=~/release/i)) {
                       $u="released";
                     }
                   }
                   $t.="\t$u";

        }
        $t=~s/=f=/$n/g;

        #@ insert selects
        if (!($_[0]=~/l/)) {                         # whorfin - no tabs!, thanks rick

             if ($_[0] =~/d|H|R/) {
                #$t=~s/NAME +/ZFS\tSEL\tSNAP/;
                $t=~s/SELNAME +/SEL\tSNAP  /;
                $z=$t; $z=~s/\@.*//;                 # name of snap

                # default, show select
                $r=$sel;
                $r=~s/Z/$n/;

               # no checkbox if there is clone base on this snap
                if ($t=~/>cloned</) {      # has a clone or set to hold
                  $r="";
                }

                # no checkbox if holded
                if ($in{'l2'}=~/release/i) {
                  if (!($t=~/>[1-9]</))  {
                     $r="";
                  }
                }  else {
                  if ($t=~/>[1-9]</)  {
                     $r="";
                  }
                }

                # no checkbox without hold activated in submenu release
                if (($t=~/>off</) && ($in{'l2'}=~/release/i)) {
                   $r="";
                }

                $t=~s/\@/\t$r\t$z\@/;
             } else {
                $t=~s/NAME/ZFS\t\tSNAP/;
                $z=$t; $z=~s/\@.*//;                 # name of snap
                $t=~s/\@/\t\t$z\@/;
             }
        }
    }

  #&print_array(@snap);


  if ($_[0]=~/l|f/) {    #commalist  list,
       @w=();
       foreach $t (@snap) {
          if (!($t=~/\@/)) { next; }                   #no snap
             if ($_[0]=~/l/) {
             $t=~s/^\s+//;  $t=~s/ .*//;               #only snap name
          } else {
             $t=~s/^\t+//; $t=~s/\t+/\t/g;
             $t=~s/<.*//;                              # no delete url
          }
          push (@w,$t);
       }
       ## thanks to rick
       if ($_[0]=~/l/) {
           # 'l' is used for retention policy, so list in REVERSE order
           @w=sort {$b cmp $a} @w;      # sort num
       } else {
           @w=sort(@w);                 # sort char
       }
       ##
       $n=join(",",@w);
       return($n);

  } else {            #print list

     if ($_[0]=~/d|H|R/) {
          &ask('begin_sel');
     }
     print "\n<div id='chb'>\n";   # select all input

     $n=join("\n",@snap);
     $n=~s/  +/\t/gs;
     $n=&list2table($n);

     print  "$n<br>\n";
     print "</div>\n"; # /chb

     if ($_[0]=~/d/) {
        &ask('confirm','destroy selected','destroy selected snaps?');
     }

     if ($_[0]=~/H/) {
        &ask('confirm','release selected','release selected snaps and destroy when defer=on?');
     }

     if ($_[0]=~/R/) {
        &ask('checkbox_defer','defer_destroy<br></b>This will detroy a holded snap immediately when you release it<b>',"enable defer_destroy");
        &ask('confirm','hold selected','hold selected snaps?');
     }

  # js for select all
  print <<EoF;
  <script type="text/javascript">
      function toggleChecked(status) {
      \$(".checkbox").each( function() {
      \$(this).attr("checked",status);
      })
      }

      function toggleChecked(status) {
      \$("#chb input").each( function() {
      \$(this).attr("checked",status);
      })
      }
</script>

EoF

     return();
  }
}

#####################
sub zfslib_pool_zdb {  # show pooldetails via zdb, parameter1="1" -> print
#####################  # show all or parameter(2)=pool
  my (@w,$w,$key,$k,$n,@p,$p,$r,$i,$v,@v,$d,@d,$id,$guid,$phys,$devid,$j,$ids,$out);

  &load_lib('get-disk.pl');   # get required values

  @p=();
  @p=split(/\t/,$zfs{'allpools'});

  #only special pool(s)
  if ($_[1] ne "")  {
    @p=$_[1];
  }

  $r="";
  $out="Pool\tVersion\tPool GUID\tVdev\tAshift\tAsize\tVdev GUID\tDisk\tDisk-GUID\tCap\tProduct/ Phys_Path/ Dev_Id/ Sn";

  ##each pool
  ###########
  foreach $p (@p) {

      $r=&exe("zdb -C $p");

      @w=();   #values
      @w=split(/\n/,$r);

      #basic pool values and search first children[0]= first vdev
      ###########################################################
      foreach $w (@w) {
             #pool-version
             if ($w=~/version: /) {
                  $key=$p."_version";
                  $disk{$key}=$w;
                  $disk{$key}=~s/.*version: //;
                  $out.="\n\n$p\t$disk{$key}";   # pool + version

             }
             #pool-guid
             if ($w=~/pool_guid: /) {
                  $key=$p."_guid";
                  $disk{$key}=$w;
                  $disk{$key}=~s/.*pool_guid: //;
                  $out.="\t$disk{$key}"
             }
             #vdevs_children
             if ($w=~/vdev_children: /) {
                  $key=$p."_vdev_children";
                  $disk{$key}=$w;
                  $disk{$key}=~s/.*vdev_children: //;
                  $out.= "\t vdevs: $disk{$key}";
             }
             #hostid
             if ($w=~/hostid: /) {
                  $key="hostid";
                  $sys{$key}=$w;
                  $sys{$key}=~s/.*hostid: //;
             }

             if ($w=~/children\[/) {         #first children -> vdev-list
                $v=$w;
                $v=~s/\[.*//;
                last;
             }

      }

      # $v="         children[0]   -> vdev1
      $r=~s/\n$v/\nvdev:/gs;
      @v=();
      @v=split(/vdev:/,$r);      # @v=list of vdevs, [0]=pool, 1..n vdevs

      #each vdev
      ##########
      for ($i=1;$i<@v;$i++) {    # each vdevs


         @d=();
         @d=split(/children/,$v[$i]);


         for ($j=0;$j<@d;$j++) {              # each member disk (value=0: vdev info)


            ## [0] get vdev info
            @w=();
            @w=split(/\n/,$d[$j]);            #type, id, guid, path, devid, phys_path,

            $guid="";

            foreach $w (@w) {                 #get vdev info

                     if ($j eq "0") {         #vdev info only
                              if ($w=~/type: /) {
                                $key=$p."_vdev_$i"."_type";                 #vdev type (mirror, raidz))
                                $disk{$key}=$w;
                                $disk{$key}=~s/.*type: //;
                                $disk{$key}=~s/'//g;
                                $out.="\n\t\t\t\tvdev $i: $disk{$key}";

                                #raid-z level
                                if  (($disk{$key} eq "raidz") && ($d[$j]=~/nparity: /)) {
                                      $key=$p."_vdev_$i"."_nparity";
                                      if ($disk{$key} eq "") {
                                        $disk{$key}=$d[$j];
                                        $disk{$key}=~s/.*nparity: //s;
                                        $disk{$key}=~s/ .*//gs;
                                        $disk{$key}=~s/\D//gs;
                                        $out.=$disk{$key};
                                      }
                                }

                                if ($d[$j]=~/ashift: /) {                    #vdev ashift, post disk info
                                  $key=$p."_vdev_$i"."_ashift";
                                  if ($disk{$key} eq "") {
                                    $disk{$key}=$d[$j];
                                    $disk{$key}=~s/.*ashift: //s;
                                    $disk{$key}=~s/ .*//gs;
                                    $disk{$key}=~s/\D//gs;
                                    $disk{$key}=~s/'//g;
                                    $out.="\t$disk{$key}";
                                  }

                                }
                                if ($d[$j]=~/asize: /) {                    #vdev asize, post disk info
                                  $key=$p."_vdev_$i"."asize";
                                  if ($disk{$key} eq "") {
                                    $disk{$key}=$d[$j];
                                    $disk{$key}=~s/.*asize: //s;
                                    $disk{$key}=~s/ .*//gs;
                                    $disk{$key}=~s/\D//gs;
                                    $disk{$key}=~s/'//g;

                                    $t=$disk{$key};
                                    if ($disk{$key} > 1000000) {
                                       $t=$t/1000000;
                                       $t=sprintf "%0.2f",$t;
                                       $t.=" MB ";
                                    }
                                    if ($disk{$key} > 1000000000) {
                                      $t=$disk{$key};
                                      $t=$t/1000000000;
                                      $t=sprintf "%0.2f",$t;
                                      $t.=" GB ";
                                    }

                                    if ($disk{$key} > 1000000000000) {
                                      $t=$disk{$key};
                                      $t=$t/1000000000000;
                                      $t=sprintf "%0.2f",$t;
                                      $t.=" TB ";
                                    }

                                    $out.="\t$t";
                                  }
                                }


                                next;
                              }

                              if ($w=~/guid: /) {                           #vdev guid
                                $key=$p."_vdev_$i"."_guid";
                                $disk{$key}=$w;
                                $disk{$key}=~s/.*guid: //;
                                $disk{$key}=~s/'//g;
                                $out.="\t$disk{$key}";
                                next;
                              }

                      } # /(0)


                      if ($w=~/guid: /) {                         #disk guid
                         $guid=$w; $guid=~s/.*guid: //;
                         next;
                      }

                      if ($w=~/\spath: /) {                       #disk id from path
                         $id=$w; $id=~s/.*path: //;
                         $id=~s/\'//g;
                         $id=~s/.*\///;
                         $id=~s/s0|p0//;                          # ex c1t1d0
                          next;
                      }


                      if ($w=~/devid: /) {                       # device id
                         $devid=$w; $devid=~s/.*devid: //;
                         $devid=~s/\'//g;
                          next;
                      }

                      if ($w=~/phys_path: /) {                       # phys id
                         $phys=$w; $phys=~s/.*phys_path: //;
                         $phys=~s/\'//g;
                          next;
                      }
                      #write to %disk
                      if (($id ne "") && (!($ids=~/$id\t/))) {
                         $key=$id."_guid";
                         $disk{$key}="$guid";

                         $key=$id."_phys";
                         $disk{$key}="$phys";

                         $key=$id."_devid";
                         $disk{$key}="$devid";

                         $key=$p."_vdev_$i"."_members";
                         $disk{$key}.="$id\t";

                         $out.="\n\t\t\t\t\t\t\t\t$id\t$guid";
                         $key="$id\_cap";
                         $out.="\t$disk{$key}";

                         $key="$id\_product";
                         $out.="\t$disk{$key}<br>$phys <br>$devid";

                         $key="$id\_sn";
                         $out.="<br>$disk{$key}";

                         $ids.="$id\t";

                      }

            }  # /w

         }   # /j

      } #i

  } #/pools

  if ($_[0] eq "1") {
        $out=&list2table($out);
        return ($out);
  }
}

##########################
 sub zfslib_pool_overview {            # print pool overview
 ############### print overview if user zfs, opr parameter "e" for poef encrypted pool
    my ($zfs,@z,$url,$u,$n,$r,@w,$dedup,$z,$out,$pool,@pools,$row,$sync,$l,$b1,$b2,$df1,$df2,$size, $avail,$used,$res,$fres,$ssb);


    #change zfs property
    if ($in{'answered'}) { &zfslib_change_poolproperty; }

    #url
    $url="<a href=\"/cgi-bin/admin.pl?id=$in{'id'}\&amp;action=\&amp;l1=$in{'l1'}\&amp;l2=$in{'l2'}\&amp;l3=$in{'l3'}\&amp;zfs==zfs=\&amp;prop==prop=\&amp;old==old=\">=text=</a>";

    $out="Pool\tVER\tRAW SIZE/ USABLE\tALLOC\tRES\tFRES\tAVAIL zfs [df -h/df -H]\tDEDUP\tFAILM\tEXP\tREPL\tAlt\tGUID\tHEALTH\tAutotrim\tSYNC\tENCRYPT\tACTION\tATIME\tPriCache\tSecCache\tACLinherit\tACLmode\t<a title='System Checkpoints'>CP</a>\t<a title='Pool is mounted readonly'>Rdonly</a>\t";

    $t="rpool"."\t$zfs{'datapools'}"; $t=~s/\t+$//;
    @pools=split(/\t/,$t);

    foreach $pool (@pools) {

        if (($_[0] eq "e") && (!($pool=~/^poef_/))) { next; }
        &log_add("zfslib.pl 2494. zpool get 'version,size,allocated,free,dedupratio,sync,failmode,autoexpand,autoreplace,altroot,guid,health' $pool");
        $zfs=&exe("zpool get 'version,size,allocated,free,dedupratio,failmode,autoexpand,autoreplace,altroot,guid,health' $pool");

        &log_add($zfs);
                                                                         # name      used         avail        refer        mountp
        $size=`sudo $sys{'sbinpath'}zfs list -H $pool`;                  # b1        31.0T        10.5T        1.29M        /b1

        if ($size eq "") {
             $size="$pool   0   0   0   /$pool";                        # ex pool io suspended
        }

        # res and fres
        $res=&zfslib_get_property('reservation',$pool);
        $fres=&zfslib_get_property('refreservation',$pool);

        #small allocation classes
        $t=`sudo /sbin/zpool status $pool`;
        $ssb="-";
        if ($t=~/\tspecial\t/s) {
          $ssb=&zfslib_get_property('special_small_blocks',$pool);       #&mess($ssb);
        }

        $t=~s/ +/\t/g;  @t=();  @t=split(/\t/,$size);
        $used=$t[1];                                                     #name used avail
        $avail=$t[2];

        if ($fres=~/\d/) {
          $size=&zfslib_val2kb($used)+&zfslib_val2kb($avail)-&zfslib_val2kb($fres);
        } else {
          $size=&zfslib_val2kb($used)+&zfslib_val2kb($avail);
        }
        $size=&zfslib_val2xb($size);                                    # useable size = used + avail - fres

        #df values: fs  size used avail
        ##
        if ($pool eq "rpool") {
             $t=`/usr/gnu/bin/df -h /`;                                  # base 1024
        } else {
             $t=`/usr/gnu/bin/df -h /$pool`;


             if ($t=~/\nrpool/s) {
               $t="";             # pool io suspended
             }
        }
        $t=~s/\n/ /gs;                                                   # data in second or third line
        $t=~s/ +/\t/g;  @t=();  @t=split(/\t/,$t);
        $df1=$t[10];    # avail power of 1024

        ##

        if ($pool eq "rpool") {
             $t=`/usr/gnu/bin/df -H /`;                                  # base 1000
        } else {
             $t=`/usr/gnu/bin/df -H /$pool`;


             if ($t=~/\nrpool/s) {
               $t="";             # pool io suspended
             }
        }


        $t=~s/\n/ /gs;                                                   # data in second or third line
        $t=~s/ +/\t/g;  @t=();  @t=split(/\t/,$t);                       # 7=name, 8=size,9=used, 10=avail, use avail only
        $df2=$t[10];       #avail power of 1000

        ##

        #each value on pos 2 in a new line
        # convert to one line with tabs
        @z=split(/\n/,$zfs);

        #poolname
            $u=$url;
            $u=~s/=zfs=/$pool/;
            $u=~s/=prop=//;
            $u=~s/=text=/$pool/;
            $u=~s/=old=/details/;

        $out.="\n$u\t";

        $row="1";
        foreach $z (@z) {
                if ($z=~/^NAME /)  { next; }

                @w=split(/ +/,$z);                                    # name  property  value

                ## size
                if ($w[1] eq 'size') {                                # repla alloc with avail from df
                   if ($size > 0) {
                     $w[2].="/ <b>$size</b>";                          # usable
                   } else {
                     $w[2].="/ <b>0</b>";
                   }
                }


                ## avail
                # less than 10%
                if ($avail > 0) {
                  if (&zfslib_val2kb($size) > 0) {
                     $t=&zfslib_val2kb($avail)/&zfslib_val2kb($size);
                     if ($t < 0.1)  { $t="<font style='color:#ba2f28'>$avail</font>"; }  else { $t=$avail; }
                  }
                } else {
                     $t="0";  # ex not avail, suspended
                }

                if ($w[1] eq 'free') {                                # size replace with avail from df
                     $w[2]="<b>$t</b> &nbsp; [$df1 /$df2]";
                }

                #edit item?
                if ($w[1] eq 'version') {
                        if (!($w[2]=~/\d/)) { $w[2]="5000"; }
                        $u=$url;
                        $u=~s/=zfs=/$pool/;
                        $u=~s/=prop=/poolupgrade/;
                        $u=~s/=text=/$w[2]/;
                        $u=~s/=old=/$w[2]/;
                        $w[2]=$u;
                }

                if ($w[1] eq 'failmode') {
                        $u=$url;
                        $u=~s/=zfs=/$pool/;
                        $u=~s/=prop=/failmode/;
                        $u=~s/=text=/$w[2]/;
                        $u=~s/=old=/$w[2]/;
                        $w[2]=$u;

                }
                if ($w[1] eq 'autoexpand') {
                        $u=$url;
                        $u=~s/=zfs=/$pool/;
                        $u=~s/=prop=/autoexpand/;
                        $u=~s/=text=/$w[2]/;
                        $u=~s/=old=/$w[2]/;
                        $w[2]=$u;

                }
                if ($w[1] eq 'autoreplace') {
                        $u=$url;
                        $u=~s/=zfs=/$pool/;
                        $u=~s/=prop=/autoreplace/;
                        $u=~s/=text=/$w[2]/;
                        $u=~s/=old=/$w[2]/;
                        $w[2]=$u;
                }


                $w[2]=~s/DEGRADED/<font style='color:blue'>DEGRADED<\/font>/gs;                #health
                $w[2]=~s/UNAVAIL/<font style='color:#ba2f28'>UNAVAIL<\/font>/gs;
                $w[2]=~s/FAULTED/<font style='color:#ba2f28'>FAULTED<\/font>/gs;

                $out.="$w[2]\t";

                # add zfs parameter to pool overview
                if ($row eq "3") {   # 4=alloc
                   if ($res=~/\d/) {$out.="$res\t"; } else { $out.="-\t";}
                   if ($fres=~/\d/) {$out.="$fres\t"; } else { $out.="-\t";}
                }
                $row++;
        }


        # add autotrim
                 $t=&exe("zpool get -H -o value autotrim $pool 2>/dev/null",'','zfslib 4883');
                 $t=~s/\n.*//s;         # bad property or on/off
                 if ($t=~/on|off/) {
                    $u=$url;
                    $u=~s/=zfs=/$pool/;
                    $u=~s/=prop=/autotrim/;
                    $u=~s/=text=/$t/;
                    $t="$u\t";
                 } else {
                    $t="-\t";
                 }
                 $out.="$t";

        #add sync
                 # get sync
                 $sync=&exe("zfs get -H  sync $pool",'','zfslib 2575');

                 if ($sync=~/error|suspended|cannot/is) { $sync="-"; }
                 $sync=~s/ +/\t/g;

                 @t=split(/\t/,$sync);
                 $u=$url;
                 $u=~s/=zfs=/$pool/;
                 $u=~s/=prop=/sync/;
                 $u=~s/=text=/$t[2]/;
                 $out.="$u\t";

        #add encrypt
                $t=&exe("$sys{'sbinpath'}zfs get encryption $pool");
                if ($t =~/bad|suspended|cannot/is) { $t="n.a."; } else {
                     $t=~s/.*encryption +//s;
                     $t=~s/ .*//s;
                }
                if (length($t) > 4)  { $t="err"; }
                if ($t eq "on") {
                   $t=&zfslib_get_property('keystatus',$pool);

                   # lock/ unlock
                        $u=$url;
                        $u=~s/=zfs=/$pool/;

                   if ($t eq 'available') {
                        $t="<b class='font_darkblue'>unlocked</b>";
                        $u=~s/=prop=/lock/;
                   } else {
                         $t="<b class='font_darkred'>locked</b>";
                         $u=~s/=prop=/unlock/;
                   }


                   $u=~s/=text=/$t/;
                   $u=~s/=old=/$w[2]/;
                   $w[2]=$u;

                        $out.="$w[2]\t";
                }   else {

                        #&mess($t);
                        $out.="$t\t";
                }
        #add clear errors
                $u=$url;
                $u=~s/=zfs=/$pool/;
                $u=~s/=prop=/clear/;
                $u=~s/=text=/clear errors/;
                $out.=$u;

        # add atime for pool
                        $t=&exe("zfs get -H atime $pool");
                        if ($t =~/on/s) { $t="on"; } else { $t="off"; }
                        $u=$url;
                        $u=~s/=zfs=/$pool/;
                        $u=~s/=prop=/atime/;
                        $u=~s/=text=/$t/;
                        $u=~s/=old=/$t/;
                        $out.="\t$u";


         # primary cache
                $t=&exe("zfs get -H primarycache $pool");
                chomp($t);
                @t=split(/\s+/,$t);
                $t=$t[2];
                $u=$url;
                $u=~s/=zfs=/$pool/;
                $u=~s/=prop=/p-cache/;
                $u=~s/=text=/$t/;
                $u=~s/=old=/$t/;
                $out.="\t$u";
        # secondary cache
                $t=&exe("zfs get -H secondarycache $pool");
                @t=split(/\s+/,$t);
                $t=$t[2];
                $u=$url;
                $u=~s/=zfs=/$pool/;
                $u=~s/=prop=/s-cache/;
                $u=~s/=text=/$t/;
                $u=~s/=old=/$t/;
                $out.="\t$u";

        # aclinherit
                $t=&exe("zfs get -H aclinherit $pool");
                @t=split(/\s+/,$t);
                $t=$t[2];
                $u=$url;
                $u=~s/=zfs=/$pool/;
                $u=~s/=prop=/aclinherit/;
                $u=~s/=text=/$t/;
                $u=~s/=old=/$t/;
                $out.="\t$u";
        # aclmode
                $t=&exe("zfs get -H aclmode $pool");
                @t=split(/\s+/,$t);
                $t=$t[2];
                $u=$url;
                $u=~s/=zfs=/$pool/;
                $u=~s/=prop=/aclmode/;
                $u=~s/=text=/$t/;
                $u=~s/=old=/$t/;
                $out.="\t$u";

        # Checkpoints
                $u=&exe("zpool status $pool | grep checkpoint:");
                if ($u=~/created/) {
                    $out.="\tyes";
                } else {
                    $out.="\tno";
                }
        # readonly
                $u=&exe("zpool get -H -o value readonly $pool");
                if ($u=~/on/) {
                    $out.="\t<font style='color:#ba2f28'>yes</font>";
                } else {
                    $out.="\tno";
                }

    }

    if (!($out =~/\d/s)) { $out.="-none-"; }
    $l= &list2table($out,'100%,200px');
    $l="<!--wsarea-->\n<div id='wsarea'>\n$l\n</div>\n<!--wsarea_end-->\n";    # realtime update of list only
    print $l;
    print "$txt{'pool_footer'}";

    #ask property
    if (($in{'prop'} ne "") && ($in{'answered'} ne "1")) { print &zfslib_ask_poolproperty; }

    #zpool status with details?
    if ($in{'old'} eq "details") {

         print "\n\nPool Properties:<hr>";
         $r=&exe("$sys{'sbinpath'}zpool get all $in{'zfs'}");
         $r=~s/  +/\t/gs;
         print &list2table($r);

         print "<pre>\n\n<hr>zpool status -v $in{'zfs'}<hr>";
         print &exe("/usr/sbin/zpool status -v $in{'zfs'}");

         print "<hr>\n";
         print "Pool details zdb -C $in{'zfs'}\n";
         print &exe("zdb -C $in{'zfs'}");

         print "\n\n</pre></body></html>";   &log_end; exit;
    }

    print "\n</body></html>";
    return;

 }


#####################
 sub zfslib_ask_poolproperty {       #hide: ask for property
 #####################        (div window in front)

    if ($in{'answered'}) { &zfslib_change_poolproperty; exit; }

    my ($ask,$close,$form,$info,$t,$opt,$r);
    $close="<a href=\"/cgi-bin/admin.pl?id=$in{'id'}\&amp;action=\&amp;l1=$in{'l1'}\&amp;l2=$in{'l2'}\&amp;l3=$in{'l3'}\&amp;list=$in{'list'}\&amp;zfs=$in{'zfs'}\"><b>\&nbsp;cancel\&nbsp;/close\&nbsp;</b></a>";

    #ask poolupgrade
    if ($in{'prop'} eq "poolupgrade") {
      $info="The new ZFS-Version must be supported, if you want to import the pool on another ZFS-Server";
      $form="Upgrade pool ". &select('poolupgrade','yes',$r);             #
    }


    #ask p-cache
    if ($in{'prop'} eq "p-cache") {
      $info="Default=all for best performance. Set none for benchmarks without cache effecs.<br>This is a pool default. You can force this property per filesystem.";
      $form="set primarycache to ". &select('p-cache','all,metadata,none',$in{'old'});             #
    }

    #ask s-cache
    if ($in{'prop'} eq "s-cache") {
      $info="Default=all for best performance. Set none for benchmarks without cache effecs.<br>This is a pool default. You can force this property per filesystem.";
      $form="set secondarycache to ". &select('s-cache','all,metadata,none',$in{'old'});             #
    }

    #ask atime
    if ($in{'prop'} eq "atime") {
      $t=&exe("zfs get -H atime $in{'zfs'}");
      chomp($t);
      if ($t =~/on/s)  { $r="off"; } else { $r="on"; }
      $info="Log access time. Set Off for best performance. <br>This is a pool default. You can force this property per filesystem.";
      $form="set atime to ". &select('atime','on,off',$r);             #
    }

    #aclmode
    if ($in{'prop'} eq "aclmode") {
      $t=&exe("zfs get -H -o value aclmode $in{'zfs'}");
      chomp($t);

      $info="ACL mode settings. Passthrough is a most open setting while the default discard restrict options via chmod. <br>This is a pool default. You can force this property per filesystem.";
      $form="set aclmode to ". &select('aclmode','passthrough,discard,groupmask,restricted',$t);             #
    }

    #aclinherit
    if ($in{'prop'} eq "aclinherit") {
      $t=&exe("zfs get -H -o value aclinherit $in{'zfs'}");
      chomp($t);
      $info="ACL-Inheritance settings. Set passthrough for a Windows alike behaviour. <br>The default value is restricted that removes write-owner/write acl. <br>This is a pool default. You can force this property per filesystem.";
      $form="set aclinherit to ". &select('aclinherit','passthrough,discard,noallow,restricted,passthrough-x',$t);             #
    }

    #ask autotrim
    if ($in{'prop'} eq "autotrim") {
      $r=&exe("zpool get -H -o value autotrim $in{'zfs'}");
      chomp($r);
      if ($r eq "on")  { $r="off"; } else { $r="on"; }
      $info="This feature enables autotrim for a pool<br>This feature is new so use with care!";
      $form="set autotrim to ". &select('autotrim','on,off',$r);             #
    }

    #ask autoexpand
    if ($in{'prop'} eq "autoexpand") {
      $r=&exe("zpool get autoexpand $in{'zfs'}"); $r=~s/^.*autoexpand\s+//s; $r=~s/\s.*//s;
      chomp($r);
      if ($r eq "on")  { $r="off"; } else { $r="on"; }
      $info="This feature enable you to expand a LUN after replacing/resilvering a drive. The resulting pool can access the expanded space without having to export and import pool or reboot the system. default is off";
      $form="set autoexpand to ". &select('autoexpand','on,off',$r);             #
    }

    #ask autoreplace
    if ($in{'prop'} eq "autoreplace") {
      $r=&exe("zpool get autoreplace $in{'zfs'}"); $r=~s/^.*autoreplace\s+//s; $r=~s/\s.*//s;
      chomp($r);
      if ($r eq "on")  { $r="off"; } else { $r="on"; }
      $info="If set to on, any new device, found in the same physical location as a device that previously belonged to the pool, is automatically formatted and replaced. default is off";
      $form="set autoreplace to ". &select('autoreplace','on,off',$r);             #
    }


    #ask failmode
    if ($in{'prop'} eq "failmode") {
        $r=&exe("zpool get failmode $in{'zfs'}"); $r=~s/^.*failmode\s+//s; $r=~s/\s.*//s;
        chomp($r);
        $form="set failmode to ". &select('failmode','wait,continue,panic',$r);             # name, values, selected-> return htmlcode   ;
        $info="Failmode property is needed for determining the behavior of a catastrophic pool failure due to a loss of device connectivity or the failure of all devices in the pool. The failmode property can be set to these values: wait, continue, or panic. The default value is wait, which means you must reconnect the device or replace a failed device and clear the error with the zpool clear command.";
    }


    #ask sync
    if ($in{'prop'} eq "sync") {
        $form="set sync mode to ". &select('sync','standard,always,disabled');             # name, values, selected-> return htmlcode   ;
        $info="How should ZFS handle sync write requests? <br>standard: depends on client request. <br>This is a pool default. You can force this property per filesystem.";
    }

    #ask lock
    if ($in{'prop'} eq "lock") {
        $form="encrypted $in{'zfs'}: <input type='text' class='input' name='text' value='lock it' style='width: 150px' readonly> ";
        $info="if you want to reaccess this ZFS, you have to unlock it with your passphrase<br>all shares are forced to be offline";
    }
    #ask unlock
    if ($in{'prop'} eq "unlock") {
        $form="unlock with Passphrase <input type='password' name='pass' value='' style='width: 150px'> ";
        $info="enter your passphrase to unlock this ZFS";
    }
    #ask clear
    if ($in{'prop'} eq "clear") {
        $form="clear error-state: ". &select('clear','yes',$r);
        $info="clear pool error-state is only possible, if there are no current errors.";
    }

     ### form ###
     $ask=&ask_pop("$form","$info","$opt");
     return ($ask);
 }


######################
 sub zfslib_change_poolproperty {                 #hide:
 #######################    set pool details
 my ($r,$s,$t);


   ##### poolupgrade
   if ($in{'prop'} eq "poolupgrade") {
        $r= &exe("zpool upgrade $in{'zfs'}");
        print $r;
        sleep(2);
        $in{'prop'}="";
        &reload;
   }


   ##### autotrim
   if ($in{'prop'} eq "autotrim") {
        $r= &exe("zpool set autotrim=$in{'autotrim'} $in{'zfs'}");
   }

   ##### sync
   if ($in{'prop'} eq "sync") {
        $r= &exe("zfs set sync=$in{'sync'} $in{'zfs'}");
   }

   #p-cache
   if ($in{'prop'} eq "p-cache") {
        $r= &exe("zfs set primarycache=$in{'p-cache'} $in{'zfs'}");
   }

   #s-cache
   if ($in{'prop'} eq "s-cache") {
        $r= &exe("zfs set secondarycache=$in{'s-cache'} $in{'zfs'}");
   }

   #atime
   #####
   if ($in{'prop'} eq "atime") {
        $r= &exe("zfs set atime=$in{'atime'} $in{'zfs'}");
   }

   #aclmode
   #####
   if ($in{'prop'} eq "aclmode") {
        $r= &exe("zfs set aclmode=$in{'aclmode'} $in{'zfs'}");
   }

   #aclinherit
   #####
   if ($in{'prop'} eq "aclinherit") {
        $r= &exe("zfs set aclinherit=$in{'aclinherit'} $in{'zfs'}");
   }

   ##### failmode
   if ($in{'prop'} eq "failmode") {
        $r= &exe("zpool set failmode=$in{'failmode'} $in{'zfs'}");
   }
   ##### autoexpand
   if ($in{'prop'} eq "autoexpand") {
        $r= &exe("zpool set autoexpand=$in{'autoexpand'} $in{'zfs'}");
   }
   ##### autoreplace
   if ($in{'prop'} eq "autoreplace") {
        $r= &exe("zpool set autoreplace=$in{'autoreplace'} $in{'zfs'}");
   }


   ##### clear
   if ($in{'prop'} eq "clear") {
        $r= &exe("zpool clear $in{'zfs'}");
        if (-f "/tmp/poolstat.log") {  &exe("rm /tmp/poolstat.log"); }
   }

   ##### lock
   if ($in{'prop'} eq "lock") {
        print "please wait...\n";
### martingo 20180306 >>> Keys sicherheitshalber rekursiv entladen; vermutlich nicht notwendig, schadet aber nicht
        #$r= &exe("$sys{'sbinpath'}zfs key -u -f $in{'zfs'}");
        $r= &exe("$sys{'sbinpath'}zfs key -u -r -f $in{'zfs'}");
### <<<
        &zfslib_refresh_svc;
   }



   ##### unlock
   if ($in{'prop'} eq "unlock") {
        print "please wait...\n";
        use Expect;
        my $exp; #->raw_pty(1);
        $exp = Expect->spawn("sudo $sys{'sbinpath'}zfs mount $in{'zfs'}") or die "Couldn't mount: $!\n";

        $exp->expect(2,
                [ 'Enter' =>
                         sub {
                             $exp->send("$in{'pass'}\r");
                             print "<br><br>\n";
                             exp_continue;
                         }
                ]
         );
         $exp->soft_close();
         &zfslib_refresh_svc;
         sleep(1);
   }

   #if (-f '/tmp/nappit/zfs.cfg') { unlink('/tmp/nappit/zfs.cfg'); }  # old
   if (-f "/tmp/nappit/zfs_list.zfs") { `sudo rm /tmp/nappit/*.zfs`;  }

   if ($r ne"") {  &mess($r); }

   $in{'prop'}="";
   &reload;


  #######
 }





####################
sub zfslib_list_pools  {   # list pools par: 'source (all,exported,destroyed,comma,nl,br,array,),state,status,action,scan,config'
####################
                      ##########################
                      # comma,nl,br,array -> form of return value
                      # header -> incl header,
                      # all all pools inkl sys
                      #   (exported) and/or D (destroyed) pools only
                      #########################
                      # default, list datapools as tablist  tank\tdata
                      # options: state,status,action,scan,config

 #$_[0]="state,status,header,imp,des,action";
  my (@w,$w,$key,$k,$n,@l,$t,@d,$d,$m,$ant,$par);
  $par=$_[0];

  if ($_[0] =~/imp|des/i) {
    if ($_[0] =~/ex/i)  {   $ant.=&exe('zpool import');   }   # get exported pools
    if ($_[0] =~/des/i) {   $ant.=&exe('zpool import -D'); }  # get destroyed pools
  } else {
          $ant=&exe('zpool status');                             # get status info
  }



  $ant=~s/\s/ /gs; $ant=~s/ +/ /gs; $ant=~s/^ +//gs;     #every item trimmed in new line
  $ant=~s/pool: /\npool:/gs;                                #in new line

  #valid pools
  @w=split(/\n/,$ant);
  foreach $n (@w) {
      if ($_[0] =~/all/i)                                         {  push (@n,$n); }
      if (($_[0] =~/ex|des/i) &&  ($n=~/pool can.* be imported/))  {
        if ($n=~/DESTROYED/) {
                      $n=~s/ id: /_D/;
        } else {
                      $n=~s/ id: /_/;
         }
         push (@n,$n);
         }
  }
  $ant=join("\n",@n);

  #  &mess($ant);


  $ant=~s/state: /\nstate:/gs;                                  #in new line
  $ant=~s/scan: /\nscan:/gs;                                    #in new line
  $ant=~s/status: /\nstatus:/gs;                                #in new line
  $ant=~s/action: /\naction:/gs;                                #in new line
  $ant=~s/config: /\nconfig:/gs;                                #in new line
  $ant=~s/id: /\nid:/gs;                                        #in new line

  # example
  # pool:syspool
  # id:1793635354   (opt)
  # state:ONLINE (DESTROYED)
  # status:The pool is formatted using an older on-disk format. The pool can still be used, but some features are unavailable.
  # action:Upgrade the pool using 'zpool upgrade'. Once this is done, the pool will no longer be accessible on older software versions.
  # scan:none requested
  # config:NAME STATE READ WRITE CKSUM syspool ONLINE 0 0 0 c1t0d0s0 ONLINE 0 0 0 errors: No known data errors

  @l=split(/\n/,$ant);

#test
#print @l; exit;
#&print_array(@l);

  my $pools;
  my $header;
  my $lastpool;
  if ($_[0]=~/header/) { $header="pool\t"; }
  if (($_[0]=~/header/i) && ($_[0]=~/state/i))  { $header.="state\t"; }
  if (($_[0]=~/header/i) && ($_[0]=~/status/i)) { $header.="status\t"; }
  if (($_[0]=~/header/i) && ($_[0]=~/action/i)) { $header.="action\t"; }
  if (($_[0]=~/header/i) && ($_[0]=~/scan/i))   { $header.="scan\t"; }
  if (($_[0]=~/header/i) && ($_[0]=~/config/i)) { $header.="config\t"; }
  $header=~s/\t$//;

  foreach my $l (@l) {
        if (!($l =~/\:/)) {  next; }                        # no value

        $l=~s/ +$//;
        @w=split(/:/,$l);                               #pool:tank
        $w[1]=~s/ +$//g;                                #rtrim

        if ($w[0] eq "pool") {
                 $pool=~s/\t+$//s;
                if (($w[1] ne $zfs{'syspool'}) || ($_[0]=~/all/))  {        # incl syspool
                        $pools.="\n$w[1]\t";        # tablist
                         $lastpool=$w[1];
                }
                 next;
        }
         #only pools

         if (!($_[0]=~/state|status|action|scan|config/))   { next; }        #only poollist
         #otherwise every pool in a line

         if (($_[0]=~/all/i) || ($lastpool ne $zfs{'syspool'}))  {

          if (($w[0] eq "state") && ($_[0]=~/state/i)) {                        # tank_state
                 $par.="nl";
                 $w[1]=~s/\t//g;   $w[1]=~s/^\s+|\s+$//g;
                $pools.="$w[1]\t";
                 next;
           }


          if (($w[0] eq "status") && ($_[0]=~/status/i)) {
                 $par.="nl";
                 $w[1]=~s/\t//g;   $w[1]=~s/^\s+|\s+$//g;
                $pools.="$w[1]\t";
                 next;
          }


          if (($w[0] eq "action") && ($_[0]=~/action/i)) {
                 $par.="nl";
                 $w[1]=~s/\t//g;    $w[1]=~s/^\s+|\s+$//g;
                 $pools.="$w[1]\t";
                 next;
          }

          if (($w[0] eq "scan")  && ($_[0]=~/scan/i)) {
                 $par.="nl";
                 $w[1]=~s/\t//g;    $w[1]=~s/^\s+|\s+$//g;
                 $pools.="$w[1]\t";
                 next;
          }


           if (($w[0] eq "config")  && ($_[0]=~/config/i)) {
                 $par.="nl";
                 $w[1]=~s/\t//g;   $w[1]=~s/^\s+|\s+$//g;
                 $pools.="$w[1]\t";
                 next;
          }
         }
  }
  ##
  $pools=~s/^\n//;  $pools=~s/\t$//s;
  $pools=~s/\t\n/\n/gs;

  if ($par=~/header/i) {
     $pools="$header\n$pools";
  }

#default tablist
  #&mess($pools);

  #rows
  if ($par=~/nl/)  {
        if ($par=~/br/)  {
                        $pools=~s/\n/<br>\n/gs;
        }
        $par="zfslib_list_pools_$_[0]_nl";
        $current{$par}=$pools;
        return ($pools);
  }

  #array
  if ($par=~/array/)  {
          @w=();
         @w=split(/\n/,$pools);
         $par="zfslib_list_pools_$_[0]_array";
         $current{$par}=$pools;
         return (@w);
  }

  #kommalist
  if ($par=~/omma/)  {
          $pools=~s/\n/,/gs;
         $par="zfslib_list_pools_$_[0]_comma";
         $current{$par}=$pools;
         return ($pools);          #komma, comma

  }
  #default tablist

  $pools=~s/\n/\t/gs;
  $par="zfslib_list_pools_$_[0]_tab";
  $current{$par}=$pools;
  return ($pools);

}

#############################     when smb set to off
sub zfslib_remember_shareacl {
#############################
    # shareacl as zfs property shareacl_00:, shareacl_01:, ..
    my (@r,@t,$r,$fi,$t,@acl);
    $t=$in{'zfs'}; $t=~s/.*\///;        #fs name only
    $fi="/$in{'zfs'}/.zfs/shares/$t";  #share acl file ex /tank/data/.zfs/shares/data
    #&mess($fi);


    # step1: delete old
    ###################
    $r=`sudo /sbin/zfs get all $fs | grep shareacl_ | grep -v @`;   chomp($r);      # without snaps
    # ex
    # shareacl_00:                  user:a:rwxpdDaARWcCos:-------:deny              local

    @t=split(/\n/,$r);
    #&print_array(@t);

    foreach my $i (@t) {
            @p=split(/\s+/,$i);               # prop=val1
            print "zfs inherit $p[1] $fs <br>";
            `sudo /sbin/zfs inherit $p[1] $in{'zfs'}`;
    }

    #step2, get share acl
    $r=`/usr/bin/ls -V $fi`; chomp($r);

    @acl=split(/\n/,$r);
    splice @acl, 0, 1;         # remove first entry (filename)

    #step3 save shareacl as prop

    for (my $i=0; $i < @acl; $i++) {
       $n="0$i"; $n=~s/^00/0/; if ($n eq "0") {$n="00"; }      # count 00, 01, 02, ..
       $acl[$i]=~s/ +//;    # remove spaces
       print "zfs set shareacl_$n:=$acl[$i] $in{'zfs'}<br>";
       `sudo /usr/sbin/zfs set shareacl_$n:=$acl[$i] $in{'zfs'}`;
    }

    sleep(1);
    #&print_array (@acl);       # debug, exit;
}

#############################     when smb set to off
sub zfslib_set_shareacl {
#############################
    # shareacl as zfs property shareacl_00:, shareacl_01:, ..
    my (@r,@t,$r,$fi,$t,@acl,$n,@n);
    $t=$in{'zfs'}; $t=~s/.*\///;        #fs name only
    $fi="/$in{'zfs'}/.zfs/shares/$t";   #share acl file ex /tank/data/.zfs/shares/data

    # full
    if ($in{'shareacl'} eq "everyone=full") { return; }   #default



    # modify
    if ($in{'shareacl'} eq "root=full, everyone=modify") {

       print   "chmod A=user:root:rwxpdDaARWcCos:-------:allow<br>";
       $r=`sudo /usr/bin/chmod A=user:root:rwxpdDaARWcCos:-------:allow $fi`;   # root always full

       print   "/usr/bin/chmod A1+everyone\@:rwxpdDaARWc--s:-------:allow $fi<br>";
       $r=`sudo /usr/bin/chmod A1+everyone\@:rwxpdDaARWc--s:-------:allow $fi`;    # other modify
       return;
    }

    # read
    if ($in{'shareacl'} eq "root=full, everyone=read") {

       print   "chmod A=user:root:rwxpdDaARWcCos:-------:allow<br>";
       $r=`sudo /usr/bin/chmod A=user:root:rwxpdDaARWcCos:-------:allow $fi`;   # root always full

       print   "/usr/bin/chmod A1+everyone\@:r-----a-R-c--s::-------:allow $fi<br>";
       $r=`sudo /usr/bin/chmod A1+everyone\@:r-----a-R-c--s:-------:allow $fi`;    # other modify
       return;
    }

    #last setting
    #&print_hash(%in);
    print "restore last share acl<br>";

    # otherwise restore  last properies
    #ex shareacl_00:=everyone@:rwxpdDaARWcCos:-------:allow
    #ex shareacl_01:=..
    # ..
    $r=`sudo /sbin/zfs get all $in{'zfs'} | grep shareacl_`;
    @t=split(/\n/,$r); @t=reverse sort(@t);        # sort

    foreach my $l (@t) {                                      #
          if ($l=~/:root:/) {   next; }
          @p=split(/ +/,$l);
          push (@n,$p[2]);
    }
    push(@n,"user:root:rwxpdDaARWcCos:-------:allow");          # allow/add root always (full)

    #
    #print_array(@n);

    for (my $i=0; $i<@n; $i++) {
        if ($i==0) {
           print "chmod A=$n[$i]<br>";
           $r=`sudo /usr/bin/chmod A=$n[$i] $fi`;
        } else  {
           print "chmod A$i+$n[$i]<br>";
           $r=`sudo /usr/bin/chmod A$n+$n[$i] $fi`;
        }
    }
    sleep(2);
}

########### eof ##########
1;
